﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;

using JabboServerCMD;
using JabboServerCMD.Core;
using JabboServerCMD.Core.Sockets;
using JabboServerCMD.Core.Settings;
using JabboServerCMD.Core.Managers;
using JabboServerCMD.Core.Systems;
using JabboServerCMD.Core.Encryption;

namespace JabboServerCMD
{
    class Program
    {
        // TODO:
        // Make a kill switch - load page and if it doesnt exist kill the server
        // 

        private static Thread serverMonitor = new Thread(new ThreadStart(monitorServer));

        static void Main(string[] args)
        {
            Initialise();

            while (true)
            {
                Console.Read();
            }
        }

        static void Initialise()
        {
            Console.WriteLine("=========================================");
            Console.WriteLine(" Jabbo V3 Server");
            Console.WriteLine(" (C) Thomas Vermaercke 2006-2011");
            Console.WriteLine("=========================================");
            Console.WriteLine("");
            Console.WriteLine("Opstarten...");

            INIFile MyINIFile = new INIFile("settings.ini");

            Config.linux = bool.Parse(MyINIFile.GetValue("config", "linux", "true"));

            if (!Config.linux)
            {
                Console.WindowHeight = Console.LargestWindowHeight - 25;
                Console.WindowWidth = Console.LargestWindowWidth - 25;
                Console.Title = "Jabbo Server";
            }

            Config.debug = bool.Parse(MyINIFile.GetValue("config", "debug", "false"));
            Config.port = int.Parse(MyINIFile.GetValue("config", "port", "3500"));
            Config.maxconn = int.Parse(MyINIFile.GetValue("config", "maxcon", "150"));
            Config.maxconnbacklog = int.Parse(MyINIFile.GetValue("config", "maxconbl", "50"));

            Config.publickey = MyINIFile.GetValue("security", "key", "");

            Config.dbHost = MyINIFile.GetValue("mysql", "host", "localhost");
            Config.dbPort = int.Parse(MyINIFile.GetValue("mysql", "port", "3306"));
            Config.dbUsername = MyINIFile.GetValue("mysql", "username", "root");
            Config.dbPassword = MyINIFile.GetValue("mysql", "password", "root");
            Config.dbName = MyINIFile.GetValue("mysql", "database", "jabbo");

            Console.WriteLine("    Opties succesvol ingelezen.");

            if (MySQL.openConnection(Config.dbHost, Config.dbPort, Config.dbName, Config.dbUsername, Config.dbPassword) == false)
                return;

            Console.WriteLine("");

            TextsManager.Init();
            UserManager.Init();
            CatalogueManager.Init();
            RankManager.Init();
            NavigatorManager.Init();

            if (SocketServer.Init(Config.port, Config.maxconn) == false)
                return;
            if (WebsiteSocketServer.Init(Config.port + 1, "127.0.0.1") == false)
                return;

            CsHTTPServer.CsHTTPServer HTTPServer;
            HTTPServer = new CsHTTPServer.MyServer(3502);
            HTTPServer.Start();


            serverMonitor.Priority = ThreadPriority.Lowest;
            serverMonitor.Start();

            Console.WriteLine("Server draait...");
            Console.WriteLine("");
        }

        private static void monitorServer()
        {
            var startTime = DateTime.Now;
            while (true)
            {
                if (!Config.linux)
                {
                    Console.Title = "Jabbo Server [Memory: " + GC.GetTotalMemory(false) / 1024 + "KB | Uptime: " + (DateTime.Now - startTime) + "]";
                }
                Thread.Sleep(6000);
            }
        }
    }
}
