document.domain=document.domain;
(function() {
    var HANDSHAKE_TIMEOUT = 30000;
    var RETRY_INTERVAL = 250;
    var RETRY_TIMEOUT = 30000;
    
    Orbited = {};
    
    Orbited.settings = {};
    Orbited.settings.hostname = document.domain;
    Orbited.settings.port = 8001;
    Orbited.settings.protocol = location.protocol.slice(0, -1);
    Orbited.settings.log = false;
    Orbited.settings.streaming = true;
    Orbited.settings.HEARTBEAT_TIMEOUT = 6000;
    Orbited.settings.POLL_INTERVAL = 2000;
    Orbited.settings.pageLoggerHeight = '200px';
    Orbited.settings.pageLoggerWidth = null;
    Orbited.settings.enableFFPrivileges = false;
    Orbited.singleton = {};
    
    
    // Orbited CometSession Errors
    Orbited.Errors = {};
    Orbited.Errors.ConnectionTimeout = 101;
    Orbited.Errors.InvalidHandshake = 102;
    Orbited.Errors.UserConnectionReset = 103;
    Orbited.Errors.Unauthorized = 106;
    Orbited.Errors.RemoteConnectionFailed = 108;
    
    Orbited.Statuses = {};
    Orbited.Statuses.ServerClosedConnection = 201;
    Orbited.Statuses.SocketControlKilled = 301;
    
    Orbited.util = {};
    
    Orbited.util.browser = null;
    if (typeof(ActiveXObject) != "undefined") {
        Orbited.util.browser = 'ie';
    } else if (navigator.userAgent.indexOf('WebKit') != -1 || navigator.userAgent.indexOf('Konqueror') != -1) {
        Orbited.util.browser = 'webkit';
    } else if (navigator.product == 'Gecko' && window.find && !navigator.savePreferences) {
        Orbited.util.browser = 'firefox';
    } else if((typeof window.addEventStream) === 'function') {
        Orbited.util.browser = 'opera';
    }
    

    ////
    // NB: Base64 code was borrowed from Dojo; we had to fix decode for not
    //     striping NULs though.  Tom Trenka from Dojo wont fix this because
    //     he claims it helped to detect and avoid broken encoded data.
    //     See http://svn.dojotoolkit.org/src/dojox/trunk/encoding/base64.js
    //     See http://bugs.dojotoolkit.org/ticket/7400
    (function(){
        Orbited.base64 = {};
        
        var p = "=";
        var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
        
        if (window.btoa && window.btoa('1') == 'MQ==') {
            Orbited.base64.encode = function(data) { return btoa(data); };
            Orbited.base64.decode = function(data) { return atob(data); };
            return;
        }
        
        Orbited.base64.encode=function(/* String */ba){
            //  summary
            //  Encode a string as a base64-encoded string
            var s=[];
            var l=ba.length;
            var rm=l%3;
            var x=l-rm;
            for (var i=0; i<x;){
            var t=ba.charCodeAt(i++)<<16|ba.charCodeAt(i++)<<8|ba.charCodeAt(i++);
            s.push(tab.charAt((t>>>18)&0x3f));
            s.push(tab.charAt((t>>>12)&0x3f));
            s.push(tab.charAt((t>>>6)&0x3f));
            s.push(tab.charAt(t&0x3f));
            }
            //  deal with trailers, based on patch from Peter Wood.
            switch(rm){
            case 2:
            t=ba.charCodeAt(i++)<<16|ba.charCodeAt(i++)<<8;
            s.push(tab.charAt((t>>>18)&0x3f));
            s.push(tab.charAt((t>>>12)&0x3f));
            s.push(tab.charAt((t>>>6)&0x3f));
            s.push(p);
            break;
            case 1:
            t=ba.charCodeAt(i++)<<16;
            s.push(tab.charAt((t>>>18)&0x3f));
            s.push(tab.charAt((t>>>12)&0x3f));
            s.push(p);
            s.push(p);
            break;
            }
            return s.join("");  //    string
        };
        
        
        Orbited.base64.decode=function(/* string */str){
            //  summary
            //  Convert a base64-encoded string to an array of bytes
            var s=str.split("");
            var out=[];
            var l=s.length;
            var tl=0;
            while(s[--l]==p){ ++tl; }   //    strip off trailing padding
            for (var i=0; i<l;){
            var t=tab.indexOf(s[i++])<<18;
            if(i<=l){ t|=tab.indexOf(s[i++])<<12; }
            if(i<=l){ t|=tab.indexOf(s[i++])<<6; }
            if(i<=l){ t|=tab.indexOf(s[i++]); }
            out.push(String.fromCharCode((t>>>16)&0xff));
            out.push(String.fromCharCode((t>>>8)&0xff));
            out.push(String.fromCharCode(t&0xff));
            }
            // strip off trailing padding
            while(tl--){ out.pop(); }
            return out.join(""); //     string
        };
    })();
    



    Orbited.loggers = {};
    Orbited.Loggers = {};
    Orbited.util.loggingSystem = null;
    
    if (window.Log4js) {
        Orbited.util.loggingSystem = 'log4js';
    }
    else if (window.console && console.firebug && console.firebug != "1.3.0") {
        Orbited.util.loggingSystem = 'firebug';
    }
    
    Orbited.getLogger = function(name) {
        if (!Orbited.loggers[name]) {
            var logger = null;
            switch (Orbited.util.loggingSystem) {
            case 'firebug':
            logger = new Orbited.Loggers.FirebugLogger(name);
            break;
            case 'log4js':
            logger = new Orbited.Loggers.Log4jsLogger(name);
            break;
            
            default:
            logger = new Orbited.Loggers.PageLogger(name);
            break;
            }
            Orbited.loggers[name] = logger;
        }
        return Orbited.loggers[name];
    };
    
    // TODO: is it confusing to have Orbited.Loggers be the various logging classes
    //     and Orbited.loggers be actual instances of logging classes?
    
    Orbited.Loggers.FirebugLogger = function(name) {
        var self = this;
        self.name = name;
        self.enabled = false;
        var padArgs = function(args) {
            var newArgs = [ name + ":" ];
            for (var i = 0; i < args.length; ++i) {
            newArgs.push(args[i]);
            }
            return newArgs;
        };
        self.log = function() {
            if (!self.enabled) { return; }
            console.log.apply(this, padArgs(arguments));
        };
        self.debug = function() {
            if (!self.enabled) { return; }
            console.debug.apply(this, padArgs(arguments));
        };
        self.info = function() {
            if (!self.enabled) { return; }
            console.info.apply(this, padArgs(arguments));
        };
        self.warn = function() {
            if (!self.enabled) { return; }
            console.warn.apply(this, padArgs(arguments));
        };
        self.error = function() {
            if (!self.enabled) { return; }
            console.error.apply(this, padArgs(arguments));
        };
        self.assert = function() {
            if (!self.enabled) { return; }
            var newArgs = [arguments[0], name + ":" ];
            for (var i = 1; i < arguments.length; ++i) {
            newArgs.push(arguments[i]);
            }
            console.assert.apply(this, newArgs);
        };
        self.trace = function() {
            if (!self.enabled) { return; }
            console.trace.apply(this, padArgs(arguments));
        };
    };
    Orbited.singleton.pageLoggerPane = null;

    Orbited.Loggers.PageLogger = function(name) {
        var self = this;
        self.enabled = false;
        self.name = name;

        var checkPane = function() {
            if (!Orbited.singleton.pageLoggerPane) {
            var p = document.createElement("div");
            p.border = "1px solid black";
            if(Orbited.settings.pageLoggerHeight) {
                p.style.height = Orbited.settings.pageLoggerHeight;
            }
            if(Orbited.settings.pageLoggerWidth) {
                p.style.height = Orbited.settings.pageLoggerWidth;
            }
            
            p.style.overflow = "scroll";
            document.body.appendChild(p);
            Orbited.singleton.pageLoggerPane = p;
            }
        };
        var show = function(data) {
            checkPane();
            var d = document.createElement('div');
            d.innerHTML = data;
            Orbited.singleton.pageLoggerPane.appendChild(d);
            Orbited.singleton.pageLoggerPane.scrollTop = Orbited.singleton.pageLoggerPane.scrollHeight;
        };
        self.log = function() {
            if (!self.enabled) { return; }
            var newArgs = [ "log", new Date(), "debug", "<b>" + name + "</b>" ];
            for (var i = 0; i < arguments.length; ++i) {
            newArgs.push(arguments[i]);
            }
            show(newArgs.join(", "));
        };
        self.debug = function() {
            if (!self.enabled) { return; }
            var newArgs = [ new Date(), "debug", "<b>" + name + "</b>" ];
            for (var i = 0; i < arguments.length; ++i) {
            newArgs.push(arguments[i]);
            }
            show(newArgs.join(", "));
        };
        self.info = function() {
            if (!self.enabled) { return; }
            var newArgs = [ new Date(), "info", "<b>" + name + "</b>" ];
            for (var i = 0; i < arguments.length; ++i) {
            newArgs.push(arguments[i]);
            }
            show(newArgs.join(", "));
        };
        self.warn = function() {
        };
        self.error = function() {
        };
        self.assert = function() {
        };
        self.trace = function() {
        };
    };


    Orbited.Loggers.Log4jsLogger = function(name) {
        var self = this;
        self.name = name;
        // NOTE: Why oh WHY doesn't Log4js accept dots in the logger names, and
        //         more importantly, why don't they have reasonble error messages?!
        var log4jsName = name;
        while (log4jsName.indexOf('.') != -1) {
            log4jsName = log4jsName.replace('.', '_');
        }
        var logger = Log4js.getLogger(log4jsName);
        self.logger = logger;
        logger.setLevel(Log4js.Level.OFF);
        
        var generateOutput = function(args) {
            var newArgs = [ name + ":" ];
            for (var i = 0; i < args.length; ++i) {
            newArgs.push(args[i]);
            }
            return newArgs.join(" ");
        };
        
        self.setLevel = function(level) {
            logger.setLevel(level);
        };
        self.addAppender = function(a) {
            logger.addAppender(a);
        };
        self.log= function() {
            // NOTE: log doesn't mean anything in Log4js. mapping it to info
            logger.info(generateOutput(arguments));
        };
        self.debug = function() {
            logger.debug(generateOutput(arguments));
        };
        self.info = function() {
            logger.info(generateOutput(arguments));
        };
        self.warn = function() {
            logger.warn(generateOutput(arguments));
        };
        self.error = function() {
            logger.error(generateOutput(arguments));
        };
        self.assert = function() {
        };
        self.trace = function() {
        };

    };
    Orbited.system = Orbited.getLogger('system');



    Orbited.CometTransports = {};

    Orbited.util.chooseTransport = function() {
        if (Orbited.settings.streaming == false || Orbited.util.browser == "webkit") {
            return Orbited.CometTransports.LongPoll;
        }
        var choices = [];
        for (var name in Orbited.CometTransports) {
            var transport = Orbited.CometTransports[name];
            if (typeof(transport[Orbited.util.browser]) == "number") {
                Orbited.system.log('viable transport: ', name);
                choices.push(transport);
            }
        }
        // TODO: sort the choices by the values of transport[Orbited.util.browser]
        //         and return the transport with the highest value.
        //    return XHRStream
        return choices[0];
    };



    var createXHR = function () {
        try { return new XMLHttpRequest(); } catch(e) {}
        try { return new ActiveXObject('MSXML3.XMLHTTP'); } catch(e) {}
        try { return new ActiveXObject('MSXML2.XMLHTTP.3.0'); } catch(e) {}
        try { return new ActiveXObject('Msxml2.XMLHTTP'); } catch(e) {}
        try { return new ActiveXObject('Microsoft.XMLHTTP'); } catch(e) {}
        throw new Error('Could not find XMLHttpRequest or an alternative.');
    };


    Orbited.legacy = {};
    //Orbited.web.connect = function() {
    //
    //}

    Orbited.CometSession = function() {
        var self = this;
        self.readyState = self.READY_STATE_INITIALIZED;
        self.onopen = function() {};
        self.onread = function() {};
        self.onclose = function() {};
        var sessionUrl = null;
        var sessionKey = null;
        var sendQueue = [];
        var packetCount = 0;
        var xhr = null;
        var handshakeTimer = null;
        var cometTransport = null;
        var pingInterval = 30000;
        var pingTimeout = 30000;
        var timeoutTimer = null;
        var lastPacketId = 0;
        var sending = false;
        var xsdClose = null;

        /*
     * This will always fire same-domain and cross-subdomain.
     * It will fire most of the time cross-port, but it's not
     * strictly guaranteed.
     * -mario
     */
        var hardClose = function() {
            var tdata = encodePackets([[++packetCount, "close"]]);
            if (xsdClose) {
                xsdClose.contentWindow.sendCloseFrame(sessionUrl.render(),tdata);
            }
            else {
                xhr.open('POST', sessionUrl.render(), !sessionUrl.isSameDomain(location.href));
                xhr.send(tdata);
            }
        }

        /*
     * self.open can only be used when readyState is INITIALIZED. Immediately
     * following a call to self.open, the readyState will be OPENING until a
     * connection with the server has been negotiated. self.open takes a url
     * as a single argument which desiginates the remote url with which to
     * establish the connection.
     */
        self.open = function(_url) {
;;;         self.logger.debug('open');
            self.readyState = self.READY_STATE_OPENING;
            sessionUrl = new Orbited.URL(_url);
            if (sessionUrl.isSameDomain(location.href)) {
                xhr = createXHR();
            }
            else {
                xhr = new Orbited.XSDR();
                if (sessionUrl.isSamePort(location.href)) {
                    xsdClose = document.createElement('iframe');
                    xsdClose.style.display = 'block';
                    xsdClose.style.width = '0';
                    xsdClose.style.height = '0';
                    xsdClose.style.border = '0';
                    xsdClose.style.margin = '0';
                    xsdClose.style.padding = '0';
                    xsdClose.style.overflow = 'hidden';
                    xsdClose.style.visibility = 'hidden';
                    var ifUrl = new Orbited.URL("");
                    ifUrl.protocol = Orbited.settings.protocol;
                    ifUrl.domain = Orbited.settings.hostname;
                    ifUrl.port = Orbited.settings.port;
                    ifUrl.path = '/static/xsdClose.html';
                    ifUrl.hash = document.domain;
                    xsdClose.src = ifUrl.render();
                    document.body.appendChild(xsdClose);
                }
            }
            if (Orbited.settings.enableFFPrivileges) {
                try {
                    netscape.security.PrivilegeManager.enablePrivilege('UniversalBrowserRead');
                } catch (ex) { }
            }

            xhr.open('GET', _url, true);
            xhr.onreadystatechange = function() {
                if (xhr.readyState == 4) {
                    if (xhr.status == 200) {
                        sessionKey = xhr.responseText;
;;;                     self.logger.debug('session key is: ', sessionKey);
                        resetTimeout();
                        // START new URL way
                        //              sessionUrl.extendPath(sessionKey)
                        // END: new URL way
                        
                        // START: old URL way
                        if (sessionUrl.path[sessionUrl.path.length] != '/') {
                            sessionUrl.path += '/';
                        }
                        sessionUrl.path += sessionKey;
                        // END: old Url way
                        var transportClass = Orbited.util.chooseTransport();
                        cometTransport = new transportClass();
                        cometTransport.timeoutResetter = resetTimeout;
                        cometTransport.isSubDomain = sessionUrl.isSubDomain(location.href);
                        cometTransport.onReadFrame = transportOnReadFrame;
                        cometTransport.onclose = transportOnClose;
                        cometTransport.connect(sessionUrl.render());
                    } else {
                        xhr = null;
                        self.readyState = self.READY_STATE_CLOSED;
                        self.onclose(Orbited.Errors.InvalidHandshake);
                    }
                }
            };
            xhr.send(null);
        };
        
        /*
     * self.send is only callable when readyState is OPEN. It will queue the data
     * up for delivery as soon as the upstream xhr is ready.
     */
        self.send = function(data) {
;;;         self.logger.debug('send', data);
            if (self.readyState != self.READY_STATE_OPEN) {
                throw new Error("Invalid readyState");
            }

//            if (cometTransport.limbo) {
//;;;             self.logger.debug('limbo is true. setting sending to false');
  //              sending = false;
    //        }

            data = Orbited.base64.encode(data);
            sendQueue.push([++packetCount, "data", data]);
;;;         self.logger.debug('sending ==', sending);
            if (!sending) {
;;;             self.logger.debug('starting send');
                doSend();
            }
        };
        
        /*
     * self.close sends a close frame to the server, at the end of the queue.
     * It also sets the readyState to CLOSING so that no further data may be
     * sent. onclose is not called immediately -- it waits for the server to
     * send a close event.
     */
        self.close = function() {
            switch(self.readyState) {
                case self.READY_STATE_CLOSING:
                case self.READY_STATE_CLOSED:
                    return;
                case self.READY_STATE_INITIALIZED:
                    // TODO: call onclose here?
                    self.readyState = self.READY_STATE_CLOSED;
                    return;
                default:
                    break;
            }
            self.readyState = self.READY_STATE_CLOSING;
            sendQueue.push([++packetCount, "close"]);
            if (!sending) {
                doSend();
            }
        };
        
        /* self.reset is a way to close immediately. The send queue will be discarded
     * and a close frame will be sent to the server. onclose is called immediately
     * without waiting for a reply from the server.
     */
        self.reset = function() {
;;;         self.logger.debug('reset');
            var origState = self.readyState;
            self.readyState = self.READY_STATE_CLOSED;
            switch(origState) {
                case self.READY_STATE_INITIALIZED:
                    self.onclose(Orbited.Errors.UserConnectionReset);
                    break;
                case self.READY_STATE_OPENING:
                    xhr.onreadystatechange = function() {};
                    xhr.abort();
                    self.onclose(Orbited.Errors.UserConnectionReset);
                    break;
                case self.READY_STATE_OPEN:
                    self.sendQueue = [];
                    self.sending = false;
                    if (xhr.readyState < 4) {
                        xhr.onreadystatechange = function() {};
                        xhr.abort();
                    }
                    doClose(Orbited.Errors.UserConnectionReset);
                    hardClose();
                    break;
                case self.READY_STATE_CLOSING:
                    // TODO: Do nothing here?
                    //     we need to figure out if we've attempted to send the close
                    //     frame yet or not If not, we do something similar to case
                    //     OPEN. either way, we should kill the transport and
                    //     trigger onclose
                    //     -mcarter 7-29-08
                    break;
                
                case self.READY_STATE_CLOSED:
                    break;
            }
        };
        
        self.cleanup = function() {
            self.readyState = self.READY_STATE_CLOSED;
            cometTransport.close();
        }
        
        var transportOnReadFrame = function(frame) {
;;;         self.logger.debug('transportOnReadFrame');
;;;         self.logger.debug('READ FRAME: ', frame.id, frame.name, frame.data ? frame.data.length : '');
            if (!isNaN(frame.id)) {
                lastPacketId = Math.max(lastPacketId, frame.id);
            }
;;;         self.logger.debug(frame);
            switch(frame.name) {
                case 'close':
                    if (self.readyState < self.READY_STATE_CLOSED) {
                        doClose(Orbited.Statuses.ServerClosedConnection);
                    }
                    break;
                case 'data':
;;;                 self.logger.debug('base64 decoding ' + frame.data.length + ' bytes of data');
                    var data = Orbited.base64.decode(frame.data);
;;;                 self.logger.debug('decode complete');
                    self.onread(data);
                    break;
                case 'open':
                    if (self.readyState == self.READY_STATE_OPENING) {
                        self.readyState = self.READY_STATE_OPEN;
;;;                     self.logger.debug('Call self.onopen()');
                        self.onopen();
                    }
                    else {
                        //TODO Throw and error?
                    }
                    break;
                case 'ping':
                    // TODO: don't have a third element (remove the null).
                    // NOTE: don't waste a request when we get a longpoll ping.
                    switch(cometTransport.name) {
                        case 'longpoll':
                            break;
                        case 'poll':
                            break;
                        default:
                            sendQueue.push([++packetCount, "ping", null]);
                            if (!sending) {
                                doSend();
                            }
                            break;
                    }
                    break;
                case 'opt':
                    var args = frame.data.split(',');
                    switch(args[0]) {
                        case 'pingTimeout':
                            pingTimeout = parseInt(args[1])*1000;
                            break;
                        case 'pingInterval':
                            pingInterval = parseInt(args[1])*1000;
                            break;
                        default:
;;;                         self.logger.warn('unknown opt key', args[0]);
                            break;
                    }
                    break;
            }
;;;         self.logger.debug("resetting timeout from transportOnReadFrame");
            resetTimeout();
        };
        var transportOnClose = function() {
;;;         self.logger.debug('transportOnClose');
            if (self.readyState < self.READY_STATE_CLOSED) {
                try {
                    doClose(Orbited.Statuses.ServerClosedConnection);
                }
                catch(e) {
                    //        Fix for navigation-close
                    return;
                }
            }
        };
        var encodePackets = function(queue) {
            //TODO: optimize this.
            var output = [];
            for (var i =0; i < queue.length; ++i) {
                var frame = queue[i];
                for (var j =0; j < frame.length; ++j) {
                    var arg = frame[j];
                    if (arg == null) {
                        arg = "";
                    }
                    if (j == frame.length-1) {
                        output.push('0');
                    }
                    else {
                        output.push('1');
                    }
                    output.push(arg.toString().length);
                    output.push(',');
                    output.push(arg.toString());
                }
            }
            return output.join("");
        };

        var doSend = function(retries) {
;;;         self.logger.debug('in doSend');
            if (typeof(retries) == "undefined") {
                retries = 0;
            }
            // TODO: I don't think this timeout formula is quite right...
            //     -mcarter 8-3-08
            if (retries*RETRY_INTERVAL >= RETRY_TIMEOUT) {
                doClose(Orbited.Errors.ConnectionTimeout);
                sending = false;
                return;
            }
            if (sendQueue.length == 0) {
;;;             self.logger.debug('sendQueue exhausted');
                sending = false;
                return;
            }
            sending = true;
;;;         self.logger.debug('setting sending=true');
            var numSent = sendQueue.length;
            sessionUrl.setQsParameter('ack', lastPacketId);
            var tdata = encodePackets(sendQueue);
;;;         self.logger.debug('post', retries, tdata);
            if (Orbited.settings.enableFFPrivileges) {
                try {
                    netscape.security.PrivilegeManager.enablePrivilege('UniversalBrowserRead');
                } catch (ex) { }
            }
            xhr.open('POST', sessionUrl.render(), true);
            // NB: its awkard, but for reusing the XHR object in IE (7 at least),
            //     we can only reset the onreadystatechange *after* we call open;
            //     if we don't do this, the XHR will stop sending data.
            // See "Reusing XMLHttpRequest Object in IE"
            //     at http://keelypavan.blogspot.com/2006/03/reusing-xmlhttprequest-object-in-ie.html
            xhr.onreadystatechange = function() {
;;;             self.logger.debug('doSend onreadystatechange');
                switch(xhr.readyState) {
                    case 4:
                        if (xhr.status == 200) {
                            resetTimeout();
                            sendQueue.splice(0, numSent);
                            return doSend();
                        }
                        else {
                            //TODO: implement retry back-off;
                            window.setTimeout(function(){doSend(++retries);},RETRY_INTERVAL);
                        }
                        break;
                }
            };
            xhr.send(tdata);
        };

        var doClose = function(code) {
;;;         self.logger.debug('doClose', code);
            unsetTimeout();
            self.readyState = self.READY_STATE_CLOSED;
            if (cometTransport != null) {
                // TODO: is this line necessary?
                cometTransport.onReadFrame = function() {};
                cometTransport.onclose = function() { };
                cometTransport.close();
            }
            self.onclose(code);
            
        };

        var resetTimeout = function() {
;;;         self.logger.debug('reset Timeout', pingInterval+pingTimeout);
            unsetTimeout();
            timeoutTimer = window.setTimeout(timedOut, pingInterval + pingTimeout);
        };
        var unsetTimeout = function() {
            window.clearTimeout(timeoutTimer);
            
        };
        var timedOut = function() {
;;;         self.logger.debug('timed out!');
            doClose(Orbited.Errors.ConnectionTimeout);
        };
    };
    Orbited.CometSession.prototype.logger = Orbited.getLogger("Orbited.CometSession");
    Orbited.CometSession.prototype.READY_STATE_INITIALIZED    = 1;
    Orbited.CometSession.prototype.READY_STATE_OPENING    = 2;
    Orbited.CometSession.prototype.READY_STATE_OPEN        = 3;
    Orbited.CometSession.prototype.READY_STATE_CLOSING    = 4;
    Orbited.CometSession.prototype.READY_STATE_CLOSED    = 5;
    
    var currentTCPSocketId = 0;
    var openSockets = {};

    Orbited.test = {};
    Orbited.test.logger = Orbited.getLogger("Orbited.test");

    Orbited.test.socketcontrol = {};
    Orbited.test.socketcontrol.kill = function(t) {
;;;     Orbited.test.logger.debug("kill ordered for socket:", t);
        if (openSockets[t.id]) {
            openSockets[t.id](Orbited.Statuses.SocketControlKilled);
            t = null;
;;;         Orbited.test.logger.debug("socket killed");
        }
        else {
;;;         Orbited.test.logger.debug("socket not found");
        }
    };

    Orbited.test.stompdispatcher = {};
    Orbited.test.stompdispatcher.send = function(dest, msg) {
;;;     Orbited.test.logger.debug("stompdispatcher dispatching "+msg+" to "+dest);
        var s = document.createElement('script');
        s.src = "http://"+Orbited.settings.hostname+":"+Orbited.settings.port+"/system/test/stomp?";
        s.src += "msg="+msg;
        s.src += "&dest="+dest;
        document.body.appendChild(s);
    };

    Orbited.TCPSocket = function() {
        var self = this;
        self.id = ++currentTCPSocketId;

        // So we don't completely ambush people used to the 0.5 api...
        if (arguments.length > 0) {
            throw new Error("TCPSocket() accepts no arguments");
        }
        self.readyState = self.READY_STATE_INITIALIZED;
        self.onopen = function() { };
        self.onread = function() { };
        self.onclose = function() { };
        var onCloseTriggered = false;
        var buffer = "";
        var session = null;
        var binary = false;
        var handshakeState = null;
        var hostname = null;
        var port = null;

        /* self.open attempts to establish a tcp connection to the specified remote
     * hostname on the specified port. When specified as true, the optional
     * argument, isBinary, will cause onread to return byte arrays, and send
     * will only accept a byte array.
     */
        self.open = function(_hostname, _port, isBinary) {
            if (self.readyState != self.READY_STATE_INITIALIZED) {
                // TODO: allow reuse from readyState == self.READY_STATE_CLOSED?
                //         Re-use makes sense for xhr due to memory concerns, but
                //         probably not for tcp sockets. How often do you reconnect
                //         in the same page?
                //         -mcarter 7-30-08
                throw new Error("Invalid readyState");
            }
            if (_hostname == false) {
                throw new Error("No hostname specified");
            }
            if (isNaN(_port)) {
                throw new Error("Invalid port specified");
            }
            // handle isBinary undefined/null case
            binary = !!isBinary;
            self.readyState = self.READY_STATE_OPENING;
            hostname = _hostname;
            port = _port;
            session = new Orbited.CometSession();
            var sessionUrl = new Orbited.URL('/tcp');
            sessionUrl.domain = Orbited.settings.hostname;
            sessionUrl.port = Orbited.settings.port;
            sessionUrl.protocol = Orbited.settings.protocol;
            sessionUrl.setQsParameter('nocache', Math.random());
            session.open(sessionUrl.render());
            session.onopen = sessionOnOpen;
            session.onread = sessionOnRead;
            session.onclose = sessionOnClose;
            handshakeState = "initial";
        };

        self.close = function() {
            if (self.readyState == self.READY_STATE_CLOSED) {
                return;
            }
            self.readyState = self.READY_STATE_CLOSED;
            doClose(Orbited.Errors.UserConnectionReset);
        };

        /* self.reset closes the connection from this end immediately. The server
     * may be notified, but there is no guarantee. The main purpose of the reset
     * function is for a quick teardown in the case of a user navigation.
     * if reset is not called when IE navigates, for instance, there will be
     * potential issues with future TCPSocket communication.
     */
        self.reset = function() {
            if (session) {session.reset();}
        };

        self.send = function(data) {
            if (self.readyState != self.READY_STATE_OPEN) {
                throw new Error("Invalid readyState");
            }
            if (!binary) {
                data = Orbited.utf8.encode(data);
            }
;;;         self.logger.debug('SEND: ', data);
            //      try {
            session.send(data);
            //      }
            //      catch(e) {
            //      alert("Why sending: typeof(data) = " + typeof(data));
            //}
        };

        var process = function() {
            var result = Orbited.utf8.decode(buffer);
            var data = result[0];
            var i = result[1];
            buffer = buffer.slice(i);
            if (data.length > 0) {
                window.setTimeout(function() { self.onread(data); }, 0);
            }
        };

        var sessionOnRead = function(data) {
            switch(self.readyState) {
                case self.READY_STATE_OPEN:
;;;                 self.logger.debug('READ: ', data);
                    if (binary) {
                        window.setTimeout(function() { self.onread(data); }, 0);
                    }
                    else {
;;;                     self.logger.debug('start buffer size:', buffer.length);
                        buffer += data;
                        process();
;;;                     self.logger.debug('end buffer size:', buffer.length);
                    }
                    break;
                case self.READY_STATE_OPENING:
                    switch(handshakeState) {
                        case 'initial':
                            // NOTE: we should only get complete payloads during
                            //     the handshake. no need to buffer, then parse
                            data = Orbited.utf8.decode(data)[0];
;;;                         self.logger.debug('initial');
;;;                         self.logger.debug('data', data);
;;;                         self.logger.debug('len', data.length);
;;;                         self.logger.debug('typeof(data)', typeof(data));
;;;                         self.logger.debug('data[0] ', data.slice(0,1));
;;;                         self.logger.debug('type ', typeof(data.slice(0,1)));
                            var result = (data.slice(0,1) == '1');
;;;                         self.logger.debug('result', result);
                            if (!result) {
;;;                             self.logger.debug('!result');
                                var errorCode = data.slice(1,4);
                                doClose(parseInt(errorCode));
                            }
                            if (result) {
                                self.readyState = self.READY_STATE_OPEN;
;;;                             self.logger.debug('tcpsocket.onopen..');
                                self.onopen();
;;;                             self.logger.debug('did onopen');
                            }
                            break;
                    }
                    break;
            }
        };
        var doClose = function(code) {
;;;         self.logger.debug('doClose', code);
            if (session) {
                if (code == Orbited.Statuses.ServerClosedConnection || code == Orbited.Errors.Unauthorized || code == Orbited.Errors.RemoteConnectionFailed) {
                    session.cleanup();
                }
                else {
                    sessionOnClose = function() {};
                    session.close();
                }
                session = null;
            }
;;;         self.logger.debug('onCloseTriggered', onCloseTriggered);
            if (!onCloseTriggered) {
;;;             self.logger.debug('triggerClose timer', code);
                onCloseTriggered = true;
                window.setTimeout(function() {
;;;                 self.logger.debug('onclose!', code);
                    self.onclose(code);
                }, 0);
            }
        };

        openSockets[self.id] = doClose;

        var sessionOnOpen = function(data) {
            // TODO: TCPSocket handshake
            var payload = hostname + ':' + port + '\n';
;;;         self.logger.debug('sessionOpen; sending:', payload);
            payload = Orbited.utf8.encode(payload);
;;;         self.logger.debug('encoded payload:', payload);
            X = payload;
            session.send(payload);
            handshakeState = 'initial';
        };

        var sessionOnClose = function(code) {
;;;         self.logger.debug('sessionOnClose');
            // If we are in the OPENING state, then the handshake code should
            // handle the close
            doClose(code);
        };
    };
    Orbited.TCPSocket.prototype.toString = function() {
        return "<Orbited.TCPSocket " + this.id + ">";
    };
    Orbited.TCPSocket.prototype.logger = Orbited.getLogger("Orbited.TCPSocket");
    Orbited.TCPSocket.prototype.READY_STATE_INITIALIZED  = 1;
    Orbited.TCPSocket.prototype.READY_STATE_OPENING         = 2;
    Orbited.TCPSocket.prototype.READY_STATE_OPEN         = 3;
    Orbited.TCPSocket.prototype.READY_STATE_CLOSING         = 4;
    Orbited.TCPSocket.prototype.READY_STATE_CLOSED         = 5;





    // XXX: the Orbited.XSDR stuff (presumably) doesn't work yet.
    //    mcarter - 8-9-08 (~rev 476)

    Orbited.singleton.XSDR = {
        receiveCbs: {},
        queues: {},
        iframes: {},
        id: 0,
        register: function(receive, queue) {
            var id = ++Orbited.singleton.XSDR.id;
            Orbited.singleton.XSDR.receiveCbs[id] = receive;
            Orbited.singleton.XSDR.queues[id] = queue;
;;;         Orbited.system.debug('id is', id);
            return id;
        }
    };
    Orbited.XSDR = function() {
        var self = this;
        var ifr = null;
        var url;
        var method;
        var data;
        var requestHeaders;
        var queue = [];
        var id = Orbited.singleton.XSDR.register(function(data) { receive(data); },queue);
        var bridgeUrl = new Orbited.URL("");
        bridgeUrl.domain = Orbited.settings.hostname;
        bridgeUrl.port = Orbited.settings.port;
        bridgeUrl.path = '/static/xsdrBridge.html';
        bridgeUrl.hash = id.toString();
        bridgeUrl.protocol = Orbited.settings.protocol;
;;;     self.logger.debug('bridgeUrl.hash is', bridgeUrl.hash);
;;;     self.logger.debug('bridgeUrl.path is', bridgeUrl.path);
;;;     self.logger.debug('bridgeUrl is', bridgeUrl.render());
        var reset = function() {
            self.responseText = "";
            self.status = null;
            self.readyState = 0;
            url = null;
            method = null;
            data = null;
            requestHeaders = {};
        };
        reset();
        self.onreadystatechange = function() { };
        self.open = function(_method, _url, async) {
            if (self.readyState == 4) {
                reset();
            }
            if (self.readyState != 0) {
                throw new Error("Invalid readyState");
            }
            if (!async) {
                throw new Error("Only Async XSDR supported");
            }
;;;         self.logger.debug('open', _method, _url, async);
            self.readyState = 1;
            url = _url;
            method = _method;
        };

        self.send = function(data) {
            if (self.readyState != 1) {
                throw new Error("Invalid readyState");
            }
;;;         self.logger.debug('send', data);
            if (!ifr) {
;;;             self.logger.debug('creating iframe');
                ifr = document.createElement("iframe");
                hideIframe(ifr);
                ifr.src = bridgeUrl.render();
;;;             self.logger.debug('set ifr.src to', ifr.src);
                document.body.appendChild(ifr);
                Orbited.singleton.XSDR.iframes[id] = ifr;
            }
            else {
                queue.push([method, url, data, requestHeaders]);
            }
        };

        self.abort = function() {
            if (self.readyState > 0 && self.readyState < 4) {
                // TODO: push an ABORT command (so as not to reload the iframe)
                //          queue.push(['ABORT']);
;;;             self.logger.debug('ABORT called');
                ifr.src = "about:blank";
                document.body.removeChild(ifr);
                ifr = null;
                self.readyState = 4;
                self.onreadystatechange();
            }
        };


        //    self.abort = function() {
        //      if (self.readyState > 0 && self.readyState < 4) {
        //          queue.push(['ABORT']);
        //      }
        //    }

        self.setRequestHeader = function(key, val) {
            if (self.readyState != 0) {
                throw new Error("Invalid readyState");
            }
            requestHeaders[key] = val;
        };

        self.getResponseHeader = function() {
            if (self.readyState < 2) {
                throw new Error("Invalid readyState");
            }
            return responseHeaders[key];
        };

        var receive = function(payload) {
;;;         self.logger.debug('received', payload);
            switch(payload[0]) {
                case 'initialized':
                    queue.push([method, url, data, requestHeaders]);
;;;                 self.logger.debug('queue is', queue);
;;;                 self.logger.debug('Orbited.singleton.XSDR.queues[id] is', Orbited.singleton.XSDR.queues[id]);
                    break;
                case 'readystatechange':
                    data = payload[1];
                    self.readyState = data.readyState;
;;;                 self.logger.debug('readystatechange', self.readyState);
                    if (data.status) {
                        self.status = data.status;
;;;                     self.logger.debug('status', data.status);
                    }
                    if (data.responseText) {
                        self.responseText += data.responseText;
;;;                     self.logger.debug('responseText', data.responseText);
                    }
;;;                 self.logger.debug('doing trigger');
                    self.onreadystatechange();
;;;                 self.logger.debug('trigger complete');
                    break;
            }
        };

        var hideIframe =function (ifr) {
            ifr.style.display = 'block';
            ifr.style.width = '0';
            ifr.style.height = '0';
            ifr.style.border = '0';
            ifr.style.margin = '0';
            ifr.style.padding = '0';
            ifr.style.overflow = 'hidden';
            ifr.style.visibility = 'hidden';
        };

    };

    if (Orbited.util.browser == "opera") {
        var pmLocation = window.postMessage && "contentWindow" || "document";
        (window.postMessage && window || document).addEventListener('message', function(e) {
            var msg = e.data.split(" ");
            var cmd = msg.shift();
            if (cmd == "event") {
                var id = msg.shift();
                var dataString = msg.join(" ");
                var data = Orbited.JSON.parse(dataString);
                Orbited.singleton.XSDR.receiveCbs[id](data);
            }
            if (cmd == "queues") {
                id = msg.shift();
                var queue = Orbited.singleton.XSDR.queues[id];
                if (queue.length > 0) {
                    data = queue.shift();
                    Orbited.singleton.XSDR.iframes[id][pmLocation].postMessage(Orbited.JSON.stringify(data), e.origin);
                }
            }
        }, false);
    }

    Orbited.XSDR.prototype.logger = Orbited.getLogger("Orbited.XSDR");
    Orbited.singleton.XSDRBridgeLogger = Orbited.getLogger('XSDRBridge');

    /* Comet Transports!
 */
    var CT_READYSTATE_INITIAL = 0;
    var CT_READYSTATE_OPEN      = 1;
    var CT_READYSTATE_CLOSED  = 2;

    Orbited.CometTransports.XHRStream = function() {
        var self = this;
        self.name = 'xhrstream';
        var url = null;
        var xhr = null;
        var ackId = null;
        var offset = 0;
        var heartbeatTimer = null;
        var retryTimer = null;
        var buffer = "";
        var retryInterval = 50;
        self.readyState = CT_READYSTATE_INITIAL;
        self.onReadFrame = function(frame) {};
        self.onread = function(packet) { self.onReadFrame(packet); };
        self.onclose = function() { };

        self.close = function() {
            if (self.readyState == CT_READYSTATE_CLOSED) {
                return;
            }
            if (xhr != null && (xhr.readyState > 1 || xhr.readyState < 4)) {
                xhr.onreadystatechange = function() { };
                xhr.abort();
                xhr = null;
            }
            self.readyState = CT_READYSTATE_CLOSED;
            window.clearTimeout(heartbeatTimer);
            window.clearTimeout(retryTimer);
            self.onclose();
        };

        self.connect = function(_url) {
            if (self.readyState == CT_READYSTATE_OPEN) {
                throw new Error("Already Connected");
            }
            url = new Orbited.URL(_url);
            if (xhr == null) {
                if (url.isSameDomain(location.href)) {
                    xhr = createXHR();
                }
                else {
                    xhr = new Orbited.XSDR();
                }
            }
            url.path += '/xhrstream';
            //      url.setQsParameter('transport', 'xhrstream')
            self.readyState = CT_READYSTATE_OPEN;
            open();
        };
        var open = function() {
            try {
                if (typeof(ackId) == "number") {
                    url.setQsParameter('ack', ackId);
                }
                if (typeof(xhr)== "undefined" || xhr == null) {
                    throw new Error("how did this happen?");
                }
                if (Orbited.settings.enableFFPrivileges) {
                    try {
                        netscape.security.PrivilegeManager.enablePrivilege('UniversalBrowserRead');
                    }
                    catch (ex) { }
                }

                xhr.open('GET', url.render(), true);
                xhr.onreadystatechange = function() {
;;;                 self.logger.debug(xhr.readyState);
                    if (self.readyState == CT_READYSTATE_CLOSED) {
                        return;
                    }
                    switch(xhr.readyState) {
                        case 2:
                            // If we can't get the status, then we didn't actually
                            // get a valid xhr response -- we got a network error
                            try {
                                var status = xhr.status;
                            }
                            catch(e) {
                                return;
                            }
                            // If we got a 200, then we're in business
                            if (status == 200) {
                                try {
                                    heartbeatTimer = window.setTimeout(heartbeatTimeout, Orbited.settings.HEARTBEAT_TIMEOUT);
                                }
                                catch(e) {
                                //                 Happens after navigation
                                    self.close();
                                    return;
                                }
                                var testtimer = heartbeatTimer;
                            }
                            // Otherwise, case 4 should handle the reconnect,
                            // so do nothing here.
                            break;
                        case 3:
                            // If we can't get the status, then we didn't actually
                            // get a valid xhr response -- we got a network error
                            try {
                                var status = xhr.status;
                            }
                            catch(e) {
                                return;
                            }
                            // We successfully established a connection, so put the
                            // retryInterval back to a short value
                            if (status == 200) {
                                retryInterval = 50;
                                process();
                            }
                            break;
                        case 4:
                            var doReconnect = true;
                            try {
                                if (xhr.status === null) {
                                    doReconnect = true;
                                }
                                else {
                                    doReconnect = false;
                                }
                            }
                            catch(e) {
                            }
                            if (doReconnect) {
                                // Expoential backoff: Every time we fail to
                                // reconnect, double the interval.
                                // TODO cap the max value.
                                retryInterval *= 2;
                                //                  self.logger.debug('retryInterval', retryInterval)
                                window.clearTimeout(heartbeatTimer);
                                retryTimer = window.setTimeout(reconnect, retryInterval);
                                return;
                            }
                            switch(xhr.status) {
                                case 200:
                                    //                  alert('finished, call process');
                                    //                 if (typeof(Orbited) == "undefined") {
                                    //                      alert('must have reloaded')
                                    //                      break
                                    //                  }
                                    //                  alert('a');
                                    //                  alert('stream over ' +  typeof(console) + ' ' + typeof(Orbited) + ' ' + Orbited + ' ...');
                                    process();
                                    offset = 0;
                                    setTimeout(open, 0);
                                    window.clearTimeout(heartbeatTimer);
                                    break;
                                case 404:
                                    self.close();
                                    break;
                                default:
                                    self.close();
                                    break;
                            }
                            break;
                    }
                };
                xhr.send(null);
            }
            catch(e) {
                self.close();
            }
        };

        var reconnect = function() {
;;;         self.logger.debug('reconnect...')
            if (xhr.readyState < 4 && xhr.readyState > 0) {
            xhr.onreadystatechange = function() {
                if (xhr.readyState == 4) {
                reconnect();
                }
            };
;;;         self.logger.debug('do abort..')
            xhr.abort();
            window.clearTimeout(heartbeatTimer);
            }
            else {
;;;         self.logger.debug('reconnect do open');
            offset = 0;
            setTimeout(open, 0);
            }
        };
        // 12,ab011,hello world
        var commaPos = -1;
        var argEnd = null;
        var frame = [];
        var process = function() {
            var stream = xhr.responseText;
            receivedHeartbeat();

            // ignore leading whitespace, such as at the start of an xhr stream
            while (stream[offset] == ' ') {
            offset += 1;
            }
            // ignore leading whitespace, such as at the start of an xhr stream
            while (stream[offset] == 'x') {
            offset += 1;
            }

            var k = 0;
            while (true) {
            k += 1;
            if (k > 2000) {
                throw new Error("Borked XHRStream transport");
            }
            if (commaPos == -1) {
                commaPos = stream.indexOf(',', offset);
            }
            if (commaPos == -1) {
                return;
            }
            if (argEnd == null) {
                argSize = parseInt(stream.slice(offset+1, commaPos));
                argEnd = commaPos +1 + argSize;
            }
            
            if (stream.length < argEnd) {
                return;
            }
            var data = stream.slice(commaPos+1, argEnd);
            frame.push(data);
            var isLast = (stream.charAt(offset) == '0');
            offset = argEnd;
            argEnd = null;
            commaPos = -1;
            if (isLast) {
                var frameCopy = frame;
                frame = [];
                receivedPacket(frameCopy);
            }
            }

        };
        var receivedHeartbeat = function() {
            window.clearTimeout(heartbeatTimer);
;;;         self.logger.debug('clearing heartbeatTimer', heartbeatTimer);
            try {
            heartbeatTimer = window.setTimeout(function() {
;;;             self.logger.debug('timer', testtimer, 'did it');
                heartbeatTimeout();
            }, Orbited.settings.HEARTBEAT_TIMEOUT);
            }
            catch(e) {
            return;
            }
            var testtimer = heartbeatTimer;

;;;         self.logger.debug('heartbeatTimer is now', heartbeatTimer);
        };
        var heartbeatTimeout = function() {
;;;         self.logger.debug('heartbeat timeout... reconnect');
            reconnect();
        };
        var receivedPacket = function(args) {
            var testAckId = parseInt(args[0]);
            if (!isNaN(testAckId)) {
            ackId = testAckId;
            }
            var packet = {
            id: testAckId,
            name: args[1],
            data: args[2]
            };
            // TODO: shouldn't we put this in a window.setTimeout so that user
            //     code won't mess up our code?
            self.onread(packet);
        };
    };
    Orbited.CometTransports.XHRStream.prototype.logger = Orbited.getLogger("Orbited.CometTransports.XHRStream");
    // XHRStream supported browsers
    Orbited.CometTransports.XHRStream.firefox = 1.0;
    Orbited.CometTransports.XHRStream.firefox2 = 1.0;
    Orbited.CometTransports.XHRStream.firefox3 = 1.0;
    Orbited.CometTransports.XHRStream.safari2 = 1.0;
    Orbited.CometTransports.XHRStream.safari3 = 1.0;





    Orbited.CometTransports.LongPoll = function() {
        var self = this;
        self.name = 'longpoll';
        var url = null;
        var xhr = null;
        var ackId = null;
        var retryTimer = null;
        var buffer = "";
        var retryInterval = 50;
        self.readyState = CT_READYSTATE_INITIAL;
        self.onReadFrame = function(frame) {};
        self.onclose = function() { };

        self.close = function() {
;;;         self.logger.debug('close');
            if (self.readyState == CT_READYSTATE_CLOSED) {
                return;
            }
            if (xhr != null && (xhr.readyState > 1 || xhr.readyState < 4)) {
                xhr.onreadystatechange = function() { };
                xhr.abort();
                xhr = null;
            }
;;;         self.logger.debug('close! self.readyState now is 2');
            self.readyState = CT_READYSTATE_CLOSED;
            window.clearTimeout(retryTimer);
            self.onclose();
        };

        self.connect = function(_url) {
;;;         self.logger.debug('connect');
            if (self.readyState == CT_READYSTATE_OPEN) {
                throw new Error("Already Connected");
            }
            url = new Orbited.URL(_url);
            if (xhr == null) {
                if (url.isSameDomain(location.href)) {
                    xhr = createXHR();
                }
                else {
                    xhr = new Orbited.XSDR();
                }
            }
            url.path += '/longpoll';
            //      url.setQsParameter('transport', 'xhrstream')
            self.readyState = CT_READYSTATE_OPEN;
            open();
        };
        var open = function() {
;;;         self.logger.debug('open... self.readyState = ' + self.readyState);
            if (self.readyState == CT_READYSTATE_CLOSED) {
                return;
            }
            try {
                if (typeof(ackId) == "number") {
                    url.setQsParameter('ack', ackId);
                }
                if (typeof(xhr)== "undefined" || xhr == null) {
                    throw new Error("how did this happen?");
                }
                if (Orbited.settings.enableFFPrivileges) {
                    try {
                        netscape.security.PrivilegeManager.enablePrivilege('UniversalBrowserRead');
                    }
                    catch (ex) { }
                }
                xhr.open('GET', url.render(), true);
                xhr.onreadystatechange = function() {
;;;                 self.logger.debug('readystate', xhr.readyState);
                    switch(xhr.readyState) {
                        case 4:
                            try {
                                var test = xhr.status;
                            }
                            catch(e) {
                                // Exponential backoff: Every time we fail to
                                // reconnect, double the interval.
                                // TODO cap the max value.
;;;                             self.logger.debug("start reconnect Timer (couldn't access xhr.status)");
                                retryInterval *= 2;
                                window.setTimeout(reconnect, retryInterval);
                                return;
                            }
                            switch(xhr.status) {
                                case 200:
                                    self.timeoutResetter();
                                    process();
;;;                                 self.logger.debug("completed request, reconnect immediately");
                                    setTimeout(open, 0);
                                    break;
                                case 404:
                                    self.close();
                                    break;
                                case null:
                                    // NOTE: for the XSDR case:
                                    // (we can always get status, but maybe its null)
                                    retryInterval *= 2;
;;;                                 self.logger.debug("start reconnect Timer (null xhr.status)");
                                    window.setTimeout(reconnect, retryInterval);
                                    break;
                                default:
                                    // TODO: do we want to retry here?
;;;                                 self.logger.debug("something broke, xhr.status=", xhr.status);
                                    self.close();
                                    break;
                            }
                            break;
                    }
                };
                xhr.send(null);
            }
            catch(e) {
                self.close();
            }
        };

        var reconnect = function() {
;;;         self.logger.debug('reconnect...');
            if (xhr.readyState < 4 && xhr.readyState > 0) {
                xhr.onreadystatechange = function() {
                    if (xhr.readyState == 4) {
                        reconnect();
                    }
                };
;;;             self.logger.debug('do abort..');
                xhr.abort();
                window.clearTimeout(heartbeatTimer);
            }
            else {
;;;             self.logger.debug('reconnect do open');
                offset = 0;
                setTimeout(open, 0);
            }
        };
        // ( ab,hello world)
        // 12,ab011,hello world
        var process = function() {
;;;         self.logger.debug('process');
            var commaPos = -1;
            var argEnd = null;
            var argSize;
            var frame = [];
            var stream = xhr.responseText;
            var offset = 0;


            var k = 0;
            while (true) {
                k += 1;
                if (k > 2000) {
                    throw new Error("Borked XHRStream transport");
                }
                if (commaPos == -1) {
                    commaPos = stream.indexOf(',', offset);
                }
                if (commaPos == -1) {
;;;                 self.logger.debug('no more commas. offset:', offset, 'stream.length:', stream.length);
                    return;
                }
                if (argEnd == null) {
                    argSize = parseInt(stream.slice(offset+1, commaPos));
                    argEnd = commaPos +1 + argSize;
                }
;;;             self.logger.assert(true);
                /*          if (stream.length < argEnd) {
    //;;;        self.logger.debug('how did we get here? stream.length:', stream.length, 'argEnd:', argEnd, 'offset:', offset)
                return
                }*/
                var data = stream.slice(commaPos+1, argEnd);
;;;             self.logger.assert(data.length == argSize, 'argSize:', argSize, 'data.length', data.length);
                if (data.length != argSize) {
                    DEBUGDATA = stream;
                }
                frame.push(data);
                var isLast = (stream.charAt(offset) == '0');
                offset = argEnd;
                argEnd = null;
                commaPos = -1;
                if (isLast) {
                    var frameCopy = frame;
                    frame = [];
                    receivedPacket(frameCopy);
                }
            }

        };
        var receivedPacket = function(args) {
            var testAckId = parseInt(args[0]);
;;;         self.logger.debug('args', args);
            if (!isNaN(testAckId)) {
                ackId = testAckId;
            }
;;;         self.logger.debug('testAckId', testAckId, 'ackId', ackId);
            var packet = {
                id: testAckId,
                name: args[1],
                data: args[2]
            };
            // TODO: shouldn't we put this in a window.setTimeout so that user
            //     code won't mess up our code?
            self.onReadFrame(packet);
        };
    };
    Orbited.CometTransports.LongPoll.prototype.logger = Orbited.getLogger("Orbited.CometTransports.LongPoll");
    // LongPoll supported browsers
    /*
Orbited.CometTransports.LongPoll.firefox = 0.9
Orbited.CometTransports.LongPoll.firefox2 = 0.9
Orbited.CometTransports.LongPoll.firefox3 = 0.9
Orbited.CometTransports.LongPoll.safari2 = 0.9
Orbited.CometTransports.LongPoll.safari3 = 0.9
Orbited.CometTransports.LongPoll.opera = 0.9
Orbited.CometTransports.LongPoll.ie = 0.9
*/



    Orbited.CometTransports.Poll = function() {
        var self = this;
        self.name = 'poll';

//        self.limbo = false;

        var url = null;
        var xhr = null;
        var ackId = null;
        var retryTimer = null;
        var buffer = "";
        var baseRetryInterval = Orbited.settings.POLL_INTERVAL;
        var retryInterval = baseRetryInterval;
        self.readyState = CT_READYSTATE_INITIAL;
        self.onReadFrame = function(frame) {};
        self.onclose = function() { };

        self.close = function() {
;;;         self.logger.debug('close...');
            if (self.readyState == CT_READYSTATE_CLOSED) {
                return;
            }
            if (xhr != null && (xhr.readyState > 1 || xhr.readyState < 4)) {
                xhr.onreadystatechange = function() { };
                xhr.abort();
                xhr = null;
            }
            self.readyState = CT_READYSTATE_CLOSED;
            window.clearTimeout(retryTimer);
            self.onclose();
        };

        self.connect = function(_url) {
;;;         self.logger.debug('connect...');
            if (self.readyState == CT_READYSTATE_OPEN) {
                throw new Error("Already Connected");
            }
            url = new Orbited.URL(_url);
            if (xhr == null) {
                if (url.isSameDomain(location.href)) {
                    xhr = createXHR();
                }
                else {
                    xhr = new Orbited.XSDR();
                }
            }
            url.path += '/poll';
            //      url.setQsParameter('transport', 'xhrstream')
            self.readyState = CT_READYSTATE_OPEN;
            open();
        };
        var open = function() {
;;;         self.logger.debug('open...');
            try {
                if (typeof(ackId) == "number") {
                    url.setQsParameter('ack', ackId);
                }
                if (typeof(xhr)== "undefined" || xhr == null) {
                    throw new Error("how did this happen?");
                }
                
                if (Orbited.settings.enableFFPrivileges) {
                    try {
                        netscape.security.PrivilegeManager.enablePrivilege('UniversalBrowserRead');
                    } catch (ex) { }
                }
                xhr.open('GET', url.render(), true);
                xhr.onreadystatechange = function() {
                    switch(xhr.readyState) {
                        case 4:
                            try {
                                var test = xhr.status;
                            }
                            catch(e) {
                                // Exponential backoff: Every time we fail to
                                // reconnect, double the interval.
                                // TODO cap the max value.
                                retryInterval *= 2;
                                window.setTimeout(reconnect, retryInterval);
                                return;
                            }
                            switch(xhr.status) {
                                case 200:
                                    self.timeoutResetter();
                                    retryInterval = baseRetryInterval;
                                    process();
                                    setTimeout(open, retryInterval);
                                    break;
                                case 404:
                                    self.close();
                                    break;
                                case null:
                                    // NOTE: for the XSDR case: Long
                                    // (we can always get status, but maybe its null)
                                    retryInterval *= 2;
                                    window.setTimeout(reconnect, retryInterval);
                                    break;
                                default:
                                    // TODO: do we want to retry here?
                                    self.close();
                                    break;
                            }
                            break;
                    }
                };
                xhr.send(null);
            }
            catch(e) {
                self.close();
            }
        };
        
        var reconnect = function() {
;;;         self.logger.debug('reconnect...');
            if (xhr.readyState < 4 && xhr.readyState > 0) {
                xhr.onreadystatechange = function() {
                    if (xhr.readyState == 4) {
                        reconnect();
                    }
                };
;;;             self.logger.debug('do abort..');
                xhr.abort();
                window.clearTimeout(heartbeatTimer);
            } else {
;;;             self.logger.debug('reconnect do open');
                offset = 0;
                setTimeout(open, 0);
            }
        };
        // 12,ab011,hello world
        var process = function() {
;;;         self.logger.debug('process...');
            var commaPos = -1;
            var argEnd = null;
            var argSize;
            var frame = [];
            var stream = xhr.responseText;
            var offset = 0;

            var k = 0;

            while (true) {
                k += 1;
                if (k > 2000) {
                    throw new Error("Borked XHRStream transport");
                }
                if (commaPos == -1) {
                    commaPos = stream.indexOf(',', offset);
                }
                if (commaPos == -1) {
;;;                 self.logger.debug('no more commas. offset:', offset, 'stream.length:', stream.length);
//                    if (offset == 0 && stream.length == 0) {
//;;;                     self.logger.debug('setting limbo to true');
  //                      self.limbo = true;
    //                }
                    return;
                }
//;;;             self.logger.debug('setting limbo to false');
  //              self.limbo = false;

                if (argEnd == null) {
                    argSize = parseInt(stream.slice(offset+1, commaPos));
                    argEnd = commaPos +1 + argSize;
                }
                //;;;        self.logger.assert(true);
                /*          if (stream.length < argEnd) {
    //;;;        self.logger.debug('how did we get here? stream.length:', stream.length, 'argEnd:', argEnd, 'offset:', offset)
                return
                }*/
                var data = stream.slice(commaPos+1, argEnd);
;;;             self.logger.assert(data.length == argSize, 'argSize:', argSize, 'data.length', data.length);
                if (data.length != argSize) {
                    DEBUGDATA = stream;
                }
                frame.push(data);
                var isLast = (stream.charAt(offset) == '0');
                offset = argEnd;
                argEnd = null;
                commaPos = -1;
                if (isLast) {
                    var frameCopy = frame;
                    frame = [];
                    receivedPacket(frameCopy);
                }
            }
            
        };
        var receivedPacket = function(args) {
;;;         self.logger.debug('receivedPacket...');
            var testAckId = parseInt(args[0]);
;;;         self.logger.debug('args', args);
            if (!isNaN(testAckId)) {
            ackId = testAckId;
            }
;;;         self.logger.debug('testAckId', testAckId, 'ackId', ackId);
            var packet = {
            id: testAckId,
            name: args[1],
            data: args[2]
            };
            // TODO: shouldn't we put this in a window.setTimeout so that user
            //     code won't mess up our code?
            self.onReadFrame(packet);
        };
    };
    Orbited.CometTransports.Poll.prototype.logger = Orbited.getLogger("Orbited.CometTransports.Poll");

    // Poll supported browsers
    /*Orbited.CometTransports.Poll.firefox = 0.5
Orbited.CometTransports.Poll.opera = 0.5
Orbited.CometTransports.Poll.ie = 0.5
*/





    Orbited.CometTransports.HTMLFile = function() {
        var self = this;
        self.name = 'htmlfile';
        var id = ++Orbited.singleton.HTMLFile.i;
        Orbited.singleton.HTMLFile.instances[id] = self;
        var htmlfile = null;
        var ifr = null;
        var url = null;
        var restartUrl = null;
        var restartTimer = null;
        // TODO: move constant to Orbited.settings
        var baseRestartTimeout = 2000;
        var restartTimeout = baseRestartTimeout;
        self.onReadFrame = function(frame) {};
        self.onread = function(packet) { self.onReadFrame(packet); };
        self.onclose = function() { };
        self.connect = function(_url) {
            if (self.readyState == CT_READYSTATE_OPEN) {
            throw new Error("Already Connected");
            }
            self.logger.debug('self.connect', _url);
            url = new Orbited.URL(_url);
            url.path += '/htmlfile';
            url.setQsParameter('frameID', id.toString());
            self.readyState = CT_READYSTATE_OPEN;
            doOpen(url.render());
        };

        var doOpenIfr = function() {

            var ifr = document.createElement('iframe');
            ifr.src = url.render();
            document.body.appendChild(ifr);
        };

        var doOpen = function(_url) {
;;;         self.logger.debug('doOpen', _url);
            htmlfile = new ActiveXObject('htmlfile'); // magical microsoft object
            htmlfile.open();
            if (self.isSubDomain) {
                htmlfile.write('<html><script>' + 'document.domain="' + document.domain + '";' + '</script></html>');
            }
            else {
                htmlfile.write('<html></html>');
            }
            htmlfile.parentWindow.Orbited = Orbited;
            htmlfile.close();
            var iframe_div = htmlfile.createElement('div');
            htmlfile.body.appendChild(iframe_div);
            ifr = htmlfile.createElement('iframe');
            iframe_div.appendChild(ifr);
            ifr.src = _url;
            restartUrl = _url;
            restartTimer = window.setTimeout(reconnect, restartTimeout);
        };

        // TODO: expose this in another way besides the public api
        self.restartingStream = function(_url) {
            restartUrl = _url;
            restartTimer = window.setTimeout(reconnect, restartTimeout);
        };

        var reconnect = function() {
;;;         self.logger.debug('doing reconnect... ' + restartTimeout);
            restartTimeout*=2;
            ifr.src = restartUrl;
          restartTimer = window.setTimeout(reconnect, restartTimeout);
        };

        self.streamStarted = function() {
;;;         self.logger.debug('stream started..');
            window.clearTimeout(restartTimer);
            restartTimer = null;
            restartTimeout = baseRestartTimeout;
        };

        self.streamClosed = function() {
;;;         self.logger.debug('stream closed!');
            window.clearTimeout(restartTimer);
            self.close();
        };

        self.receive = function(id, name, data) {
            packet = {
            id: id,
            name: name,
            data: data
            };
            self.onread(packet);
        };

        self.close = function() {
            if (self.readyState == CT_READYSTATE_CLOSED) {
            return;
            }
;;;         self.logger.debug('close called, clearing timer');
            window.clearTimeout(restartTimer);
            self.readyState = CT_READYSTATE_CLOSED;
            ifr.src = 'about:blank';
            htmlfile = null;
            CollectGarbage();
            self.onclose();
        };

    };
    Orbited.CometTransports.HTMLFile.prototype.logger = Orbited.getLogger("Orbited.CometTransports.HTMLFile");
    // HTMLFile supported browsers
    Orbited.CometTransports.HTMLFile.ie = 1.0;
    Orbited.singleton.HTMLFile = {
        i: 0,
        instances: {}
    };




    Orbited.CometTransports.SSE = function() {
        var self = this;
        self.name = 'sse';
        self.onReadFrame = function(frame) {};
        self.onclose = function() { };
        self.readyState = CT_READYSTATE_INITIAL;
        var heartbeatTimer = null;
        var source = null;
        var url = null;
        var lastEventId = -1;

        self.close = function() {
            if (self.readyState == CT_READYSTATE_CLOSED) {
                return;
            }
            // TODO: can someone test this and get back to me? (No opera at the moment)
            //     : -mcarter 7-26-08
            self.readyState = CT_READYSTATE_CLOSED;
            doClose();
            self.onclose();
        };

        self.connect = function(_url) {
            if (self.readyState == CT_READYSTATE_OPEN) {
                throw new Error("Already Connected");
            }
            url = new Orbited.URL(_url);
            url.path += '/sse';
            self.readyState = CT_READYSTATE_OPEN;
            doOpen();
        };
        doClose = function() {
            source.removeEventSource(source.getAttribute('src'));
            source.setAttribute('src',"");
            if (opera.version() < 9.5) {
                document.body.removeChild(source);
            }
            source = null;
        };
        doOpen = function() {
            /*
        if (typeof(lastEventId) == "number") {
            url.setQsParameter('ack', lastEventId)
        }
*/
            source = document.createElement("event-source");
            source.setAttribute('src', url.render());
            // NOTE: without this check opera 9.5 would make two connections.
            if (opera.version() < 9.5) {
            document.body.appendChild(source);
            }
            source.addEventListener('payload', receivePayload, false);
            
            //      source.addEventListener('heartbeat', receiveHeartbeat, false);
            // start up the heartbeat timer...
            //      receiveHeartbeat();
        };
        
        var receivePayload = function(event) {
            var data = eval(event.data);
            if (typeof(data) != 'undefined') {
            for (var i = 0; i < data.length; ++i) {
                var packet = data[i];
                receive(packet[0], packet[1], packet[2]);
            }
            }
            
        };
        /*    var receiveHeartbeat = function() {
               window.clearTimeout(heartbeatTimer);
               heartbeatTimer = window.setTimeout(reconnect, Orbited.settings.HEARTBEAT_TIMEOUT)
              }      */
        var receive = function(id, name, data) {
            var tempId = parseInt(id);
            if (!isNaN(tempId)) {
            // NOTE: The old application/x-dom-event-stream transport doesn't
            //         allow us to put in the lastEventId on reconnect, so we are
            //         bound to get double copies of some of the events. Therefore
            //         we are going to throw out the duplicates. Its not clear to
            //         me that this is a perfect solution.
            //         -mcarter 8-9-08
            //          if (tempId <= lastEventId) {
            //          return
            //          }
            lastEventId = tempId;
            }
            // NOTE: we are dispatching null-id packets. Is this correct?
            //     -mcarter 9-8-08
            packet = {
            id: id,
            name: name,
            data: data
            };
            self.onReadFrame(packet);
        };
    };
    Orbited.CometTransports.SSE.prototype.logger = Orbited.getLogger("Orbited.CometTransports.SSE");
    
    Orbited.CometTransports.SSE.opera = 1.0;
    Orbited.CometTransports.SSE.opera8 = 1.0;
    Orbited.CometTransports.SSE.opera9 = 1.0;
    Orbited.CometTransports.SSE.opera9_5 = 0.8;



    /* This is an old implementation of the URL class. Jacob is cleaning it up
 * -mcarter, 7-30-08
 *
 * Jacob is actually throwing this away and rewriting from scratch
 * -mcarter 11-14-08
 */
    Orbited.URL = function(_url) {
        var self = this;
        var protocolIndex = _url.indexOf("://");
        if (protocolIndex != -1) self.protocol = _url.slice(0,protocolIndex);
        else protocolIndex = -3;

        var domainIndex = _url.indexOf('/', protocolIndex+3);
        if (domainIndex == -1) domainIndex=_url.length;
            
        var hashIndex = _url.indexOf("#", domainIndex);
        if (hashIndex != -1) self.hash = _url.slice(hashIndex+1);
        else hashIndex = _url.length;
            
        var uri = _url.slice(domainIndex, hashIndex);
        var qsIndex = uri.indexOf('?');
        if (qsIndex == -1) qsIndex=uri.length;

        self.path = uri.slice(0, qsIndex);
        self.qs = uri.slice(qsIndex+1);
        if (self.path == "") self.path = "/";
            
        var domain = _url.slice(protocolIndex+3, domainIndex);
        var portIndex = domain.indexOf(":");
        if (portIndex == -1) {
            self.port = 80;
            portIndex = domain.length;
        }
        else {
            self.port = parseInt(domain.slice(portIndex+1));
        }
        if (isNaN(this.port)) throw new Error("Invalid _url");
            
        self.domain = domain.slice(0, portIndex);

        self.render = function() {
            var output = "";
            if (typeof(self.protocol) != "undefined")
            output += self.protocol + "://";
            output += self.domain;
            if (self.port != 80 && typeof(self.port) != "undefined" && self.port != null)
            if (typeof(self.port) != "string" || self.port.length > 0)
                output += ":" + self.port;
            if (typeof(self.path) == "undefined" || self.path == null)
            output += '/';
            else
            output += self.path;
            if (self.qs.length > 0)
            output += '?' + self.qs;
            if (typeof(self.hash) != "undefined" && self.hash.length > 0)
            output += "#" + self.hash;
            return output;
        };
        self.isSamePort = function(_url) {
            _url = new Orbited.URL(_url);
            return _url.port == self.port;
        }
        self.isSameDomain = function(_url) {
            _url = new Orbited.URL(_url);
            
            if (!_url.domain || !self.domain)
            return true;
            return (_url.port == self.port && _url.domain == self.domain);
        };
        self.isSameParentDomain = function(_url) {
            _url = new Orbited.URL(_url);
            if (_url.domain == self.domain) {
            return true;
            }
            var orig_domain = _url.domain;
            var parts = document.domain.split('.');
            //      var orig_domain = document.domain
            for (var i = 0; i < parts.length-1; ++i) {
            var new_domain = parts.slice(i).join(".");
            if (orig_domain == new_domain)
                return true;
            }
            return false;
        };
        self.isSubDomain = function(_url) {
            _url = new Orbited.URL(_url);
            if (!_url.domain || !self.domain) {
                return false;
            }
            return (_url.port == self.port && self.domain.indexOf("."+_url.domain) > 0);
//            return (_url.port == self.port && _url.domain == self.domain.split('.').slice(1).join('.'));
        };
        var decodeQs = function(qs) {
            //      alert('a')
            if (qs.indexOf('=') == -1) return {};
            var result = {};
            var chunks = qs.split('&');
            for (var i = 0; i < chunks.length; ++i) {
            var cur = chunks[i];
            var pieces = cur.split('=');
            result[pieces[0]] = pieces[1];
            }
            return result;
        };
        var encodeQs = function(o) {
            var output = "";
            for (var key in o)
            output += "&" + key + "=" + o[key];
            return output.slice(1);
        };
        self.setQsParameter = function(key, val) {
            var curQsObj = decodeQs(self.qs);
            curQsObj[key] = val;
            self.qs = encodeQs(curQsObj);
        };

        self.mergeQs = function(qs) {
            var newQsObj = decodeQs(qs);
            for (key in newQsObj) {
            curQsObj[key] = newQsObj[key];
            }
        };
        self.removeQsParameter = function(key) {
            var curQsObj = decodeQs(self.qs);
            delete curQsObj[key];
            self.qs = encodeQs(curQsObj);
        };

        self.merge = function(targetUrl) {
            if (typeof(self.protocol) != "undefined" && self.protocol.length > 0) {
            self.protocol = targetUrl.protocol;
            }
            if (targetUrl.domain.length > 0) {
            self.domain = targetUrl.domain;
            self.port = targetUrl.port;
            }
            self.path = targetUrl.path;
            self.qs = targetUrl.qs;
            self.hash = targetUrl.hash;
        };

    };

    Orbited.utf8 = {};
    Orbited.utf8.decode = function(s) {
        var ret = [];
        var j = 0;
        function pad6(str) {
            while(str.length < 6) { str = "0" + str; } return str;
        }
        for (var i=0; i < s.length; i++) {
            if ((s.charCodeAt(i) & 0xf8) == 0xf0) {
                if (s.length -j < 4) { break; }
                j+=4;
                ret.push(String.fromCharCode(parseInt(
                    (s.charCodeAt(i) & 0x07).toString(2) +
                    pad6((s.charCodeAt(i+1) & 0x3f).toString(2)) +
                    pad6((s.charCodeAt(i+2) & 0x3f).toString(2)) +
                    pad6((s.charCodeAt(i+3) & 0x3f).toString(2))
                    , 2)));
                i += 3;
            } else if ((s.charCodeAt(i) & 0xf0) == 0xe0) {
                if (s.length -j < 3) { break; }
                j+=3;
                ret.push(String.fromCharCode(parseInt(
                    (s.charCodeAt(i) & 0x0f).toString(2) +
                    pad6((s.charCodeAt(i+1) & 0x3f).toString(2)) +
                    pad6((s.charCodeAt(i+2) & 0x3f).toString(2))
                    , 2)));
                i += 2;
            } else if ((s.charCodeAt(i) & 0xe0) == 0xc0) {
                if (s.length -j < 2) { break }
                j+=2;
                ret.push(String.fromCharCode(parseInt(
                    (s.charCodeAt(i) & 0x1f).toString(2) +
                    pad6((s.charCodeAt(i+1) & 0x3f).toString(2), 6)
                    , 2)));
                i += 1;
            } else {
                j+=1;
                ret.push(String.fromCharCode(s.charCodeAt(i)));
            }
        }
        return [ret.join(""), j];
    };
    
    // TODO rename to encode
    Orbited.utf8.encode = function(text) {
        var ret = [];

        function pad(str, len) {
            while(str.length < len) { str = "0" + str; } return str;
        }
        var e = String.fromCharCode;
        for (var i=0; i < text.length; i++) {
            var chr = text.charCodeAt(i);
            if (chr <= 0x7F) {
            ret.push(e(chr));
            } else if(chr <= 0x7FF) {
            var binary = pad(chr.toString(2), 11);
            ret.push(e(parseInt("110"    + binary.substr(0,5), 2)));
            ret.push(e(parseInt("10"    + binary.substr(5,6), 2)));
            } else if(chr <= 0xFFFF) {
            var binary = pad(chr.toString(2), 16);
            ret.push(e(parseInt("1110"    + binary.substr(0,4), 2)));
            ret.push(e(parseInt("10"    + binary.substr(4,6), 2)));
            ret.push(e(parseInt("10"    + binary.substr(10,6), 2)));
            } else if(chr <= 0x10FFFF) {
            var binary = pad(chr.toString(2), 21);
            ret.push(e(parseInt("11110" + binary.substr(0,3), 2)));
            ret.push(e(parseInt("10"    + binary.substr(3,6), 2)));
            ret.push(e(parseInt("10"    + binary.substr(9,6), 2)));
            ret.push(e(parseInt("10"    + binary.substr(15,6), 2)));
            }
        }
        return ret.join("");
    };

        /*
         * We create Orbited.JSON whether or not some other JSON
         * exists. This is because Orbited.JSON is compatible with
         * JSON.js (imported by xsdrBridge), whereas various other
         * JSONs, including the one that ships with Prototype, are
         * not, leading to dumb errors.
         *     -mario
         */

        Orbited.JSON = function () {

            function f(n) {
            // Format integers to have at least two digits.
            return n < 10 ? '0' + n : n;
            }

            Date.prototype.toJSON = function (key) {

            return this.getUTCFullYear()   + '-' +
                f(this.getUTCMonth() + 1) + '-' +
                f(this.getUTCDate())       + 'T' +
                f(this.getUTCHours())       + ':' +
                f(this.getUTCMinutes())   + ':' +
                f(this.getUTCSeconds())   + 'Z';
            };

            String.prototype.toJSON =
            Number.prototype.toJSON =
            Boolean.prototype.toJSON = function (key) {
                return this.valueOf();
            };

            var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
            escapeable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
            gap,
            indent,
            meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
            },
            rep;


            function quote(string) {
            
            // If the string contains no control characters, no quote characters, and no
            // backslash characters, then we can safely slap some quotes around it.
            // Otherwise we must also replace the offending characters with safe escape
            // sequences.
            
            escapeable.lastIndex = 0;
            return escapeable.test(string) ?
                '"' + string.replace(escapeable, function (a) {
                var c = meta[a];
                if (typeof c === 'string') {
                    return c;
                }
                return '\\u' + ('0000' +
                        (+(a.charCodeAt(0))).toString(16)).slice(-4);
                }) + '"' :
            '"' + string + '"';
            }
            

            function str(key, holder) {

            // Produce a string from holder[key].

            var i,        // The loop counter.
            k,        // The member key.
            v,        // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

            // If the value has a toJSON method, call it to obtain a replacement value.

            if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
                value = value.toJSON(key);
            }

            // If we were called with a replacer function, then call the replacer to
            // obtain a replacement value.

            if (typeof rep === 'function') {
                value = rep.call(holder, key, value);
            }

            // What happens next depends on the value's type.

            switch (typeof value) {
            case 'string':
                return quote(value);

            case 'number':

                // JSON numbers must be finite. Encode non-finite numbers as null.

                return isFinite(value) ? String(value) : 'null';

            case 'boolean':
            case 'null':

                // If the value is a boolean or null, convert it to a string. Note:
                // typeof null does not produce 'null'. The case is included here in
                // the remote chance that this gets fixed someday.

                return String(value);

                // If the type is 'object', we might be dealing with an object or an array or
                // null.

            case 'object':

                // Due to a specification blunder in ECMAScript, typeof null is 'object',
                // so watch out for that case.

                if (!value) {
                return 'null';
                }

                // Make an array to hold the partial results of stringifying this object value.

                gap += indent;
                partial = [];

                // If the object has a dontEnum length property, we'll treat it as an array.

                if (typeof value.length === 'number' &&
                !(value.propertyIsEnumerable('length'))) {

                // The object is an array. Stringify every element. Use null as a placeholder
                // for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

                // Join all of the elements together, separated with commas, and wrap them in
                // brackets.

                v = partial.length === 0 ? '[]' :
                    gap ? '[\n' + gap +
                    partial.join(',\n' + gap) + '\n' +
                    mind + ']' :
                    '[' + partial.join(',') + ']';
                gap = mind;
                return v;
                }

                // If the replacer is an array, use it to select the members to be stringified.

                if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    k = rep[i];
                    if (typeof k === 'string') {
                    v = str(k, value);
                    if (v) {
                        partial.push(quote(k) + (gap ? ': ' : ':') + v);
                    }
                    }
                }
                } else {

                // Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                    v = str(k, value);
                    if (v) {
                        partial.push(quote(k) + (gap ? ': ' : ':') + v);
                    }
                    }
                }
                }

                // Join all of the member texts together, separated with commas,
                // and wrap them in braces.

                v = partial.length === 0 ? '{}' :
                gap ? '{\n' + gap + partial.join(',\n' + gap) + '\n' +
                mind + '}' : '{' + partial.join(',') + '}';
                gap = mind;
                return v;
            }
            }

            // Return the JSON object containing the stringify and parse methods.

            return {
            stringify: function (value, replacer, space) {

                // The stringify method takes a value and an optional replacer, and an optional
                // space parameter, and returns a JSON text. The replacer can be a function
                // that can replace values, or an array of strings that will select the keys.
                // A default replacer method can be provided. Use of the space parameter can
                // produce text that is more easily readable.

                var i;
                gap = '';
                indent = '';

                // If the space parameter is a number, make an indent string containing that
                // many spaces.

                if (typeof space === 'number') {
                for (i = 0; i < space; i += 1) {
                    indent += ' ';
                }

                // If the space parameter is a string, it will be used as the indent string.

                } else if (typeof space === 'string') {
                indent = space;
                }

                // If there is a replacer, it must be a function or an array.
                // Otherwise, throw an error.

                rep = replacer;
                if (replacer && typeof replacer !== 'function' &&
                (typeof replacer !== 'object' ||
                 typeof replacer.length !== 'number')) {
                throw new Error('JSON.stringify');
                }

                // Make a fake root object containing our value under the key of ''.
                // Return the result of stringifying the value.

                return str('', {'': value});
            },


            parse: function (text, reviver) {

                // The parse method takes a text and an optional reviver function, and returns
                // a JavaScript value if the text is a valid JSON text.

                var j;

                function walk(holder, key) {

                // The walk method is used to recursively walk the resulting structure so
                // that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        v = walk(value, k);
                        if (v !== undefined) {
                        value[k] = v;
                        } else {
                        delete value[k];
                        }
                    }
                    }
                }
                return reviver.call(holder, key, value);
                }


                // Parsing happens in four stages. In the first stage, we replace certain
                // Unicode characters with escape sequences. JavaScript handles many characters
                // incorrectly, either silently deleting them, or treating them as line endings.

                cx.lastIndex = 0;
                if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' + ('0000' +
                            (+(a.charCodeAt(0))).toString(16)).slice(-4);
                });
                }

                // In the second stage, we run the text against regular expressions that look
                // for non-JSON patterns. We are especially concerned with '()' and 'new'
                // because they can cause invocation, and '=' because it can cause mutation.
                // But just to be safe, we want to reject all unexpected forms.

                // We split the second stage into 4 regexp operations in order to work around
                // crippling inefficiencies in IE's and Safari's regexp engines. First we
                // replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
                // replace all simple value tokens with ']' characters. Third, we delete all
                // open brackets that follow a colon or comma or that begin the text. Finally,
                // we look to see that the remaining characters are only whitespace or ']' or
                // ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

                if (/^[\],:{}\s]*$/.
                test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@').
replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']').
                             replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

                    // In the third stage we use the eval function to compile the text into a
                    // JavaScript structure. The '{' operator is subject to a syntactic ambiguity
                    // in JavaScript: it can begin a block or an object literal. We wrap the text
                    // in parens to eliminate the ambiguity.

                    j = eval('(' + text + ')');

                    // In the optional fourth stage, we recursively walk the new structure, passing
                    // each name/value pair to a reviver function for possible transformation.

                    return typeof reviver === 'function' ?
                    walk({'': j}, '') : j;
                }

                          // If the text is not JSON parseable, then a SyntaxError is thrown.

                          throw new SyntaxError('JSON.parse');
                         }
                    };
                   }();
            })();


        // Try to auto detect the Orbited port and hostname
        (function() {
            try {
            var scripts = document.getElementsByTagName('script');
            for (var i = 0; i < scripts.length; ++i) {
                var script = scripts[i];
                if (script.src.match('/static/Orbited\.js$')) {
                var url = new Orbited.URL(script.src);
                if (url.render().indexOf('http') != 0) {
                    var url = new Orbited.URL(window.location.toString());
                }
                Orbited.settings.hostname = url.domain;
                Orbited.settings.port = url.port;
                break;
                }
            }
            } catch(e) {
            //    alert("Error! " + e.name + ": " + e.message);
            }
        })();
var client = {
	sound : false,
	connection : true,
	still_going : true,
	z_top : 0,
	active : true,
	mode : "main",
	init : function() {
		window.status = texts["client_title"]+" "+client_vers;
		 // DEVONLY
		/*if (window.console && window.console.firebug) {
			$("preload_logo").innerHTML = "<font face='verdana' color='white'>"+texts["nofirebug"]+"</font>";
		}
		else {
			open_script("preload.php?r="+Math.round(9000*Math.random()));
		}*/
		open_script("preload.php?r="+Math.round(9000*Math.random()));
	},
	start : function () {
		if(this.connection) {
			var req = ajaxCreate();
			req.open("GET", "prelogin.php?r="+Math.round(9000*Math.random()), true);
			req.onreadystatechange = function() {
				if (req.readyState == 4) {
					$("prelogin_div").innerHTML = req.responseText;
					setTimeout(function(){var callback = client.loadlogo_gone(); fade_out('preload_div', callback);},250);
				}
			};
			req.send(null);
		}
	},
	loadlogo_gone : function () {
		var output = new Object();
		output.complete = function() {
			ShowContent("loadview");
			HideContent("preload_div");
			setTimeout(function(){preload.init();}, 500);
		};
		return output;
	},
	loadbar_gone : function () {
		var output = new Object();
		output.complete = function() {
			HideContent("loadview");
			packets.init();
		};
		return output;
	},
	allErr : function (message, url, line) {
		//client.error(message, line); !!!
		debug.error("Error: "+message+" on line "+line);
		return true;
	},
	loaded : function () {
		this.z_top = (get_top_real()*1)+1;
		setInterval(function(){client.checks();}, 500);
		setTimeout(function(){var callback = client.loadbar_gone(); fade_out('loadview', callback);}, 250);
	},
	clicked : function () {
		laatst_actief = tijdstip;
	},
	checks : function () {
		if (window.console && window.console.firebug) {
			//this.connection = false; DEVONLY
		}
		if (!this.connection && this.still_going) {
			this.still_going = false;
			room.leave();
			set_top("connection_lost_layer");
			ShowContent('connection_lost_layer');
			conn.close();
		}
		if (this.connection) {
			if ((laatst_actief+60) >= tijdstip) {
				if (this.active == false) {
					if (user.logged_in) {
						packets.send.activeagain();
						debug.yellow("Client is active again");
					}
					this.active = false;
				}
				this.active = true;
			}
			else {
				if (this.active == true) {
					if (user.logged_in) {
						packets.send.inactive();
						debug.yellow("Client is inactive");
					}
					this.active = false;
				}
			}
		}
	},
	notify : function (message, title) {
		if (!title) {
			title = texts["notify_default"];
		}
		setTimeout(function(){add_alert(title, message);},0);
	},
	logged_in : function () {
		avatars.init();
		$("messenger_text_1_jouwnaam").innerHTML = ucfirst(user.name);
		$("messenger_mission").value = user.messenger;
		ShowContent("mainview_div");
		mainview.showtool();
	},
	logged_in2 : function () {
		login.doing = false;
		user.logged_in = true;
		packets.send.login_done();
		// eerst logged_in true zetten, dan allerlei data afhandelen
		
		$('yourface1').style.backgroundImage = "url('avatar.php?figure="+user.clothes+"&size=b&direction=3&head_direction=3&gesture=sml&action=std')";
		$('yourface2').style.backgroundImage = "url('avatar.php?figure="+user.clothes+"&size=b&direction=3&head_direction=3&gesture=sml&action=std')";
		catalogus.moneyupdate();
		
		setInterval(function(){room.chat.show_chat.move();}, 3000);
		setInterval(function(){navi.updateask();}, 60000); // elke minuut navigator update, als die geopend is
		
		HideContent("login_div");
		removechilds("login_div"); // onnodige divs en classes verwijderen
		removechilds("register");
		login = null;
		register = null;
		
		set_top("navigator");
		ShowContent("navigator");
		navi.opened = true;
	},
	error : function (message, line) {
		var error = new Error(message);
		$("disconnected1").innerHTML = texts["client_error_title"];
		$("disconnected2").innerHTML = texts["client_error_message"]+"<br><b>"+error+" on line "+line+"</b>";
		this.connection = false;
	},
	exit : function () {
		room.leave();
		this.active = false;
		this.connection = false;
	},
	exit2 : function () {
		room.leave();
		HideContent("login_div");
		HideContent("mainview_div");
		HideContent("windows_div");
		this.active = false;
	},
	block : function () {
		room.tiles.click = false;
		set_top("block");
		ShowContent("block");
	},
	unblock : function () {
		if (client.mode != "main") {
			room.tiles.click = true;
		}
		HideContent("block");
	}	
};
var user = {
	logged_in : false,
	id : 0,
	name : "",
	rights : new Array(),
	money : 0,
	badge : 0,
	badges : 0,
	email : "",
	clothes : "",
	mission : "",
	messenger : "",
	sessionearned : 0
};
var Core = {
	CatchEnter : function (e, type) {
		try {
			var characterCode = e.keyCode;
			if(characterCode == 13) {
				switch(type) {
					case "roomchat":
						room.chat.new_chat();
						return false;
					break;
					case "chatbox":
						chatbox.go();
						return false;
					break;
					case "login":
						login.attempt();
						return false;
					break;
					case "debug":
						debug.userinput();
						return false;
					case "messenger":
						messenger.search();
						return false;
					case "navi":
						navi.search();
						return false;
					break;
					default:
						return true;
					break;
				}
			}
			return true;
		}
		catch(err) {
			return true;
		}
	}
};
function ajaxCreate() {
	try {
		if(window.XMLHttpRequest) {
			return new XMLHttpRequest();
		}
		else if(window.ActiveXObject) {
			return new ActiveXObject("Microsoft.XMLHTTP");
		}
	}
	catch (e) {
		return false;
	}
}
function ajaxSync(url, data) {
	var req = ajaxCreate();
	if(data != null)  {
		req.open("POST", url, false);
		req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
		req.setRequestHeader("Content-length", data.length);
	}
	else {
		req.open("GET", url, false);
	}
	req.send(data);
	return req;
}
function ajaxAsync(url, data, callback) {
	var req = ajaxCreate();
	if (data != null) {
		req.open("POST", url, true);
		req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
		req.setRequestHeader("Content-length", data.length);
	}
	else {
		req.open("GET", url, true);
	}
	req.onreadystatechange = function() { if(req.readyState == 4) callback.complete(req); };
	req.send(data);
	return req;
}
function open_script(script_location) {
	if (client.connection) {
		var script = document.createElement('script');
		script.type = 'text/javascript';
		script.src = script_location;
		try {
			document.getElementsByTagName('head').item(0).appendChild(script);
		}
		catch(err) {
			debug.error("Runscript Error: "+err.description);
		}
	}
}
function run_script(jscode) {
	if (client.connection) {
		var script = document.createElement("script");
		script.type = "text/javascript"; 
		script.text = jscode;
		try {
			document.getElementsByTagName('head').item(0).appendChild(script);
		}
		catch(err) {
			debug.error("Run_script Error: "+err.description);
		}
	}
}
function get_script(url) {
	if (client.connection) {
		var req = ajaxCreate();
		req.open("GET", url, true);
		req.onreadystatechange = function() { if (req.readyState == 4) run_script(req.responseText); };
		req.send(null);
	}
}
var client_button = 0;
var alert_nr = 0;
var ctrl_status = false;
var alt_status = false;
var fading = false;
function get_top() {
	client.z_top++;
	return(client.z_top);
}
function set_top(el) {
	$(el).style.zIndex = ((get_top())*1)+1;
}
function get_top_real()
{
	if (document.all)
	{
		topZ = 0;
		for (i = 0; i < document.all.length; i++)
		{
			dit_element = document.all(i);
			topZ = Math.max(dit_element.style.zIndex, topZ);
		}
	}
	else
	{
		alles = new Array();
		alles = document.getElementsByTagName('*');
		topZ = 0;
		for (i = 0; i < alles.length; i++)
		{
			dezeZ = alles[i].style.zIndex;
			if (dezeZ == '')
			{
				dezeZ = 0;
			}
			if (parseInt(dezeZ) > parseInt(topZ))
			{
				topZ = dezeZ;
			}
		}
	}
	return(topZ);
}
function stop_tclick() {
	if (client.mode != "main") {
		HideContent('tile_hover');
		room.tiles.click = false;
	}
}
function start_tclick() {
	if (client.mode != "main") {
		room.tiles.click = true;
	}
}
function add_alert(alert_title, alert_message, alert_icon) {
	alert_nr++;
	client_button++;
	var alert_call = "alert_"+alert_nr;
	var new_alert = document.createElement("div");
	new_alert.id = alert_call;
	new_alert.style.position = "absolute";
	new_alert.style.left = 0;
	new_alert.style.top = 0;
	new_alert.style.zIndex = ((get_top())*1)+1;
	new_alert.style.visibility="hidden";
	new_alert.innerHTML = "<table border='0' cellpadding='0' cellspacing='0'><tr><td width='17' height='35' style='background: url(\"images/alert/left_top.gif\");' onmousedown='dragStart(event, \""+alert_call+"\")'></td><td height='35' style='background: url(\"images/alert/center_top.gif\") repeat-x; padding: 4px 0px 0px 0px;' valign='top' align='middle' onmousedown='dragStart(event, \""+alert_call+"\")'><table border='0' height='15' cellpadding='0' cellspacing='0' style='font-weight: bold;'><tr><td width='4'></td><td><div style='padding: 2px 4px 2px 4px; color: #eeeeee; background-color: #6794a7; font-weight: bold;' class='eigenfont'>"+alert_title+"</div></td><td width='4'></td></tr></table></td><td width='18' height='35' style='background: url(\"images/alert/right_top.gif\");' onmousedown='dragStart(event, \""+alert_call+"\")'></td></tr><tr><td width='17' height='50' style='background: url(\"images/alert/left_center.gif\");'></td><td bgcolor='#efefef' align='middle' class='clientfont'>&nbsp;&nbsp;"+alert_message+"&nbsp;&nbsp;</td><td width='18' height='50' style='background: url(\"images/alert/right_center.gif\");'></td></tr><tr><td width='17' height='30' style='background: url(\"images/alert/left_center.gif\");'></td><td bgcolor='#efefef' align='middle'><table id='c_knop_"+client_button+"' onclick='del_alert("+client_button+")' onmousedown='button_down("+client_button+",2,true)' onmouseup='button_up("+client_button+",2,true)' onmouseout='button_up("+client_button+",2,true)' border='0' cellpadding='0' cellspacing='0'><tr><td width='11' height='23'><img id='c_knop_"+client_button+"_left' src='images/buttons/2_left1.gif'></td><td id='c_knop_"+client_button+"_center' height='23' style='background: url(\"images/buttons/2_center1.gif\") repeat-x; padding: 5px 0px 0px 0px;' valign='top'><div id='c_knop_"+client_button+"_text' class='eigenfont' style='color: #000000;'><b>"+texts["notify_ok"]+"</b></div></td><td width='11' height='23'><img id='c_knop_"+client_button+"_right' src='images/buttons/2_right1.gif'></td></tr></table></td><td width='18' height='30' style='background: url(\"images/alert/right_center.gif\");'></td></tr><tr><td width='17' height='17' style='background: url(\"images/alert/left_bottom.gif\");'></td><td style='background: url(\"images/alert/center_bottom.gif\") repeat-x;'></td><td width='18' height='17' style='background: url(\"images/alert/right_bottom.gif\");'></td></tr></table>";
	addEvent(new_alert, 'mouseover', function() { stop_tclick(); });
	addEvent(new_alert, 'mouseout', function() { start_tclick(); });
	addEvent(new_alert, 'click', function() { set_top(alert_call); });
	$("alerts").appendChild(new_alert);
	var object = $(alert_call);
	setTimeout(function(){object_center(alert_call); fade_in(alert_call, false); object.style.visibility = "visible";object.style.width = object.clientWidth; object.style.height = object.clientHeight;},0);
}
function del_alert(id) {
	var this_alert = $("alert_"+id);
	if (this_alert) {
		var callback = del_alert2(this_alert);
		fade_out("alert_"+id, callback);
	}
}
function del_alert2(this_alert) {
	var output = new Object();
	output.complete = function() {
		$("alerts").removeChild(this_alert);
		start_tclick();
	};
	return output;
}
function button_down(button_nr, button_type, client) {
	if (client) {
		$("c_knop_"+button_nr+"_left").src = "images/buttons/"+button_type+"_left2.gif";
		$("c_knop_"+button_nr+"_center").style.background = "url(images/buttons/"+button_type+"_center2.gif) repeat-x";
		$("c_knop_"+button_nr+"_right").src = "images/buttons/"+button_type+"_right2.gif";
	}
	else {
		$("knop_"+button_nr+"_left").src = "images/buttons/"+button_type+"_left2.gif";
		$("knop_"+button_nr+"_center").style.background = "url(images/buttons/"+button_type+"_center2.gif) repeat-x";
		$("knop_"+button_nr+"_right").src = "images/buttons/"+button_type+"_right2.gif";
	}
}
function button_up(button_nr, button_type, client) {
	if (client) {
		$("c_knop_"+button_nr+"_left").src = "images/buttons/"+button_type+"_left1.gif";
		$("c_knop_"+button_nr+"_center").style.background = "url(images/buttons/"+button_type+"_center1.gif) repeat-x";
		$("c_knop_"+button_nr+"_right").src = "images/buttons/"+button_type+"_right1.gif";
	}
	else {
		$("knop_"+button_nr+"_left").src = "images/buttons/"+button_type+"_left1.gif";
		$("knop_"+button_nr+"_center").style.background = "url(images/buttons/"+button_type+"_center1.gif) repeat-x";
		$("knop_"+button_nr+"_right").src = "images/buttons/"+button_type+"_right1.gif";
	}
}
function movelayer(layername, endposx, endposy, steps, intervals, powr, movecallback)
{
	var move_layer = $(layername);
	if (!move_layer.currentPos) {
		move_layer.currentPos = [parseInt(move_layer.style.left),parseInt(move_layer.style.top)];
	}
	domovelayer(move_layer,move_layer.currentPos,[endposx,endposy],steps,intervals,powr,movecallback);
}
function easeInOut(minValue,maxValue,totalSteps,actualStep,powr)
{
	var delta = maxValue - minValue;
	var stepp = minValue+(Math.pow(((1 / totalSteps)*actualStep),powr)*delta);
	return Math.ceil(stepp);
}
function domovelayer(elem,startPos,endPos,steps,intervals,powr,movecallback)
{
	if (elem.posChangeMemInt) window.clearInterval(elem.posChangeMemInt);
	var actStep = 0;
	elem.posChangeMemInt = window.setInterval(
		function() {
			elem.currentPos = [
				easeInOut(startPos[0],endPos[0],steps,actStep,powr),
				easeInOut(startPos[1],endPos[1],steps,actStep,powr)
				];
			elem.style.left = elem.currentPos[0]+"px";
			elem.style.top = elem.currentPos[1]+"px";
			actStep++;
			if (actStep > steps)
			{
				elem.currentPos = null;
				window.clearInterval(elem.posChangeMemInt);
				if (movecallback)
				{
					movecallback.complete();
				}
			}
		}
		,intervals);
}
function set_trans(el, trans) {
	var object = $(el).style;
	object.opacity = trans;
	object.MozOpacity = trans;
	object.KhtmlOpacity = trans;
	object.filter = "alpha(opacity="+trans*100+")";
}
function fade_in(el, callback) {
	if ($(el))
	{
		set_trans(el, 0);
		ShowContent(el);
		do_fade_in(el, callback);
	}
}
function fade_out(el, callback)
{
	if ($(el))
	{
		set_trans(el, 1);
		ShowContent(el);
		do_fade_out(el, callback);
	}
}
function do_fade_in(el, callback)
{
	if ($(el))
	{
		var opacity = parseFloat($(el).style.opacity);
		if (opacity > 0.84)
		{
			set_trans(el, 1);
			if (callback)
			{
				callback.complete();
			}
		}
		else
		{
			opacity = opacity + 0.15;
			set_trans(el, opacity);
			setTimeout(function(){do_fade_in(el, callback);}, 50);
		}
	}
	return true;
}
function do_fade_out(el, callback)
{
	if ($(el))
	{
		var opacity = parseFloat($(el).style.opacity);
		if (opacity < 0.16)
		{
			set_trans(el, 0);
			if (callback)
			{
				callback.complete();
			}
		}
		else
		{
			opacity = opacity - 0.15;
			set_trans(el, opacity);
		    setTimeout(function(){do_fade_out(el, callback);}, 50);
		}
	}
	return true;
}
function createCookie(name,value,days) {
	if (days) {
		var date = new Date();
		date.setTime(date.getTime()+(days*24*60*60*1000));
		var expires = "; expires="+date.toGMTString();
	}
	else var expires = "";
	document.cookie = name+"="+value+expires+"; path=/";
}
function readCookie(name) {
	var nameEQ = name + "=";
	var ca = document.cookie.split(';');
	for(var i=0;i < ca.length;i++) {
		var c = ca[i];
		while (c.charAt(0)==' ') c = c.substring(1,c.length);
		if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
	}
	return null;
}
function eraseCookie(name) {
	createCookie(name,"",-1);
}
function addEvent(obj, eventType, fn, useCapture)
{
    if (obj.addEventListener) {
        obj.addEventListener(eventType, fn, useCapture);
        return true;
    } else {
        if (obj.attachEvent) {
            var r = obj.attachEvent("on"+eventType, fn);
            return r;
        }
    }
}
function isset(what) {
	return Boolean(typeof what != 'undefined' && what);
}
function object_center(el) {
	var object = $(el);
	object.style.left = (document.documentElement.scrollLeft+document.body.clientWidth-object.clientWidth)/2+"px";
	object.style.top = (document.documentElement.scrollTop+document.body.clientHeight-object.clientHeight)/2+"px";
	return true;
}
function HideContent (div)
{
	if(typeof(div) == "object")
	{
		try
		{
			for(var i = 0; i < div.length; i++)
			{
				if($(div[i])) $(div[i]).hide();
				else debug.error("Couldn't hide: " + div[i]);
			}
		}
		catch (err)
		{
			debug.error(err);
		}
	}
	else if(typeof(div) == "string")
	{
		if($(div)) $(div).hide();
		else debug.error("Couldn't hide: " + div);
	}
	else debug.error("lol");
}
function ShowContent(d) {
	if (d.length < 1) {
		return;
	}
	if ($(d)) {
		$(d).style.display = "";
	}
	else {
		debug.error("Couldn't show '"+d+"'");
	}
}
function ReverseContent(d) {
	if (d.length < 1) {
		return;
	}
	if ($(d).style.display == "none") {
		ShowContent(d);
	}
	else {
		HideContent(d);
	}
}
function fade_out_done(d)
{
	var output = new Object();
	output.complete = function()
	{
		HideContent(d);
		fading = false;
	};
	return output;
}
function fade_in_done()
{
	var output = new Object();
	output.complete = function()
	{
		fading = false;
	};
	return output;
}
function reverse_fade(d)
{
	if (d.length < 1)
	{
		return;
	}
	if ($(d).style.display == "none")
	{
		fading = true;
		var callback = fade_in_done();
		fade_in(d, callback);
	}
	else
	{
		fading = true;
		var callback = fade_out_done(d);
		fade_out(d, callback);
	}
}
function in_array(search_term, search_in)
{
	var i = search_in.length;
	if (i > 0) {
		do {
			if (search_in[i] === search_term) {
				return true;
			}
		} while (i--);
	}
	return false;
}
function textCounter(field, maxlimit)
{
	if (field.value.length > maxlimit) {
		field.value = field.value.substring(0, maxlimit);
	}
}
function isValidEmail(str) {
	return (str.indexOf(".") > 2) && (str.indexOf("@") > 0);
}
function arr_dump(arr,level)
{
	var dumped_text = "";
	if(!level)
	{
		level = 0;
	}
	var level_padding = "";
	for (var j=0; j<level; j++)
	{
		level_padding += "    ";
	}
	if(typeof(arr) == 'object')
	{
		for(var item in arr)
		{
			value = arr[item];
			if(typeof(value) == 'object')
			{
				dumped_text += level_padding + "'" + item + "' ...\n";
				dumped_text += arr_dump(value,level);
			}
			else
			{
				dumped_text += level_padding + "    '" + item + "' => \"" + value + "\"\n";
			}
		}
	}
	else
	{
		dumped_text = "===>"+arr+"<===("+typeof(arr)+")";
	}
	return dumped_text;
}
function removechilds(layer)
{
	var holder = $(layer);
	while(holder.hasChildNodes()) {
		holder.removeChild(holder.lastChild);
	}
}
function htmlEntities(str) {
    return String(str).replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
}
function ucfirst (str) {
    str += '';
    var f = str.charAt(0).toUpperCase();
    return f + str.substr(1);
}
var max_window_x, max_window_y;
function dragStart(event, id)
{
	var el;
	var x, y;
	if (id)
	{
		dragObj.elNode = $(id);
	}
	else
	{
		dragObj.elNode = document.all ? window.event.srcElement : event.target;
		if (dragObj.elNode.nodeType == 3)
		{
			dragObj.elNode = dragObj.elNode.parentNode;
		}
	}
	x = document.all ? window.event.clientX + document.documentElement.scrollLeft + document.body.scrollLeft : event.clientX + window.scrollX;
	y = document.all ? window.event.clientY + document.documentElement.scrollTop + document.body.scrollLeft : event.clientY + window.scrollY;
	var dragwindow_width = parseInt(dragObj.elNode.style.width);
	var dragwindow_height = parseInt(dragObj.elNode.style.height);
	max_window_x = client_width - dragwindow_width + 10;
	if (id != "camera") {
		max_window_y = client_height - dragwindow_height + 10;
	}
	else {
		max_window_y = client_height - 150;
	}
	dragObj.cursorStartX = x;
	dragObj.cursorStartY = y;
	dragObj.elStartLeft  = parseInt(dragObj.elNode.style.left, 10);
	dragObj.elStartTop   = parseInt(dragObj.elNode.style.top,  10);
	if (isNaN(dragObj.elStartLeft))
	{
		dragObj.elStartLeft = 0;
	}
	if (isNaN(dragObj.elStartTop))
	{
		dragObj.elStartTop  = 0;
	}
	x = dragObj.elStartLeft + x - dragObj.cursorStartX;
	y = dragObj.elStartTop  + y - dragObj.cursorStartY;
	if (x < -10) {
		x = -10;
	}
	if (x > max_window_x) {
		x = max_window_x;
	}
	if (y < -10) {
		y = -10;
	}
	if (y > max_window_y) {
		y = max_window_y;
	}
	set_top(id); 
	if (document.all)
	{
		document.attachEvent("onmousemove", dragGo);
		document.attachEvent("onmouseup",   dragStop);
		window.event.cancelBubble = true;
		window.event.returnValue = false;
	}
	else
	{
		document.addEventListener("mousemove", dragGo,   true);
		document.addEventListener("mouseup",   dragStop, true);
		event.preventDefault();
	}
}
function dragGo(event)
{
	var x, y;
	x = document.all ? window.event.clientX + document.documentElement.scrollLeft + document.body.scrollLeft : event.clientX + window.scrollX;
	y = document.all ? window.event.clientY + document.documentElement.scrollTop + document.body.scrollLeft : event.clientY + window.scrollY;
	x = dragObj.elStartLeft + x - dragObj.cursorStartX;
	y = dragObj.elStartTop  + y - dragObj.cursorStartY;
	if (x < -10) {
		x = -10;
	}
	if (x > max_window_x) {
		x = max_window_x;
	}
	if (y < -10) {
		y = -10;
	}
	if (y > max_window_y) {
		y = max_window_y;
	}
	dragObj.elNode.style.left = x+"px";
	dragObj.elNode.style.top  = y+"px";
	if (document.all)
	{
		window.event.cancelBubble = true;
		window.event.returnValue = false;
	}
	else
	{
		event.preventDefault();
	}
}
function dragStop(event)
{
	if (document.all)
	{
		document.detachEvent("onmousemove", dragGo);
		document.detachEvent("onmouseup",   dragStop);
	}
	else
	{
		document.removeEventListener("mousemove", dragGo, true);
		document.removeEventListener("mouseup", dragStop, true);
	}
}
function checkKeycode(e)
{
	var keycode;
	if (window.event) {
		keycode = window.event.keyCode;
	}
	else {
		if (e) {
			keycode = e.which;
		}
	}
	if (keycode == 112 || keycode == 114 || keycode == 116 || keycode == 118 || keycode == 122) {
		if (keycode != 116) {
			if (document.all) {
				$('trashtext').focus();
				event.keyCode = 0;
			}
			else {
				e.stopPropagation();
				e.preventDefault();
			}
		}
		else {
			client.mode = "main";
			location.reload();
		}
		return false;
	}
	if (keycode == 17) {
		ctrl_status = true;
	}
	if (keycode == 18) {
		alt_status = true;
	}
	if (keycode == 33) {
		if (document.all) {
			$('trashtext').focus();
			event.keyCode = 0;
		}
		else {
			e.stopPropagation();
			e.preventDefault();
		}
		debug.show();
	}
	client.clicked();
}
function checkKeys() {
	ctrl_status = false;
	alt_status = false;
}
var dragObj = new Object();
dragObj.zIndex = 0;
document.onkeydown = checkKeycode;
document.onkeyup = checkKeys;
//window.onerror = client.allErr; // DEVONLY
var omitformtags=["input", "textarea", "select"];
omitformtags=omitformtags.join("|");
function disableselect(e)
{
	if (omitformtags.indexOf(e.target.tagName.toLowerCase())==-1)
	{
		return false;
	}
}
function reEnable() {
	return true;
}
if (typeof document.onselectstart!="undefined") {
	document.onselectstart=new Function ("return false");
}
else {
	document.onmousedown = disableselect;
	document.onmouseup = reEnable;
}
function clickIE4()
{
	if (event.button==2)
	{
		return false;
	}
}
function clickNS4(e)
{
	if (document.layers||$&&!document.all)
	{
		if (e.which==2||e.which==3)
		{
			return false;
		}
	}
}
if (document.layers) {
	document.captureEvents(Event.MOUSEDOWN);
	document.onmousedown=clickNS4;
}
else if (document.all&&!$) {
	document.onmousedown=clickIE4;
}
document.oncontextmenu=new Function("return false;");
addEvent(document, 'click', client.clicked);
function removeWho(who) {
	if(typeof who== 'string') who=$(who);
	if(who && who.parentNode)who.parentNode.removeChild(who);
}
Array.prototype.remove = function(from, to) {
  var rest = this.slice((to || from) + 1 || this.length);
  this.length = from < 0 ? this.length + from : from;
  return this.push.apply(this, rest);
};
var speed = {
	start:function (){
	    speedDate = new Date();
	    speedTime  = speedDate.getTime();
	},
	stop:function (){
	    speedDate = new Date();
	    return (speedDate.getTime()-speedTime);
	}
}
var tijdstip = laatst_actief = 0;
setInterval(function(){tijdstip++;}, 1000);
window.offscreenBuffering = true;
addEvent(window, 'load', client.init);
var lib = {
	// script that calls for extra scripts in scripts/libraries when needed, makes sure they are only called once
	loaded : new Array(),
	load : function (lib) {
		if (!in_array(lib, this.loaded)) {
			get_script("js.php?lib="+lib);
			this.loaded.push(lib);
		}
	}
};

var debug = {
	active : false,
	show: function() {
    },
    add: function(text) {
    },
    error: function(text) {
    },
    yellow: function(text) {
    },
    green: function(text) {
    }
};
var preload = {
	img : new Array(),
	loaded : new Array(),
	count : 0,
	init : function() {
		var len = preImg.length;
		for (var i = 0; i < len; i++) {
			this.img[i] = document.createElement('img');
	        this.img[i].setAttribute('src', preImg[i]);
		}
		var len = this.img.length;
		for (i = 0; i < len; i++) {
			this.loaded[i] = false;
		}
		this.load();
	},
	adjustbar : function() {
		var procent = Math.floor((this.count*100)/preImg.length);
		$("grijsbar").style.width = procent+"%";
		$("zwartbar").style.width = (100-procent)+"%";
	},
	load : function() {
		var len = this.img.length;
		if (this.count == len) {
			this.adjustbar();
			client.loaded();
			return;
		}
		for (i = 0; i <= len; i++) {
			if (this.loaded[i] == false && this.img[i].complete) {
				this.adjustbar();
				this.loaded[i] = true;
				this.count++;
			}
		}
		setTimeout(function(){preload.load();},10);
	}
};
if (!this.JSON) {
    JSON = {};
}
(function () {

    function f(n) {
        return n < 10 ? '0' + n : n;
    }

    if (typeof Date.prototype.toJSON !== 'function') {

        Date.prototype.toJSON = function (key) {

            return this.getUTCFullYear()   + '-' +
                 f(this.getUTCMonth() + 1) + '-' +
                 f(this.getUTCDate())      + 'T' +
                 f(this.getUTCHours())     + ':' +
                 f(this.getUTCMinutes())   + ':' +
                 f(this.getUTCSeconds())   + 'Z';
        };

        String.prototype.toJSON =
        Number.prototype.toJSON =
        Boolean.prototype.toJSON = function (key) {
            return this.valueOf();
        };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        rep;


    function quote(string) {

        escapable.lastIndex = 0;
        return escapable.test(string) ?
            '"' + string.replace(escapable, function (a) {
                var c = meta[a];
                return typeof c === 'string' ? c :
                    '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
            }) + '"' :
            '"' + string + '"';
    }


    function str(key, holder) {

        var i,
            k,
            v,
            length,
            mind = gap,
            partial,
            value = holder[key];

        if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }

        if (typeof rep === 'function') {
            value = rep.call(holder, key, value);
        }

        switch (typeof value) {
        case 'string':
            return quote(value);

        case 'number':

            return isFinite(value) ? String(value) : 'null';

        case 'boolean':
        case 'null':

            return String(value);

        case 'object':

            if (!value) {
                return 'null';
            }

            gap += indent;
            partial = [];

            if (Object.prototype.toString.apply(value) === '[object Array]') {

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

                v = partial.length === 0 ? '[]' :
                    gap ? '[\n' + gap +
                            partial.join(',\n' + gap) + '\n' +
                                mind + ']' :
                          '[' + partial.join(',') + ']';
                gap = mind;
                return v;
            }

            if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    k = rep[i];
                    if (typeof k === 'string') {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            } else {

                for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            }

            v = partial.length === 0 ? '{}' :
                gap ? '{\n' + gap + partial.join(',\n' + gap) + '\n' +
                        mind + '}' : '{' + partial.join(',') + '}';
            gap = mind;
            return v;
        }
    }

    if (typeof JSON.stringify !== 'function') {
        JSON.stringify = function (value, replacer, space) {

            var i;
            gap = '';
            indent = '';

            if (typeof space === 'number') {
                for (i = 0; i < space; i += 1) {
                    indent += ' ';
                }

            } else if (typeof space === 'string') {
                indent = space;
            }

            rep = replacer;
            if (replacer && typeof replacer !== 'function' &&
                    (typeof replacer !== 'object' ||
                     typeof replacer.length !== 'number')) {
                throw new Error('JSON.stringify');
            }

            return str('', {'': value});
        };
    }

    if (typeof JSON.parse !== 'function') {
        JSON.parse = function (text, reviver) {

            var j;

            function walk(holder, key) {

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                        if (Object.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }

            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' +
                        ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

            if (/^[\],:{}\s]*$/.
test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@').
replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']').
replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

                j = eval('(' + text + ')');

                return typeof reviver === 'function' ?
                    walk({'': j}, '') : j;
            }

            throw new SyntaxError('JSON.parse');
        };
    }
})();
/*
 * Configurable variables. You may need to tweak these to be compatible with
 * the server-side, but the defaults work in most cases.
 */
var hexcase = 0;  /* hex output format. 0 - lowercase; 1 - uppercase        */
var b64pad  = ""; /* base-64 pad character. "=" for strict RFC compliance   */
var chrsz   = 8;  /* bits per input character. 8 - ASCII; 16 - Unicode      */

/*
 * These are the functions you'll usually want to call
 * They take string arguments and return either hex or base-64 encoded strings
 */
function hex_md5(s){ return binl2hex(core_md5(str2binl(s), s.length * chrsz));}
function b64_md5(s){ return binl2b64(core_md5(str2binl(s), s.length * chrsz));}
function str_md5(s){ return binl2str(core_md5(str2binl(s), s.length * chrsz));}
function hex_hmac_md5(key, data) { return binl2hex(core_hmac_md5(key, data)); }
function b64_hmac_md5(key, data) { return binl2b64(core_hmac_md5(key, data)); }
function str_hmac_md5(key, data) { return binl2str(core_hmac_md5(key, data)); }

function core_md5(x, len)
{
  /* append padding */
  x[len >> 5] |= 0x80 << ((len) % 32);
  x[(((len + 64) >>> 9) << 4) + 14] = len;

  var a =  1732584193;
  var b = -271733879;
  var c = -1732584194;
  var d =  271733878;

  for(var i = 0; i < x.length; i += 16)
  {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;

    a = md5_ff(a, b, c, d, x[i+ 0], 7 , -680876936);
    d = md5_ff(d, a, b, c, x[i+ 1], 12, -389564586);
    c = md5_ff(c, d, a, b, x[i+ 2], 17,  606105819);
    b = md5_ff(b, c, d, a, x[i+ 3], 22, -1044525330);
    a = md5_ff(a, b, c, d, x[i+ 4], 7 , -176418897);
    d = md5_ff(d, a, b, c, x[i+ 5], 12,  1200080426);
    c = md5_ff(c, d, a, b, x[i+ 6], 17, -1473231341);
    b = md5_ff(b, c, d, a, x[i+ 7], 22, -45705983);
    a = md5_ff(a, b, c, d, x[i+ 8], 7 ,  1770035416);
    d = md5_ff(d, a, b, c, x[i+ 9], 12, -1958414417);
    c = md5_ff(c, d, a, b, x[i+10], 17, -42063);
    b = md5_ff(b, c, d, a, x[i+11], 22, -1990404162);
    a = md5_ff(a, b, c, d, x[i+12], 7 ,  1804603682);
    d = md5_ff(d, a, b, c, x[i+13], 12, -40341101);
    c = md5_ff(c, d, a, b, x[i+14], 17, -1502002290);
    b = md5_ff(b, c, d, a, x[i+15], 22,  1236535329);

    a = md5_gg(a, b, c, d, x[i+ 1], 5 , -165796510);
    d = md5_gg(d, a, b, c, x[i+ 6], 9 , -1069501632);
    c = md5_gg(c, d, a, b, x[i+11], 14,  643717713);
    b = md5_gg(b, c, d, a, x[i+ 0], 20, -373897302);
    a = md5_gg(a, b, c, d, x[i+ 5], 5 , -701558691);
    d = md5_gg(d, a, b, c, x[i+10], 9 ,  38016083);
    c = md5_gg(c, d, a, b, x[i+15], 14, -660478335);
    b = md5_gg(b, c, d, a, x[i+ 4], 20, -405537848);
    a = md5_gg(a, b, c, d, x[i+ 9], 5 ,  568446438);
    d = md5_gg(d, a, b, c, x[i+14], 9 , -1019803690);
    c = md5_gg(c, d, a, b, x[i+ 3], 14, -187363961);
    b = md5_gg(b, c, d, a, x[i+ 8], 20,  1163531501);
    a = md5_gg(a, b, c, d, x[i+13], 5 , -1444681467);
    d = md5_gg(d, a, b, c, x[i+ 2], 9 , -51403784);
    c = md5_gg(c, d, a, b, x[i+ 7], 14,  1735328473);
    b = md5_gg(b, c, d, a, x[i+12], 20, -1926607734);

    a = md5_hh(a, b, c, d, x[i+ 5], 4 , -378558);
    d = md5_hh(d, a, b, c, x[i+ 8], 11, -2022574463);
    c = md5_hh(c, d, a, b, x[i+11], 16,  1839030562);
    b = md5_hh(b, c, d, a, x[i+14], 23, -35309556);
    a = md5_hh(a, b, c, d, x[i+ 1], 4 , -1530992060);
    d = md5_hh(d, a, b, c, x[i+ 4], 11,  1272893353);
    c = md5_hh(c, d, a, b, x[i+ 7], 16, -155497632);
    b = md5_hh(b, c, d, a, x[i+10], 23, -1094730640);
    a = md5_hh(a, b, c, d, x[i+13], 4 ,  681279174);
    d = md5_hh(d, a, b, c, x[i+ 0], 11, -358537222);
    c = md5_hh(c, d, a, b, x[i+ 3], 16, -722521979);
    b = md5_hh(b, c, d, a, x[i+ 6], 23,  76029189);
    a = md5_hh(a, b, c, d, x[i+ 9], 4 , -640364487);
    d = md5_hh(d, a, b, c, x[i+12], 11, -421815835);
    c = md5_hh(c, d, a, b, x[i+15], 16,  530742520);
    b = md5_hh(b, c, d, a, x[i+ 2], 23, -995338651);

    a = md5_ii(a, b, c, d, x[i+ 0], 6 , -198630844);
    d = md5_ii(d, a, b, c, x[i+ 7], 10,  1126891415);
    c = md5_ii(c, d, a, b, x[i+14], 15, -1416354905);
    b = md5_ii(b, c, d, a, x[i+ 5], 21, -57434055);
    a = md5_ii(a, b, c, d, x[i+12], 6 ,  1700485571);
    d = md5_ii(d, a, b, c, x[i+ 3], 10, -1894986606);
    c = md5_ii(c, d, a, b, x[i+10], 15, -1051523);
    b = md5_ii(b, c, d, a, x[i+ 1], 21, -2054922799);
    a = md5_ii(a, b, c, d, x[i+ 8], 6 ,  1873313359);
    d = md5_ii(d, a, b, c, x[i+15], 10, -30611744);
    c = md5_ii(c, d, a, b, x[i+ 6], 15, -1560198380);
    b = md5_ii(b, c, d, a, x[i+13], 21,  1309151649);
    a = md5_ii(a, b, c, d, x[i+ 4], 6 , -145523070);
    d = md5_ii(d, a, b, c, x[i+11], 10, -1120210379);
    c = md5_ii(c, d, a, b, x[i+ 2], 15,  718787259);
    b = md5_ii(b, c, d, a, x[i+ 9], 21, -343485551);

    a = safe_add(a, olda);
    b = safe_add(b, oldb);
    c = safe_add(c, oldc);
    d = safe_add(d, oldd);
  }
  return Array(a, b, c, d);

}

function md5_cmn(q, a, b, x, s, t)
{
  return safe_add(bit_rol(safe_add(safe_add(a, q), safe_add(x, t)), s),b);
}
function md5_ff(a, b, c, d, x, s, t)
{
  return md5_cmn((b & c) | ((~b) & d), a, b, x, s, t);
}
function md5_gg(a, b, c, d, x, s, t)
{
  return md5_cmn((b & d) | (c & (~d)), a, b, x, s, t);
}
function md5_hh(a, b, c, d, x, s, t)
{
  return md5_cmn(b ^ c ^ d, a, b, x, s, t);
}
function md5_ii(a, b, c, d, x, s, t)
{
  return md5_cmn(c ^ (b | (~d)), a, b, x, s, t);
}

function core_hmac_md5(key, data)
{
  var bkey = str2binl(key);
  if(bkey.length > 16) bkey = core_md5(bkey, key.length * chrsz);

  var ipad = Array(16), opad = Array(16);
  for(var i = 0; i < 16; i++)
  {
    ipad[i] = bkey[i] ^ 0x36363636;
    opad[i] = bkey[i] ^ 0x5C5C5C5C;
  }

  var hash = core_md5(ipad.concat(str2binl(data)), 512 + data.length * chrsz);
  return core_md5(opad.concat(hash), 512 + 128);
}

function safe_add(x, y)
{
  var lsw = (x & 0xFFFF) + (y & 0xFFFF);
  var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return (msw << 16) | (lsw & 0xFFFF);
}

function bit_rol(num, cnt)
{
  return (num << cnt) | (num >>> (32 - cnt));
}

function str2binl(str)
{
  var bin = Array();
  var mask = (1 << chrsz) - 1;
  for(var i = 0; i < str.length * chrsz; i += chrsz)
    bin[i>>5] |= (str.charCodeAt(i / chrsz) & mask) << (i%32);
  return bin;
}

function binl2str(bin)
{
  var str = "";
  var mask = (1 << chrsz) - 1;
  for(var i = 0; i < bin.length * 32; i += chrsz)
    str += String.fromCharCode((bin[i>>5] >>> (i % 32)) & mask);
  return str;
}

function binl2hex(binarray)
{
  var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i++)
  {
    str += hex_tab.charAt((binarray[i>>2] >> ((i%4)*8+4)) & 0xF) +
           hex_tab.charAt((binarray[i>>2] >> ((i%4)*8  )) & 0xF);
  }
  return str;
}

function binl2b64(binarray)
{
  var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i += 3)
  {
    var triplet = (((binarray[i   >> 2] >> 8 * ( i   %4)) & 0xFF) << 16)
                | (((binarray[i+1 >> 2] >> 8 * ((i+1)%4)) & 0xFF) << 8 )
                |  ((binarray[i+2 >> 2] >> 8 * ((i+2)%4)) & 0xFF);
    for(var j = 0; j < 4; j++)
    {
      if(i * 8 + j * 6 > binarray.length * 32) str += b64pad;
      else str += tab.charAt((triplet >> 6*(3-j)) & 0x3F);
    }
  }
  return str;
}
var comet = {
	connect : function(host, port) {
		try  {
			conn = new Orbited.TCPSocket();
			conn.onopen = this.onopen;
			conn.onread = this.onread;
			conn.onclose = this.onclose;
			conn.open(host, port);
		}
		catch(err) {
			//server bridge is offline
			debug.error(err);
			client.exit();
		}
    },
    send : function(data) {
	    if (conn) {
        	conn.send(data+"#");
    	}
        if (debug.active) {
        	debug.add("SENT: "+data);
    	}
    },
    onread : function(data) {
	    var to_parse = data.split("#");
	    
	    if (debug.active) {
		    var n=to_parse.length-1;
			var k=n;
			do {
			var i=k-n;
				debug.add("RECV: "+to_parse[i]);
				packets.receive(to_parse[i]);
			}
			while(--n);
		}
		else {
			var n=to_parse.length-1;
			var k=n;
			do {
			var i=k-n;
				packets.receive(to_parse[i]);
			}
			while(--n);
		}
    },
    onopen : function() {
	    debug.green("Connection with server established!");
    },
    onclose : function(code) {
        debug.error("Connection with server closed!");
        set_top("connection_lost_layer");
		ShowContent('connection_lost_layer');
        client.exit();
    }
};
var packets = {
	init : function() {
		comet.connect(document.domain, 3500);
	},
	receive : function(packet) {
		var header = parseInt(packet.substr(0,3), 10); // 10: decimaal stelsel
		var content = packet.substr(3);
		if (content != "") {
			var data = JSON.parse(content);
		}
		else {
			var data = "";
		}
		switch(header) 
		{
		case 1:
			// connection accepted by the server, show login screen!
			comet.send("006"); // tell the server you've received the ping request
			login.show();
			break;    
		case 2:
			debug.yellow("Connection is about to be closed.");
			client.connection = false;
			break;
		case 3:
			debug.error("Packet rejected by server.");
			break;
		case 5:
			// server pings
			comet.send("006"); // tell the server you've received the ping request
			break;
		case 6:
			// server is still here
			break;
		case 8:
			// user money update
			user.money = data.Cr;
			user.mission = data.M;
			user.clothes = data.C;
			user.badge = data.B;
			catalogus.moneyupdate();
			break;
		case 12:
			// Server RAM
			debug.green("Server RAM usage: "+(Math.round((data.R/1024)*100)/100)+" MB / "+data.R+" KB");
			break;
		case 13:
			// alert
			client.notify(data.M, data.T);
			break;
		case 16:
			// registercheck answer
			register.regstart_checked(data.A);
			break;
		case 18:
			// login succesful
			login.succes();
			break;
		case 19:
			// login failed
			login.error(data.R);
			break;
		case 20:
			// update user data
			user.id = data.I;
			user.name = data.UN;
			user.rights = data.R.split(";");
			user.money = data.MY;
			user.badge = data.CB;
			user.badges = data.B;
			user.clothes = data.C;
			user.mission = data.M;
			user.messenger = data.MM;
			inv.login_data = data.ID; // store inventory data
			break;
		case 24:
			// messenger search answer
			messenger.searched(data.I, data.M, data.C, data.L, data.O, data.A);
			break;
		case 27:
			// messenger buddy requests
			var len = data.R.length;
			var count = 0;
			for(x in data.R) {
				if(count >= len) break;
				else {
					var request = {
						I: data.R[x].I,
						N: data.R[x].N
					}
					messenger.requests.push(request);
					count++;
				}
			}
			messenger.update(1);
			break;
		case 29:
			// friend list update
			messenger.buddies = data.F;
			messenger.update(1);
			// messages are initialised when receiving this packet, so:
			if (!messenger.loaded) {
				messenger.loaded = true;
				if (messenger.messages.length > 0 && client.sound) {
					soundManager.play("messenger");
				}
			}
			break;
		case 31:
			// messenger message
			var message = {
				I: data.I,
				FI: data.FI,
				FN: data.FN,
				C: data.C,
				M: data.M,
				D: data.D
			}
			messenger.messages.push(message);
			messenger.update(1);
			if (messenger.loaded && client.sound) {
				soundManager.play("messenger");
			}
			break;
		case 33:
			// update money data
			user.money = data.M;
			user.sessionearned = data.S;
			catalogus.moneyupdate();
			break;
		case 35:
			// buy furniture answer
			catalogus.buyanswer(data.A);
			break;
		case 37:
			crcodes.checkedok();
			break;
		case 38:
			crcodes.checkedbad(data.E);
			break;
		case 40:
			// navigator data coming in
			if (data.P != null) { navi.lists.publics = data.P; }
			if (data.R != null) { navi.lists.privates = data.R; }
			if (data.O != null) { navi.lists.own = data.O; }
			navi.update_publics();
			navi.update_guestrooms();
			navi.update_own();
			break;
		case 41:
			// room data + furniture
			room.data = {
				id : data.I,
				name : data.N,
				descr : data.D,
				owner : data.O,
				ownername : data.ON,
				floor : data.Fl,
				wall : data.Wl,
				lang : data.L,
				breed : data.B,
				door : data.Do,
				mode : data.M,
				rights : data.R,
				holes : data.H,
				voted : data.V,
				score : data.S
			};
			furni.data = data.F;
			room.load();
			break;
		case 43:
			// room loading failed
			room.fail('error');
			break;
		case 44:
			// room loading done serverside
			room.loadedserver = true;
			room.loaded();
			break;
		case 45:
			// client kicked from room
			client.exit2();
			break;
		case 47:
			// move/add furniture
			furni.packet(data);
			break;
		case 48:
			// remove furniture
			furni.remove(data.I);
			break;
		case 49:
			// move furniture
			furni.moved(data.I, data.T, data.S, data.SH, data.H, data.A, data.F);
			break;
		case 52:
			// chat bubble
			room.chat.show_chat.render(data.U, data.I, data.M);
			break;
		case 53:
			// user is spamming
			room.chat.spam.init(3);
			break;
		case 55:
			client.notify("Reported by: "+data.U+"<br>Message: "+data.M, "Help Report");
			break;
		case 56:
			avatars.add(data.I, data.U, data.M, data.C, data.B, (parseInt(data.X)+1)+"_"+(parseInt(data.Y)+1), data.H, data.D, data.Brb, data.S, data.F, data.R);	
			break;
		case 57:
			avatars.move(data.I, parseInt(data.X)+1, parseInt(data.Y)+1, data.H);
			break;
		case 58:
			avatars.del(data.I);
			break;
		case 61:
			// room is full
			room.fail('full');
			break;
		case 63:
			// open chatbox
			chatbox.open();
			break;
		case 64:
			avatars.stopmove(data.I);
			break;
		case 66:
			room.voted(data.S);
			break;
		case 68:
			// navigator search data coming in
			navi.lists.searched = data.R;
			navi.update_searched();
			break;
		case 71:
			// create room answer
			matic.answer(data.A);
			break;
		case 72:
			// club status update
			club.update(data.S, data.D, data.PM, data.RM);
			break;
		case 74:
			// give avatar drink
			avatars.givedrink(data.I, data.D);
			break;
		case 76:
			// opens picture
			camera.loadpicture(data.C, data.CA);
			break;
		case 79:
			// update rights of avatar
			avatars.updaterights(data.I, data.R);
			break;
		case 80:
			// update sit status of avatar
			avatars.updatesit(data.I, data.S);
			break;
		case 81:
			// stop moving and sit
			avatars.stopmovesit(data.I, data.S);
			break;
		case 82:
			// update brb status
			avatars.updatebrb(data.I, data.Brb);
			break;
		default:
			debug.error("Unknown packet header.");
			break;
		}
	},
	send :
	{
		inactive : function () {
			comet.send("009");
		},
		activeagain : function () {
			comet.send("010");
		},
		requestram : function () {
			comet.send("011"); // request server ram usage
		},
		login : function (try_name, try_pass) {
			// login
			var data = {
				U: try_name,
				P: hex_md5(hex_md5(try_name)+hex_md5(try_pass))
			};
			comet.send("017"+JSON.stringify(data));
		},
		login_done : function () {
			// login complete clientside
			comet.send("021");
		},
		register_start_check : function () {
			// check if register is valid
			var data = {
				IP: 0
			};
			comet.send("015"+JSON.stringify(data));
		},
		globalalert : function (message) {
			// global alert
			var data = {
				M: message.replace("#", "")
			};
			comet.send("014"+JSON.stringify(data));
		},
		naviupdate : function () {
			// request navigator update
			comet.send("039");
		},
		messengermission : function (mission) {
			// change messenger mission
			var data = {
				M: mission
			};
			comet.send("022"+JSON.stringify(data));
		},
		messengersearch : function (searchname) {
			var data = {
				N: searchname.replace("#", "")
			};
			comet.send("023"+JSON.stringify(data));
		},
		addbuddy : function (id) {
			var data = {
				I: id
			};
			comet.send("025"+JSON.stringify(data));
		},
		delbuddy : function (id) {
			var data = {
				I: id
			};
			comet.send("026"+JSON.stringify(data));
		},
		writemessage : function (id, message) {
			var data = {
				I: id,
				M: message
			};
			comet.send("030"+JSON.stringify(data));
		},
		delmessage : function (id) {
			var data = {
				I: id
			};
			comet.send("032"+JSON.stringify(data));
		},
		messengeranswerrequest : function (id, answer) {
			var data = {
				I: id,
				A: answer
			};
			comet.send("028"+JSON.stringify(data));
		},
		room_go : function (id) {
			// go to a room
			var data = {
				I: parseInt(id)
			};
			comet.send("042"+JSON.stringify(data));
		},
		room_chat : function (to, message) {
			//send the chat to the server
			if (message.replace("#", "") != "") {
				var data = {
					T: to,
					M: message.replace("#", "")
				};
				comet.send("051"+JSON.stringify(data));
			}
		},
		room_leave : function () {
			//leave the room
			comet.send("046");
		},
		report : function (message) {
			//send the report
			var data = {
				M: message.replace("#", "")
			};
			comet.send("054"+JSON.stringify(data));
		},
		coupon : function (coupon) {
			//send coupon code
			var data = {
				C: coupon
			};
			comet.send("036"+JSON.stringify(data));
		},
		buyfurni : function (cat, furni, amount, mode, to, message) {
			// buy furniture
			var data = {
				C: cat,
				F: furni,
				A: amount,
				M: mode,
				T: to,
				Me: message
			};
			comet.send("034"+JSON.stringify(data));
		},
		banktransfer : function (to, amount) {
			var data = {
				U: to,
				M: amount
			};
			comet.send("RT"+JSON.stringify(data));
		},
		walkrequest : function (x, y) {
			var data = {
				X: parseInt(x)-1,
				Y: parseInt(y)-1
			};
			comet.send("059"+JSON.stringify(data));
		},
		furni_move : function (id, tile) {
			var data = {
				I: parseInt(id),
				T: tile
			};
			comet.send("050"+JSON.stringify(data));
		},
		furni_turn : function (id) {
			var data = {
				I: parseInt(id)
			};
			comet.send("049"+JSON.stringify(data));
		},
		furni_action : function (id) {
			var data = {
				I: parseInt(id)
			};
			comet.send("069"+JSON.stringify(data));
		},
		furni_pickup : function (id) {
			var data = {
				I: parseInt(id),
				T: "inv"
			};
			comet.send("050"+JSON.stringify(data));
		},
		botkick : function (id) {
			var data = {
				I: parseInt(id)
			};
			comet.send("061"+JSON.stringify(data));
		},
		openchat : function () {
			comet.send("062");
		},
		vote : function (vote) {
			var data = {
				V: vote
			};
			comet.send("065"+JSON.stringify(data));
		},
		getdrink : function (drink) {
			var data = {
				D: drink
			};
			comet.send("073"+JSON.stringify(data));
		},
		navisearch : function (searchtext) {
			var data = {
				S: searchtext
			};
			comet.send("067"+JSON.stringify(data));
		},
		createroom : function (new_name, new_desc, new_type, new_safe, new_pass) {
			var data = {
				N: new_name,
				D: new_desc,
				T: parseInt(new_type),
				S: parseInt(new_safe),
				P: new_pass
			};
			comet.send("070"+JSON.stringify(data));
		},
		openpicture : function (id) {
			var data = {
				I: id
			};
			comet.send("075"+JSON.stringify(data));
		},
		giverights : function (id) {
			var data = {
				I: id
			};
			comet.send("077"+JSON.stringify(data));
		},
		takerights : function (id) {
			var data = {
				I: id
			};
			comet.send("078"+JSON.stringify(data));
		}
	}
};
var animate = {
	drop : {
		g : 3.724, // gravitational constant 1/100th scale in px / ms2
		start : function (id, x, y) {
			obj = $(id);
			if (isset(obj.timer)) {
				clearTimeout(obj.timer);
			}
			obj.style.top = document.documentElement.scrollTop-obj.clientHeight+"px";
			obj.style.left = x;
			obj.endpos = y;
			obj.ti = new Date; // record initial time in miliseconds
			
			obj.timer = setTimeout(function(){animate.drop.go(id);},100); // start dropping
		},
		go : function (id) {
			obj = $(id);
			clearTimeout(obj.timer); // clear timeout
			var tc = new Date; // get current time in miliseconds
			var td = tc.getTime() - obj.ti.getTime(); // get difference between timestamps
			var tPos = this.g * td * td / 2500; // determine top position for object
			// if we're not at the bottom, do it again
			if (tPos < obj.endpos) {
				obj.style.top = tPos + "px"; // set object's top position
				obj.timer = setTimeout(function(){animate.drop.go(id);},1); // do it again
			} else {
				obj.b = true;
				obj.style.top = obj.endpos + "px";
				setTimeout(function(){animate.drop.bounce(id);},50);
			}
		},
		bounce : function (id) {
			obj = $(id);
			clearTimeout(obj.timer); // clear timeout
			if (obj.b) {
				obj.b = false;
				obj.style.top = obj.endpos - 5 + "px";
				obj.timer = setTimeout(function(){animate.drop.bounce(id);},100);
			} else {
				obj.style.top = obj.endpos + "px";
			}
		}
	}
};
/*
Load Indicator script

 Add:
	OBJECT = getBusyOverlay(parent[,overlay[,busy]]);

	parent		== element to add the overlay (e.g. $(id) or 'viewport')

	overlay 	== OBJECT e.g. {color: 'black', opacity: 0.5, ...}
		color	== STR 'black' or '#000000' or 'rgb(0,0,0)' Default: 'white'
		opacity	== FLOAT 0.0 - 1.0  Default: 0.0
		text	== STR e.g. "loading" Default: ''
		style	== STR e.g. "color: black;" or "my_text_class" Default: ''

	busy		== OBJECT e.g. {color: '#fff', size: 48, ...}
		color	== STR '#000000' - '#ffffff' or '#000' - '#fff' Default: '#000'
		size	== INT 16 - 512 (pixel) Default: 32
		type	== STR 'circle|oval|polygon|rectangle|tube' or 'c|o|p|r|t' Default: 'tube'
		iradius	== INT 6 - 254 (pixel) Default: 8
		weight	== INT 1 - 254 (pixel) Default: 3
		count	== INT 5 - 36 (rays) Default: 12
		speed	== INT 30 - 1000 (millisec) Default: 96
		minopac	== FLOAT 0.0 - 0.5  Default: 0.25

 Remove:
	OBJECT.remove();
	
 Set Overlay text:
	OBJECT.settext(string);
*/

function onIEWinResize(event) {
	function parseWidth(val) {return (isNaN(parseInt(val,10))?0:parseInt(val,10));}
	if(!event) {event=window.event;} var i,cs,parent=this, div=parent.getElementsByTagName("div");
	if(div.length>0) {if(parent.currentStyle){cs=parent.currentStyle;}else if(document.defaultView&&document.defaultView.getComputedStyle){cs=document.defaultView.getComputedStyle(parent,"");}else{cs=parent.style;}
		for(i=0; i<div.length; i++) {if(div[i].className=='buzy_ele') {
				div[i].style.height=(parent.offsetHeight-parseWidth(cs.borderBottomWidth)-parseWidth(cs.borderTopWidth));
				div[i].style.width=(parent.offsetWidth-parseWidth(cs.borderLeftWidth)-parseWidth(cs.borderRightWidth)); 
				div[i].firstChild.style.height=div[i].style.height; div[i].firstChild.style.width=div[i].style.width; 
				break;
			}
		}
	}
}
function onIEVPResize(event) {
	if(!event) {event=window.event;} var vp=$('viewport'); if(vp) {
		if(typeof document.documentElement!='undefined') {vp.style.width=document.documentElement.clientWidth+'px'; vp.style.height=document.documentElement.clientHeight+'px';}
		else {vp.style.width=document.getElementsByTagName('body')[0].clientWidth+'px'; vp.style.height=document.getElementsByTagName('body')[0].clientHeight+'px';}
	}
}
function onIEVPScroll(event) {
	if(!event) {event=window.event;} var vp=$('viewport'); if(vp) {
		if(typeof document.documentElement!='undefined') {vp.style.left=document.documentElement.scrollLeft+'px'; vp.style.top=document.documentElement.scrollTop+'px';}
		else {vp.style.left=document.getElementsByTagName('body')[0].scrollLeft+'px'; vp.style.top=document.getElementsByTagName('body')[0].scrollTop+'px';}
	}
}
function getBusyOverlay(parent,overlay,busy) {
	if((typeof(parent)==='object' || parent=='viewport') && document.getElementsByTagName) {
		function parseWidth(val) {return (isNaN(parseInt(val,10))?0:parseInt(val,10));}
		var isIE,isVL,isCV,isWK,isGE,i,b,o,lt,rt,lb,rb,cz,cs,size,viewport,inner,outer,string,canvas,context,ctrl,opacity,color,text,styles,waiting=true;
		if(parent=='viewport') {viewport=$('viewport');
			if(!viewport) {viewport=document.createElement('div'); viewport.id='viewport'; cz=viewport.style;
				cz.backgroundColor='transparent'; cz.position='fixed'; cz.overflow='hidden';
				cz.display='block'; cz.zIndex=999999; cz.left='0px'; cz.top='0px'; cz.zoom=1;
				cz.width='100%'; cz.height='100%'; cz.margin='0px'; cz.padding='0px';
				if(document.all&&!window.opera&&!window.XMLHttpRequest) {cz.position='absolute'; 
					if(typeof document.documentElement!='undefined') {cz.width=document.documentElement.clientWidth+'px'; cz.height=document.documentElement.clientHeight+'px';}
					else {cz.width=document.getElementsByTagName('body')[0].clientWidth+'px'; cz.height=document.getElementsByTagName('body')[0].clientHeight+'px';}
				}document.getElementsByTagName("body")[0].appendChild(viewport);
			}else {viewport.style.display='block';
				if(document.all&&!window.opera&&!window.XMLHttpRequest) { 
					if(typeof document.documentElement!='undefined') {viewport.style.width=document.documentElement.clientWidth+'px';	viewport.style.height=document.documentElement.clientHeight+'px';}
					else {viewport.style.width=document.getElementsByTagName('body')[0].clientWidth+'px';	viewport.style.height=document.getElementsByTagName('body')[0].clientHeight+'px';}
				}
			}parent=viewport;
		}
		if(parent.currentStyle){cs=parent.currentStyle;}else if(document.defaultView&&document.defaultView.getComputedStyle){cs=document.defaultView.getComputedStyle(parent,"");}else{cs=parent.style;}
		while(cs.display.search(/block|inline-block|table|inline-table|list-item/i)<0) {parent=parent.parentNode; if(parent.currentStyle){cs=parent.currentStyle;}else if(document.defaultView&&document.defaultView.getComputedStyle){cs=document.defaultView.getComputedStyle(parent,"");}else{cs=parent.style;} if(parent.tagName.toUpperCase()=='BODY') {parent="";}}
		if(typeof(parent)==='object') {
			if(!overlay) {overlay=new Object(); overlay['opacity']=0; overlay['color']='white'; overlay['text']=''; overlay['style']='';}
			if(!busy) {busy=new Object(); busy['size']=32; busy['color']='#000'; busy['type']='tube'; busy['iradius']=8; busy['weight']=3; busy['count']=12; busy['speed']=96; busy['minopac']=.25;}
			opacity=Math.max(0.0,Math.min(1.0,(typeof overlay['opacity']==='number'?overlay['opacity']:0)||0)); color=(typeof overlay['color']==='string'?overlay['color']:'white');
			text=(typeof overlay['text']==='string'?overlay['text']:''); styles=(typeof overlay['style']==='string'?overlay['style']:'');
			canvas=document.createElement("canvas"); isCV=canvas.getContext?1:0; 
			isWK=navigator.userAgent.indexOf('WebKit')>-1?1:0; isGE=navigator.userAgent.indexOf('Gecko')>-1&&window.updateCommands?1:0;
			isIE=navigator.appName=='Microsoft Internet Explorer'&&window.navigator.systemLanguage&&!window.opera?1:0; 
			isVL=document.all&&document.namespaces?1:0; outer=document.createElement('div'); parent.style.position=(cs.position=='static'?'relative':cs.position);
			cz=parent.style.zIndex>=0?(parent.style.zIndex-0+2):2; if(isIE && !cs.hasLayout) {parent.style.zoom=1;}
			outer.style.position='absolute'; outer.style.overflow='hidden'; outer.style.display='block'; outer.style.zIndex=cz; 
			outer.style.left=0+'px'; outer.style.top=0+'px'; outer.style.width='100%'; outer.style.height='100%';
			if(isIE) {outer.className='buzy_ele'; outer.style.zoom=1; outer.style.margin='0px'; outer.style.padding='0px'; outer.style.height=(parent.offsetHeight-parseWidth(cs.borderBottomWidth)-parseWidth(cs.borderTopWidth)); outer.style.width=(parent.offsetWidth-parseWidth(cs.borderLeftWidth)-parseWidth(cs.borderRightWidth));}
			if(typeof(cs.borderRadius)=="undefined"){
				if(typeof(cs.MozBorderRadius)!="undefined"){
					lt=parseFloat(cs.MozBorderRadiusTopleft)-Math.min(parseFloat(cs.borderLeftWidth),parseFloat(cs.borderTopWidth));
					rt=parseFloat(cs.MozBorderRadiusTopright)-Math.min(parseFloat(cs.borderRightWidth),parseFloat(cs.borderTopWidth));
					lb=parseFloat(cs.MozBorderRadiusBottomleft)-Math.min(parseFloat(cs.borderLeftWidth),parseFloat(cs.borderBottomWidth));
					rb=parseFloat(cs.MozBorderRadiusBottomright)-Math.min(parseFloat(cs.borderRightWidth),parseFloat(cs.borderBottomWidth));
					outer.style.MozBorderRadiusTopleft=lt+"px"; outer.style.MozBorderRadiusTopright=rt+"px"; outer.style.MozBorderRadiusBottomleft=lb+"px"; outer.style.MozBorderRadiusBottomright=rb+"px";
				}else if(typeof(cs.WebkitBorderRadius)!="undefined"){
					lt=parseFloat(cs.WebkitBorderTopLeftRadius)-Math.min(parseFloat(cs.borderLeftWidth),parseFloat(cs.borderTopWidth));
					rt=parseFloat(cs.WebkitBorderTopRightRadius)-Math.min(parseFloat(cs.borderRightWidth),parseFloat(cs.borderTopWidth));
					lb=parseFloat(cs.WebkitBorderBottomLeftRadius)-Math.min(parseFloat(cs.borderLeftWidth),parseFloat(cs.borderBottomWidth));
					rb=parseFloat(cs.WebkitBorderBottomRightRadius)-Math.min(parseFloat(cs.borderRightWidth),parseFloat(cs.borderBottomWidth));
					outer.style.WebkitBorderTopLeftRadius=lt+"px"; outer.style.WebkitBorderTopRightRadius=rt+"px"; outer.style.WebkitBorderBottomLeftRadius=lb+"px"; outer.style.WebkitBorderBottomRightRadius=rb+"px";
				}
			}else {
				lt=parseFloat(cs.borderTopLeftRadius)-Math.min(parseFloat(cs.borderLeftWidth),parseFloat(cs.borderTopWidth));
				rt=parseFloat(cs.borderTopRightRadius)-Math.min(parseFloat(cs.borderRightWidth),parseFloat(cs.borderTopWidth));
				lb=parseFloat(cs.borderBottomLeftRadius)-Math.min(parseFloat(cs.borderLeftWidth),parseFloat(cs.borderBottomWidth));
				rb=parseFloat(cs.borderBottomRightRadius)-Math.min(parseFloat(cs.borderRightWidth),parseFloat(cs.borderBottomWidth));
				outer.style.borderTopLeftRadius=lt+"px"; outer.style.borderTopRightRadius=rt+"px"; outer.style.borderBottomLeftRadius=lb+"px"; outer.style.borderBottomRightRadius=rb+"px";
			}
			parent.appendChild(outer); inner=document.createElement('div');
			inner.style.position='absolute'; inner.style.cursor='progress'; inner.style.display='block'; 
			inner.style.zIndex=(cz-1); inner.style.left=0+'px'; inner.style.top=0+'px';
			inner.style.width=100+'%'; inner.style.height=100+'%'; inner.style.backgroundColor=color;
			if(isIE) {inner.style.zoom=1; inner.style.margin='0px'; inner.style.padding='0px'; inner.style.height=outer.style.height; inner.style.width=outer.style.width; }
			if(typeof(cs.borderRadius)=="undefined"){
				if(typeof(cs.MozBorderRadius)!="undefined"){inner.style.MozBorderRadiusTopleft=lt+"px"; inner.style.MozBorderRadiusTopright=rt+"px"; inner.style.MozBorderRadiusBottomleft=lb+"px"; inner.style.MozBorderRadiusBottomright=rb+"px";}else 
				if(typeof(cs.WebkitBorderRadius)!="undefined"){inner.style.WebkitBorderTopLeftRadius=lt+"px"; inner.style.WebkitBorderTopRightRadius=rt+"px"; inner.style.WebkitBorderBottomLeftRadius=lb+"px"; inner.style.WebkitBorderBottomRightRadius=rb+"px";}
			}else {inner.style.borderTopLeftRadius=lt+"px"; inner.style.borderTopRightRadius=rt+"px"; inner.style.borderBottomLeftRadius=lb+"px"; inner.style.borderBottomRightRadius=rb+"px";}
			if(isIE) {inner.style.filter="progid:DXImageTransform.Microsoft.Alpha(opacity="+parseInt(opacity*100)+")";}else {inner.style.opacity=opacity;}
			outer.appendChild(inner); size=Math.max(16,Math.min(512,(typeof busy['size']==='number'?(busy['size']==0?32:busy['size']):32)));
			if(isVL){
				if(document.namespaces['v']==null) {
					var e=["shape","shapetype","group","background","path","formulas","handles","fill","stroke","shadow","textbox","textpath","imagedata","line","polyline","curve","roundrect","oval","rect","arc","image"],s=document.createStyleSheet(); 
					for(i=0; i<e.length; i++) {s.addRule("v\\:"+e[i],"behavior: url(#default#VML);");} document.namespaces.add("v","urn:schemas-microsoft-com:vml");
				} 
			}
			if(!isCV){canvas=document.createElement("div");}
			canvas.style.position='absolute'; canvas.style.cursor='progress'; canvas.style.zIndex=(cz-0+1);
			canvas.style.top='50%'; canvas.style.left='50%'; canvas.style.marginTop='-'+(size/2)+'px'; canvas.style.marginLeft='-'+(size/2)+'px';
			canvas.width=size; canvas.height=size; canvas.style.width=size+"px"; canvas.style.height=size+"px";
			outer.appendChild(canvas);
			if(text!=""){
				string=document.createElement('div'); string.style.position='absolute'; string.style.overflow='hidden'; 
				string.style.cursor='progress'; string.style.zIndex=(cz-0+1); string.style.top='50%'; string.style.left='0px';
				string.style.marginTop=2+(size/2)+'px'; string.style.textAlign='center'; string.style.width=100+'%'; string.style.height='auto';
				if(styles!="") {string.innerHTML='<span '+(styles.match(/:/i)?'style':'class')+'="'+styles+'">'+text+'</span>';}
				else {string.innerHTML='<span>'+text+'</span>';} outer.appendChild(string);
			}
			if(isGE){outer.style.MozUserSelect="none"; inner.style.MozUserSelect="none"; canvas.style.MozUserSelect="none";}else 
			if(isWK){outer.style.KhtmlUserSelect="none"; inner.style.KhtmlUserSelect="none"; canvas.style.KhtmlUserSelect="none";}else 
			if(isIE){outer.unselectable="on"; inner.unselectable="on"; canvas.unselectable="on";}
			if(isVL){ctrl=getBusyVL(canvas,busy['color'],busy['size'],busy['type'],busy['iradius'],busy['weight'],busy['count'],busy['speed'],busy['minopac']); ctrl.start();}else 
			if(isCV){ctrl=getBusyCV(canvas.getContext("2d"),busy['color'],busy['size'],busy['type'],busy['iradius'],busy['weight'],busy['count'],busy['speed'],busy['minopac']); ctrl.start();}
			else {ctrl=getBusy(canvas,busy['color'],busy['size'],busy['type'],busy['iradius'],busy['weight'],busy['count'],busy['speed'],busy['minopac']); ctrl.start();}
			if(isIE) {parent.onresize=onIEWinResize; if(parent.id=='viewport'&&!window.XMLHttpRequest) {window.onresize=onIEVPResize; window.onscroll=onIEVPScroll;}}
			return {
				remove: function (){if(waiting){waiting=false; ctrl.stop(); delete ctrl; parent.removeChild(outer); if(parent.id=='viewport') {parent.style.display='none';}}},
				settext: function (v){if(string&&typeof(v)=='string') {string.firstChild.innerHTML=v; return false;}}
			};
		}
	}
}
function getBusy(obj,cl,sz,tp,ir,w,ct,sp,mo) {
	function getHEX(v) {var col=v||'#000000';
		if(!col.match(/^#[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f]$/i)) {
			if(v.match(/^#[0-9a-f][0-9a-f][0-9a-f]$/i)) {col='#'+v.substr(1,1)+v.substr(1,1)+v.substr(2,1)+v.substr(2,1)+v.substr(3,1)+v.substr(3,1);}
		}return col;
	}
	var running=false,i=0,os=0,al=0,f=100,c,h,p,t,x,y,v,hp,ph,sh,ele=new Array();
	c=getHEX(cl); tp=tp||"t"; t=(tp.match(/^[coprt]/i)?tp.substr(0,1).toLowerCase():'t');
	ct=Math.max(5,Math.min(36,ct||12)); sp=Math.max(30,Math.min(1000,sp||96));
	sz=Math.max(16,Math.min(512,sz||32)); ir=Math.max(1,Math.min((sz/2)-2,ir||sz/4));
	w=Math.max(1,Math.min((sz/2)-ir,w||sz/10)); mo=Math.max(0,Math.min(0.5,mo||0.25));
	al=360/ct; hp=(Math.PI/2)*-1; ph=Math.PI/180; w=(t!='c'?parseInt((w/2)*3):w); v=parseInt((sz/2)-(w/2));		
	for(i=0;i<ct;i++) {
		sh=document.createElement('div'); x=Math.round(v+v*Math.cos(hp+(i+1)*al*ph)); y=Math.round(v+v*Math.sin(hp+(i+1)*al*ph));
		sh.style.position='absolute'; sh.style.margin='0px'; sh.style.width=w+'px'; sh.style.height=w+'px';
		sh.style.lineHeight='1px'; sh.style.fontSize='0px'; sh.style.top=y+'px'; sh.style.left=x+'px'; sh.style.backgroundColor=c;
		if(document.all&&!window.opera) {sh.style.filter="progid:DXImageTransform.Microsoft.Alpha(opacity="+parseInt(Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1))))*100)+")";
		}else {sh.style.opacity=Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1)))); }
		obj.appendChild(sh); ele[i]=sh;
	}
	function nextLoop(){
		if(!running) {return;} os=(os+1)%ct; 
		if(document.all&&!window.opera) {for(i=0;i<ct;i++){al=((os+i)%ct); ele[al].style.filter="progid:DXImageTransform.Microsoft.Alpha(opacity="+parseInt(Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1))))*100)+")";}}
		else {for(i=0;i<ct;i++){al=((os+i)%ct); ele[al].style.opacity=Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1))));}} 
		setTimeout(nextLoop,sp);
	}
	nextLoop(0);
	return {
		start: function (){if(!running){running=true; nextLoop(0);}},
		stop: function (){running=false; for(i=0;i<ct;i++) {if(document.all&&!window.opera) {ele[i].style.filter="progid:DXImageTransform.Microsoft.Alpha(opacity=0)";}else {ele[i].setAttribute('opacity',0);}}},
		pause: function (){running=false; }
	};
}
function getBusyVL(obj,cl,sz,tp,ir,w,ct,sp,mo) {
	function getHEX(v) {var col=v||'#000000';
		if(!col.match(/^#[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f]$/i)) {
			if(v.match(/^#[0-9a-f][0-9a-f][0-9a-f]$/i)) {col='#'+v.substr(1,1)+v.substr(1,1)+v.substr(2,1)+v.substr(2,1)+v.substr(3,1)+v.substr(3,1);}
		}return col;
	}
	var running=false,os=0,al=0,f=100,c,i,h,p,t,x,y,hs,qs,hw,qw,rp,sh,fl,ele=new Array();
	c=getHEX(cl); tp=tp||"t"; t=(tp.match(/^[coprt]/i)?tp.substr(0,1).toLowerCase():'t');
	ct=Math.max(5,Math.min(36,ct||12)); sp=Math.max(30,Math.min(1000,sp||96));
	sz=Math.max(16,Math.min(512,sz||32)); ir=Math.max(1,Math.min((sz/2)-2,ir||sz/4));
	w=Math.max(1,Math.min((sz/2)-ir,w||sz/10)); mo=Math.max(0,Math.min(0.5,mo||0.25));
	h=(sz/2)-ir; x=sz/2; y=x; al=360/ct; hs=parseInt((sz/2)*f); qs=parseInt(hs/2); 
	hw=parseInt((w/2)*f); qw=parseInt(hw/2); rp=hs-parseInt(ir*f); 
	switch(t) {
		case "c": p='m '+hs+','+(rp-hw)+' ar '+(hs-hw)+','+(rp-hw-hw)+','+(hs+hw)+','+rp+','+(hs-hw)+','+(rp-hw-hw)+','+(hs-hw)+','+(rp-hw-hw)+' e'; break;
		case "p": p='m '+(hs-qw)+',0 l '+(hs-hw)+','+rp+','+(hs+hw)+','+rp+','+(hs+qw)+',0 x e'; break;
		case "o": p='m '+hs+','+(rp-qs)+' ar '+(hs-hw)+',0,'+(hs+hw)+','+rp+','+(hs-hw)+',0,'+(hs-hw)+',0 e'; break;
		case "t": p='m '+(hs-hw)+','+rp+' l '+(hs-hw)+','+hw+' qy '+hs+',0 qx '+(hs+hw)+','+hw+' l '+(hs+hw)+','+rp+' x e'; break;
		default: p='m '+(hs-hw)+',0 l '+(hs-hw)+','+rp+','+(hs+hw)+','+rp+','+(hs+hw)+',0 x e'; break;
	} 
	for(i=0;i<ct;i++) {
		sh=document.createElement('v:shape'); sh.setAttribute('filled','t'); sh.setAttribute('stroked','f');
		sh.setAttribute('coordorigin','0,0'); sh.setAttribute('coordsize',(sz*f)+','+(sz*f));
		sh.setAttribute('path',p); sh.style.rotation=(i*al); sh.style.position='absolute'; sh.style.margin='0px';
		sh.style.width=sz+'px'; sh.style.height=sz+'px'; sh.style.top='-1px'; sh.style.left='-1px';
		obj.appendChild(sh); fl=document.createElement('v:fill');
		fl.setAttribute('opacity',Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1)))));
		fl.setAttribute('color',c); sh.appendChild(fl); ele[i]=fl;
	}
	function nextLoop(){
		if(!running) {return;} os=(os+1)%ct; 
		if(document.documentMode==8) {
			for(i=0;i<ct;i++) {al=((os+i)%ct); ele[al].opacity=Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1))));}
		}else {
			for(i=0;i<ct;i++) {al=((os+i)%ct); ele[al].setAttribute('opacity',Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1)))));}
		}
		setTimeout(nextLoop,sp);
	}
	nextLoop(0);
	return {
		start: function (){if(!running){running=true; nextLoop(0);}},
		stop: function (){running=false; for(i=0;i<ct;i++) {ele[i].setAttribute('opacity',0);}},
		pause: function (){running=false; }
	};
}
function getBusyCV(ctx,cl,sz,tp,ir,w,ct,sp,mo) {
	function getRGB(v){
		function hex2dec(h){return(Math.max(0,Math.min(parseInt(h,16),255)));}
		var r=0,g=0,b=0; v=v||'#000'; if(v.match(/^#[0-9a-f][0-9a-f][0-9a-f]$/i)) {
			r=hex2dec(v.substr(1,1)+v.substr(1,1)),g=hex2dec(v.substr(2,1)+v.substr(2,1)),b=hex2dec(v.substr(3,1)+v.substr(3,1));
		}else if(v.match(/^#[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f]$/i)) {
			r=hex2dec(v.substr(1,2)),g=hex2dec(v.substr(3,2)),b=hex2dec(v.substr(5,2));
		} return r+','+g+','+b;
	}
	function drawOval(ctx,w,h){ctx.beginPath(); ctx.moveTo(-w/2,h/2); ctx.quadraticCurveTo(-w/2,0,0,0); ctx.quadraticCurveTo(w/2,0,w/2,h/2); ctx.quadraticCurveTo(w/2,h,0,h); ctx.quadraticCurveTo(-w/2,h,-w/2,h/2); ctx.fill();}
	function drawTube(ctx,w,h){ctx.beginPath(); ctx.moveTo(w/2,0); ctx.lineTo(-w/2,0); ctx.lineTo(-w/2,h-(w/2)); ctx.quadraticCurveTo(-w/2,h,0,h); ctx.quadraticCurveTo(w/2,h,w/2,h-(w/2)); ctx.fill();}
	function drawPoly(ctx,w,h){ctx.beginPath(); ctx.moveTo(w/2,0); ctx.lineTo(-w/2,0); ctx.lineTo(-w/4,h); ctx.lineTo(w/4,h); ctx.fill();}
	function drawCirc(ctx,r,z){ctx.beginPath(); ctx.arc(r,r,r,0,Math.PI*2,false); ctx.fill();}  
	var running=false,os=0,al=0,c,i,h,t,x,y;
	c=getRGB(cl); tp=tp||"t"; t=(tp.match(/^[coprt]/i)?tp.substr(0,1).toLowerCase():'t');
	ct=Math.max(5,Math.min(36,ct||12)); sp=Math.max(30,Math.min(1000,sp||96));
	sz=Math.max(16,Math.min(512,sz||32)); ir=Math.max(1,Math.min((sz/2)-2,ir||sz/4));
	w=Math.max(1,Math.min((sz/2)-ir,w||sz/10)); mo=Math.max(0,Math.min(0.5,mo||0.25));
	h=(sz/2)-ir; x=sz/2; y=x;
	function nextLoop(){
		if(!running) {return;} os=(os+1)%ct; ctx.clearRect(0,0,sz,sz); ctx.save(); ctx.translate(x,y);
		for(i=0;i<ct;i++) {al=2*((os+i)%ct)*Math.PI/ct; 
			ctx.save(); ctx.translate(ir*Math.sin(-al),ir*Math.cos(-al)); ctx.rotate(al);
			ctx.fillStyle='rgba('+c+','+Math.min(1,Math.max(mo,1-((ct+1-i)/(ct+1))))+')';
			switch(t) {case "c": drawCirc(ctx,w/2,h); break; case "o": drawOval(ctx,w,h); break; case "p": drawPoly(ctx,w,h); break; case "t": drawTube(ctx,w,h); break; default: ctx.fillRect(-w/2,0,w,h); break;} ctx.restore();
		} ctx.restore();
		setTimeout(nextLoop,sp);
	}
	nextLoop(0);
	return {
		start: function (){if(!running){running=true; nextLoop(0);}},
		stop: function (){running=false; ctx.clearRect(0,0,sz,sz); },
		pause: function (){running=false; }
	};
}

var login = {
	unreal : false,
	doing : false,
	try_name : "",
	try_pass : "",
	show : function () {
		if (login_direct == "ja") {
			$("login_naam").value = login_direct_name;
			$("login_wachtwoord").value = login_direct_pass;
			try_login();
		}
		if (readCookie('jabbo_sess') == "ja") {
			$("login_naam").value = readCookie('jabbo_name');
			$("login_wachtwoord").value = readCookie('jabbo_pass');
		}
		mainview.show();
		ShowContent("login_div");
		register.close_button();
		$("login_naam").focus();
		client.mode = "main";
		// cleaning up
		removechilds("preload_div");
		removechilds("loadview");
	},
	attempt : function () {
		this.try_name = $("login_naam").value.toLowerCase();
		this.try_pass = $("login_wachtwoord").value;
		if (this.try_name != "" && this.try_pass != "") {
			client.block();
			HideContent("knop_1");
			ShowContent("verbinden_img");
			login.doing = true;
			client.block();
			HideContent("knop_1");
			ShowContent("verbinden_img");
			if (mainview.animations_done) {
				login.go();
			}
			else {
				login.unreal = true;
			}
		}
		else {
			client.notify(texts["login_fill_both"]);
		}
	},
	go : function () {
		if (login.unreal) {
			login.unreal = false;
		}
		HideContent("register_box");
		packets.send.login(this.try_name, this.try_pass);
	},
	succes : function () {
		if (in_array("debug", user.rights)) {
			lib.load("debug");
		}
		user.name = $("login_naam").value.toLowerCase();
		user.pass = $("login_wachtwoord").value;
		this.try_name = "";
		this.try_pass = "";
		createCookie('jabbo_name', user.name, 0);
		createCookie('jabbo_pass', user.pass, 0);
		createCookie('jabbo_sess', 'ja', 0);
		register.close_button();
		furni.loaddb();
	},
	dbloaded : function () {
		$('welcome_jabbo_img').src = "avatar.php?figure="+user.clothes+"&action=wav&size=b&direction=3&head_direction=3&gesture=sml";
		this.loadwindows();
	},
	loadwindows : function () {
		if (client.connection) {
			var req = ajaxCreate();
			req.open("GET", "windows.php?r="+Math.round(9000*Math.random()), true);
			req.onreadystatechange = function() {
				if (req.readyState == 4) {
					$("windows_div").innerHTML = req.responseText;
					HideContent("login_hide");
					$("welcome_jabbo_text").innerHTML = texts["login_welcome"]+", "+ucfirst(user.name);
					ShowContent("welcome_jabbo");
					ShowContent("welcome_jabbo_text");
					client.logged_in();
					setTimeout(function(){client.logged_in2();}, 3000);
					client.unblock();
				}
			};
			req.send(null);
		}
	},
	error : function (errcode) {
		switch(errcode) {
			case "error":
				ShowContent("knop_1");
				HideContent("verbinden_img");
				client.unblock();
				client.exit2();
				client.notify(texts["login_fail"]);
				break;
			case "try":
				ShowContent("knop_1");
				HideContent("verbinden_img");
				client.unblock();
				client.exit2();
				client.notify(texts["login_max"]);
				break;
			case "mail":
				ShowContent("knop_1");
				HideContent("verbinden_img");
				client.notify(texts["login_mail"]);
				client.unblock();
				break;
			case "full":
				ShowContent("knop_1");
				HideContent("verbinden_img");
				client.notify(texts["login_full"]);
				client.unblock();
				break;
			case "ban":
				$("login_naam").value = "";
				$("login_wachtwoord").value = "";
				ShowContent("knop_1");
				HideContent("verbinden_img");
				client.notify(texts["login_ban"]);
				client.unblock();
				break;
			case "wrong":
				$("login_naam").value = "";
				$("login_wachtwoord").value = "";
				ShowContent("knop_1");
				HideContent("verbinden_img");
				client.notify(texts["login_wrong"]);
				client.unblock();
				break;
			default:
				debug.error("Unknown login errorcode.");
		}
		login.doing = false;
		ShowContent("register_box");
	}
};
var register = {
	step : 1,
	kruisje1 : false,
	kruisje2: false,
	error_message : "",
	clear : function () {
		HideContent(["register_upperlayer_1", "register_upperlayer_2", "register_upperlayer_3", "register_laden", "register_ballon", "register_plaatje1", "register_ballon_text", "register_text1", "knop_37", "knop_38", "knop_39", "register_text2", "register_text3", "register_kruisje1", "register_text4", "knop_35", "knop_36", "register_text5", "register_char_back", "register_char_name_back", "register_text6", "register_char_name_div", "register_text7", "register_plaatje2", "register_info_back1", "register_info_back2", "register_info_back3", "register_text8", "register_text9", "register_text10", "register_info_input_div_1", "register_info_input_div_2", "register_info_input_div_3", "register_info_input_div_4"]);
	},
	open : function () {
		if(!login.doing) {
			$("register").style.left = 621;
			$("register").style.top = 71;
			register.clear();
			register.step = 1;
			register.kruisje1 = false;
			$("register_kruisje1").src = "images/register/kruisje1.gif";
			$("register_char_name").value = "";
			ShowContent("register_upperlayer_3");
			ShowContent("register_laden");
			ShowContent("register");
			HideContent("login_div");
			packets.send.register_start_check();
		}
	},
	close : function () {
		HideContent("register");
		register.clear();
	},
	close_button : function () {
		ShowContent("login_div");
		HideContent("register");
		register.clear();
	},
	step1 : function () {
		register.step = 1;
		$("register_text5").innerHTML = "1/6";
		register.clear();
		ShowContent("register_upperlayer_1");
		ShowContent("register_ballon");
		ShowContent("register_plaatje1");
		ShowContent("register_ballon_text");
		ShowContent("register_text1");
		ShowContent("knop_37");
		ShowContent("knop_38");
		ShowContent("knop_39");
	},
	step2 : function () {
		register.step = 2;
		$("register_text5").innerHTML = "2/6";
		$('register_text3').scrollTop = 0;
		register.clear();
		ShowContent("register_upperlayer_2");
		ShowContent("register_text2");
		ShowContent("register_text3");
		ShowContent("register_kruisje1");
		ShowContent("register_text4");
		ShowContent("knop_35");
		ShowContent("knop_36");
		ShowContent("register_text5");
	},
	step3 : function () {
		if (register.kruisje1) {
			register.step = 3;
			$("register_text5").innerHTML = "3/6";
			register.clear();
			ShowContent("register_upperlayer_2");
			ShowContent("knop_35");
			ShowContent("knop_36");
			ShowContent("register_text5");
			ShowContent("register_char_back");
			ShowContent("register_char_name_back");
			ShowContent("register_text6");
			ShowContent("register_char_name_div");
			$("register_char_name").focus();
		}
		else {
			client.notify(texts[13]);
		}	
	},
	step4 : function () {
		if (register.step == 4) {
			$("register_text5").innerHTML = "4/6";
			$("register_info_pass1").value = "";
			$("register_info_pass2").value = "";
			register.clear();
			ShowContent("register_upperlayer_2");
			ShowContent("knop_35");
			ShowContent("knop_36");
			ShowContent("register_text5");
			ShowContent("register_text7");
			ShowContent("register_plaatje2");
			ShowContent("register_info_back1");
			ShowContent("register_info_back2");
			ShowContent("register_info_back3");
			ShowContent("register_text8");
			ShowContent("register_info_input_div_1");
			ShowContent("register_info_input_div_2");
			ShowContent("register_info_input_div_3");
			ShowContent("register_info_input_div_4");
		}
		else {
			if ($("register_char_name").value != "")
			{
				register.clear();
				ShowContent("register_upperlayer_3");
				ShowContent("register_laden");
				register.name_check();
			}
			else {
				client.notify(texts[14]);
			}
		}
	},
	step5 : function () {
		if (register.step == "5") {
			register.kruisje2 = true;
			$("register_kruisje2").src = "images/register/kruisje4.gif";
			$("register_text5").innerHTML = "5/6";
			register.clear();
			ShowContent("register_upperlayer_2");
			ShowContent("knop_35");
			ShowContent("knop_36");
			ShowContent("register_text5");
			ShowContent("register_text9");
			$("reg_info_lastcheck_div").innerHTML = "<b>"+texts[15]+" "+$("register_char_name").value+"<br><br>"+texts[16]+" "+$("register_info_birth").value+"<br><br>"+texts[17]+" "+$("register_info_mail").value+"</b>";
		}
		else {
			this.check_info.pass1 = $("register_info_pass1").value;
			this.check_info.pass2 = $("register_info_pass2").value;
			this.check_info.mail = $("register_info_mail").value;
			this.check_info.birth = $("register_info_birth").value;
			this.check_info.firstcheck = true;
			this.check_info.pass_firstcheck = true;
			this.check_info.mail_firstcheck = true;
			this.check_info.birth_firstcheck = true;
			if (this.check_info.pass1 == "") {
				this.check_info.firstcheck = false;
				this.check_info.pass_firstcheck = false;
			}
			if (this.check_info.pass1 != this.check_info.pass2) {
				this.check_info.firstcheck = false;
				this.check_info.pass_firstcheck = false;
			}
			if (!isValidEmail(this.check_info.mail)) {
				this.check_info.firstcheck = false;
				this.check_info.mail_firstcheck = false;
			}
			if (this.check_info.birth != "") {
				this.birtharray = this.check_info.birth.split("-");
				if (this.birtharray.length != 3)
				{
					this.check_info.firstcheck = false;
					this.check_info.birth_firstcheck = false;
				}
				else
				{
					if(this.birtharray[0] > 31)
					{
						this.check_info.firstcheck = false;
						this.check_info.birth_firstcheck = false;
					}
					if(this.birtharray[1] > 12)
					{
						this.check_info.firstcheck = false;
						this.check_info.birth_firstcheck = false;
					}
					if(this.birtharray[2] != 4)
					{
						this.check_info.firstcheck = false;
						this.check_info.birth_firstcheck = false;
					}
				}
			}
			else {
				this.check_info.firstcheck = false;
				this.check_info.birth_firstcheck = false;
			}
			if (this.check_info.firstcheck) {
				register.clear();
				ShowContent("register_upperlayer_3");
				ShowContent("register_laden");
				this.check_info.go();
			}
			else {
				this.error_message = "";
				this.error_message += "<b>"+texts[18]+"</b><br>";
				if (!this.check_info.pass_firstcheck) {
					this.error_message += "<br>"+texts[19];
				}
				if (!this.check_info.birth_firstcheck) {
					this.error_message += "<br>"+texts[20];
				}
				if (!this.check_info.mail_firstcheck) {
					this.error_message += "<br>"+texts[21];
				}
				client.notify(this.error_message);
			}
		}
	},
	step6 : function () {
		register.clear();
		ShowContent("register_upperlayer_3");
		ShowContent("register_laden");
		register.send();
	},
	check_info :
	{
		pass1 : "",
		pass2 : "",
		mail : "",
		birth : "",
		firstcheck : true,
		pass_firstcheck : true,
		mail_firstcheck : true,
		birth_firstcheck : true,
		go : function () {
			
		}
	},
	jong : function () {
		register.close();
		register.jong_send();
		client.exit2();
		client.notify(texts[12]);
	},
	jong_send : function () {
		
	},
	push_kruisje : function () {
		if (register.kruisje1) {
			register.kruisje1 = false;
			$("register_kruisje1").src = "images/register/kruisje1.gif";
		}
		else {
			register.kruisje1 = true;
			$("register_kruisje1").src = "images/register/kruisje2.gif";
		}
	},
	push_kruisje2 : function () {
		if (register.kruisje2) {
			register.kruisje2 = false;
			$("register_kruisje2").src = "images/register/kruisje3.gif";
		}
		else {
			register.kruisje2 = true;
			$("register_kruisje2").src = "images/register/kruisje4.gif";
		}
	},
	done : function () {
		register.step = 6;
		$("register_text5").innerHTML = "6/6";
		register.clear();
		ShowContent("register_upperlayer_2");
		ShowContent("register_text10");
	},
	login : function () {
		$("login_naam").value = $("register_char_name").value;
		$("login_wachtwoord").value = $("register_info_pass1").value;
		register.close_button();
		login.attempt();
	},
	push_back : function () {
		run_script("register.step"+((register.step*1) - 1)+"();");
	},
	push_next : function () {
		run_script("register.step"+((register.step*1) + 1)+"();");
	},
	regstart_checked : function (answer) {
		switch(answer) 
		{
		case "ok":
			register.step1();
			break;
		case "cclosed":
			register.close();
			client.exit();
			break;
		case "ban":
			client.ecxit2();
			client.notify(texts[22]);
			break;
		case "closed":
			register.close_button();
			client.notify(texts["register_closed"]);
			break;
		case "age":
			register.close();
			client.exit2();
			client.notify(texts[24]);
			break;
		case "multi":
			register.close_button();
			client.notify(texts[25]);
			break;
		}
	}
};
/*
function checked_register_name(checked_waarde)
{
	if (checked_waarde == "ok")
	{
		register.step = "4";
		register_step4();
	}
	else
	{
		if (checked_waarde == "unacceptable")
		{
			$("register_char_name").value = "";
			start_alert(texts[26]);
			register_step3();
		}
		else
		{
			if (checked_waarde == "long")
			{
				$("register_char_name").value = "";
				start_alert(texts[27]);
				register_step3();
			}
			else
			{
				if (checked_waarde == "short")
				{
					$("register_char_name").value = "";
					start_alert(texts[28]);
					register_step3();
				}
				else
				{
					if (checked_waarde == "multi")
					{
						$("register_char_name").value = "";
						start_alert(texts[29]);
						register_step3();
					}
					else
					{
						if (checked_waarde == "char")
						{
							$("register_char_name").value = "";
							start_alert(texts[30]);
							register_step3();
						}
						else
						{
							if (checked_waarde == "empty")
							{
								$("register_char_name").value = "";
								start_alert(texts[31]);
								register_step3();
							}
						}
					}
				}
			}
		}
	}
}
function checked_register_info(checked_waarde)
{
	checked_waarde1 = checked_waarde.split("<ajax>")[0];
	checked_waarde2 = checked_waarde.split("<ajax>")[1];
	if (checked_waarde1 == "ok")
	{
		if (checked_waarde2 == "ok")
		{
			register.step = "5";
			register_step5();
		}
		else
		{
			if (checked_waarde2 == "error")
			{
				$("register_info_pass1").value = "";
				$("register_info_pass2").value = "";
				start_alert(texts[32]);
				register_step4();
			}
			else
			{
				if (checked_waarde2 == "age")
				{
					register.close();
					exit_client3();
					start_alert(texts[33]);
				}
				else
				{
					$("register_info_pass1").value = "";
					$("register_info_pass2").value = "";
					register_step4();
				}
			}
		}
	}
	else
	{
		if (checked_waarde1 == "long")
		{
			$("register_info_pass1").value = "";
			$("register_info_pass2").value = "";
			start_alert(texts[34]);
			register_step4();
		}
		else
		{
			if (checked_waarde1 == "short")
			{
				$("register_info_pass1").value = "";
				$("register_info_pass2").value = "";
				start_alert(texts[35]);
				register_step4();
			}
			else
			{
				$("register_info_pass1").value = "";
				$("register_info_pass2").value = "";
				register_step4();
			}
		}
	}
}

function register_jong_send()
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
			ajax_vrij = "nee";
			xmlhttp.open("GET", "register_jong.php?r="+Math.round(9000*Math.random()),true);
	    	xmlhttp.onreadystatechange=function()
	    	{
			    if (xmlhttp.readyState==4)
	       		{
			       	if(xmlhttp.status != 200)
	 				{
		 				verbinding = "nee";
	 				}
	 				ajax_vrij = "ja";
	       		}
	    	}
	    	xmlhttp.send(null);
    	}
    	else
    	{
	    	setTimeout("register_jong_send();", 1000);
    	}
    }
}
function register_name_check()
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
			ajax_vrij = "nee";
			xmlhttp.open("GET", "check_register_name.php?name="+$("register_char_name").value+"&r="+Math.round(9000*Math.random()),true);
	    	xmlhttp.onreadystatechange=function()
	    	{
			    if (xmlhttp.readyState==4)
	       		{
			       	if(xmlhttp.status == 200)
	 				{
		 				checked_register_name(xmlhttp.responseText);
	 				}
	 				else if(xmlhttp.status == 404)
	 				{
		  				verbinding = "nee";
	 				}
	 				else
	 				{
		  				verbinding = "nee";
	 				}
	 				ajax_vrij = "ja";
	       		}
	    	}
	    	xmlhttp.send(null);
    	}
    	else
    	{
	    	setTimeout("register_name_check();", 1000);
    	}
    }
}
function register_info_check()
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
			ajax_vrij = "nee";
			xmlhttp.open("GET", "check_register_info.php?pass="+$("register_info_pass1").value+"&birth="+$("register_info_birth").value+"&r="+Math.round(9000*Math.random()),true);
	    	xmlhttp.onreadystatechange=function()
	    	{
			    if (xmlhttp.readyState==4)
	       		{
			       	if(xmlhttp.status == 200)
	 				{
		 				checked_register_info(xmlhttp.responseText);
	 				}
	 				else if(xmlhttp.status == 404)
	 				{
		  				verbinding = "nee";
	 				}
	 				else
	 				{
		  				verbinding = "nee";
	 				}
	 				ajax_vrij = "ja";
	       		}
	    	}
	    	xmlhttp.send(null);
    	}
    	else
    	{
	    	setTimeout("register_info_check();", 1000);
    	}
    }
}
function register_send()
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
			ajax_vrij = "nee";
			xmlhttp.open("GET", "registergo.php?name="+$("register_char_name").value+"&pass="+$("register_info_pass1").value+"&mail="+$("register_info_mail").value+"&birth="+$("register_info_birth").value+"&automail="+register.kruisje2+"&r="+Math.round(9000*Math.random()),true);
	    	xmlhttp.onreadystatechange=function()
	    	{
			    if (xmlhttp.readyState==4)
	       		{
			       	if(xmlhttp.status == 200)
	 				{
		 				register_done();
	 				}
	 				else if(xmlhttp.status == 404)
	 				{
		  				verbinding = "nee";
	 				}
	 				else
	 				{
		  				verbinding = "nee";
	 				}
	 				ajax_vrij = "ja";
	       		}
	    	}
	    	xmlhttp.send(null);
    	}
    	else
    	{
	    	setTimeout("register_send();", 1000);
    	}
    }
}*/
var mainview = {
	animations_done : false,
	show: function () {
		this.closewindows();
		ShowContent("backview_div");
		ShowContent("mainview_div");
		client.mode = "main";
		this.black._open();
	},
	showopen: function () {
		this.black.opened = false;
		this.closewindows();
		$('navigator').style.left = 585;
		$('navigator').style.top = 20;
		navi.open();
		ShowContent("backview_div");
		ShowContent("mainview_div");
		this.showtool();
		client.mode = "main";
		this.black._open();
	},
	showdirect: function () {
		this.black.opened = true;
		this.closewindows();
		$('navigator').style.left = 585;
		$('navigator').style.top = 20;
		navi.open();
		ShowContent("backview_div");
		ShowContent("mainview_div");
		this.showtool();
		client.mode = "main";
	},
	hide : function () {
		this.closewindows();
		HideContent("mainview_div");
		HideContent("backview_div");
	},
	closewindows : function () {
		if (user.logged_in) {
			HideContent("catalogus");
			HideContent("codes_layer");
			HideContent("askcamera");
			HideContent("camera");
			HideContent("picture");
			HideContent("donate_layer");
			chatbox.close();
			messenger.close();
			navi.close();
			matic.close();
			HideContent("help_layer");
		}
	},
	black :
	{
		opened : false,
		_open : function () {
			if (!mainview.black.opened) {
				mainview.animations_done = false;
				$('main_logo').style.top = -168;
				client.block();
				set_top("main_black_div");
				ShowContent("main_black_div");
				movecallback = this._opened();
				$('main_black_1').style.top = 0;
				$('main_black_2').style.top = Math.ceil((client_height/2));
				movelayer('main_black_1', 0, (Math.ceil((client_height/2)))-((Math.ceil((client_height/2)))*2), 30, 20, 0.8, movecallback);
				movelayer('main_black_2', 0, client_height, 30, 20, 0.8, false);
			}
		},
		_close : function () {
			if (mainview.black.opened) {
				mainview.animations_done = false;
				client.block();
				set_top("main_black_div");
				ShowContent("main_black_div");
				movecallback = this._closed();
				$('main_black_1').style.top = (Math.ceil((client_height/2)))-((Math.ceil((client_height/2)))*2);
				$('main_black_2').style.top = client_height;
				movelayer('main_black_1', 0, 0, 30, 20, 0.8, movecallback);
				movelayer('main_black_2', 0, Math.ceil((client_height/2)), 30, 20, 0.8, false);
			}
		},
		_opened : function () {
			to_return = new Object();
			to_return.complete = function() {
				client.unblock();
				mainview.black.opened = true;
				HideContent("main_black_div");
				mainview.open_logo();
			}
			return to_return;
		},
		_closed : function () {
			to_return = new Object();
			to_return.complete = function() {
				mainview.animations_done = true;
				client.unblock();
				mainview.black.opened = false;
				room.go2(); // ga naar de kamers!
				//public.go2();
				mainview.hide();
				HideContent("main_black_div");
			}
			return to_return;
		}
	},
	open_logo : function () {
		mainview.animations_done = true;
		ShowContent("main_logo");
		movecallback = this.logo_opened();
		movelayer('main_logo', 0, 0, 10, 40, 0.5, movecallback);
	},
	logo_opened : function () {
		to_return = new Object();
		to_return.complete = function() {
			mainview.animations_done = true;
			if (!user.logged_in && login.unreal)
			{
				login.go();
			}
		}
		return to_return;
	},
	showtool : function () {
		$('toolbar_div').style.top = 55;
		movelayer('toolbar_div', 0, 0, 10, 20, 0.9, false);
	}
}
var chatbox = {
	opened : false,
	open : function () {
		if (!room.loading) {
			if (!chatbox.opened) {
				$('chatbox_text').innerHTML = "";
				ShowContent("chatbox_load");
				chatbox.opened = true;
				chatbox.update();
				set_top('chatbox');
				ShowContent('chatbox');
				$("chatbox_input").focus();
			}
		}
	},
	close : function () {
		if (this.opened) {
			this.opened = false;
			HideContent('chatbox');
		}
	},
	reverse : function () {
		if (this.opened) {
			this.close();
		}
		else {
			this.open();
		}
	},
	update : function () {
		if (client.connection && this.opened) {
			var req = ajaxCreate();
			req.open("GET", "action.php?method=chat&r="+Math.round(9000*Math.random()),true);
			req.onreadystatechange = function()
			{
				if (req.readyState == 4)
				{
					$('chatbox_text').innerHTML = "<table border='0' class='clientfont'>"+req.responseText+"</table>";
		        	$('chatbox_text').scrollTop = $('chatbox_text').scrollHeight;
		        	HideContent("chatbox_load");
		        	setTimeout(function(){chatbox.update()},500)
				}
			}
			req.send(null);
		}
	},
	go : function () {
		if (client.connection && this.opened) {
			var chattext = $('chatbox_input').value;
			$('chatbox_input').value = "";
     		$('chatbox_input').focus();
			var req = ajaxCreate();
			req.open("POST", "action.php?method=chat&r="+Math.round(9000*Math.random()),true);
			req.onreadystatechange = function()
			{
				if (req.readyState == 4)
				{
					$('chatbox_text').innerHTML = "<table border='0' class='clientfont'>"+req.responseText+"</table>";
		        	$('chatbox_text').scrollTop = $('chatbox_text').scrollHeight;
				}
			}
			req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    		req.send("text="+chattext);
		}
	}
};
var room = {
	loadedclient: false,
	loadedserver: false,
	loading: false,
	id : 0,
	data : null,
	start_x : 0,
	start_y : 0,
	bigholes : new Array(),
	checkedholes : new Array(),
	go : function (id) {
		if (!this.loading) {
			client.mode = "main";
			if (this.id == 0) {
				this.loading = true;
				this.id = id;
				mainview.black._close();
			}
			else {
				mainview.closewindows();
				this.loading = true;
				this.id = id;
				this.go2();
			}
		}
	},
	go2 : function () {
		if (this.loading) {
			this.loadedclient = false;
			this.loadedserver = false; 
			furni.obj = null;
			furni.dragimg = null;
			this.tiles.click = false;
			set_top('room_div');
			set_top('load_layer');
			ShowContent('load_layer');
			ShowContent('room_div');
			packets.send.room_go(this.id);
		}
	},
	fail : function (error) {
		if (this.loading) {
			this.loading = false;
			this.id = 0;
			mainview.showdirect();
			HideContent('load_layer');
			HideContent('room_div');
			debug.error("Loading room failed.");
			if (error == "error") {
				client.notify(texts["room_loadfail"], texts["notify_error"]);
			}
			else {
				client.notify(texts["room_full"]);
			}
		}
	},
	load : function () {
		if (this.loading) {
			client.mode = "room";
			$("roominfo1").innerHTML = this.data.name;
			$("roominfo2").innerHTML = texts["room_owner"]+": "+ucfirst(this.data.ownername);
			if (room.data.voted) {
				$('voted_text').innerHTML = texts["room_score"]+" "+room.data.score;
				HideContent('vote_bad');
				HideContent('vote_good');
				HideContent('vote_text');
				ShowContent('voted_text');
			}
			else {
				$('voted_text').innerHTML = "";
				ShowContent('vote_bad');
				ShowContent('vote_good');
				ShowContent('vote_text');
				HideContent('voted_text');
			}
			$('roomchat').value = texts["room_chatclick"];
			$("roomchat").style.width = "580px";
			$('trashtext').focus();
			furni.click('');
			this.roomgen();
			this.tiles.init();
			furni.inv(); // after loading inventory login data, this will call on furni.init(), which includes preload, will call on room.loaded() when done
			inv.set("closeforce");
			document.onmousemove = room.mousemove;
			document.onmouseup = room.mousedown;
		}
	},
	loaded : function () {
		if (this.loading) {
			if (this.loadedclient && this.loadedserver) {
				ShowContent("room_div");
				HideContent("load_layer");
				this.loading = false;
				this.tiles.click = true;
			}
		}
	},
	leave : function () {
		if (client.mode == "room") {
			this.tiles.click = false;
			this.id = 0;
			furni.obj = null;
			furni.dragimg = null;
			HideContent("room_div");
			removechilds("room_back");
			removechilds("furniture");
			removechilds("dragfurni");
			removechilds("roomchats");
			removechilds("nametags");
			mainview.showopen();
			packets.send.room_leave();
		}
	},
	walls : {
		turn : new Array(),
		startx : new Array(),
		starty : new Array(),
		endx : new Array(),
		endy : new Array(),
		clear : function () {
			this.turn = new Array();
			this.startx = new Array();
			this.starty = new Array();
			this.endx = new Array();
			this.endy = new Array();
		},
		add : function (turn, x, y) {
			var gendata = genarray[room.data.mode];
			this.turn.push(turn);
			this.startx.push(x);
			this.starty.push(y);
			this.endx.push(x+gendata[8]);
			this.endy.push(y+gendata[12]);
		}
	},
	roomgen : function () {
		removechilds("room_back");
		removechilds("furniture");
		removechilds("dragfurni");
		removechilds("roomchats");
		removechilds("nametags");
		inv.prepare();
		var left = 0;
		var top = 0;
		var gendata = genarray[this.data.mode];
		
		this.tiles.width = gendata[8];
		this.tiles.height = this.tiles.width/2;
		
		left -= gendata[4];
		top += gendata[5];
		left += this.data.breed * gendata[8];
		
		var width = (((parseInt(this.data.lang) + parseInt(this.data.breed)) / 2) * (gendata[8]*2)) + gendata[6];
		var height = (((parseInt(this.data.lang) + parseInt(this.data.breed)) / 2) * gendata[8]) + gendata[7];
		
		left += Math.round((client_width - width) / 2);
		top += Math.round((client_height - height) / 2);
		
		if ((left / gendata[8]) % 2 == 0) {
			var row = Math.floor(top / gendata[8]) + Math.floor(left / (2 * gendata[8]));
			var col = -Math.floor(top / gendata[8]) + Math.floor((left + gendata[8]) / (2 * gendata[8]));
		}
		else {
			var row = Math.floor((top + gendata[8] / 2) / gendata[8]) + Math.floor(left / (2 * gendata[8]));
			var col = -Math.floor((top + gendata[8] / 2) / gendata[8]) + Math.floor((left + gendata[8]) / (2 * gendata[8]));
		}
		
		var start_x = floor_x = row * gendata[8] + col * gendata[8];
		var start_y = floor_y = (row * (gendata[8] / 2)) - (col * (gendata[8] / 2));
		
		this.start_x = start_x;
		this.start_y = start_y;
		start_x = floor_x -= this.tiles.width/2;
		start_y = floor_y -= this.tiles.height/2;
		
		this.bigholes = new Array();
		this.checkedholes = new Array();
		walls = { turn : new Array(), startx : new Array(), starty : new Array(), endx : new Array(), endy : new Array() };
		this.bighole([1,1]);
			
		var deurgeweest = false;
		for (i1 = 1; i1 <= this.data.lang; i1++) {
			for (i2 = 1; i2 <= this.data.breed; i2++) {
				if (!in_array(i2+"_"+i1, this.data.holes)) {
					// dit is geen gat
					this.addtoback("images/rooms/floors/"+this.data.mode+"/"+this.data.floor+".gif", floor_x, floor_y, 3);
					var door = 0;
					if (i2+"_"+(i1-1) == this.data.door && !deurgeweest) {
						var x = floor_x + gendata[0];
						var y = floor_y - gendata[1];
						this.addtoback("images/rooms/door.php?door=1&wall="+this.data.wall+"&mode="+this.data.mode+"&img=1", x, y, 1);
						this.addtoback("images/rooms/door.php?door=1&wall="+this.data.wall+"&mode="+this.data.mode+"&img=2", x, y, 3);
						door = 1;
						deurgeweest = true;
						this.walls.add(1, x, y);
					}
					if ((i2-1)+"_"+i1 == this.data.door && !deurgeweest) {
						var x = floor_x + gendata[2];
						var y = floor_y - gendata[3];
						this.addtoback("images/rooms/door.php?door=2&wall="+this.data.wall+"&mode="+this.data.mode+"&img=1", x, y, 1);
						this.addtoback("images/rooms/door.php?door=2&wall="+this.data.wall+"&mode="+this.data.mode+"&img=2", x, y, 3);
						door = 2;
						deurgeweest = true;
						this.walls.add(2, x, y);
					}
					if (i1 == 1) {
						if (door != 1) {
							var x = floor_x + gendata[0];
							var y = floor_y - gendata[1];
							this.addtoback("images/rooms/walls/"+this.data.mode+"/"+this.data.wall+"/1.gif", x, y, 3);
							this.walls.add(1, x, y);
						}
					}
					if (i2 == 1) {
						if (door != 2) {
							var x = floor_x + gendata[2];
							var y = floor_y - gendata[3];
							this.addtoback("images/rooms/walls/"+this.data.mode+"/"+this.data.wall+"/2.gif", x, y, 3);
							this.walls.add(2, x, y);
						}
					}
					if (in_array(i2+"_"+(i1-1), this.bigholes)) {
						if (door != 1) {
							var x = floor_x + gendata[0];
							var y = floor_y - gendata[1];
							this.addtoback("images/rooms/walls/"+this.data.mode+"/"+this.data.wall+"/1.gif", x, y, 3);
							this.walls.add(1, x, y);
						}
					}
					if (in_array((i2-1)+"_"+i1, this.bigholes)) {
						if (door != 2) {
							var x = floor_x + gendata[2];
							var y = floor_y - gendata[3];
							this.addtoback("images/rooms/walls/"+this.data.mode+"/"+this.data.wall+"/2.gif", x, y, 3);
							this.walls.add(2, x, y);
						}
					}
					if ((i2+1) > this.data.breed || in_array((i2+1)+"_"+i1, this.data.holes)) {
						if ((i1-1) < 1 || in_array(i2+"_"+(i1-1), this.bigholes)) {
							var x = floor_x - gendata[9];
							var y = floor_y - gendata[11];
							this.addtoback("images/rooms/walls/"+this.data.mode+"/"+this.data.wall+"/l.gif", x, y, 5);
						}
					}
					if ((i1+1) > this.data.lang || in_array(i2+"_"+(i1+1), this.data.holes)) {
						if ((i2-1) < 1 || in_array((i2-1)+"_"+(i1-1), this.bigholes)) {
							var x = floor_x + gendata[10];
							var y = floor_y - gendata[11];
							this.addtoback("images/rooms/walls/"+this.data.mode+"/"+this.data.wall+"/r.gif", x, y, 5);
						}
					}
				}
				this.addtile(i2+"_"+i1);
				floor_x -= gendata[8];
				floor_y += (gendata[8] / 2);
			}
			floor_x = start_x + (i1 * gendata[8]);
			floor_y = start_y + (i1 * (gendata[8] / 2));
		}
		this.addtile("0_0"); // for posters
		
		// deurtje
		var obj = document.createElement("div");
		obj.id = "tile"+this.data.door;
		obj.style.position = "absolute";
		obj.style.zIndex = 2;
		$("room_back").appendChild(obj);
	},
	bighole : function (tile) {
		if (tile[0] > 0 && tile[1] > 0 && tile[0] <= this.data.breed && tile[1] <= this.data.lang) {
			if (!in_array(tile[0]+"_"+tile[1], this.checkedholes)) {
				if (in_array(tile[0]+"_"+tile[1], this.data.holes)) {
					this.checkedholes.push(tile[0]+"_"+tile[1]);
					if (this.notile([(tile[0]-1),tile[1]]) && this.notile([tile[0],(tile[1]-1)])) {
						this.bigholes.push(tile[0]+"_"+tile[1]);
					}
					this.bighole([(tile[0]-1),tile[1]]);
					this.bighole([tile[0],(tile[1]+1)]);
					this.bighole([(tile[0]+1),tile[1]]);
					this.bighole([tile[0],(tile[1]-1)]);
				}
			}
		}
	},
	notile : function (tile) {
		var answer = false;
		if (tile[0] < 1 || tile[1] < 1 || tile[0] > this.data.breed || tile[1] > this.data.lang) {
			answer = true;
		}
		if (in_array(tile[0]+"_"+tile[1], this.data.holes)) {
			answer = true;
		}
		return answer;
	},
	addtoback : function (img, x, y, z) {
		var obj = document.createElement("img");
		obj.src = img;
		obj.style.position = "absolute";
		obj.style.left = x;
		obj.style.top = y;
		obj.style.zIndex = z;
		$("room_back").appendChild(obj);
	},
	addtile : function (tile) {
		var obj = document.createElement("div");
		obj.id = "tile"+tile;
		var z = ((tile.split(/_/)[0])*1)+((tile.split(/_/)[1])*1);
		if (z < 2) {
			z = 2;
		}
		if (tile == "0_0") {
			z = 1;
		}
		obj.style.position = "absolute"; // make z-index apply to nodes correctly
		obj.style.zIndex = z+2; // +2 -> we don't want furniture etc to be beneath walls
		// add as first node
		//$("furniture").insertBefore(obj, $("furniture").firstChild);
		$("furniture").appendChild(obj);
	},
	tiles : {
		click : false,
		width : 0,
		height: 0,
		posx : null,
		posy : null,
		nul_row : 0,
		nul_col : 0,
		init : function () {
			$("tile_hover").src = "images/tile_hover_"+room.data.mode+".gif";
			this.posx = new Array();
			this.posy = new Array();
			
			var x = room.start_x - (this.width/2);
			var y = room.start_y - (this.height/2);
			for (i1 = 1; i1 <= room.data.lang; i1++) {
				for (i2 = 1; i2 <= room.data.breed; i2++) {
					this.posx[i2+"_"+i1] = x;
					this.posy[i2+"_"+i1] = y;
					
					if ((i2-1)+"_"+i1 == room.data.door) {
						this.posx[room.data.door] = x+(this.width);
						this.posy[room.data.door] = y-(this.height);
					}
					else if (i2+"_"+(i1-1) == room.data.door) {
						this.posx[room.data.door] = x-(this.width)
						this.posy[room.data.door] = y-(this.height);
					}
					
					x -= this.width;
					y += this.height;
				}
				x = room.start_x + (i1 * this.width) - (this.width/2);
				y = room.start_y + (i1 * this.height) - (this.height/2);
			}
			if (parseInt(room.start_x / this.width) % 2 == 0) {
				this.nul_row = Math.floor(room.start_y / this.width) + Math.floor(room.start_x / (2 * this.width));
				this.nul_col = -Math.floor(room.start_y / this.width) + Math.floor((room.start_x + this.width) / (2 * this.width));
			}
			else {
				this.nul_row = Math.floor((room.start_y + this.width / 2) / this.width) + Math.floor(room.start_x / (2 * this.width));
				this.nul_col = -Math.floor((room.start_y + this.width / 2) / this.width) + Math.floor((room.start_x + this.width) / (2 * this.width));
			}
		},
		check : function (x, y, mode) {
			if (parseInt(x / this.width) % 2 == 0) {
				var row = Math.floor(y / this.width) + Math.floor(x / (2 * this.width));
				var col = -Math.floor(y / this.width) + Math.floor((x + this.width) / (2 * this.width));
			}
			else {
				var row = Math.floor((y + this.width / 2) / this.width) + Math.floor(x / (2 * this.width));
				var col = -Math.floor((y + this.width / 2) / this.width) + Math.floor((x + this.width) / (2 * this.width));
			}
			if (mode) {
				var tile1 = -(col-this.nul_col)+1;
				var tile2 = row-this.nul_row+1;
				if (tile1 > 0 && tile2 > 0 && tile1 <= room.data.breed && tile2 <= room.data.lang && !in_array(tile1+"_"+tile2, room.data.holes)) {
					var hover_x = (row * this.width + col * this.width)-(this.width/2);
					var hover_y = ((row * (this.width / 2)) - (col * (this.width / 2)))-(this.height/2);
					var to_return = new Array(true, tile1, tile2, hover_x, hover_y);
					$('tile_hover').style.left = hover_x;
					$('tile_hover').style.top = hover_y;
					ShowContent('tile_hover');
				}
				else {
					var to_return = new Array(false, 0, 0, 0, 0);
					$('tile_hover').hide();
				}
				return to_return;
			}
			else {
				tile1 = -(col-this.nul_col)+1;
				tile2 = row-this.nul_row+1;
				$('infotext').innerHTML = tile1+" "+tile2; // DEVONLY
				if (tile1 > 0 && tile2 > 0 && tile1 <= room.data.breed && tile2 <= room.data.lang && !in_array(tile1+"_"+tile2, room.data.holes)) {
					$('tile_hover').style.left = (row * this.width + col * this.width)-(this.width/2);
					$('tile_hover').style.top = ((row * (this.width / 2)) - (col * (this.width / 2)))-(this.height/2);
					ShowContent('tile_hover');
				}
				else {
					$('tile_hover').hide();
				}
			}
		}
	},
	mousemove : function (positions) {
		// drag furniture + tile hover
		var posx = document.all ? window.event.clientX : positions.pageX;
		var posy = document.all ? window.event.clientY : positions.pageY;
		if (furni.obj != null && client.mode == "room") {
			furni.move(posx, posy);
		}
		else {
			//no object is being dragged, just hover the tile please
			if (room.tiles.click) {
				room.tiles.check(posx, posy, false);
			}
		}
	},
	mousedown : function (positions) {
		// drop furniture + walking
		if (furni.obj != null && client.mode == "room") {
			var posx = document.all ? window.event.clientX : positions.pageX;
			var posy = document.all ? window.event.clientY : positions.pageY;
			var this_tegel = room.tiles.check(posx, posy, true);
			furni.dragstop([posx,posy], this_tegel[0], [this_tegel[1],this_tegel[2]]);
		}
		else {
			//no object is being dragged, just start walking please - if this is really a tile!
			if (room.tiles.click && client.mode != "main" && !ctrl_status && !alt_status && furni.obj == null) {
				var posx = document.all ? window.event.clientX : positions.pageX;
				var posy = document.all ? window.event.clientY : positions.pageY;
				var this_tegel = room.tiles.check(posx, posy, true);
				if (this_tegel[0]) {
					packets.send.walkrequest(this_tegel[1], this_tegel[2]);
				}
			}
		}
	},
	chat : {
		expand : function () {
			var reg = new RegExp("<(.|\n)*?>");
  			$("roomchat").value = $("roomchat").value.replace(reg, "");
			var size = ($("roomchat").value.length*13)+30;
			if (size > 44 && size < 580) {
				$("roomchat").style.width = size+"px";
			}
			else {
				if (size < 44) {
					$("roomchat").style.width = "44px";
				}
				else if (size > 575) {
					$("roomchat").style.width = "580px";
				}
			}
		},
		new_chat : function () {
			var chat_box = $('roomchat').value;
			if(chat_box != '') {
				packets.send.room_chat("R", chat_box);
			}
			$('roomchat').value = "";
			$("roomchat").style.width = "44px";
			$('roomchat').focus();
		},
		spam : {
			count : 0,
			init : function (count) {
				this.count = count;
				this.spam();
			},
			spam : function (count) {
				$('roomchat').disabled = true;
		
				this.count = this.count - 1;

				$('roomchat').value = texts["room_spam"]+" " + this.count;
				room.chat.expand();
				if (this.count <= 0 ) {
					$('roomchat').disabled = false;
					$('roomchat').value = "";
					$("roomchat").style.width = "44px";
					$('roomchat').focus();
					clearTimeout(spamtimer);
					return;
				}
				var spamtimer = setTimeout(function(){room.chat.spam.spam(this.count)},1000);
			}
		},
		show_chat : {
			chat_id : 0,
			render : function (name, id, message) {
				if (client.mode != "main") {
					if($("ava_"+id)) {
						var node = $("roomchats");
						var len = node.childNodes.length;
						for (i = 0; i < len; i++) {
							var this_node = node.childNodes[i];
							if (this_node) {
								var new_top = parseInt(this_node.style.top)-22;
								if(new_top > 0) {
									this_node.style.top = new_top;
								}
								else {
									node.removeChild(this_node);
								}
							}
						}
						var chatlength = Math.floor(((message.length*6.5)+(name.length*6.1))/2);
						var middle_x = parseInt($("ava_"+id).style.left);
						middle_x += ($("ava_"+id).clientWidth)/2;
						middle_x -= chatlength;
						
						if (middle_x < 0) {
							middle_x = 0;
						}
						if ((middle_x+chatlength) > client_width) {
							middle_x = client_width - chatlength;
						}
						$("roomchats").innerHTML += "<div id='chattext_"+this.chat_id+"' style='position:absolute; left:"+middle_x+"px; top:144px; z-index:150;'><table onclick='avatars.click("+id+")' border='0' cellpadding='0' cellspacing='0'><tr><td width='6' height='19'><img src='images/roomchat/1.gif'></td><td height='19' style='background: url(\"images/roomchat/2.gif\") repeat-x; padding: 3px 0px 0px 0px; background-color: white;' valign='top'><div class='eigenfont' style='color: black;'><b>"+name+"</b>: "+message+"</div></td><td width='10' height='19'><img src='images/roomchat/3.gif'></td></tr></table></div>";
						this.chat_id++;
					}
				}
			},	
			move : function () {
				if (client.mode != "main") {
					var node = $("roomchats");
					var len = node.childNodes.length;
					for (i = 0; i < len; i++) {
						var this_node = node.childNodes[i];
						if (this_node) {
							var new_top = parseInt(this_node.style.top)-22;
							if(new_top > 0) {
								new Effect.Move(this_node.id, { y:-22 });
							}
							else {
								node.removeChild(this_node);
							}
						}
					}
				}
			}
		}
	},
	button_down : function (button, buttontype) {
		button.style.backgroundPosition = "0px -17px";
	},
	button_up : function (button, buttontype) {
		button.style.backgroundPosition = "0px 0px";
	},
	vote : function (vote) {
		if (!room.data.voted) {
			packets.send.vote(vote);
		}
	},
	voted : function (score) {
		$('voted_text').innerHTML = texts["room_score"]+" "+score;
		HideContent('vote_bad');
		HideContent('vote_good');
		HideContent('vote_text');
		ShowContent('voted_text');
		room.data.voted = true;
	}
};

var furni = {
	db : null, // data of all furni types
	data : null, // data of furni in room
	obj : null, // object that is being dragged
	dragimg : null,
	put_down : false, // is the object ready to be put down?
	drop : false, // can object be placed on this tile?
	tile_map : null,
	stack_map : null,
	loaddb : function () {
		if (client.connection) {
			var req = ajaxCreate();
			req.open("GET", "action.php?method=loadfurni&r="+Math.round(9000*Math.random()), true);
			req.onreadystatechange = function() {
				if (req.readyState == 4) {
					if (req.responseText != "") {
						var answer = JSON.parse(req.responseText);
						furni.db = answer[0];
						furni.manager.stats_total = answer[1];
						debug.yellow("Loaded info of "+answer[1]+" pieces of furniture");
						login.dbloaded();
					}
					else {
						login.error('error');
					}
				}
			};
			req.send(null);
		}
	},
	manager : {
		errors : false,
		stats_loaded : 0,
		stats_total : 0,
		img_loaded : null,
		img_preload : null,
		img_count : null,
		preload : null,
		callback : null,
		amount : 0,
		amount2 : 0,
		check : function () {
			var yes = 0;
			
			var len = furni.db.length;
			var count = 0;
			for(x in furni.db) {
				count++;
				if (furni.db[x].loaded) {
					yes++;
				}
			}
			this.stats_loaded = yes;
			this.stats_total = count;
			debug.yellow("Furniture manager counted: "+yes+" of "+count+" pieces preloaded.");
		},
		load : function (toload, addimg, callback) {
			// usage : you can call addimg and callback false
			
			var imgtoload = new Array();
			this.errors = false;
			this.img_loaded = new Array();
			this.img_preload = new Array()
			this.img_count = 0;
			this.callback = callback;
			this.amount = toload.length;
			if (addimg) {
				this.amount2 = addimg.length;
			}
			else {
				this.amount2 = 0;
			}
			
			var toload_length = toload.length;
			for (i = 0; i < toload_length; i++) {
				if (isset(furni.db[toload[i]])) {
					if (!furni.db[toload[i]].loaded) {
						if (furni.db[toload[i]].afb == 1) {
							if (furni.db[toload[i]].action == 1) {
								imgtoload.push("images/furni/"+toload[i]+"/1.gif");
								imgtoload.push("images/furni/"+toload[i]+"/1b.gif");
							}
							else {
								imgtoload.push("images/furni/"+toload[i]+"/1.gif");
							}
						}
						else if (furni.db[toload[i]].afb > 2) {
							if (furni.db[toload[i]].action == 1) {
								imgtoload.push("images/furni/"+toload[i]+"/1.gif");
								imgtoload.push("images/furni/"+toload[i]+"/2.gif");
								imgtoload.push("images/furni/"+toload[i]+"/3.gif");
								imgtoload.push("images/furni/"+toload[i]+"/4.gif");
								imgtoload.push("images/furni/"+toload[i]+"/1b.gif");
								imgtoload.push("images/furni/"+toload[i]+"/2b.gif");
								imgtoload.push("images/furni/"+toload[i]+"/3b.gif");
								imgtoload.push("images/furni/"+toload[i]+"/4b.gif");
							}
							else {
								imgtoload.push("images/furni/"+toload[i]+"/1.gif");
								imgtoload.push("images/furni/"+toload[i]+"/2.gif");
								imgtoload.push("images/furni/"+toload[i]+"/3.gif");
								imgtoload.push("images/furni/"+toload[i]+"/4.gif");
							}
						}
						else {
							if (furni.db[toload[i]].action == 1) {
								imgtoload.push("images/furni/"+toload[i]+"/1.gif");
								imgtoload.push("images/furni/"+toload[i]+"/2.gif");
								imgtoload.push("images/furni/"+toload[i]+"/1b.gif");
								imgtoload.push("images/furni/"+toload[i]+"/2b.gif");
							}
							else {
								imgtoload.push("images/furni/"+toload[i]+"/1.gif");
								imgtoload.push("images/furni/"+toload[i]+"/2.gif");
							}
						}
						imgtoload.push("images/furni/"+toload[i]+"/icon.gif");
						furni.db[toload[i]].loaded = true;
					}
				}
				else {
					debug.error("Furni manager failed to load '"+toload[i]+"'");
					this.amount--;
					this.errors = true;
				}
			}
			
			if (addimg) {
				var addimg_length = addimg.length;
				for (i = 0; i < addimg_length; i++) {
					imgtoload.push(addimg[i]);
				}
			}
			
			var imgtoload_length = imgtoload.length;
			for (i = 0; i < imgtoload_length; i++) {
				this.img_preload[i] = document.createElement('img');
		        this.img_preload[i].setAttribute('src', imgtoload[i]);
			}
			var img_preload_length = this.img_preload.length;
			for (i = 0; i < img_preload_length; i++) {
				this.img_loaded[i] = false;
			}
			this.check_load();
		},
		check_load : function () {
			if (this.img_count == this.img_preload.length) {
				this.loaded();
				return;
			}
			var img_preload_length = this.img_preload.length;
			for (i = 0; i <= img_preload_length; i++) {
				if (this.img_loaded[i] == false && this.img_preload[i].complete) {
					this.img_loaded[i] = true;
					this.img_count++;
				}
			}
			setTimeout(function(){furni.manager.check_load();},10);
		},
		loaded : function () {
			this.stats_loaded = this.stats_loaded + this.amount;
			debug.yellow("Furniture manager preloaded "+this.amount+" new piece(s) + "+this.amount2+" additional image(s), "+this.stats_loaded+"/"+this.stats_total+" loaded");
			this.callback.complete();
		}
	},
	init : function () {
		// intialise and show furniture
		this.obj = null;
		this.dragimg = null;
		var preload_furni = new Array();
		
		var len = this.data.length;
		var count = 0;
		
		for(x in this.data) {
			if(count >= len) break;
			else {
				count++;
				preload_furni.push(this.data[x].F);
			}
		}
		// laadt ook de deuren, die door PHP GD gegenereert moeten worden
		var preload_images = new Array("images/rooms/door.php?door=1&wall="+room.data.wall+"&mode="+room.data.mode+"&img=1", "images/rooms/door.php?door=1&wall="+room.data.wall+"&mode="+room.data.mode+"&img=2", "images/rooms/door.php?door=2&wall="+room.data.wall+"&mode="+room.data.mode+"&img=1", "images/rooms/door.php?door=2&wall="+room.data.wall+"&mode="+room.data.mode+"&img=2");
		var callback = furni.init2();
		furni.manager.load(preload_furni, preload_images, callback); // preload furniture
	},
	init2 : function () {
		var output = new Object();
		output.complete = function() {
			var len = furni.data.length;
			var count = 0;
			
			for(x in furni.data) {
				if(count >= len) break;
				else {
					count++;
					furni.add(furni.data[x], false);
				}
			}
			debug.green("Room furniture data parsed.");
			furni.heightmaps(); // set heightmaps, after furni init
			room.loadedclient = true;
			room.loaded();
		};
		return output;
	},
	inv : function () {
		if (!inv.loaded) {
			// preload inventory login data
			this.obj = null;
			this.dragimg = null;
			var preload_furni = new Array();
			
			var len = inv.login_data.length;
			var count = 0;
			
			for(x in inv.login_data) {
				if(count >= len) break;
				else {
					count++;
					preload_furni.push(inv.login_data[x].F);
				}
			}
			var callback = furni.inv2();
			furni.manager.load(preload_furni, false, callback); // preload furniture
		}
		else {
			furni.inv2().complete();
		}
	},
	inv2 : function () {
		// add inventory login data
		var output = new Object();
		output.complete = function() {
			if (!inv.loaded) {
				var len = inv.login_data.length;
				var count = 0;
				
				for(x in inv.login_data) {
					if(count >= len) break;
					else {
						count++;
						furni.add(inv.login_data[x], false);
					}
				}
				inv.login_data = null;
				inv.loaded = true;
				debug.green("Inventory login data parsed.");
				furni.init();
			}
			else {
				furni.init();
			}
		};
		return output;
	},
	getpos : function (furniname, turn, action, tile) {
		var info = this.db[furniname];
		if (info.soort != "poster") {
			var x = room.tiles.posx[tile];
			var y = room.tiles.posy[tile];
			tile = tile.split(/_/);
			x += parseInt(info.change_x);
			y += parseInt(info.change_y);
			if (turn == 1 || turn == 3) {
				y -= Math.floor(((parseInt(info.lang)/2)-0.5)*33);
				y += Math.floor(((parseInt(info.breed)/2)-0.5)*33);
			}
			if (turn == 3 || turn == 4) {
				x += parseInt(info.turn_x);
				y += parseInt(info.turn_y);
			}
			if (action == 1) {
				if (turn == 2) {
					x += parseInt(info.action_x);
				}
				y += parseInt(info.action_y);
			}
		}
		else {
			var x = parseInt(tile.split("|")[0]);
			var y =  parseInt(tile.split("|")[1]);
			if (action == 1) {
				if (turn == 2) {
					x += parseInt(info.action_x);
				}
				y += parseInt(info.action_y);
			}
		}
		return [x,y];
	},
	checkmuur : function (posx, posy, startx, starty, endx, endy, uitvoertype) {
		if (uitvoertype == 1) {
			if (posx >= startx && posx <= endx && posy >= starty && posy <= endy) {
				return true;
			}
			else {
				return false;
			}
		}
		else {
			if (posx >= startx && posx <= endx) {
				return true;
			}
			else {
				return false;
			}
		}
	},
	checkmuren : function (posx, posy, uitvoertype) {
		var to_return = false;
	    var to_return2 = 1;
	    
		var len = room.walls.turn.length;
		for (i = 0; i < len; i++) {
			if (this.checkmuur(posx, posy, room.walls.startx[i], room.walls.starty[i], room.walls.endx[i], room.walls.endy[i], uitvoertype)) {
				to_return = true;
				to_return2 = room.walls.turn[i];
			}
		}
		if (uitvoertype == 1) {
			return to_return;
		}
		else {
			return to_return2;
		}	
	},
	packet : function (data) {
		
		/* preload images first, add to furnimanager etc
		var image = new Image();
image.src = 'image.jpg';
image.onload = function() {
// sometimes called
alert('image loaded');
};
*/
		var furninode = $("furni_"+data.I);
		if (client.mode != "main") {
			var handrefresh = false;
			var pagerefresh = false;
			var newfurni = false;
			if (data.T == "inv") {
				handrefresh = true;
				pagerefresh = true;
				if (!furninode) {
					newfurni = true;
					this.add(data, false);
				}
				else {
					// weghalen uit kamer, in de hand zetten
					furninode.tile = data.T;
					furninode.style.zIndex = parseInt(data.S);
					furninode.tile = data.T;
					furninode.turn = data.H;
					furninode.action = data.A;
					$("tile"+data.T).appendChild(furninode);
					HideContent("furni_"+data.I);
					this.to_icon(data.I, true); // true: force update
					furni.click(""); // item zit nu in hand, podium leegmaken
				}
			}
			else {
				// place in room
				if (!furninode) {
					this.add(data, true); // true: animation
				}
				else {
					// it's in the room already, just move it please!
					if (furninode.tile == "inv") {
						handrefresh = true;
					}
					if (this.db[data.F].soort != "poster") {
						var pos = this.getpos(data.F, data.H, data.A, data.T);
					}
					else {
						var pos = this.getpos(data.F, data.H, data.A, data.T);
						data.T = "0_0";
					}
					furninode.style.left = pos[0];
					furninode.style.top = pos[1];
					furninode.tile = data.T;
					furninode.turn = data.H;
					furninode.action = data.A;
					if (this.db[data.F].soort != "tapijt") {
						furninode.style.zIndex = parseInt(data.S);
						$("tile"+data.T).appendChild(furninode);
					}
					else {
						if (data.T != "inv") {
							furninode.style.zIndex = parseInt(data.T.split(/_/)[0])+parseInt(data.T.split(/_/)[1]);
							$("tile0_0").appendChild(furninode);
						}
						else {
							furninode.style.zIndex = parseInt(data.S);
							$("tile"+data.T).appendChild(furninode);
						}
					}
					this.to_furni(data.I, true); // true: force update
				}
			}
			if (handrefresh) {
				if (pagerefresh) {
					inv.page = Math.ceil($("tileinv").childNodes.length/9);
				}
				if (inv.open) {
					inv.content("close");
					inv.content("open");
				}
				else {
					inv.set("open");
				}
			}
			this.heightmaps();
		}
		else {
			if (data.T == "inv") {
				// gewoon toevoegen aan hand, we zitten in het hoteloverzicht
				this.add(data, false);
			}
		}
	},
	add : function (data, animation) {
		// add a piece of furniture
		
		var data = { // conversions
			id : data.I,
			tile : data.T,
			stacknr : data.S,
			stackheight : data.SH,
			turn : data.H,
			action : data.A,
			furni : data.F
		};
		
		if (isset(this.db[data.furni])) {
			var info = this.db[data.furni];
			var image = "";
			
			var hoekshow = data.turn;
			if (info.soort == "stoel" && info.afb == 1) {
				hoekshow = 1;
			}
			if (data.action == 1) {
				image = "images/furni/"+data.furni+"/"+hoekshow+"b.gif";
			}
			else {
				image = "images/furni/"+data.furni+"/"+hoekshow+".gif";
			}
			if (data.tile == "inv") {
				image = "images/furni/"+data.furni+"/icon.gif";
				var pos = [0,0];
			}
			else {
				if (info.soort != "poster") {
					var pos = this.getpos(data.furni, data.turn, data.action, data.tile);
				}
				else {
					var pos = this.getpos(data.furni, data.turn, data.action, data.tile);
					data.tile = "0_0";
				}
			}
			
			var obj = document.createElement("img");
			obj.id = "furni_"+data.id;
			obj.db = data.id;
			obj.tile = data.tile;
			obj.stacknr = data.stacknr;
			obj.stackheight = data.stackheight;
			obj.turn = data.turn;
			obj.action = data.action;
			obj.furni = data.furni;
			obj.style.left = pos[0];
			obj.style.top = pos[1];
			obj.src = image;
			obj.style.position = "absolute";
			obj.className = "item_solid";
			if (data.tile == "inv") {
				obj.imgstate = false; // true = furniture / false = icon
				obj.style.display = "none";
			}
			else {
				obj.imgstate = true; // true = furniture / false = icon
			}
			if (animation) {
				obj.style.display = "none";
			}
			addEvent(obj, 'mousedown', function() { furni.click(data.id); });
			if (info.action == 1) {
		    	addEvent(obj, 'dblclick', function() { furni.action(data.id); });
			}
			//$("tile"+data.tile).insertBefore(obj, $("tile"+data.tile).firstChild);
			if (info.soort != "tapijt") {
				obj.style.zIndex = parseInt(data.stacknr);
				$("tile"+data.tile).appendChild(obj);
			}
			else {
				if (data.tile != "inv") {
					obj.style.zIndex = parseInt(data.tile.split(/_/)[0])+parseInt(data.tile.split(/_/)[1]);
					$("tile0_0").appendChild(obj);
				}
				else {
					obj.style.zIndex = parseInt(data.stacknr);
					$("tile"+data.tile).appendChild(obj);
				}
			}
			if (animation) {
				animate.drop.start("furni_"+data.id, pos[0], pos[1]);
				ShowContent("furni_"+data.id);
			}
		}
		else {
			debug.error("Couldn't load furniture: '"+data.furni+"'");
		}
	},
	remove : function (id) {
		var furninode = $("furni_"+id);
		if (furninode) {
			furninode.parentNode.removeChild(furninode);
		}
		if (this.obj != null) {
			if (this.obj.db == id) {
				removechilds("dragfurni");
				this.obj = null;
				this.dragimg = null;
				this.click("");
			}
		}
	},
	click : function (id) {
		if (this.obj == null) {
			avatars.clicked = 0;
			if (id != "") {
				var furni = $("furni_"+id);
				var info = this.db[furni.furni];
				var info_show = true;
				
				if (furni.tile == "inv") {
					if (info.soort == "camera") {
						camera.camid = id;
						set_top("askcamera");
						ShowContent("askcamera");
						object_center("askcamera");
					}
					else {
						this.dragstart(id);
					}
					info_show = false;
				}
				else {
					if (ctrl_status) {
						if (info.soort != "poster") {
							this.turn(id);
							info_show = false;
						}
					}
					if (alt_status) {
						if (info.soort != "poster") {
							this.dragstart(id);
							info_show = false;
						}
					}
				}
				if (info_show) {
					if (furni.furni == "picture") {
						camera.showpicture(id);
						
					}
					ShowContent("infoshade");
					$('infotext').innerHTML = info.name;
					$('infodesc').innerHTML = info.descr;
					$('infoicon').src = "images/furni/"+furni.furni+"/icon.gif";
					$('infopop').style.backgroundImage = "url('images/blank.gif')";
					$('infobadge').src = 'images/blank.gif';
					$('buttons').innerHTML = "";
					if (room.data.rights) {
						if (info.soort != "poster") {
							$('buttons').innerHTML += '<div onclick="furni.dragstart(\''+id+'\')" onmousedown="room.button_down(this, \'move\')" onmouseup="room.button_up(this, \'move\')" style="width: 84px; background-image:url(\'images/rooms/move.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
							if (this.turncheck(id)) {
								$('buttons').innerHTML += '<div onclick="furni.turn(\''+id+'\')" onmousedown="room.button_down(this, \'turn\')" onmouseup="room.button_up(this, \'turn\')" style="width: 69px; background-image:url(\'images/rooms/turn.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
							}
						}
						if (room.data.owner == user.id || in_array("always_pickup", user.rights)) {
							$('buttons').innerHTML += '<div onclick="furni.pickup(\''+id+'\')" onmousedown="room.button_down(this, \'pickup\')" onmouseup="room.button_up(this, \'pickup\')" style="width: 66px; background-image:url(\'images/rooms/pickup.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
						}
						if (in_array("config_furni", user.rights)) {
							$('buttons').innerHTML += '<div onclick="room.furnitools('+id+')" onmousedown="room.button_down(this, \'config\')" onmouseup="room.button_up(this, \'config\')" style="width: 64px; background-image:url(\'images/rooms/config.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
						}
					}
				}
			}
			else {
				HideContent("infoshade");
				$('infotext').innerHTML = "";
				$('infodesc').innerHTML = "";
				$('infoicon').src = 'images/blank.gif';
				$('infopop').style.backgroundImage = "url('images/blank.gif')";
				$('infobadge').src = 'images/blank.gif';
				$('buttons').innerHTML = "";
			}
		}
	},
	dragstart : function (id) {
		if (room.data.rights && client.mode == "room") {
			this.obj = $("furni_"+id);
			this.put_down = false;
			if (this.obj.tile != "inv") {
				this.obj.className = "item_trans";
			}
			else {
				HideContent(this.obj.id);
			}
			var dragimg = document.createElement("img");
			dragimg.id = "dragimg";
			dragimg.src = this.obj.src;
			dragimg.style.position = "absolute";
			dragimg.style.left = this.obj.style.left;
			dragimg.style.top = this.obj.style.top;
			dragimg.style.zIndex = ((get_top())*1)+1;
			dragimg.style.className = "item_trans2";
			dragimg.imgstate = this.obj.imgstate;
			removechilds("dragfurni");
			$("dragfurni").appendChild(dragimg);
			this.dragimg = $("dragimg");
			this.drop = false;
			setTimeout(function(){furni.put_down = true;},100);
		}
		return false;
	},
	move : function (posx, posy) {
		if (this.obj != null && client.mode == "room") {
			// move furniture!
			var info = this.db[this.obj.furni];
			var changepos = true;
			var imgstate = this.dragimg.imgstate; // to determine whether the object should become an icon or furni image
			if (inv.open) {
				if (posx <= client_width && posy <= 162 && posx >= client_width-267 && posy >= 0) {
					var handstatus = true;
					imgstate = false;
					this.dragimg.className = 'item_solid';
				}
				else {
					var handstatus = false;
					imgstate = true;
				}
			}
			else {
				var handstatus = false;
				imgstate = true;
			}
			var force = false;
			var dragwidth = this.dragimg.width;
			var dragheight = this.dragimg.height;
			var dragx = Math.ceil(dragwidth/2);
			var dragy = Math.ceil(dragheight/2);
			var max_x = client_width - dragwidth;
			var max_y = client_height - dragheight;
			var x = (posx - dragx);
			var y = (posy - dragy);
			if (x < 0) {
				x = 0;
			}
			if (y < 0) {
				y = 0;
			}
			if (x > max_x) {
				x = max_x;
			}
			if (y > max_y) {
				y = max_y;
			}
			this.drop = true;
			if (!handstatus) { // it's not in the inventory, let's try to put in on a tile!
				var tile = room.tiles.check(posx, posy, true);
				if (info.soort == "poster") {
					// poster!
					imgstate = true;
					var turn = 1;
					if (this.checkmuren(posx, posy, 2) == 1) {
						// rotatie 1
						var check1 = this.checkmuren(x, y+(dragwidth/4), 1);
						var check2 = this.checkmuren((x+dragwidth), y, 1);
						var check3 = this.checkmuren(x, (y+dragheight), 1);
						var check4 = this.checkmuren((x+dragwidth), ((y+dragheight)-(dragwidth/4)), 1);
						var check2_1 = this.checkmuren(x, y+(dragwidth/4), 2);
						var check2_2 = this.checkmuren((x+dragwidth), y, 2);
						var check2_3 = this.checkmuren(x, (y+dragheight), 2);
						var check2_4 = this.checkmuren((x+dragwidth), ((y+dragheight)-(dragwidth/4)), 2);
						turn = 1;
					}
					else {
						// rotatie 2
						var check1 = this.checkmuren(x, y, 1);
						var check2 = this.checkmuren((x+dragwidth), y+(dragwidth/4), 1);
						var check3 = this.checkmuren(x, ((y+dragheight)-(dragwidth/4)), 1);
						var check4 = this.checkmuren((x+dragwidth), (y+dragheight), 1);
						var check2_1 = this.checkmuren(x, y, 2);
						var check2_2 = this.checkmuren((x+dragwidth), y+(dragwidth/4), 2);
						var check2_3 = this.checkmuren(x, ((y+dragheight)-(dragwidth/4)), 2);
						var check2_4 = this.checkmuren((x+dragwidth), (y+dragheight), 2);
						turn = 2;
					}
					
					if (check1 && check2 && check3 && check4 && check2_1 == check2_2 && check2_1 == check2_3 && check2_1 == check2_4) {
						changepos = true;
						this.dragimg.style.zIndex = 3;
						this.obj.turn = turn;
						this.drop = true;
						this.dragimg.className = 'item_solid';
						force = true;
					}
					else
					{
						this.drop = false;
						this.dragimg.className = 'item_trans';
						changepos = true;
						set_top(this.dragimg.id);
					}
				}
				else {
					if (tile[0]) { // is it on a tile?
						// furniture might be multiple tiles big, let's check
						var multiple_check = this.multiple_check(this.obj, tile[1], tile[2], this.obj.turn, info.lang, info.breed);
						if (multiple_check[1] == 0) {
							// none are marked 'used', let's check for stackable and free tiles
							if(multiple_check[2] != 0) {
								// some tiles are stackable, whilst all other are free -> stacking time!
								var multi_check = "stack";
							}
							else {
								// all tiles are free -> normal placement
								var multi_check = "free";
							}
						}
						else {
							// some are marked 'used', furniture cannot be moved
							var multi_check = "used";
						}
						//debug.add(multiple_check[0]+" "+multiple_check[1]+" "+multiple_check[2]+" "+multi_check);
						if (multi_check == "free") {
							this.drop = true;
							imgstate = true;
							this.dragimg.className = 'item_solid';
							var pos = this.getpos(this.obj.furni, this.obj.turn, this.obj.action, tile[1]+"_"+tile[2]);
							x = pos[0];
							y = pos[1];
						}
						else if (multi_check == "stack") {
							this.drop = false;
							changepos = true;
							imgstate = false;
						}
						else {
							// furniture cannot be placed
							this.drop = false;
							changepos = true;
							imgstate = false;
						}
					}
					else {
						// furniture is not on a tile
						this.drop = false;
						set_top(this.dragimg.id);
						imgstate = false;
						this.dragimg.className = 'item_trans2';
					}
				}
			}
			if (handstatus  && inv.open) {
				// make sure one can drop in inventory
				this.drop = true;
			}
			if (imgstate) {
				if (info.soort == "poster") {
					this.to_furni(this.obj.db, force);
				}
				else {
					this.to_furni(this.obj.db, false);
				}
			}
			else {
				this.to_icon(this.obj.db, false);
			}
			if (changepos) {
				this.dragimg.style.left = x;
				this.dragimg.style.top = y;
				if (info.soort == "poster") {
					if (handstatus) {
						set_top(this.dragimg.id);
					}
				}
			}
			return false;
		}
	},
	dragstop : function (pos, is_tile, droptile) {
		if (this.obj != null && this.drop && this.put_down && client.mode == "room") {
			if (this.db[this.obj.furni].soort != "poster") {
				var do_drop = false;
				if (is_tile) {
					sendtile = droptile[0]+"_"+droptile[1];
					do_drop = true;
				}
				else if (inv.open && pos[0] <= client_width && pos[1] <= 162 && pos[0] >= client_width-267 && pos[1] >= 0) {
					do_drop = true;
					sendtile = "inv";
				}
				if (do_drop) {
					removechilds("dragfurni");
					if (this.obj.tile != "inv") {
						this.obj.className = 'item_solid';
					}
					else {
						HideContent(this.obj.id);
						inv.content("open");
					}
					// check of het niet gewoon in de inventory blijft staan
					if (!(sendtile == "inv" && this.obj.tile == "inv")) {
						// check eerst nog voor doneren
						if (this.obj.tile == "inv" && sendtile != "inv" && room.data.owner != user.id) {
							// doneren!
							if (in_array("always_pickup", user.rights)) {
								packets.send.furni_move(this.obj.db, sendtile);
							}
							else {
								this.donate.start(this.obj.db, sendtile);
							}
						}
						else {
							packets.send.furni_move(this.obj.db, sendtile);
						}
					}
					this.obj = null;
					this.dragimg = null;
				}
			}
			else {
				// poster, andere soort cordinaten doorgeven
				removechilds("dragfurni");
				HideContent(this.obj.id);
				inv.content("open");
				if (!(inv.open && pos[0] <= client_width && pos[1] <= 162 && pos[0] >= client_width-267 && pos[1] >= 0)) {
					if (room.data.owner != user.id) {
						// doneren!
						if (in_array("always_pickup", user.rights)) {
							packets.send.furni_move(this.obj.db, parseInt(this.dragimg.style.left)+"|"+parseInt(this.dragimg.style.top)+"@"+this.obj.turn);
						}
						else {
							this.donate.start(this.obj.db, parseInt(this.dragimg.style.left)+"|"+parseInt(this.dragimg.style.top)+"@"+this.obj.turn);
						}
					}
					else {
						packets.send.furni_move(this.obj.db, parseInt(this.dragimg.style.left)+"|"+parseInt(this.dragimg.style.top)+"@"+this.obj.turn);
					}
				}
				this.obj = null;
				this.dragimg = null;
			}
		}
	},
	donate : {
		id : null,
		tile : null,
		start: function (id, tile) {
			this.id = id;
			this.tile = tile;
			set_top("donate_layer");
			ShowContent("donate_layer");
		},
		proceed : function () {
			packets.send.furni_move(this.id, this.tile);
			this.id = null;
			this.tile = null;
			HideContent("donate_layer");
		},
		stop : function () {
			this.id = null;
			this.tile = null;
			HideContent("donate_layer");
		}
	},
	to_furni : function (id, update) {
		var furni = $("furni_"+id);
		var dragimg = $("dragimg");
		var info = this.db[furni.furni];
		var h = furni.turn;
		if (info.soort == "stoel" && info.afb == 1) {
			h = 1;
		}
		var extra = "";
		if (furni.action == 1) {
			extra = "b";
		}
		var img = "images/furni/"+furni.furni+"/"+h+extra+".gif";
		if (update) {
			if (dragimg != null) {
				dragimg.imgstate = true;
				dragimg.src = img;
			}
			else {
				furni.imgstate = true;
				furni.src = img;
			}
		}
		else {
			if (!dragimg.imgstate) {
				dragimg.imgstate = true;
				dragimg.src = img;
			}
		}
	},
	to_icon : function (id, update) {
		var furni = $("furni_"+id);
		var dragimg = $("dragimg");
		var img = "images/furni/"+furni.furni+"/icon.gif";
		if (update) {
			furni.imgstate = false;
			furni.src = img;
		}
		else {
			if (dragimg.imgstate) {
				dragimg.imgstate = false;
				dragimg.src = img;
				
			}
		}
	},
	turncheck : function (id) {
		// requiring rewrite
		var furni = $("furni_"+id);
		var info = this.db[furni.furni];
		
		var hoeken = info.afb;
		var doen = true;
		if (hoeken == 1) {
			doen = false;
		}
		if (info.soort == "stoel" && hoeken == 1) {
			hoeken = 4;
			doen = true;
		}
		if (furni.h == hoeken) {
			var check_draai = 1;
		}
		else {
			var check_draai = (furni.h*1)+1;
		}
		/*var check_tegels_array = maak_tegel_array(furni.tile, check_draai, info.lang, info.breed);
		var aantal_vrij = aantal_bezet = aantal_huidig = 0;
		for (i = 0; i < check_tegels_array.length; i++) // LENGTH APART, SPEED!
		{
			var this_check = vrijtegel(check_tegels_array[i], furni_nr, furni.tile);
			if (this_check == "vrij")
			{
				aantal_vrij++;
			}
			if (this_check == "bezet")
			{
				aantal_bezet++;
			}
			if (this_check == "huidig")
			{
				aantal_huidig++;
			}
		}
		if (aantal_bezet == 0)
		{
			if ((aantal_vrij+aantal_huidig) == check_tegels_array.length)
			{
				resultaat = "ja";
			}
			else
			{
				resultaat = "nee";
			}
		}
		else
		{
			resultaat = "nee";
		}
		if (resultaat == "nee")
		{
			doen = false;
		}*/
		if (furni.tile == "inv") {
			doen = false;
		}
		if (furni.soort == "poster") {
			doen = false;
		}
		return doen;
	},
	multiple_check : function (furni, tile1, tile2, h, lang, breed) {
		// function used for furniture that has a length of breath larger than 1, to see whether the other tiles are free
		var info = this.db[furni.furni];
	    var furnitiles = new Array();
		if (furni.tile != "inv") {
			var tile = furni.tile.split(/_/);
	        var furnilengte = 0;
	        var furnibreedte = 0;
	        switch (furni.turn) {
	            case 1:
	                furnilengte = info.lang;
	                furnibreedte = info.breed;
	                break;
	            case 2:
	                furnilengte = info.breed;
	                furnibreedte = info.lang;
	                break;
	            case 3:
	                furnilengte = info.lang;
	                furnibreedte = info.breed;
	                break;
	            case 4:
	                furnilengte = info.breed;
	                furnibreedte = info.lang;
	                break;
	        }
	        for (i1 = 0; i1 < furnilengte; i1++) {
				for (i2 = 0; i2 < furnibreedte; i2++) {
					furnitiles.push((parseInt(tile[0])-i1)+"_"+(parseInt(tile[1])+i2));
				}
			}
		}
		var free = 0;
		var used = 0;
		var stack = 0;
		switch(h) {
		case 1:
			lengte = lang;
			breedte = breed;
			break;
		case 2:
			lengte = breed;
			breedte = lang;
		break;
		case 3:
			lengte = lang;
			breedte = breed;
		break;
		case 4:
			lengte = breed;
			breedte = lang;
		break;
		}
		for (i1 = 0; i1 < lengte; i1++) {
			for (i2 = 0; i2 < breedte; i2++) {
				var tile = (tile1-i1)+"_"+(tile2+i2);
				if (in_array(tile, furnitiles)) {
					 // this is one of the current tiles!
					free++;
				}
				else {
					if (isset(this.stack_map[tile])) {
						if (info.soort != "tapijt") {
							switch(this.tile_map[tile]) {
							case 0: // free
								free++;
							break;
							case 1: // used
								used++;
							break;
							case 2: // stackable
								stack++;
							break;
							case 3: // chair
								used++; // chairs are never stackable anyway
							break;
							}
						}
						else {
							if (!in_array(tile, room.data.holes)) {
								free++;
							}
							else {
								used++;
							}
						}
					}
					else {
						used++; // tile does not even exist, make sure furniture is not placed!!
					}
				}
			}
		}
		return new Array(free, used, stack);
	},
	heightmaps : function () {
		/*
		tile_map options:
			0: free
			1: used
			2: stackable
			3: chair
		*/
		this.stack_map = new Array(); // this is in fact just to retrieve the highest furniture in a nice array for all tiles
		this.tile_map = new Array(); // data of all the furniture that is stacked highest, used for furniture placing and avatar walking
		// first create neccesary arrays
		for (i1 = 1; i1 <= room.data.lang; i1++) {
			for (i2 = 1; i2 <= room.data.breed; i2++) {
				var tile = i2+"_"+i1;
				this.stack_map[tile] = new Array();
				this.tile_map[tile] = 0; // free tile
			}
		}
		// then put info in them
		for (i1 = 1; i1 <= room.data.lang; i1++) {
			for (i2 = 1; i2 <= room.data.breed; i2++) {
				var tile = i2+"_"+i1;
				var node = $("tile"+tile);
				for(var i = 0; i < node.childNodes.length; i++) {
					if (node.childNodes[i]) {
						var furni = node.childNodes[i];
						if (furni.furni == "avatar") {
							this.stack_map[(i2)+"_"+(i1)][1] = "avatar";
						}
						else {
							var info = this.db[furni.furni];
							switch(furni.turn) {
							case 1:
								var lengte = info.lang;
								var breedte = info.breed;
								break;
							case 2:
								var lengte = info.breed;
								var breedte = info.lang;
								break;
							case 3:
								var lengte = info.lang;
								var breedte = info.breed;
								break;
							case 4:
								var lengte = info.breed;
								var breedte = info.lang;
								break;
							}
							for (i3 = 0; i3 < lengte; i3++) {
								for (i4 = 0; i4 < breedte; i4++) {
									this.stack_map[(i2-i3)+"_"+(i1+i4)][furni.stacknr] = furni.db;
								}
							}
						}
					}
				}
			}
		}
		// generate tile_map, now that we can see which items are stacked highest
		for (i1 = 1; i1 <= room.data.lang; i1++) {
			for (i2 = 1; i2 <= room.data.breed; i2++) {
				var tile = i2+"_"+i1;
				var this_stack_map = this.stack_map[tile];
				var amount = this_stack_map.length;
				if (amount != 0) {
					amount -= 1;
					var id = this_stack_map[amount];
					if (id == "avatar") {
						this.tile_map[tile] = 1;
					}
					else {
						var furni = $("furni_"+id);
						var info = this.db[furni.furni];
						var result = 1; // normal furniture
						if (info.stacking == "yes") {
							result = 2; // stackable furniture
						}
						else if (info.soort == "stoel") {
							// why use else? -> chair is never stackable
							result = 3; //chair
						}
						this.tile_map[tile] = result; // set tile_map
					}
				}
			}
		}
		// oh, and don't place furniture in holes!
		var len = room.data.holes.length;
		for (i = 0; i < len; i++) {
			this.tile_map[room.data.holes[i]] = 1;
		}
	},
	turn : function (id) {
		// turn furniture
		packets.send.furni_turn(id);
	},
	action : function (id) {
		if (!(ctrl_status || alt_status)) {
			// turn furnture on/off if possible
			packets.send.furni_action(id);
		}
	},
	pickup : function (id) {
		//pick up furni
		packets.send.furni_pickup(id);
		furni.click("");
	}
};
var avatars = {
	pictures : new Array(),
	avapos : null,
	clicked : 0,
	init : function () {
		this.avapos = new Array();
		var direction = new Array(1,2,3,4,5,6,7,8);
		var count = 0;
		var len = direction.length;
		for (i = 0; i < len; i++) {
			this.avapos[direction[i]] = new Array();
			this.avapos[direction[i]][0] = count; //std
			count += 64;
			this.avapos[direction[i]][1] = count; //wlk
			count += 64;
		}
		for (i = 1; i <= 4; i++) { // zitten
			this.avapos[(i*2)][2] = count;
			count += 64;
		}
		
		debug.yellow("Avatar sprite positions calculated");
	},
	url : function (figure, data) {
		return "avatar.php?figure="+figure+data;
	},
	add : function (id, name, mission, clothes, badge, tile, hoek, drink, brb, sit, friend, rights) {
		if (!$("ava_"+id)) {
			var tile1 = (tile.split(/_/)[0])*1;
			var tile2 = (tile.split(/_/)[1])*1;
			if (room.data.mode == "s") {
				var ava_x = room.tiles.posx[tile]-2;
				var ava_y = room.tiles.posy[tile]-65;
			}
			if (room.data.mode == "b") {
				var ava_x = room.tiles.posx[tile]+4;
				var ava_y = room.tiles.posy[tile]-85;
			}
			var newava = document.createElement("div");
			newava.id = "ava_"+id;
			newava.name = name;
			newava.mission = mission;
			newava.tile = [tile1, tile2];
			newava.clothes = clothes;
			newava.badge = badge;
			newava.drink = drink;
			newava.brb = brb;
			newava.sit = sit;
			newava.hoek = hoek;
			newava.friend = friend;
			newava.rights = rights;
			newava.moving = false;
			newava.ani = false;
			newava.step = 1;
			newava.next = new Array();
			newava.end = false;
			newava.furni = "avatar"; // for heightmaps
			addEvent(newava, 'click', function() { avatars.click(id); });
			addEvent(newava, 'mouseover', function() { avatars.tagin(id); });
			addEvent(newava, 'mouseout', function() { avatars.tagout(id); });
			newava.style.position = "absolute";
			newava.style.left = ava_x;
			newava.style.top = ava_y;
			newava.style.width = "64px";
			newava.style.height = "110px";
			newava.style.zIndex = 2; // VOORLOPIG, ZAL NIET WERKEN MET STAPELEN! (om boven deurtjes te komen)
			newava.className = "item_trans2";
			newava.style.backgroundImage = "url('images/ghost.gif')";
			
			if (sit == "") {
				newava.style.zIndex = 2; // VOORLOPIG, ZAL NIET WERKEN MET STAPELEN! (om boven deurtjes te komen)
				$("tile"+tile).appendChild(newava);
			}
			else {
				newava.style.zIndex = tile1+tile2;
				$("tile"+sit.split("|")[1]).appendChild(newava);
			}
			
			if (drink == "camera") {
				this.loadpicture("avatar3.php?figure="+clothes+"&brb="+brb, id);
			}
			else {
				this.loadpicture("avatar2.php?figure="+clothes+"&brb="+brb, id);
			}
				
			var image = document.createElement("img");
			image.src = "images/nametag.php?t="+ucfirst($("ava_"+id).name);
			image.id = "tag_"+id;
			image.user = id;
			image.style.position = "absolute";
			image.style.display = "none";
			$("nametags").appendChild(image);
			set_top("tag_"+id);
			
			furni.heightmaps();
			
			if (sit == "") {
				this.set(id, newava.hoek, 0);
			}
			else {
				this.set(id, newava.hoek, 2);
			}
			ShowContent("ava_"+id);
		}
	},
	loadpicture : function (url, id) {
		if (!avatars.pictures[url]) {
			avatars.pictures[url] = document.createElement("img");
			avatars.pictures[url].src = url;
			avatars.pictures[url].loaded = false;
			avatars.pictureloaded(url, id);
		}
		else {
			if (avatars.pictures[url].loaded) {
				$("ava_"+id).style.backgroundImage = "url('"+url+"')";
				if ($("ava_"+id).brb == 0) {
					$("ava_"+id).className = "item_solid";
				}
				else {
					$("ava_"+id).className = "item_trans2";
				}
			}
			else {
				avatars.pictureloaded(url, id);
			}
		}	
	},
	pictureloaded : function (url, id) {
		if (avatars.pictures[url].complete) {
			avatars.pictures[url].loaded = true;
			$("ava_"+id).style.backgroundImage = "url('"+url+"')";
			if ($("ava_"+id).brb == 0) {
				$("ava_"+id).className = "item_solid";
			}
			else {
				$("ava_"+id).className = "item_trans2";
			}
		}
		else {
			setTimeout(function(){avatars.pictureloaded(url, id);},10);
		}
	},
	givedrink : function (id, drink) {
		var avatar = $("ava_"+id);
		if (avatar) {
			avatar.drink = drink;
			if (drink == "camera") {
				this.loadpicture("avatar3.php?figure="+avatar.clothes+"&brb="+avatar.brb, id);
			}
			else {
				this.loadpicture("avatar2.php?figure="+avatar.clothes+"&brb="+avatar.brb, id);
			}
		}
	},
	updatebrb : function (id, brb) {
		var avatar = $("ava_"+id);
		if (avatar) {
			avatar.brb = brb;
			if (avatar.drink == "camera") {
				this.loadpicture("avatar3.php?figure="+avatar.clothes+"&brb="+avatar.brb, id);
			}
			else {
				this.loadpicture("avatar2.php?figure="+avatar.clothes+"&brb="+avatar.brb, id);
			}
			if (avatar.brb == 0) {
				$("ava_"+id).className = "item_solid";
			}
			else {
				$("ava_"+id).className = "item_trans2";
			}
		}
	},
	del : function (id) {
		var avatar = $("ava_"+id);
		if (avatar) {
			avatar.parentNode.removeChild(avatar);
			furni.heightmaps();
		}
		if (this.clicked == id) {
			furni.click("");
		}
	},
	click : function(id) {
		var data = $("ava_"+id);
		if (data) {
			this.clicked = id;
			ShowContent("infoshade");
			$('infotext').innerHTML = ucfirst(data.name);
			$('infodesc').innerHTML = data.mission;
			$('infopop').style.backgroundImage = "url('avatar2.php?figure="+data.clothes+"&brb=0')";
			if (data.badge != 0) {
				$('infobadge').src = "images/badges/"+data.badge+".gif";
			}
			else {
				$('infobadge').src = "images/blank.gif";
			}
			$('infoicon').src = "images/blank.gif";
			$('buttons').innerHTML = "";
			if (id > 0) {
				// normale avatar
				if (id != user.id) {
					if (id != room.data.owner) {
						// rechten
						if (data.rights) {
							$('buttons').innerHTML += '<div onclick="avatars.takerights('+id+')" onmousedown="room.button_down(this, \'rights2\')" onmouseup="room.button_up(this, \'rights2\')" style="width: 118px; background-image:url(\'images/rooms/rights2.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
						}
						else {
							$('buttons').innerHTML += '<div onclick="avatars.giverights('+id+')" onmousedown="room.button_down(this, \'rights1\')" onmouseup="room.button_up(this, \'rights1\')" style="width: 89px; background-image:url(\'images/rooms/rights1.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
						}
						
					}
					// vriend gedoe
				}
				else {
					// dansen, zwaaien
				}
			}
			else if (id < 0) { 
				// een bot
				if (in_array("kick_bot", user.rights)) {
					$('buttons').innerHTML += '<div onclick="avatars.kickbot('+id+')" onmousedown="room.button_down(this, \'kick\')" onmouseup="room.button_up(this, \'kick\')" style="width: 49px; background-image:url(\'images/rooms/kick.gif\'); height: 17px; z-index:3; background-position: 0px -0px; background-repeat:no-repeat; float: right;"></div>';
				}
			}
		}

	},
	tagin : function(id) {
		if($("ava_"+id)) {
			if ($("tag_"+id)) {
				if (!$("ava_"+id).moving && furni.obj == null) {
					
					var middle_x = parseInt($("ava_"+id).style.left);
					middle_x += ($("ava_"+id).clientWidth)/2;
					middle_x -= ($("tag_"+id).width)/2;
					
					$("tag_"+id).style.left = middle_x;
					$("tag_"+id).style.top = parseInt($("ava_"+id).style.top)-15;
					
					set_trans("tag_"+id, 0);
					ShowContent("tag_"+id);
					Effect.Fade("tag_"+id, { duration: 0.2, from: 0, to: 1 });
				}
			}
		}
		else {
			debug.error("Avatar "+id+" not loaded");
		}
	},
	tagout : function(id) {
		if($("ava_"+id)) {
			if ($("tag_"+id)) {
				Effect.Fade("tag_"+id, { duration: 0.2, from: 1, to: 0 });
			}
		}
		else {
			debug.error("Avatar "+id+" not loaded");
		}
	},
	set : function (id, hoek, action) {
		if ($("ava_"+id)) {
			$("ava_"+id).style.backgroundPosition = "-"+this.avapos[hoek][action]+"px 0px";
		}
		
	},
	move : function (id, x, y, hoek) {
		if($("ava_"+id)) {
			HideContent("tag_"+id);
			if ($("ava_"+id).moving) {
				$("ava_"+id).next.push([x, y, hoek]);
			}
			else {
				var data = $("ava_"+id);
				
				var tile = x+"_"+y;
				if (room.data.mode == "s") {
					var ava_x = room.tiles.posx[tile]-2;
					var ava_y = room.tiles.posy[tile]-65;
				}
				if (room.data.mode == "b") {
					var ava_x = room.tiles.posx[tile]+4;
					var ava_y = room.tiles.posy[tile]-85;
				}
				$("ava_"+id).moving = true;
				
				$("ava_"+id).tile = [x, y];
				$("ava_"+id).hoek = hoek;
				$("tile"+tile).appendChild($("ava_"+id));
				
				if (!data.ani) {
					$("ava_"+id).ani = true;
					$("ava_"+id).step = 1;
					this.animate(id);
				}
				
				var movecallback = avatars.moved(id);
				movelayer('ava_'+id, ava_x, ava_y, 25, 20, 1, movecallback);
				
				//set_grid(); -> not only grid, but also heightmap for furni, right?
			}
		}
		else {
			debug.error("Avatar "+id+" not loaded");
		}
	},
	stopmove : function (id) {
		if($("ava_"+id)) {
			if ($("ava_"+id).moving) {
				$("ava_"+id).end = true;
				$("ava_"+id).sit = "";
			}
			else {
				$("ava_"+id).ani = false;
				$("ava_"+id).end = false;
				$("ava_"+id).sit = "";
				avatars.set(id, $("ava_"+id).hoek, 0);
			}
			furni.heightmaps();
		}
	},
	stopmovesit : function (id, sit) {
		if($("ava_"+id)) {
			if ($("ava_"+id).moving) {
				$("ava_"+id).end = true;
				$("ava_"+id).sit = sit;
			}
			else {
				$("ava_"+id).ani = false;
				$("ava_"+id).end = false;
				$("ava_"+id).sit = sit;
				$("ava_"+id).hoek = parseInt($("ava_"+id).sit.split("|")[0]);
				avatars.set(id, $("ava_"+id).hoek, 2);
				$("ava_"+id).style.zIndex = $("ava_"+id).tile[0]+$("ava_"+id).tile[1];
				$("tile"+$("ava_"+id).sit.split("|")[1]).appendChild($("ava_"+id));
			}
			furni.heightmaps();
		}
	},
	moved : function (id) {
		output = {
			complete : function () {
				if($("ava_"+id)) {
					$("ava_"+id).moving = false;
					if ($("ava_"+id).next.length > 0) {
						var nextmove = $("ava_"+id).next.shift();
						avatars.move(id, nextmove[0], nextmove[1], nextmove[2]);
					}
					else {
						if ($("ava_"+id).end) {
							$("ava_"+id).ani = false;
							$("ava_"+id).end = false;
							if ($("ava_"+id).sit == "") {
								$("ava_"+id).style.zIndex = 2;
								avatars.set(id, $("ava_"+id).hoek, 0);
							}
							else {
								$("ava_"+id).hoek = parseInt($("ava_"+id).sit.split("|")[0]);
								avatars.set(id, $("ava_"+id).hoek, 2);
								$("ava_"+id).style.zIndex = $("ava_"+id).tile[0]+$("ava_"+id).tile[1];
								$("tile"+$("ava_"+id).sit.split("|")[1]).appendChild($("ava_"+id));
							}
						}
					}
					furni.heightmaps();
				}
			}
		};
		return output;
	},
	animate : function (id) {
		if($("ava_"+id)) {
			if ($("ava_"+id).ani) {
				this.set(id, $("ava_"+id).hoek, $("ava_"+id).step);
				if ($("ava_"+id).step == 1) {
					$("ava_"+id).step = 0;
				}
				else {
					$("ava_"+id).step = 1;
				}
				var animateavatars = setTimeout(function(){avatars.animate(id);}, 150);
			}
		}
	},
	kickbot : function (id) {
		if($("ava_"+id)) {
			var bot_id = id - id*2;
			packets.send.botkick(bot_id);
			furni.click("");
		}
	},
	giverights : function (id) {
		if($("ava_"+id)) {
			packets.send.giverights(id);
			furni.click("");
		}
	},
	takerights : function (id) {
		if($("ava_"+id)) {
			packets.send.takerights(id);
			furni.click("");
		}
	},
	updaterights : function (id, rights) {
		if($("ava_"+id)) {
			$("ava_"+id).rights = rights;
			if (id == user.id) {
				// jouw rechten!
				room.data.rights = rights;
			}
		}
	},
	updatesit : function (id, sit) {
		if($("ava_"+id)) {
			$("ava_"+id).sit = sit;
			if (sit == "") {
				avatars.set(id, $("ava_"+id).hoek, 0);
				var tile = $("ava_"+id).tile;
				$("ava_"+id).style.zIndex = 2;
				$("tile"+tile[0]+"_"+tile[1]).appendChild($("ava_"+id));
			}
			else {
				$("ava_"+id).hoek = parseInt($("ava_"+id).sit.split("|")[0]);
				avatars.set(id, $("ava_"+id).hoek, 2);
				$("ava_"+id).style.zIndex = $("ava_"+id).tile[0]+$("ava_"+id).tile[1];
				$("tile"+$("ava_"+id).sit.split("|")[1]).appendChild($("ava_"+id));
			}
		}
	}
};
var inv = {
	loaded : false,
	icon_posx : new Array(137,157,177,113,133,153,89,109,129),
	icon_posy : new Array(26,42,58,42,58,74,58,74,90),
	login_data : null, // array of furni that is sent upon server login, and is to be parsed only after PHP login
	page : 1,
	pages : 1,
	open : false,
	back_able : false,
	next_able : false,
	clear : function () {
		// function normally never to be used
		removechilds("tileinv"); // remove all furniture placed in inventory
		this.open = false; // set status to closed
	},
	prepare : function () {
		$('inventory').style.x = 730;
		$('inventory').style.y = 0;
		this.content('close');
		this.open = false; // set status to closed
	},
	opened : function () {
		output = {
			complete : function () {
				$("inventory").style.backgroundPosition = "0px 0px";
				ShowContent("inventory_back");
				ShowContent("inventory_next");
				ShowContent("inventory_close");
				inv.content('open');
			}
		};
		return output;
	},
	set : function (way) {
		switch(way) {
		case "reverse":
			if (this.open) {
				this.set("close");
			}
			else {
				this.set("open");
			}
			break;
		case "open":
			$("inventory").style.backgroundPosition = "-267px 0px";
			var movecallback = inv.opened();
			movelayer("inventory", client_width-267, 0, 10, 20, 0.5, movecallback);
			this.open = true;
			break;
		case "close":
			this.content('close');
			HideContent("inventory_back");
			HideContent("inventory_next");
			HideContent("inventory_close");
			$("inventory").style.backgroundPosition = "-267px 0px";
			movelayer("inventory", client_width, -162, 20, 20, 0.5, false);
			this.open = false;
			break;
		case "closeforce":
			this.content('close');
			HideContent("inventory_back");
			HideContent("inventory_next");
			HideContent("inventory_close");
			$("inventory").style.backgroundPosition = "-267px 0px";
			$("inventory").style.left = client_width;
			$("inventory").style.top = "-162";
			this.open = false;
			break;
		}
	},
	content : function (way) {
		var inv_furni = new Array();
		var node = $("tileinv");
		var len = node.childNodes.length;
		for(var i = 0; i < len; i++) {
			if (node.childNodes[i]) {
				inv_furni.push(node.childNodes[i].id);
			}
		}
		this.pages = Math.ceil(inv_furni.length/9);
		if (this.pages == 0) {
			this.pages = 1;
		}
		var inv_content = new Array();
		var len = inv_furni.length;
		if (len <= 9) {
			for (i = 0; i < len; i++) {
				inv_content[i] = inv_furni[i];
			}
		}
		else {
			var end = this.page * 9;
			var start = end - 9;
			if (end > inv_furni.length) {
				end = inv_furni.length;
			}
			for (i = start; i < end; i++) {
				inv_content.push(inv_furni[i]);
			}
		}
		if (this.page > this.pages) {
			this.page--;
		}
		if (way == "open") {
			for (i = 0; i < inv_content.length; i++) {
				var this_furni = $(inv_content[i]);
				this_furni.style.left = client_width-this.icon_posx[i];
				this_furni.style.top = this.icon_posy[i];
				ShowContent(inv_content[i]);
			}
			if (this.pages > 1) {
				if (this.page != 1) {
					$('inventory_back').className = 'knop_solid';
					this.back_able = true;
				}
				else {
					$('inventory_back').className = 'knop_trans';
					this.back_able = false;
				}
				if (this.page != this.pages) {
					$('inventory_next').className = 'knop_solid';
					this.next_able = true;
				}
				else {
					$('inventory_next').className = 'knop_trans';
					this.next_able = false;
				}
			}
			else {
				$('inventory_back').className = 'knop_trans';
				$('inventory_next').className = 'knop_trans';
				this.back_able = this.next_able = false;
			}
		}
		else if (way == "close") {
			var len = inv_furni.length;
			for (i = 0; i < len; i++) {
				if (furni.obj != null) {
					if (inv_content[i] != furni.obj.id) {
						HideContent(inv_furni[i]);
					}
				}
				else {
					HideContent(inv_furni[i]);
				}
			}
		}
	},
	go_back: function () {
		if (this.back_able) {
			this.content('close');
			if (this.page != 1) {
				this.page--;
			}
			this.content('open');
		}
	},
	go_next : function () {
		if (this.next_able) {
			this.content('close');
			if (this.page != this.pages) {
				this.page++;
			}
			this.content('open');
		}
	}
};
var navi = {
	opened : false,
	lists :
	{
		publics : { },
		privates : { },
		own : { },
		searched : { }
	},
	selectedroom : null,
	open : function () {
		if (!this.opened) {
			this.opened = true;
			this.updateask();
			set_top('navigator');
			ShowContent('navigator');
		}
	},
	close : function () {
		if (this.opened) {
			this.opened = false;
			HideContent('navigator');
		}
	},
	reverse : function () {
		if (this.opened) {
			this.close();
		}
		else {
			this.open();
		}
	},
	updateask : function () {
		if (this.opened) {
			packets.send.naviupdate();
		}
	},
	update_publics : function () {
		var html = "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='5' height='2'></td></tr>";
		var len = this.lists.publics.length;
		var count = 0;
		for(x in this.lists.publics) {
			if(count >= len) break; //fix to avoid prototype breaking our loop
			else {
				var room = this.lists.publics[x];
				if (room.U != 0) {
					var image_extra = "2";
				}
				else {
					var image_extra = "";
				}
				html += "<tr><td><img src='images/navi/links.gif'></td><td onclick="+'"'+"navi.roomclick("+x+", 'public');"+'"'+" width='227' bgcolor='#DADADA'><div class='clientfont'><img src='images/navi/sloten/1.gif'>&nbsp;"+room.N+"</div></td><td><img src='images/navi/rechts.gif'></td><td width='2'><font color='black' size='2'>&nbsp;</font></td><td><div onclick="+'"'+"navi.goclick("+x+", 'public');"+'"'+" style='position:relative;'><img src='images/navi/ga"+image_extra+".gif' border='0' style='cursor: default;'><div style='position:absolute; left:0; top:0; width:42; height:16; text-align:right;' class='clientfont'><u>"+texts["navi_go"]+"</u></div></div></td></tr><tr><td colspan='5' height='2'></td></tr>";
				count++;
			}
		}
		if (count < 1) {
			html += "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='2' height='8'></td></tr><tr><td width='8'></td><td><div class='clientfont'>"+texts["navi_no_publics"]+"</div></td></tr></table>";
			html += "</table>";
		}
		html += "</table>";
		$("list_publics").innerHTML = html;
	},
	update_guestrooms : function () {
		var html = "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='5' height='2'></td></tr>";
		var len = this.lists.privates.length;
		var count = 0;
		for(x in this.lists.privates) {
			if(count >= len) break; //fix to avoid prototype breaking our loop
			else {
				var room = this.lists.privates[x];
				if (room.U != 0) {
					var eenderde = room.M/3;
					if (room.U < Math.round(eenderde)) {
						var image_extra = "2";
					}
					else if (room.U < Math.round(eenderde*2)) {
						var image_extra = "3";
					}
					else if (room.U <= room.M) {
						var image_extra = "4";
					}
				}
				else {
					var image_extra = "";
				}
				html += "<tr><td><img src='images/navi/links.gif'></td><td onclick="+'"'+"navi.roomclick("+x+", 'private');"+'"'+" width='227' bgcolor='#DADADA'><div class='clientfont'><img src='images/navi/sloten/"+room.S+".gif'>&nbsp;"+room.N+"</div></td><td><img src='images/navi/rechts.gif'></td><td width='2'><font color='black' size='2'>&nbsp;</font></td><td><div onclick="+'"'+"navi.goclick("+x+", 'private');"+'"'+" style='position:relative;'><img src='images/navi/ga"+image_extra+".gif' border='0' style='cursor: default;'><div style='position:absolute; left:0; top:0; width:42; height:16; text-align:right;' class='clientfont'><u>"+texts["navi_go"]+"</u></div></div></td></tr><tr><td colspan='5' height='2'></td></tr>";
				count++;
			}
		}
		if (count < 1) {
			html += "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='2' height='8'></td></tr><tr><td width='8'></td><td><div class='clientfont'>"+texts["navi_no_guestrooms"]+"</div></td></tr></table>";
			html += "</table>";
		}
		html += "</table>";
		$("list_guestrooms").innerHTML = html;
	},
	update_own : function () {
		var html = "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='5' height='2'></td></tr>";
		var len = this.lists.own.length;
		var count = 0;
		for(x in this.lists.own) {
			if(count >= len) break; //fix to avoid prototype breaking our loop
			else {
				var room = this.lists.own[x];
				if (room.U != 0) {
					var eenderde = room.M/3;
					if (room.U < Math.round(eenderde)) {
						var image_extra = "2";
					}
					else if (room.U < Math.round(eenderde*2)) {
						var image_extra = "3";
					}
					else if (room.U <= room.M) {
						var image_extra = "4";
					}
				}
				else {
					var image_extra = "";
				}
				html += "<tr><td><img src='images/navi/links.gif'></td><td onclick="+'"'+"navi.roomclick("+x+", 'own');"+'"'+" width='227' bgcolor='#DADADA'><div class='clientfont'><img src='images/navi/sloten/"+room.S+".gif'>&nbsp;"+room.N+"</div></td><td><img src='images/navi/rechts.gif'></td><td width='2'><font color='black' size='2'>&nbsp;</font></td><td><div onclick="+'"'+"navi.goclick("+x+", 'own');"+'"'+" style='position:relative;'><img src='images/navi/ga"+image_extra+".gif' border='0' style='cursor: default;'><div style='position:absolute; left:0; top:0; width:42; height:16; text-align:right;' class='clientfont'><u>"+texts["navi_go"]+"</u></div></div></td></tr><tr><td colspan='5' height='2'></td></tr>";
				count++;
			}
		}
		if (count < 1) {
			html += "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='2' height='8'></td></tr><tr><td width='8'></td><td><div class='clientfont'>"+texts["navi_no_ownrooms"]+"</div></td></tr></table>";
			html += "</table>";
		}
		html += "</table>";
		$("list_own").innerHTML = html;
	},
	update_searched : function () {
		var html = "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='5' height='2'></td></tr>";
		var len = this.lists.searched.length;
		var count = 0;
		for(x in this.lists.searched) {
			if(count >= len) break; //fix to avoid prototype breaking our loop
			else {
				var room = this.lists.searched[x];
				if (room.U != 0) {
					var eenderde = room.M/3;
					if (room.U < Math.round(eenderde)) {
						var image_extra = "2";
					}
					else if (room.U < Math.round(eenderde*2)) {
						var image_extra = "3";
					}
					else if (room.U <= room.M) {
						var image_extra = "4";
					}
				}
				else {
					var image_extra = "";
				}
				html += "<tr><td><img src='images/navi/links.gif'></td><td onclick="+'"'+"navi.roomclick("+x+", 'searched');"+'"'+" width='227' bgcolor='#DADADA'><div class='clientfont'><img src='images/navi/sloten/"+room.S+".gif'>&nbsp;"+room.N+"</div></td><td><img src='images/navi/rechts.gif'></td><td width='2'><font color='black' size='2'>&nbsp;</font></td><td><div onclick="+'"'+"navi.goclick("+x+", 'searched');"+'"'+" style='position:relative;'><img src='images/navi/ga"+image_extra+".gif' border='0' style='cursor: default;'><div style='position:absolute; left:0; top:0; width:42; height:16; text-align:right;' class='clientfont'><u>"+texts["navi_go"]+"</u></div></div></td></tr><tr><td colspan='5' height='2'></td></tr>";
				count++;
			}
		}
		if (count < 1) {
			html += "<table border='0' cellpadding='0' cellspacing='0'><tr><td colspan='2' height='8'></td></tr><tr><td width='8'></td><td><div class='clientfont'>"+texts["navi_no_finds"]+"</div></td></tr></table>";
			html += "</table>";
		}
		html += "</table>";
		$("list_searched").innerHTML = html;
	},
	byid : function(id) {
		id = parseInt(id);
		if(id != null && id != "NaN") {
			room.go(id);
		}
		else {	
			debug.add("ID not accepted");
		}
	},
	clearbuttons : function () {
		HideContent("knop_24");
		HideContent("knop_2");
		HideContent("knop_3");
		HideContent("knop_23");
		HideContent("knop_30");
		HideContent("knop_31");
		HideContent("knop_32");
		HideContent("knop_33");
		HideContent("knop_34");
	},
	clear : function () {
		$('big_tab1').src = "images/navi/tabs/1a.gif";
		$('big_tab2').src = "images/navi/tabs/2a.gif";
		$('tabtitel_src').src = "images/blank.gif";
		$('roomschrijf_1').innerHTML = "";
		$('roomschrijf_2').innerHTML = "";
		$('roomschrijf_3').innerHTML = "";
		$('small_tab1').src = "images/navi/small_tabs/1a.gif";
		$('small_tab2').src = "images/navi/small_tabs/2a.gif";
		$('small_tab3').src = "images/navi/small_tabs/3a.gif";
		HideContent("list_publics");
		HideContent("list_guestrooms");
		HideContent("list_searched");
		HideContent("list_own");
		HideContent("list_favo");
		HideContent("tabtitel_1");
		HideContent("tabtitel_2");
		HideContent("tabtitel_3");
		HideContent("tabtitel_4");
		HideContent("tabtitel_5");
		HideContent("small_tab1_div");
		HideContent("small_tab2_div");
		HideContent("small_tab3_div");
		HideContent("zoeklayer");
		HideContent("zoekvak");
		HideContent("knop_22");
		HideContent("eigenlayer");
		HideContent("knop_26");
		HideContent("maakmachine");
		HideContent("wijziglayer");
		HideContent("knop_7");
		HideContent("knop_29");
		HideContent("knop_8");
		HideContent("navi_change_text_div");
		HideContent('change_pass_div');
		HideContent("navi_blanc_layer");
		HideContent("navi_del_text_div1");
		HideContent("navi_del_text_div2");
		HideContent("navi_del_text_div3");
		this.selectedroom = null;
	},
	bigtab1 : function () {
		this.clearbuttons();
		this.clear();
		$('big_tab1').src = "images/navi/tabs/1b.gif";
		$('tabtitel_src').src = "images/navi/icons/8.gif";
		$('roomschrijf_1').innerHTML = texts["navi_publics"];
		$('roomschrijf_3').innerHTML = texts["navi_publics_help"];
		ShowContent("list_publics");
		ShowContent("tabtitel_1");
	},
	bigtab2 : function () {
		this.clearbuttons();
		this.clear();
		$('big_tab2').src = "images/navi/tabs/2b.gif";
		$('tabtitel_src').src = "images/navi/icons/7.gif";
		$('roomschrijf_1').innerHTML = texts["navi_guestrooms"];
		$('roomschrijf_3').innerHTML = texts["navi_guestrooms_help"];
		ShowContent("list_guestrooms");
		ShowContent("tabtitel_2");
		ShowContent("small_tab1_div");
		ShowContent("small_tab2_div");
		ShowContent("small_tab3_div");
	},
	smalltab1 : function () {
		this.clearbuttons();
		this.clear();
		$('tabtitel_src').src = "images/navi/icons/3.gif";
		$('roomschrijf_1').innerHTML = texts["navi_guestrooms"];
		$('roomschrijf_3').innerHTML = texts["navi_searchhelp"];
		ShowContent("tabtitel_3");
		$('small_tab1').src = "images/navi/small_tabs/1b.gif";
		ShowContent("small_tab1_div");
		ShowContent("small_tab2_div");
		ShowContent("small_tab3_div");
		ShowContent("list_searched");
		ShowContent("zoeklayer");
		ShowContent("zoekvak");
		ShowContent("knop_22");
		$('navi_search').focus();
	},
	smalltab2 : function () {
		this.clearbuttons();
		this.clear();
		$('tabtitel_src').src = "images/navi/icons/1.gif";
		$('roomschrijf_1').innerHTML = texts["navi_guestrooms"];
		$('roomschrijf_3').innerHTML = texts["navi_ownrooms_help"];
		ShowContent("tabtitel_4");
		ShowContent("small_tab1_div");
		ShowContent("small_tab2_div");
		ShowContent("small_tab3_div");
		$('small_tab2').src = "images/navi/small_tabs/2b.gif";
		ShowContent("list_own");
		ShowContent("eigenlayer");
		ShowContent("knop_26");
		ShowContent("maakmachine");
	},
	smalltab3 : function () {
		this.clearbuttons();
		this.clear();
		$('tabtitel_src').src = "images/navi/icons/2.gif";
		$('roomschrijf_1').innerHTML = texts["navi_guestrooms"];
		$('roomschrijf_3').innerHTML = texts["navi_favo_help"];
		ShowContent("tabtitel_5");
		ShowContent("small_tab1_div");
		ShowContent("small_tab2_div");
		ShowContent("small_tab3_div");
		$('small_tab3').src = "images/navi/small_tabs/3b.gif";
		ShowContent("list_favo");
	},
	roomclick : function (pos, list) {
		switch(list) {
			case "public":
				this.selectedroom = this.lists.publics[pos];
				break;
			case "private":
				this.selectedroom = this.lists.privates[pos];
				break;
			case "own":
				this.selectedroom = this.lists.own[pos];
				break;
			case "searched":
				this.selectedroom = this.lists.searched[pos];
				break;
		}
		this.clearbuttons();
		
		if (this.selectedroom.T == "public") {
			$('tabtitel_src').src = "images/navi/public/"+this.selectedroom.I+".gif";
			$('roomschrijf_2').innerHTML = "";
			ShowContent("knop_24");
		}
		else {
			$('roomschrijf_2').innerHTML = texts["navi_roomowner"]+": "+ucfirst(this.selectedroom.O);
			switch(this.selectedroom.S) {
				case 1:
					$('tabtitel_src').src = "images/navi/icons/4.gif";
				break;
				case 2:
					$('tabtitel_src').src = "images/navi/icons/5.gif";
				break;
				case 3:
					$('tabtitel_src').src = "images/navi/icons/6.gif";
				break;
			}
			if (this.selectedroom.O == user.name) {
				ShowContent("knop_24");
				ShowContent("knop_23");
			}
			else {
				ShowContent("knop_24");
				if (this.selectedroom.F == 0)
				{
					ShowContent("knop_2");
					HideContent("knop_3");
				}
				else {
					ShowContent("knop_3");
					HideContent("knop_2");
				}
			}
		}
		$('roomschrijf_1').innerHTML = this.selectedroom.N+" "+this.selectedroom.U+"/"+this.selectedroom.M;
		if (in_array("view_room_id", user.rights)) {
			$('roomschrijf_1').innerHTML += " (ID: "+this.selectedroom.I+")";
		}
		$('roomschrijf_3').innerHTML = this.selectedroom.D;
	},
	goclick : function (pos, list) {
		switch(list) {
			case "public":
				this.selectedroom = this.lists.publics[pos];
				break;
			case "private":
				this.selectedroom = this.lists.privates[pos];
				break;
			case "own":
				this.selectedroom = this.lists.own[pos];
				break;
			case "searched":
				this.selectedroom = this.lists.searched[pos];
				break;
			break;
		}
		this.go();
	},
	go : function () {
		if (this.selectedroom != null) {
			if (this.selectedroom.T == "private") {
				if (this.selectedroom.O == user.name) {
					setTimeout(function(){room.go(navi.selectedroom.I);},1);
				}
				else {
					switch(this.selectedroom.S) {
						case 1:
							setTimeout(function(){room.go(navi.selectedroom.I);},1);
						break;
						case 2:
							if (in_array("visit_locked", user.rights)) {
								setTimeout(function(){room.go(navi.selectedroom.I);},1);
							}
							else {
								client.notify(texts["navi_locked"]);
							}
						break;
						case 3:
							if (in_array("visit_locked", user.rights)) {
								setTimeout(function(){room.go(navi.selectedroom.I);},1);
							}
							else {
								$("navi_go_room_pass_text").innerHTML = "<img src='images/navi/icons/6.gif'><br><br><b>"+texts["navi_pass"]+"<br>"+this.selectedroom.N+"</div></b><br><br>"+texts["navi_pass"]+"<br><br><input type='password' size='15' maxlength='15' name='navi_room_pass' id='navi_room_pass' class='clientfont'><br><br>";
								ShowContent("navi_blanc_layer");
								ShowContent("navi_go_room_pass");
								ShowContent("knop_30");
								ShowContent("knop_31");
							}
						break;
					}
				}
			}
			else if (this.selectedroom.T == "public")  {
				setTimeout(function(){room.go(navi.selectedroom.I);},1);
			}
		}
	},
	search : function () {
		var searchfor = $('navi_search').value;
		if (searchfor != "") {
			$('navi_search').value = "";
			packets.send.navisearch(searchfor);
		}
	}
};

/*
function enterpass(id, pass)
{
	pass_form = $("navi_room_pass").value;
	pass_good = pass;
	if (pass_form != "")
	{
		if (pass_form == pass_good)
		{
			show_room(id);
			stop_enterpass();
		}
		else
		{
			start_alert(texts[72]);
			$("navi_room_pass").value = "";
		}
	}
}
function stop_enterpass()
{
	pass_form = "";
	pass_good = "";
	$("navi_room_pass").value = "";
	$("navi_go_room_pass_text").innerHTML = "";
	HideContent("navi_blanc_layer");
	HideContent("navi_go_room_pass");
	HideContent("knop_30");
	HideContent("knop_31");
}


function change_room(naam, omschrijving, type, safe, van, id, favo, pass)
{
	del_progres = 0;
	change_current_pass = pass;
	$("change_id").value = id;
	$("change_name").value = naam;
	$("change_desc").value = omschrijving;
	if (safe == 1)
	{
		$("change_safety_1").checked = true;
		HideContent('change_pass_div');
	}
	if (safe == 2)
	{
		$("change_safety_2").checked = true;
		HideContent('change_pass_div');
	}
	if (safe == 3)
	{
		$("change_safety_3").checked = true;
		ShowContent('change_pass_div');
		$('change_pass_div').innerHTML = '<b>'+texts[99]+':</b><br><input type="password" size="15" maxlength="15" id="change_pass1" name="change_pass1" class="clientfont" value="'+change_current_pass+'"><br><br><b>'+texts[100]+':</b><br><input type="password" size="15" maxlength="15" id="change_pass2" name="change_pass2" class="clientfont" value="'+change_current_pass+'">';
	}
	ShowContent("wijziglayer");
	ShowContent("knop_7");
	ShowContent("knop_29");
	ShowContent("knop_8");
	ShowContent("navi_change_text_div");
}
function change_safe_pass()
{
	change_check1 = $("change_safety_1").checked;
	change_check2 = $("change_safety_2").checked;
	change_check3 = $("change_safety_3").checked;
	if (change_check1)
	{
		$('change_pass_div').innerHTML = '';
		HideContent('change_pass_div');
	}
	if (change_check2)
	{
		$('change_pass_div').innerHTML = '';
		HideContent('change_pass_div');
	}
	if (change_check3)
	{
		$('change_pass_div').innerHTML = '<b>'+texts[99]+':</b><br><input type="password" size="15" maxlength="15" id="change_pass1" name="change_pass1" class="clientfont" value="'+change_current_pass+'"><br><br><b>'+texts[100]+':</b><br><input type="password" size="15" maxlength="15" id="change_pass2" name="change_pass2" class="clientfont" value="'+change_current_pass+'">';
		ShowContent('change_pass_div');
	}
}
function save_change_room() 
{
	change_check1 = $("change_safety_1").checked;
	change_check2 = $("change_safety_2").checked;
	change_check3 = $("change_safety_3").checked;
	if (change_check1)
	{
		new_safe = 1;
	}
	if (change_check2)
	{
		new_safe = 2;
	}
	if (change_check3)
	{
		new_safe = 3;
	}
	if (new_safe == 3)
	{
		change_pass_check();
	}
	else
	{
		change_send();
	}
}
function change_pass_check()
{
	pass1 = $("change_pass1").value;
	pass2 = $("change_pass2").value;
	if ((pass1 != "") && (pass2 != ""))
	{
		if ((pass1.length > 3) && (pass2.length > 3))
		{
			if (pass1 == pass2)
			{
				if (pass1.length <= 15)
				{
					change_send();
				}
				else
				{
					start_alert(texts[101]);
				}
			}
			else
			{
				start_alert(texts[102]);
			}
		}
		else
		{
			start_alert(texts[103]);
		}
	}
	else
	{
		if ($("change_pass_div").style.visibility == 'visible')
		{
			start_alert(texts[104]);
		}
		else
		{
			$("change_pass1").value = "";
			$("change_pass2").value = "";
			ShowContent("change_pass_div");
		}
	}
}
function change_send(new_id)
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
			new_id = $("change_id").value;
			new_name = $("change_name").value;
			new_desc = $("change_desc").value;
			change_check1 = $("change_safety_1").checked;
			change_check2 = $("change_safety_2").checked;
			change_check3 = $("change_safety_3").checked;
			new_pass = "";
			if (change_check1)
			{
				new_safe = 1;
			}
			if (change_check2)
			{
				new_safe = 2;
			}
			if (change_check3)
			{
				new_safe = 3;
			}
			if (new_safe == 3)
			{
				new_pass = $("change_pass1").value;
			}
			ajax_vrij = "nee";
			xmlhttp.open("GET", "changeroom.php?id="+new_id+"&eigenaar="+sessie_naam+"&naam="+new_name+"&schrijf="+new_desc+"&safe="+new_safe+"&pass="+new_pass+"&r="+Math.round(9000*Math.random()),true);
		    xmlhttp.onreadystatechange=function()
		    {
			    if (xmlhttp.readyState==4)
		        {
			        if(xmlhttp.status == 200)
		 			{
						$('list_eigen').innerHTML = xmlhttp.responseText;
			        	start_alert(texts[105]);
			        	navi.smalltab2();
		 			}
		 			else if(xmlhttp.status == 404)
		 			{
		  				verbinding = "nee";
		 			}
		 			else
		 			{
		  				verbinding = "nee";
		 			}
		 			ajax_vrij = "ja";
		        }
		    }
		    xmlhttp.send(null);
	    }
	    else
    	{
	    	setTimeout("change_send('"+new_id+"');", 1000);
    	}
    }
}
function del_change_room()
{
	if (del_progres == 0)
	{
		straks_del_progres = 1;
		ShowContent("navi_blanc_layer");
		ShowContent("navi_del_text_div1");
		ShowContent("knop_32");
		ShowContent("knop_33");
	}
	if (del_progres == 1)
	{
		straks_del_progres = 2;
		ShowContent("navi_blanc_layer");
		HideContent("navi_del_text_div1");
		ShowContent("navi_del_text_div2");
		ShowContent("knop_32");
		ShowContent("knop_33");
	}
	if (del_progres == 2)
	{
		if (verbinding == "ja")
		{
			if (ajax_vrij == "ja")
			{
		 		ajax_vrij = "nee";
				dell_id = $("change_id").value;
				xmlhttp.open("GET", "dellroom.php?id="+dell_id+"&eigenaar="+sessie_naam+"&r="+Math.round(9000*Math.random()),true);
    			xmlhttp.onreadystatechange=function()
	    		{
	    			if (xmlhttp.readyState==4)
        			{
	        			
	        			if(xmlhttp.status == 200)
		 				{
					 		$('list_eigen').innerHTML = xmlhttp.responseText;
							straks_del_progres = 0;
							HideContent("navi_del_text_div2");
							HideContent("knop_32");
							HideContent("knop_33");
							ShowContent("navi_del_text_div3");
							ShowContent("knop_34");
		 				}
		 				else if(xmlhttp.status == 404)
		 				{
			  				verbinding = "nee";
		 				}
		 				else
		 				{
			  				verbinding = "nee";
		 				}
		 				ajax_vrij = "ja";
        			}
    			}
    			xmlhttp.send(null);
			}
			else
    		{
		    	setTimeout("del_change_room();", 1000);
    		}
		}
	}
	del_progres = straks_del_progres;
}
function stop_del_change_room()
{
	del_progres = 0;
	HideContent("navi_blanc_layer");
	HideContent("navi_del_text_div1");
	HideContent("navi_del_text_div2");
	HideContent("knop_30");
	HideContent("knop_31");
	HideContent("knop_32");
	HideContent("knop_33");
}
function stop_change_room()
{
	HideContent("wijziglayer");
	HideContent("knop_7");
	HideContent("knop_29");
	HideContent("knop_8");
	HideContent("navi_change_text_div");
	HideContent('change_pass_div');
	HideContent("navi_blanc_layer");
	HideContent("navi_del_text_div1");
	HideContent("navi_del_text_div2");
	change_current_pass = '';
}


function add_favo(kamerid)
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
		 	ajax_vrij = "nee";
			xmlhttp.open("GET", "add_favo.php?kamerid="+kamerid+"&sessie_naam="+sessie_naam+"&r="+Math.round(9000*Math.random()),true);
    		xmlhttp.onreadystatechange=function()
    		{
	    		if (xmlhttp.readyState==4)
		       	{
			       	if(xmlhttp.status == 200)
		 			{
			 			$('list_favoriet').innerHTML = xmlhttp.responseText;
		 			}
		 			else if(xmlhttp.status == 404)
		 			{
		  				verbinding = "nee";
		 			}
		 			else
		 			{
		  				verbinding = "nee";
		 			}
		 			ajax_vrij = "ja";
		       	}
    		}
    		xmlhttp.send(null);
		}
		else
    	{
	    	setTimeout("add_favo("+kamerid+");", 1000);
    	}
	}
}
function del_favo(kamerid)
{
	if (verbinding == "ja")
	{
		if (ajax_vrij == "ja")
		{
		 	ajax_vrij = "nee";
			xmlhttp.open("GET", "del_favo.php?kamerid="+kamerid+"&sessie_naam="+sessie_naam+"&r="+Math.round(9000*Math.random()),true);
    		xmlhttp.onreadystatechange=function()
    		{
	    		if (xmlhttp.readyState==4)
        		{
	        		if(xmlhttp.status == 200)
		 			{
			 			$('list_favoriet').innerHTML = xmlhttp.responseText;
		 			}
		 			else if(xmlhttp.status == 404)
		 			{
		  				verbinding = "nee";
		 			}
		 			else
		 			{
		  				verbinding = "nee";
		 			}
		 			ajax_vrij = "ja";
        		}
    		}
    		xmlhttp.send(null);
		}
		else
    	{
	    	setTimeout("del_favo("+kamerid+");", 1000);
    	}
	}
}

*/
var matic = {
	roomtype : 1,
	counttiles : new Array(104, 94, 36, 84, 80, 80),
	naviclick : function () {
		setTimeout(function(){matic.open();},1);	
	},
	open : function () {
		set_top('matic_back');
		ShowContent('matic_back');
		set_top('matic');
		ShowContent('matic');
		ShowContent("matic_1");
		HideContent("matic_2");
		$("matic_2_text").innerHTML = "";
		HideContent("matic_3");
		HideContent("matic_4");
		$("matic_4_text").innerHTML = "";
		HideContent("matic_5");
		HideContent("matic_6");
		HideContent("matic_7");
		HideContent("matic_lijn_2");
		HideContent("matic_lijn_3");
		ShowContent("matic_knop_1");
		ShowContent("matic_knop_2");
		HideContent("matic_knop_3");
		HideContent("matic_knop_4");
		HideContent("matic_knop_5");
		HideContent("matic_knop_6");
		HideContent("matic_knop_7");
		HideContent("matic_knop_8");
		$("matic_name").value = '';
		$("matic_desc").value = '';
		this.roomtype = 1;
		$("matic_5_text").innerHTML = texts["matic_size"]+": 104 "+texts["matic_tiles"];
		$("matic_type_image").src = "images/matic/roomtype/1.gif";
		$("matic_safety_1").checked = true;
		$("matic_pass1").value = "";
		$("matic_pass2").value = "";
	},
	close : function () {
		HideContent('matic_back');
		HideContent('matic');
	},
	button2 : function () {
		HideContent("matic_1");
		ShowContent("matic_2");
		$("matic_2_text").innerHTML = "<b>"+texts["matic_createyourown"]+"</b><br>"+texts["matic_canmodifysettings"];
		ShowContent("matic_3");
		ShowContent("matic_4");
		$("matic_4_text").innerHTML = "1/3";
		ShowContent("matic_lijn_2");
		ShowContent("matic_lijn_3");
		HideContent("matic_knop_1");
		HideContent("matic_knop_2");
		ShowContent("matic_knop_3");
		ShowContent("matic_knop_4");
		$("matic_name").focus();
	},
	button3: function () {
		if ($("matic_name").value == "") {
			client.notify("Geef je kamer een naam!");
		}
		else {
			$("matic_2_text").innerHTML = texts["matic_chooselayout"];
			$("matic_4_text").innerHTML = "2/3";
			HideContent("matic_3");
			ShowContent("matic_5");
			HideContent("matic_knop_3");
			HideContent("matic_knop_4");
			ShowContent("matic_knop_5");
			ShowContent("matic_knop_6");
		}
	},
	button4 : function () {
		$("matic_2_text").innerHTML = "<b>"+texts["matic_createyourown"]+"</b><br>"+texts["matic_canmodifysettings"];
		$("matic_4_text").innerHTML = "1/3";
		ShowContent("matic_3");
		HideContent("matic_5");
		ShowContent("matic_knop_3");
		ShowContent("matic_knop_4");
		HideContent("matic_knop_5");
		HideContent("matic_knop_6");
	},
	button5 : function () {
		$("matic_2_text").innerHTML = texts["matic_security"];
		$("matic_4_text").innerHTML = "3/3";
		HideContent("matic_5");
		ShowContent("matic_6");
		HideContent("matic_knop_5");
		HideContent("matic_knop_6");
		ShowContent("matic_knop_7");
		ShowContent("matic_knop_8");
	},
	button6 : function () {
		$("matic_2_text").innerHTML = texts["matic_chooselayout"];
		$("matic_4_text").innerHTML = "2/3";
		ShowContent("matic_5");
		HideContent("matic_6");
		HideContent("matic_7");
		ShowContent("matic_knop_5");
		ShowContent("matic_knop_6");
		HideContent("matic_knop_7");
		HideContent("matic_knop_8");
	},
	button7 : function () {
		var new_safe = 1;
		if ($("matic_safety_1").checked) {
			new_safe = 1;
		}
		else if ($("matic_safety_2").checked) {
			new_safe = 2;
		}
		else if ($("matic_safety_3").checked) {
			new_safe = 3;
		}
		if (new_safe != 0) {
			if (new_safe == 3) {
				matic.checkpass();
			}
			else {
				matic.send();
			}
		}
	},
	checkpass : function () {
		var pass1 = $("matic_pass1").value;
		var pass2 = $("matic_pass2").value;
		if ((pass1 != "") && (pass2 != "")) {
			if ((pass1.length > 3) && (pass2.length > 3)) {
				if (pass1 == pass2) {
					if (pass1.length <= 15) {
						HideContent("matic_7");
						matic.send();
					}
					else {
						HideContent("matic_6");
						ShowContent("matic_7");
						client.notify(texts["matic_longpwd"]);
					}
				}
				else {
					HideContent("matic_6");
					ShowContent("matic_7");
					client.notify(texts["matic_checkpwd"]);
				}
			}
			else {
				HideContent("matic_6");
				ShowContent("matic_7");
				client.notify(texts["matic_shortpwd"]);
			}
		}
		else {
			if ($("matic_7").style.visibility == 'visible') {
				HideContent("matic_6");
				ShowContent("matic_7");
				client.notify(texts["matic_fillall"]);
			}
			else {
				$("matic_pass1").value = "";
				$("matic_pass2").value = "";
				HideContent("matic_6");
				ShowContent("matic_7");
			}
		}
	},
	send : function () {
		var new_name = $("matic_name").value;
		var new_desc = $("matic_desc").value;
		var new_type = parseInt(this.roomtype);
		var new_pass = "";
		var new_safe = 0;
		if ($("matic_safety_1").checked) {
			new_safe = 1;
		}
		else if ($("matic_safety_2").checked) {
			new_safe = 2;
		}
		else if ($("matic_safety_3").checked) {
			new_safe = 3;
		}
		if (new_safe == 3)
		{
			new_pass = $("matic_pass1").value;
		}
		packets.send.createroom(new_name, new_desc, new_type, new_safe, new_pass);
	},
	arrow1: function () {
		var roomtype = parseInt(this.roomtype);
		if (roomtype == 1) {
			roomtype = 6;
		}
		else {
			roomtype--;
		}
		this.roomtype = roomtype;
		$("matic_5_text").innerHTML = texts["matic_size"]+": "+this.counttiles[(roomtype-1)]+" "+texts["matic_tiles"];
		$("matic_type_image").src = "images/matic/roomtype/"+roomtype+".gif";
	},
	arrow2 : function () {
		var roomtype = parseInt(this.roomtype);
		if (roomtype == 6) {
			roomtype = 1;
		}
		else {
			roomtype++;
		}
		this.roomtype = roomtype;
		$("matic_5_text").innerHTML = texts["matic_size"]+": "+this.counttiles[(roomtype-1)]+" "+texts["matic_tiles"];
		$("matic_type_image").src = "images/matic/roomtype/"+roomtype+".gif";
	},
	answer : function (answer) {
		this.close();
		if (answer == "error") {
			client.notify(texts["matic_fail"]);
		}
		else {
			var id = parseInt(answer);
			if(id != null && id != "NaN") {
				room.go(id, "private");
			}
			else {
				client.notify(texts["matic_fail"]);
				debug.add("Create Room answer ID not accepted");
			}
		}
	}
}
var messenger = {
	loaded : false,
	opened : false,
	changetime : false,
	thistab : 1,
	buddies : new Array(),
	requests : new Array(),
	messages : new Array(),
	message : null,
	request : null,
	searchfor : "",
	searchedid : 0,
	addable : false,
	selected : 0,
	selectedinfo : null,
	writebutton_enable : false,
	delbutton_enable : false,
	open : function () {
		if (!this.opened) {
			this.opened = true;
			set_top('messenger');
			ShowContent('messenger');
			this.tab1();
		}
	},
	close : function () {
		if (this.opened) {
			this.opened = false;
			HideContent('messenger');
		}
	},
	reverse : function () {
		if (this.opened) {
			this.close();
		}
		else {
			this.open();
		}
	},
	update : function (mode) {
		if (this.messages.length != 0 || this.requests.length != 0) {
			$('messenger_icon1').src = "images/messenger2.gif";
			$('messenger_icon2').src = "images/icon1_2.gif";
		}
		else {
			$('messenger_icon1').src = "images/messenger.gif";
			$('messenger_icon2').src = "images/icon1.gif";
		}
		$('messenger_link_1').innerHTML = this.messages.length+" "+texts["messenger_newmessages"];
		$('messenger_link_2').innerHTML = this.requests.length+" "+texts["messenger_newrequests"];
		var friends_text = "";
		if (this.buddies.length == 0) {
			friends_text += "<div class='messengerfont' style='padding: 5px;'>"+texts["messenger_nofriends"]+"</div>";
		}
		else {
			// first the online buddies
			var len = this.buddies.length;
			var count = 0;
			for(x in this.buddies) {
				if(count >= len) break;
				else {
					var buddy = this.buddies[x];
					if (buddy.O == "online") {
						friends_text += "<div onclick='messenger.selectfriend("+buddy.I+", "+x+")' style='position:relative; width:200; height:40;'>";
						friends_text += "<div class='messengerfont' style='position:absolute; left:29; top:4; width:170; height:8; font-weight: bold;'>"+ucfirst(buddy.N)+"</div>";
						friends_text += "<div style='position:absolute; left:6; top:4;'><img src='images/messenger/face.gif'></div>";
						friends_text += "<div class='messengerfont' style='position:absolute; left:29; top:14; width:170; height:8;'>Online</div>";
						friends_text += "<div class='messengerfont' style='position:absolute; left:29; top:24; width:170; height:8;'>"+buddy.M+"</div>";
						friends_text += "<div style='position:absolute; left:0; top:39;'><img src='images/messenger/streep2.gif'></div>";
						if(buddy.I == this.selected && mode==1) {
							friends_text += "<div id='messenger_buddy_select_"+buddy.I+"' style='position:absolute; left:1; top:1;'><img src='images/messenger/selected.gif'></div>";
						}
						else {
							friends_text += "<div id='messenger_buddy_select_"+buddy.I+"' style='position:absolute; left:1; top:1; display:none;'><img src='images/messenger/selected.gif'></div>";
						}
						friends_text += "</div>";
					}
					count++;
				}
			}
			// then the offline buddies
			var len = this.buddies.length;
			var count = 0;
			for(x in this.buddies) {
				if(count >= len) break;
				else {
					var buddy = this.buddies[x];
					if (buddy.O != "online") {
						friends_text += "<div onclick='messenger.selectfriend("+buddy.I+", "+x+")' style='position:relative; width:200; height:40;'>";
						friends_text += "<div class='messengerfont' style='position:absolute; left:29; top:4; width:170; height:8; font-weight: bold;'>"+ucfirst(buddy.N)+"</div>";
						friends_text += "<div class='messengerfont' style='position:absolute; left:29; top:14; width:170; height:8;'>Laatste bezoek "+buddy.O.split(/ /)[0]+"</div>";
						friends_text += "<div class='messengerfont' style='position:absolute; left:29; top:24; width:170; height:8;'>"+buddy.M+"</div>";
						friends_text += "<div style='position:absolute; left:0; top:39;'><img src='images/messenger/streep2.gif'></div>";
						if(buddy.I == this.selected && mode==1) {
							friends_text += "<div id='messenger_buddy_select_"+buddy.I+"' style='position:absolute; left:1; top:1;'><img src='images/messenger/selected.gif'></div>";
						}
						else {
							friends_text += "<div id='messenger_buddy_select_"+buddy.I+"' style='position:absolute; left:1; top:1; display:none;'><img src='images/messenger/selected.gif'></div>";
						}
						friends_text += "</div>";
					}
					count++;
				}
			}
		}
		$('messenger_vrienden').innerHTML = friends_text;
		
	},
	missiontext : function () {
		if (this.changetime) {
			clearTimeout(this.changetime);
		}
  		this.changetime = setTimeout(function(){messenger.updatemission();}, 3000);
	},
	missionreg : function () {
		var reg = new RegExp("<(.|\n)*?>");
  		$("messenger_mission").value = $("messenger_mission").value.replace(reg, "");
	},
	updatemission : function () {
		this.changetime = false;
		packets.send.messengermission($("messenger_mission").value);
	},
	selectfriend : function(id, pos) {
		if(this.selected == id) {
			HideContent('messenger_buddy_select_'+id);
			this.selected = 0;
			this.selectedinfo = null;
			messenger.writebutton_enable = false;
			messenger.delbutton_enable = false;
			$('knop_18').className = 'knop_trans';
			$('knop_11').className = 'knop_trans';
		}
		else {
			if(this.selected != 0) {
				HideContent('messenger_buddy_select_'+this.selected);
			}
			this.selected = id;
			this.selectedinfo = this.buddies[pos];
			messenger.writebutton_enable = true;
			messenger.delbutton_enable = true;
			ShowContent('messenger_buddy_select_'+this.selected);
			$('knop_18').className = 'knop_solid';
			$('knop_11').className = 'knop_solid';
		}
	},
	clear : function () {
		messenger.selected = 0;
		this.selectedinfo = null;
		messenger.writebutton_enable = false;
		messenger.delbutton_enable = false;
		$("messenger_send_message_text").value = "";
		$('knop_18').className = 'knop_trans';
		$('knop_11').className = 'knop_trans';
		$('knop_21').className = 'knop_trans';
		HideContent("messenger_vergroot");
		HideContent("messenger_gevonden_naam");
		HideContent("messenger_gevonden_missie");
		HideContent("messenger_gevonden_last_online");
		HideContent("messenger_gevonden_online_status");
		HideContent("messenger_text_1");
		HideContent("messenger_text_2");
		HideContent("messenger_text_3");
		HideContent("messenger_text_4");
		HideContent("messenger_text_5");
		HideContent("messenger_text_6");
		HideContent("messenger_text_7");
		HideContent("messenger_text_8");
		HideContent("messenger_text_9");
		HideContent("messenger_text_10");
		HideContent("messenger_text_11");
		$('messenger_tab1').src = "images/messenger/"+texts["lang"]+"/1.gif";
		$('messenger_tab2').src = "images/messenger/"+texts["lang"]+"/2.gif";
		$('messenger_tab3').src = "images/messenger/"+texts["lang"]+"/3.gif";
		$('messenger_tab4').src = "images/messenger/"+texts["lang"]+"/4.gif";
		$("trashtext").focus();
		messenger.update(2);
		$('messengerface1').style.backgroundImage = "url('images/blank.gif')";
		$('messengerface2').style.backgroundImage = "url('images/blank.gif')";
	},
	tab1 : function () {
		if (this.thistab != 1) {
			this.thistab = 1;
			this.clear();
			ShowContent("messenger_text_1");
			$('messenger_tab1').src = "images/messenger/"+texts["lang"]+"/1b.gif";
		}
	},
	tab2 : function () {
		if (this.thistab != 2) {
			this.thistab = 2;
			this.clear();
			ShowContent("messenger_text_2");
			$('messenger_tab2').src = "images/messenger/"+texts["lang"]+"/2b.gif";
		}
	},
	tab3 : function () {
		if (this.thistab != 3) {
			this.thistab = 3;
			this.clear();
			ShowContent("messenger_text_3");
			$('messenger_tab3').src = "images/messenger/"+texts["lang"]+"/3b.gif";
			$("messenger_zoeknaam").focus();
		}
	},
	tab4 : function () {
		if (this.thistab != 4) {
			this.thistab = 4;
			this.clear();
			ShowContent("messenger_text_4");
			$('messenger_tab4').src = "images/messenger/"+texts["lang"]+"/4b.gif";
		}
	},
	lasttab : function () {
		if (this.thistab != 0) {
			lasttab = this.thistab;
			this.thistab = 0;
			run_script("messenger.tab"+lasttab+"();");
		}
	},
	friendhelp : function () {
		this.clear();
		$('messenger_tab2').src = "images/messenger/"+texts["lang"]+"/2b.gif";
		ShowContent("messenger_text_7");
	},
	show_writehelp : function () {
		HideContent("messenger_text_9");
		ShowContent("messenger_text_10");
	},
	stop_writehelp : function () {
		HideContent("messenger_text_10");
		ShowContent("messenger_text_9");
	},
	search : function () {
		var searchname = $('messenger_zoeknaam').value;
		if (searchname != "") {
			$('messenger_zoeknaam').value = "";
			this.searchfor = searchname;
			this.searchedid = 0;
			packets.send.messengersearch(searchname);
		}
	},
	searched : function (userid, mission, clothes, last_online, online, addable) {
		this.searchedid = userid;
		this.addable = addable;
		$('knop_21').className = 'knop_trans';
		if (this.searchedid == 0) {
			$('messengerface1').style.backgroundImage = "url('images/blank.gif')";
			$('messenger_gevonden_naam').innerHTML = texts["messenger_notfound"];
			HideContent("messenger_vergroot");
			ShowContent("messenger_gevonden_naam");
			HideContent("messenger_gevonden_missie");
			HideContent("messenger_gevonden_last_online");
			HideContent("messenger_gevonden_online_status");
		}
		else {
			$('messengerface1').style.backgroundImage = "url('avatar.php?figure="+clothes+"&size=b&direction=3&head_direction=3&gesture=sml&action=std')";
			ShowContent("messenger_vergroot");
			ShowContent("messenger_gevonden_naam");
			ShowContent("messenger_gevonden_missie");
			ShowContent("messenger_gevonden_last_online");
			ShowContent("messenger_gevonden_online_status");
			$('messenger_gevonden_naam').innerHTML = ucfirst(this.searchfor.toLowerCase());
			$('messenger_gevonden_missie').innerHTML = mission;
			$('messenger_gevonden_last_online').innerHTML = last_online;
			$('messenger_gevonden_online_status').innerHTML = online;
			if (online) {
				$('messenger_gevonden_online_status').innerHTML = "Online";
			}
			else {
				$('messenger_gevonden_online_status').innerHTML = "Offline";
			}
			if (this.addable) {
				$('knop_21').className = 'knop_solid';
			}
		}
		
	},
	show_message : function () {
		if (this.messages.length > 0) {
			this.message = this.messages[0];
			$('messenger_text_11_text1').innerHTML = texts["messenger_sender"]+": "+ucfirst(this.message.FN);
			$('messenger_text_11_text2').innerHTML = this.message.D;
			$('messenger_message_div').innerHTML = this.message.M;
			this.clear();
			$('messengerface2').style.backgroundImage = "url('avatar.php?figure="+this.message.C+"&size=b&direction=3&head_direction=3&gesture=sml&action=std')";
			$('messenger_tab1').src = "images/messenger/"+texts["lang"]+"/1b.gif";
			ShowContent("messenger_text_11");
		}
	},
	answer : function () {
		if (this.messages.length > 0 && this.message != null) {
			this.clear();
			this.selected = this.message.FI;
			this.selectedinfo = { N : this.message.FN }
			this.startwrite();
			packets.send.delmessage(this.message.I);
			this.message = null;
			this.messages.shift();
			this.update(1);
		}
	},
	del_message : function () {
		if (this.messages.length > 0 && this.message != null) {
			packets.send.delmessage(this.message.I);
			this.message = null;
			this.messages.shift();
			this.update(1);
			this.thistab = 0;
			this.tab1();
			this.show_message();
		}
	},
	show_vv : function () {
		if (this.requests.length > 0) {
			this.request = this.requests[0];
			$('messenger_text_6_text').innerHTML = ucfirst(this.request.N)+"<br>"+texts["messenger_request_text"];
			this.clear();
			$('messenger_tab1').src = "images/messenger/"+texts["lang"]+"/1b.gif";
			ShowContent("messenger_text_6");
		}
	},
	search_vv : function () {
		if (this.addable) {
			this.addable = false;
			$('knop_21').className = 'knop_trans';
			$('messenger_text_5_text').innerHTML = this.searchfor+texts["messenger_request_sent"];
			this.clear();
			$('messenger_tab3').src = "images/messenger/"+texts["lang"]+"/3b.gif";
			ShowContent("messenger_text_5");
			packets.send.addbuddy(this.searchedid);
		}
	},
	answer_vv : function (answer) {
		if (this.requests.length > 0 && this.request != null) {
			packets.send.messengeranswerrequest(this.request.I, answer);
			this.request = null;
			this.requests.shift();
			this.update(1);
			this.thistab = 0;
			this.tab1();
			this.show_vv();
		}
	},
	messagereg : function () {
		var reg = new RegExp("<(.|\n)*?>");
  		$("messenger_send_message_text").value = $("messenger_send_message_text").value.replace(reg, "");
  		textCounter($('messenger_send_message_text'),216);
	},
	startwrite : function () {
		$("messenger_send_message_text").value = "";
		$('messenger_text_9_text').innerHTML = ucfirst(this.selectedinfo.N);
		HideContent("messenger_text_2");
		ShowContent("messenger_text_9");
		$("messenger_send_message_text").focus();
	},
	writefriend : function () {
		this.messagereg();
		var message = $("messenger_send_message_text").value;
		if (message != "") {
			packets.send.writemessage(this.selected, message);
			this.lasttab();
		}
	},
	startdelbuddy : function () {
		$('messenger_text_8_text').innerHTML = texts["messenger_ask_del1"]+"<br><br>"+ucfirst(this.selectedinfo.N)+"<br>"+texts["messenger_ask_del2"];
		HideContent("messenger_text_2");
		ShowContent("messenger_text_8");
	},
	friend_del : function () {
		packets.send.delbuddy(this.selected);
		messenger.thistab = 0;
		this.tab2();
	}
};
var catalogus = {
	cats : null, // categories
	gowait : null,
	actief : null,
	init_done : false,
	loading : false,
	went : false,
	buy_cat : 0,
	buy_furni : "",
	buy_price : "",
	vakje_licht : 15,
	icons_active : { },
	newcat : null,
	addimg : null,
	aantal : null,
	blz : 0,
	blz_amount : 0,
	layout2_pijl1 : 0,
	layout2_pijl2: 0,
	buy_mode : "",
	init : function () {
		if (!this.init_done && !this.loading) {
			this.create_vakjes();
			if(client.connection) {
				var req = ajaxCreate();
				req.open("GET", "action.php?method=catinit&r="+Math.round(9000*Math.random()), true);
				req.onreadystatechange = function() {
					if (req.readyState == 4) {
						catalogus.cats = JSON.parse(req.responseText);
						catalogus.init2();
					}
				};
				req.send(null);
			}
		}
	},
	init2 : function () {
		if (!this.init_done && !this.loading) {
			var sidebar = "";
			var hoeveel = 0;
			sidebar += "<table border='0' cellpadding='0' cellspacing='0'>";
			var cats_length = this.cats.length;
			for (i = 0; i < cats_length; i++) {
				sidebar += "<tr><td colspan='3' height='1' style='background-color: #a0a0a4;'></td></tr>";
				sidebar += "<tr><td><img src='images/shop/links1.gif' id='srclinks_"+(i+1)+"'></td>";
				sidebar += "<td id='srccatknop_"+(i+1)+"' class='clientfont' onclick=\"catalogus.go('"+(i+1)+"');\" width='90' height='20' style='cursor: pointer; background-color: #dddddd; overflow: hidden;'>";
				sidebar += "<b>"+this.cats[i].name+"</b>";
				sidebar += "</td><td width='1' style='background-color: #a0a0a4;'></tr>";
				hoeveel++;
			}
			sidebar += "</table>";
			$("cat_sidebar_div").innerHTML = sidebar;
			this.actief = 0;
			this.init_done = true;
			debug.yellow(hoeveel+" categories loaded in Catalogue");
			if (this.gowait == "club") {
				club.gocat();
				this.gowait = null;
			}
			else {
				this.go(1);
			}
		}
	},
	create_vakjes : function () {
		var container = $("catvakjes");
		var count = 0;
		var vak_x = 21;
		var vak_y = 171;
		for (kolom = 0; kolom < 5; kolom++) {
			vak_x = 21;
			for (rij = 0; rij < 3; rij++) {
				
				var image = document.createElement("img");
				image.src = "images/shop/vakje1.gif";
				image.id = "vakje_"+count;
				image.style.position = "absolute";
				image.style.left = vak_x;
				image.style.top = vak_y;
				image.style.zIndex = 1;
				container.appendChild(image);
				
				var image = document.createElement("img");
				image.src = "images/shop/icon.gif";
				image.id = "icon_"+count;
				image.style.position = "absolute";
				image.style.left = vak_x;
				image.style.top = vak_y;
				image.style.zIndex = 2;
				image.count = count;
				addEvent(image, 'click', function() { catalogus.clickfurni(this.count); });
				container.appendChild(image);
				
				count++;
				vak_x += 39;
			}
			vak_y += 39;
		}
	},
	clear_icons : function () {
		for (i = 0; i < 15; i++) {
			$("icon_"+i).src = "images/shop/icon.gif";
			this.icons_active[i] = false;
		}
		if (this.vakje_licht != 15) {
			$("vakje_"+this.vakje_licht).src = "images/shop/vakje1.gif";
			this.vakje_licht = 15;
		}
		$("cat_layout2_pijl1").src = "images/shop/pijl1_1.gif";
		$("cat_layout2_pijl2").src = "images/shop/pijl2_1.gif";
	},
	clear_cat : function () {
		HideContent(["cat_frontpage", "cat_camera", "cat_club", "cat_market", "cat_presents", "cat_norares", "cat_productpage1", "cat_layout2", "cat_logo"]);
	},
	go : function (catid) {
		if (!this.loading && this.actief != catid && this.init_done) {
			var category = this.cats[(parseInt(catid)-1)];
			if (this.went) {
				$("srclinks_"+this.actief).src = "images/shop/links1.gif";
				$("srccatknop_"+this.actief).style.background = "rgb(221, 221, 221)";
			}
			$("srclinks_"+catid).src = "images/shop/links2.gif";
			$("srccatknop_"+catid).style.background = "rgb(240, 240, 240)";
			if(category.loaded) {
				this.buy_cat = 0;
				this.buy_furni = "";
				this.buy_price = 0;
				switch(category.layout) {
				case "frontpage":
					$("cat_front_logo").src = "images/shop/shop_img/"+category.header+".gif";
					$("cat_front_wow").innerHTML = category.wow1;
					$("cat_front_img1").src = "images/shop/shop_img/"+category.side[0]+".gif";
					$("cat_front_text1").innerHTML = category.desc;
					this.clear_cat();
					ShowContent("cat_frontpage");
					break;
				case "camera":
					$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
					$("cat_camera_img1").src = "images/shop/shop_img/"+category.side[0]+".gif";
					$("cat_camera_text1").innerHTML = category.desc;
					$("cat_camera_text2").innerHTML = category.misc['t1'];
					this.clear_cat();
					ShowContent("cat_logo");
					ShowContent("cat_camera");
					this.buy_cat = category.id;
					this.buy_furni = furni.db[category.furnis[0]].furni;
					this.buy_price = category.prices[0];
					break;
				case "club":
					$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
					$("cat_club_img1").src = "images/shop/shop_img/"+category.side[0]+".gif";
					$("cat_club_text").innerHTML = category.desc;
					this.clear_cat();
					ShowContent("cat_logo");
					ShowContent("cat_club");
					break;
				case "market":
					$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
					this.clear_cat();
					ShowContent("cat_logo");
					ShowContent("cat_market");
					break;
				case "presents":
					$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
					$("cat_presents_img1").src = "images/shop/shop_img/"+category.side[0]+".gif";
					$("cat_presents_img2").src = "images/shop/shop_img/"+category.side[1]+".gif";
					$("cat_presents_text1").innerHTML = category.desc;
					$("cat_presents_text2").innerHTML = category.misc['t1'];
					this.clear_cat();
					ShowContent("cat_logo");
					ShowContent("cat_presents");
					break;
				case "norares":
					$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
					$("cat_norares_img").src = "images/shop/shop_img/"+category.side[0]+".gif";
					$("cat_norares_text").innerHTML = category.desc;
					this.clear_cat();
					ShowContent("cat_logo");
					ShowContent("cat_norares");
					break;
				case "productpage1":
					if (category.furnis.length == 1) {
						HideContent("cat_productpage1_wow1_div");
						HideContent("cat_productpage1_wow2_div");
						$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
						$("cat_productpage1_desc").innerHTML = category.desc;
						var furni_data = furni.db[category.furnis[0]];
						var furni_price = category.prices[0];
						if (furni_price == 1) {
							var cr_text = texts["money1"];
						}
						else {
							var cr_text = texts["money2"];
						}
						$("cat_productpage1_text1").innerHTML = furni_data.name;
						$("cat_productpage1_text2").innerHTML = furni_data.descr;
						$("cat_productpage1_text3").innerHTML = furni_price+" "+cr_text;
						if (furni_data.afb != 1) {
							$("cat_productpage1_img").src = "images/furni/"+furni_data.furni+"/2.gif";
						}
						else {
							$("cat_productpage1_img").src = "images/furni/"+furni_data.furni+"/1.gif";
						}
						if (category.wow1 != "") {
							$("cat_productpage1_wow1").innerHTML = category.wow1;
							ShowContent("cat_productpage1_wow1_div");
						}
						if (category.wow2 != "") {
							$("cat_productpage1_wow2").innerHTML = category.wow2;
							ShowContent("cat_productpage1_wow2_div");
						}
						this.clear_cat();
						ShowContent("cat_logo");
						ShowContent("cat_productpage1");
						this.buy_cat = category.id;
						this.buy_furni = furni_data.furni;
						this.buy_price = furni_price;
					}
					break;
				case "layout2":
					$("cat_layout2_text1").innerHTML = "";
					$("cat_layout2_text3").innerHTML = "";
					HideContent("cat_productpage1_prijsvak");
					HideContent("knop_25");
					this.clear_icons();
					HideContent("cat_layout2_wow1");
					HideContent("cat_layout2_wow2");
					HideContent("cat_layout2_page");
					HideContent("cat_layout2_arrows");
					$("cat_layout2_page_text").innerHTML = "";
					$("cat_logo").src = "images/shop/shop_img/"+category.header+".gif";
					$("cat_layout2_desc").innerHTML = category.desc;
					$("cat_layout2_sideimg").src = "images/shop/shop_img/"+category.side[0]+".gif";
					$("cat_layout2_text2").innerHTML = category.details;
					this.aantal = category.furnis.length;
					this.blz = 1;
					var loopaf = this.aantal;
					if (this.aantal > 15) {
						loopaf = 15;
						this.blz_amount = Math.ceil(this.aantal/15);
						this.layout2_pijl1 = 0;
						$("cat_layout2_pijl1").src = "images/shop/pijl1_1.gif";
						if (this.blz_amount > 1) {
							this.layout2_pijl2 = 2;
							$("cat_layout2_pijl2").src = "images/shop/pijl2_2.gif";
						}
						else {
							this.layout2_pijl2 = 0;
							$("cat_layout2_pijl2").src = "images/shop/pijl2_2.gif";
						}
						$("cat_layout2_page_text").innerHTML = "1/"+this.blz_amount;
						ShowContent("cat_layout2_page");
						ShowContent("cat_layout2_arrows");
					}
					for (i = 0; i < loopaf; i++) {
						$("icon_"+i).src = "images/furni/"+category.furnis[i]+"/icon.gif";
						this.icons_active[i] = true;
					}
					if (category.wow1 != "") {
						$("cat_layout2_wow1_text").innerHTML = category.wow1;
						ShowContent("cat_layout2_wow1");
					}
					if (category.wow2 != "") {
						$("cat_layout2_wow2_text").innerHTML = category.wow2;
						ShowContent("cat_layout2_wow2");
					}
					this.clear_cat();
					ShowContent("cat_logo");
					ShowContent("cat_layout2");
					break;
				default:
					debug.error("Error loading catalogue page: unknown type");
					client.notify(texts["cat_loadfail"], texts["notify_error"]);
					this.unload();
				}
				this.went = true;
				this.actief = catid;
			}
			else {
				ShowContent("cat_load_div");
				this.loading = true;
				this.newcat = catid;
				get_script("action.php?method=catload&id="+category.id+"&putin="+(parseInt(catid)-1)+"&r="+Math.round(9000*Math.random()));
			}
		}
	},
	load2 : function () {
		if (this.loading) {
			var preload_furni = new Array();
			var len = this.cats[(parseInt(this.newcat)-1)]['furnis'].length;
			for (i = 0; i < len; i++) {
				preload_furni.push(this.cats[(parseInt(this.newcat)-1)].furnis[i]);
			}
			var callback = catalogus.load3();
			furni.manager.load(preload_furni, this.addimg, callback);
		}
	},
	load3 : function () {
		var output = new Object();
		output.complete = function() {
			if (catalogus.loading) {
				if (!furni.manager.errors) {
					catalogus.loading = false;
					catalogus.cats[(parseInt(catalogus.newcat)-1)].loaded = true;
					HideContent("cat_load_div");
					catalogus.go(catalogus.newcat);
				}
				else {
					catalogus.unload();
					client.notify(texts["cat_nosuchfurni"], texts["notify_error"]);
				}
			}
		};
		return output;
	},
	unload : function () {
		HideContent("cat_load_div");
		this.loading = false;
		if (this.went) {
			$("srclinks_"+this.actief).src = "images/shop/links2.gif";
			$("srccatknop_"+this.actief).style.background = "rgb(240, 240, 240)";
		}
		$("srclinks_"+this.newcat).src = "images/shop/links1.gif";
		$("srccatknop_"+this.newcat).style.background = "rgb(221, 221, 221)";
	},
	moneyupdate : function () {
		if (user.logged_in) {
			if (user.money == 1) {
				var aanhangsel1 = texts["money1"];
			}
			else {
				var aanhangsel1 = texts["money2"];
			}
			if (user.sessionearned == 1) {
				var aanhangsel2 = texts["money1"];
			}
			else {
				var aanhangsel2 = texts["money2"];
			}
			$("purse_text1").innerHTML = texts["catalogue_youHave"]+"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font size='4' color='#eeeeee'>"+user.money+"</font>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+aanhangsel1;
			$("purse_text2").innerHTML = texts["catalogue_youHave"]+"&nbsp;&nbsp;"+user.money+"&nbsp;&nbsp;"+aanhangsel1;
			$("purse_text3").innerHTML = texts["catalogue_earned1"]+"&nbsp;&nbsp;"+user.sessionearned+"&nbsp;&nbsp;"+aanhangsel2+" "+texts["catalogue_earned2"];
		}
	},
	clickfurni : function (vak_nr) {
		if (this.icons_active[vak_nr]) {
			var lijstnr = ((this.blz-1)*15)+vak_nr;
			var furni_data = furni.db[this.cats[(parseInt(this.actief)-1)].furnis[lijstnr]];
			var furni_price = this.cats[(parseInt(this.actief)-1)].prices[lijstnr];
			
			if (this.vakje_licht != vak_nr) {
				if (this.vakje_licht != 15) {
					$("vakje_"+this.vakje_licht).src = "images/shop/vakje1.gif";
				}
				this.vakje_licht = vak_nr;
				$("vakje_"+vak_nr).src = "images/shop/vakje2.gif";
			}
			if (furni_price == 1) {
				var cr_text = texts["money1"];
			}
			else {
				var cr_text = texts["money2"];
			}
			if (furni_data.afb != 1) {
				$("cat_layout2_sideimg").src = "images/furni/"+furni_data.furni+"/2.gif";
			}
			else {
				$("cat_layout2_sideimg").src = "images/furni/"+furni_data.furni+"/1.gif";
			}
			$("cat_layout2_text1").innerHTML = furni_data.name;
			$("cat_layout2_text2").innerHTML = furni_data.descr;
			$("cat_layout2_text3").innerHTML = furni_price+" "+cr_text;
			HideContent("cat_layout2_wow1");
			HideContent("cat_layout2_wow2");
			ShowContent("cat_productpage1_prijsvak");
			ShowContent("knop_25");
			this.buy_cat = this.cats[(parseInt(this.actief)-1)].id;
			this.buy_furni = furni_data.furni;
			this.buy_price = furni_price;
		}
	},
	show_blz : function (blz) {
		if (blz != 0) {
			this.blz = blz;
			this.clear_icons();
			if (this.blz == 1) {
				this.layout2_pijl1 = 0;
				$("cat_layout2_pijl1").src = "images/shop/pijl1_1.gif";
			}
			else {
				this.layout2_pijl1 = this.blz-1;
				$("cat_layout2_pijl1").src = "images/shop/pijl1_2.gif";
			}
			if (this.blz != this.blz_amount) {
				this.layout2_pijl2 = this.blz+1;
				$("cat_layout2_pijl2").src = "images/shop/pijl2_2.gif";
			}
			else {
				this.layout2_pijl2 = 0;
				$("cat_layout2_pijl2").src = "images/shop/pijl2_1.gif";
			}
			$("cat_layout2_page_text").innerHTML = this.blz+"/"+this.blz_amount;
			var tot_item = this.blz*15;
			var van_item = tot_item-15;
			if (tot_item > this.aantal) {
				tot_item = this.aantal;
			}
			for (i = van_item; i < tot_item; i++) {
				$("icon_"+(i-van_item)).src = "images/furni/"+this.cats[(parseInt(this.actief)-1)].furnis[i]+"/icon.gif";
				this.icons_active[(i-van_item)] = true;
			}
		}
	},
	buy : function () {
		var buy_name = furni.db[this.buy_furni].name;
		if (user.money >= this.buy_price) {
			if (this.buy_price == 1) {
				var cr_text1 = texts["money1"];
			}
			else {
				var cr_text1 = texts["money2"];
			}
			if (user.money == 1) {
				var cr_text2 = texts["money1"];
			}
			else {
				var cr_text2 = texts["money2"];
			}
			this.buy_mode = "self";
			$("koop_aantal").disabled = false;
			$("kado_voorwie").value = '';
			$("kado_text").value = '';
			ShowContent("koop_aantal_div");
			$("koop_aantal").value = '1';
			ShowContent("knop_4");
			ShowContent("knop_6");
			HideContent("koop2_layer");
			HideContent("koop2_prijs");
			HideContent("koop2_jouwgeld");
			HideContent("koop2_kado");
			HideContent("kado_voorwie_layer");
			HideContent("kado_text_layer");
			ShowContent("koop1_prijs");
			ShowContent("koop1_jouwgeld");
			ShowContent("koop1_kado");
			$("koop1_prijs").innerHTML = "<b>"+buy_name+" "+texts["catalogue_costs"]+" "+this.buy_price+" "+cr_text1+"</b>";
			$("koop1_jouwgeld").innerHTML = texts["catalogue_yourMoney1"]+" "+user.money+" "+cr_text2+" "+texts["catalogue_yourMoney2"];
			$("koop1_kado").innerHTML = "<img src='images/shop/kruisje1.gif' align='bottom' onclick='catalogus.buy_present();'> "+texts["catalogue_buyAsPresent"];
			$("koop2_prijs").innerHTML = "<b>"+buy_name+" kost "+this.buy_price+" "+cr_text1+"</b>";
			$("koop2_jouwgeld").innerHTML = texts["catalogue_yourMoney1"]+" "+user.money+" "+cr_text2+" "+texts["catalogue_yourMoney2"];
			$("koop2_kado").innerHTML = "<img src='images/shop/kruisje2.gif' align='bottom' onclick='catalogus.buy_self();'>";
			setTimeout(function(){set_top('kopen'); ShowContent('kopen');},1)
		}
		else {
			if (this.buy_price == 1) {
				var cr_text = texts["money1"];
			}
			else {
				var cr_text = texts["money2"];
			}
			client.notify("<b>'"+buy_name+"' "+texts["catalogue_costs"]+" "+this.buy_price+" "+cr_text+"</b><br><br>"+texts["buy_poor"]);
		}
	},
	buy_stop : function () {
		HideContent("koop_aantal_div");
		HideContent("knop_4");
		HideContent("knop_6");
		HideContent("koop2_layer");
		HideContent("koop2_prijs");
		HideContent("koop2_jouwgeld");
		HideContent("koop2_kado");
		HideContent("kado_voorwie_layer");
		HideContent("kado_text_layer");
		HideContent("koop1_prijs");
		HideContent("koop1_jouwgeld");
		HideContent("koop1_kado");
		HideContent('kopen');
	},
	buy_present : function () {
		this.buy_mode = 'present';
		$("koop_aantal").value = '1';
		$("koop_aantal").disabled = true;
		ShowContent("koop2_layer");
		ShowContent("koop2_prijs");
		ShowContent("koop2_jouwgeld");
		ShowContent("koop2_kado");
		ShowContent("kado_voorwie_layer");
		ShowContent("kado_text_layer");
		HideContent("koop1_prijs");
		HideContent("koop1_jouwgeld");
		HideContent("koop1_kado");
	},
	buy_self : function () {
		koop_modus = 'self';
		$("koop_aantal").disabled = false;
		HideContent("koop2_layer");
		HideContent("koop2_prijs");
		HideContent("koop2_jouwgeld");
		HideContent("koop2_kado");
		HideContent("kado_voorwie_layer");
		HideContent("kado_text_layer");
		ShowContent("koop1_prijs");
		ShowContent("koop1_jouwgeld");
		ShowContent("koop1_kado");
	},
	buy_amount : function () {
		if (this.buy_mode == "present") {
			$("koop_aantal").value = '1';
		}
		else {
			$("koop_aantal").value = parseInt($("koop_aantal").value);
		}
		if ($("koop_aantal").value <= 0 || $("koop_aantal").value == "NaN")
		{
			$("koop_aantal").value = '1';
		}
	},
	buy_ok : function () {
		if (user.money >= (this.buy_price*parseInt($("koop_aantal").value))) {
			packets.send.buyfurni(this.buy_cat, this.buy_furni, $("koop_aantal").value, this.buy_mode, $("kado_voorwie").value, $("kado_text").value)
			this.buy_stop();
		}
		else {
			if (this.buy_price == 1) {
				var cr_text = texts["money1"];
			}
			else {
				var cr_text = texts["money2"];
			}
			if (parseInt($("koop_aantal").value) > 1) {
				var amount_text = " X "+parseInt($("koop_aantal").value);
			}
			else {
				var amount_text = "";
			}
			client.notify("<b>'"+furni.db[this.buy_furni].name+"'"+amount_text+" "+texts["catalogue_costs"]+" "+(this.buy_price*parseInt($("koop_aantal").value))+" "+cr_text+"</b><br><br>"+texts["buy_poor"]);
			$("koop_aantal").value = Math.floor(user.money/this.buy_price);
		}
	},
	buyanswer : function (answer) {
		switch(answer) 
		{
		case "ok":
			client.notify(texts["catalogue_itsyours"]);
			if (client.sound) {
				soundManager.play("buy");
			}
			break;
		case "price":
			if (this.buy_furni != "") {
				if (this.buy_price == 1) {
					var cr_text = texts["money1"];
				}
				else {
					var cr_text = texts["money2"];
				}
				if (parseInt($("koop_aantal").value) > 1) {
					var amount_text = " X "+parseInt($("koop_aantal").value);
				}
				else {
					var amount_text = "";
				}
				client.notify("<b>'"+furni.db[this.buy_furni].name+"'"+amount_text+" "+texts["catalogue_costs"]+" "+(this.buy_price*parseInt($("koop_aantal").value))+" "+cr_text+"</b><br><br>"+texts["buy_poor"]);
			}
			break;
		case "error":
			client.notify(texts["buy_error"]);
			break;
		default:
			debug.error("Unknown buying furniture answer.");
			client.notify(texts["buy_error"]);
		break;
		}
	}
}
var crcodes = {
	show : function () {
		HideContent("cr_code_checking");
		$("cr_code_input").value = "";
		setTimeout(function(){set_top('codes_layer'); ShowContent('codes_layer'); $('cr_code_input').focus();},1)
	},
	check : function () {
		var code_try = $("cr_code_input").value;
		if (code_try != "") {
			ShowContent("cr_code_checking");
			$("cr_code_toptext").innerHTML = texts["crcode_wait"];
			$("trashtext").focus();
			packets.send.coupon(code_try);
		}
	},
	checkedok : function () {
		$("cr_code_toptext").innerHTML = texts["crcode_enter"];
		$("cr_code_input").value = "";
		HideContent("cr_code_checking");
		HideContent("codes_layer");
		$("trashtext").focus();
	},
	checkedbad : function (error) {
		client.notify(error);
		$("cr_code_toptext").innerHTML = texts["crcode_enter"];
		$("cr_code_input").value = "";
		HideContent("cr_code_checking");
		$('cr_code_input').focus();
	}
}
var club = {
	member : false,
	restingdays : 0,
	passedmonths : 0,
	restingmonths : 0,
	update : function (clubstatus, days, pm, rm) {
		this.member = clubstatus;
		this.restingdays = days;
		this.passedmonths = pm;
		this.restingmonths = rm;
		if (this.member) {
			client.notify(this.restingdays);
		}
	},
	moreclick : function () {
		setTimeout(function(){club.more();},1);	
	},
	more : function () {
		set_top('catalogus');
		ShowContent('catalogus');
		if (!catalogus.init_done && !catalogus.loading) {
			catalogus.gowait = "club";
			catalogus.init();
		}
		else {
			this.gocat();
		}
	},
	gocat : function () {
		var len = catalogus.cats.length;
		for (i = 0; i < len; i++) {
			if (catalogus.cats[i].name == "Jabbo Club") {
				catalogus.go(i+1);
			}
		}
	}
}
var camera = {
	camid : null,
	loader : null,
	savepic : null,
	savenum : null,
	picturepic : null,
	pictureloader : null,
	pictureloading : false,
	taking : false,
	saving : false,
	drag : function () {
		if (this.camid != null) {
			HideContent("askcamera");
			furni.dragstart(this.camid);
			this.camid = null;
		}
	},
	open : function () {
		if (this.camid != null) {
			HideContent("askcamera");
			set_top("camera");
			this.reEnable();
			ShowContent("camera");
			this.camid = null;
			packets.send.getdrink("camera");
		}
	},
	take : function () {
		if (this.savenum == null && client.connection && !this.taking && !this.saving) {
			this.taking = true;
			$("cameratake").src = "images/camera/take1.gif";
			camera.loader = getBusyOverlay($("savepic"),{color: 'black', opacity: 0.5},{color: '#fff', size: 48});
			var x = parseInt($("camera").style.left)+29;
			var y = parseInt($("camera").style.top)+24;
			var req = ajaxCreate();
			req.open("GET", "camera.php?id="+room.id+"&x="+x+"&y="+y+"&r="+Math.round(9000*Math.random()), true);
			req.onreadystatechange = function() {
				if (req.readyState == 4) {
					camera.savenum = req.responseText;
					camera.savepic = document.createElement("img");
					camera.savepic.src = "cache/camera/"+camera.savenum+".gif";
					camera.savepic.style.border = '1px solid black';
					camera.savepic.style.position = "absolute";
					camera.saveload();
				}
			};
			req.send(null);
		}
	},
	saveload : function () {
		if (this.savepic.complete) {
			// show cached image, and ask to save
			camera.loader.remove();
			camera.taking = false;
			$("savepic").appendChild(this.savepic);
			$("cameradel").src = "images/camera/del2.gif";
			$("camerasave").src = "images/camera/save2.gif";
			$("cameracaptionimg").src = "images/camera/caption2.gif";
			$("camera_caption").value = "";
			ShowContent("camera_caption");
			$("camera_caption").focus();
		}
		else {
			setTimeout(function(){camera.saveload();},10);
		}
	},
	save : function () {
		if (this.savenum != null && client.connection && !this.taking && !this.saving) {
			this.saving = true;
			camera.loader = getBusyOverlay($("savepic"),{color: 'black', opacity: 0.5},{color: '#fff', size: 48});
			var saveid = this.savenum;
			this.captionreg();
			var caption = $("camera_caption").value.replace(/\n/g, "||");
			this.savenum = null;
			this.resetGUI();
			$("cameratake").src = "images/camera/take1.gif";
			var req = ajaxCreate();
			req.open("GET", "camerasave.php?id="+saveid+"&caption="+caption+"&r="+Math.round(9000*Math.random()), true);
			req.onreadystatechange = function() {
				if (req.readyState == 4) {
					if (req.responseText == "error") {
						client.notify(texts["camera_savefail"], texts["notify_error"]);
					}
					camera.loader.remove();
					camera.reEnable();
					camera.saving = false;
				}
			};
			req.send(null);
		}
	},
	deletepic : function () {
		if (this.savenum != null && !this.taking) {
			this.reEnable();
		}
	},
	reEnable : function () {
		this.resetGUI();
		removechilds("savepic");
		this.savenum = null;
	},
	resetGUI: function () {
		$("cameratake").src = "images/camera/take2.gif";
		$("cameradel").src = "images/camera/del1.gif";
		$("camerasave").src = "images/camera/save1.gif";
		$("cameracaptionimg").src = "images/camera/caption1.gif";
		HideContent("camera_caption");
		$("trashtext").focus();
	},
	captionreg : function () {
		var reg = new RegExp("<(.|\n)*?>");
  		$("camera_caption").value = $("camera_caption").value.replace(reg, "");
  		textCounter($('camera_caption'),76);
	},
	showpicture : function (id) {
		if (!this.pictureloading) {
			$('picturecaption').innerHTML = "";
			set_top("picture");
			ShowContent("picture");
			camera.pictureloader = getBusyOverlay($("picturepic"),{color: 'black', opacity: 0.5},{color: '#fff', size: 48});
			packets.send.openpicture(id);
			camera.pictureloading = true;
		}
	},
	loadpicture : function (code, caption) {
		if (this.pictureloading) {
			$('picturecaption').innerHTML = caption;
			camera.picturepic = document.createElement("img");
			camera.picturepic.src = "photos/"+code+".gif";
			camera.picturepic.style.border = '1px solid black';
			camera.picturepic.style.position = "absolute";
			camera.pictureloaded();
		}
	},
	pictureloaded : function () {
		if (this.picturepic.complete) {
			// show cached image
			camera.pictureloader.remove();
			camera.pictureloading = false;
			$("picturepic").appendChild(this.picturepic);
		}
		else {
			setTimeout(function(){camera.pictureloaded();},10);
		}
	}	
};

var help = {
	 report : function() {
		var message = prompt("Enter your report...");
		if (message != null && message != "")
		{
			packets.send.report(message);
		}
		else
		{
			client.notify("Abuse of the <strong>help reports</strong> will result in a ban");
		}
	}
};

/*jslint white: false, onevar: true, undef: true, nomen: false, eqeqeq: true, plusplus: false, bitwise: true, regexp: false, newcap: true, immed: true */
/*global window, SM2_DEFER, sm2Debugger, console, document, navigator, setTimeout, setInterval, clearInterval, Audio */

(function(window) {

var soundManager = null;

function SoundManager(smURL, smID) {

  this.flashVersion = 8;             // version of flash to require, either 8 or 9. Some API features require Flash 9.
  this.debugMode = false;             // enable debugging output (div#soundmanager-debug, OR console if available+configured)
  this.debugFlash = false;           // enable debugging output inside SWF, troubleshoot Flash/browser issues
  this.useConsole = true;            // use firebug/safari console.log()-type debug console if available
  this.consoleOnly = false;          // if console is being used, do not create/write to #soundmanager-debug
  this.waitForWindowLoad = false;    // force SM2 to wait for window.onload() before trying to call soundManager.onload()
  this.nullURL = 'about:blank';      // path to "null" (empty) MP3 file, used to unload sounds (Flash 8 only)
  this.allowPolling = true;          // allow flash to poll for status update (required for whileplaying() events, peak, sound spectrum functions to work.)
  this.useFastPolling = false;       // uses lower flash timer interval for higher callback frequency, best combined with useHighPerformance
  this.useMovieStar = true;          // enable support for Flash 9.0r115+ (codename "MovieStar") MPEG4 audio formats (AAC, M4V, FLV, MOV etc.)
  this.bgColor = '#ffffff';          // movie (.swf) background color, eg. '#000000'
  this.useHighPerformance = false;   // position:fixed flash movie can help increase js/flash speed, minimize lag
  this.flashPollingInterval = null;  // msec for polling interval. Defaults to 50 unless useFastPolling = true.
  this.flashLoadTimeout = 1000;      // msec to wait for flash movie to load before failing (0 = infinity)
  this.wmode = null;                 // string: flash rendering mode - null, transparent, opaque (last two allow layering of HTML on top)
  this.allowScriptAccess = 'always'; // for scripting the SWF (object/embed property), either 'always' or 'sameDomain'
  this.useFlashBlock = false;        // *requires flashblock.css, see demos* - allow recovery from flash blockers. Wait indefinitely and apply timeout CSS to SWF, if applicable.
  this.useHTML5Audio = false;        // Beta feature: Use HTML5 Audio() where API is supported (most Safari, Chrome versions), Firefox (no MP3/MP4.) Ideally, transparent vs. Flash API where possible.
  this.html5Test = /^probably$/i;    // HTML5 Audio().canPlayType() test. /^(probably|maybe)$/i if you want to be more liberal/risky.
  this.useGlobalHTML5Audio = true;   // (experimental) if true, re-use single HTML5 audio object across all sounds. Enabled by default on mobile devices/iOS.
  this.requireFlash = false;         // (experimental) if true, prevents "HTML5-only" mode when flash present. Allows flash to handle RTMP/serverURL, but HTML5 for other cases

  this.audioFormats = {
    // determines HTML5 support, flash requirements
    // eg. if MP3 or MP4 required, Flash fallback is used if HTML5 can't play it
    // shotgun approach to MIME testing due to browser variance
    'mp3': {
      'type': ['audio/mpeg; codecs="mp3"','audio/mpeg','audio/mp3','audio/MPA','audio/mpa-robust'],
      'required': true
    },
    'mp4': {
      'related': ['aac','m4a'], // additional formats under the MP4 container
      'type': ['audio/mp4; codecs="mp4a.40.2"','audio/aac','audio/x-m4a','audio/MP4A-LATM','audio/mpeg4-generic'],
      'required': true
    },
    'ogg': {
      'type': ['audio/ogg; codecs=vorbis'],
      'required': false
    },
    'wav': {
      'type': ['audio/wav; codecs="1"','audio/wav','audio/wave','audio/x-wav'],
      'required': false
    }
  };

  this.defaultOptions = {
    'autoLoad': false,             // enable automatic loading (otherwise .load() will be called on demand with .play(), the latter being nicer on bandwidth - if you want to .load yourself, you also can)
    'stream': true,                // allows playing before entire file has loaded (recommended)
    'autoPlay': false,             // enable playing of file as soon as possible (much faster if "stream" is true)
    'loops': 1,                    // how many times to repeat the sound (position will wrap around to 0, setPosition() will break out of loop when >0)
    'onid3': null,                 // callback function for "ID3 data is added/available"
    'onload': null,                // callback function for "load finished"
    'whileloading': null,          // callback function for "download progress update" (X of Y bytes received)
    'onplay': null,                // callback for "play" start
    'onpause': null,               // callback for "pause"
    'onresume': null,              // callback for "resume" (pause toggle)
    'whileplaying': null,          // callback during play (position update)
    'onstop': null,                // callback for "user stop"
    'onfailure': null,             // callback function for when playing fails
    'onfinish': null,              // callback function for "sound finished playing"
    'onbeforefinish': null,        // callback for "before sound finished playing (at [time])"
    'onbeforefinishtime': 5000,    // offset (milliseconds) before end of sound to trigger beforefinish (eg. 1000 msec = 1 second)
    'onbeforefinishcomplete': null,// function to call when said sound finishes playing
    'onjustbeforefinish': null,    // callback for [n] msec before end of current sound
    'onjustbeforefinishtime': 200, // [n] - if not using, set to 0 (or null handler) and event will not fire.
    'multiShot': true,             // let sounds "restart" or layer on top of each other when played multiple times, rather than one-shot/one at a time
    'multiShotEvents': false,      // fire multiple sound events (currently onfinish() only) when multiShot is enabled
    'position': null,              // offset (milliseconds) to seek to within loaded sound data.
    'pan': 0,                      // "pan" settings, left-to-right, -100 to 100
    'type': null,                  // MIME-like hint for file pattern / canPlay() tests, eg. audio/mp3
    'usePolicyFile': false,        // enable crossdomain.xml request for audio on remote domains (for ID3/waveform access)
    'volume': 100                  // self-explanatory. 0-100, the latter being the max.
  };

  this.flash9Options = {      // flash 9-only options, merged into defaultOptions if flash 9 is being used
    'isMovieStar': null,      // "MovieStar" MPEG4 audio mode. Null (default) = auto detect MP4, AAC etc. based on URL. true = force on, ignore URL
    'usePeakData': false,     // enable left/right channel peak (level) data
    'useWaveformData': false, // enable sound spectrum (raw waveform data) - WARNING: CPU-INTENSIVE: may set CPUs on fire.
    'useEQData': false,       // enable sound EQ (frequency spectrum data) - WARNING: Also CPU-intensive.
    'onbufferchange': null,   // callback for "isBuffering" property change
    'ondataerror': null       // callback for waveform/eq data access error (flash playing audio in other tabs/domains)
  };

  this.movieStarOptions = { // flash 9.0r115+ MPEG4 audio options, merged into defaultOptions if flash 9+movieStar mode is enabled
    'bufferTime': 3,        // seconds of data to buffer before playback begins (null = flash default of 0.1 seconds - if AAC playback is gappy, try increasing.)
    'serverURL': null,      // rtmp: FMS or FMIS server to connect to, required when requesting media via RTMP or one of its variants
    'onconnect': null,      // rtmp: callback for connection to flash media server
    'duration': null        // rtmp: song duration (msec)
  };

  this.version = null;
  this.versionNumber = 'V2.97a.20110123';
  this.movieURL = null;
  this.url = (smURL || null);
  this.altURL = null;
  this.swfLoaded = false;
  this.enabled = false;
  this.o = null;
  this.movieID = 'sm2-container';
  this.id = (smID || 'sm2movie');
  this.swfCSS = {
    'swfBox': 'sm2-object-box',
    'swfDefault': 'movieContainer',
    'swfError': 'swf_error', // SWF loaded, but SM2 couldn't start (other error)
    'swfTimedout': 'swf_timedout',
    'swfLoaded': 'swf_loaded',
    'swfUnblocked': 'swf_unblocked', // or loaded OK
    'sm2Debug': 'sm2_debug',
    'highPerf': 'high_performance',
    'flashDebug': 'flash_debug'
  };
  this.oMC = null;
  this.sounds = {};
  this.soundIDs = [];
  this.muted = false;
  this.debugID = 'soundmanager-debug';
  this.debugURLParam = /([#?&])debug=1/i;
  this.specialWmodeCase = false;
  this.didFlashBlock = false;

  this.filePattern = null;
  this.filePatterns = {
    'flash8': /\.mp3(\?.*)?$/i,
    'flash9': /\.mp3(\?.*)?$/i
  };

  this.baseMimeTypes = /^\s*audio\/(?:x-)?(?:mp(?:eg|3))\s*(?:$|;)/i; // mp3
  this.netStreamMimeTypes = /^\s*audio\/(?:x-)?(?:mp(?:eg|3))\s*(?:$|;)/i; // mp3, mp4, aac etc.
  this.netStreamTypes = ['aac', 'flv', 'mov', 'mp4', 'm4v', 'f4v', 'm4a', 'mp4v', '3gp', '3g2']; // Flash v9.0r115+ "moviestar" formats
  this.netStreamPattern = new RegExp('\\.(' + this.netStreamTypes.join('|') + ')(\\?.*)?$', 'i');
  this.mimePattern = this.baseMimeTypes;

  this.features = {
    'buffering': false,
    'peakData': false,
    'waveformData': false,
    'eqData': false,
    'movieStar': false
  };

  this.sandbox = {
    /*
    'type': null,
    'types': {
      'remote': 'remote (domain-based) rules',
      'localWithFile': 'local with file access (no internet access)',
      'localWithNetwork': 'local with network (internet access only, no local access)',
      'localTrusted': 'local, trusted (local+internet access)'
    },
    'description': null,
    'noRemote': null,
    'noLocal': null
    */
  };

  this.hasHTML5 = null; // switch for handling logic
  this.html5 = { // stores canPlayType() results, etc. treat as read-only.
    // mp3: boolean
    // mp4: boolean
    'usingFlash': null // set if/when flash fallback is needed
  };
  this.ignoreFlash = false; // used for special cases (eg. iPad/iPhone/palm OS?)

  // --- private SM2 internals ---

  var SMSound,
  _s = this, _sm = 'soundManager', _smc = _sm+'::', _h5 = 'HTML5::', _id, _ua = navigator.userAgent, _wl = window.location.href.toString(), _fV = this.flashVersion, _doc = document, _win = window, _doNothing, _init, _on_queue = [], _debugOpen = true, _debugTS, _didAppend = false, _appendSuccess = false, _didInit = false, _disabled = false, _windowLoaded = false, _wDS, _wdCount = 0, _initComplete, _mixin, _addOnEvent, _processOnEvents, _initUserOnload, _go, _delayWaitForEI, _waitForEI, _setVersionInfo, _handleFocus, _beginInit, _strings, _initMovie, _dcLoaded, _didDCLoaded, _getDocument, _createMovie, _die, _setPolling, _debugLevels = ['log', 'info', 'warn', 'error'], _defaultFlashVersion = 8, _disableObject, _failSafely, _normalizeMovieURL, _oRemoved = null, _oRemovedHTML = null, _str, _flashBlockHandler, _getSWFCSS, _toggleDebug, _loopFix, _policyFix, _complain, _idCheck, _waitingForEI = false, _initPending = false, _smTimer, _onTimer, _startTimer, _stopTimer, _needsFlash = null, _featureCheck, _html5OK, _html5Only = false, _html5CanPlay, _html5Ext,  _dcIE, _testHTML5, _event, _slice = Array.prototype.slice, _HTML5_networkStates, _HTML5_readyStates, _HTML5_errorCodes, _useGlobalHTML5Audio = false, _hasFlash, _detectFlash, _badSafariFix,
  _is_pre = _ua.match(/pre\//i), _is_iDevice = _ua.match(/(ipad|iphone|ipod)/i), _isMobile = (_ua.match(/mobile/i) || _is_pre || _is_iDevice), _isIE = _ua.match(/MSIE/i), _isWebkit = _ua.match(/webkit/i), _isSafari = (_ua.match(/safari/i) && !_ua.match(/chrome/i)),
  _isBadSafari = (!_wl.match(/usehtml5audio/i) && !_wl.match(/sm2\-ignorebadua/i) && _isSafari && _ua.match(/OS X 10_6_(3|4|5|6)/i)), // Safari 4 and 5 occasionally fail to load/play HTML5 audio on Snow Leopard due to bug(s) in QuickTime X and/or other underlying frameworks. :/ Known Apple "radar" bug. https://bugs.webkit.org/show_bug.cgi?id=32159
  _hasConsole = (typeof console !== 'undefined' && typeof console.log !== 'undefined'), _isFocused = (typeof _doc.hasFocus !== 'undefined'?_doc.hasFocus():null), _tryInitOnFocus = (typeof _doc.hasFocus === 'undefined' && _isSafari), _okToDisable = !_tryInitOnFocus;

  this._use_maybe = (_wl.match(/sm2\-useHTML5Maybe\=1/i)); // temporary feature: #sm2-useHTML5Maybe=1 forces loose canPlay() check
  this._overHTTP = (_doc.location?_doc.location.protocol.match(/http/i):null);
  this.useAltURL = !this._overHTTP; // use altURL if not "online"
  this._global_a = null;

  if (_is_iDevice || _is_pre) {
    // during HTML5 beta period (off by default), may as well force it on Apple + Palm, flash support unlikely
    _s.useHTML5Audio = true;
    _s.ignoreFlash = true;
    if (_s.useGlobalHTML5Audio) {
      _useGlobalHTML5Audio = true;
    }
  }

  if (_is_pre || this._use_maybe) {
    // less-strict canPlayType() checking option
    _s.html5Test = /^(probably|maybe)$/i;
  }

  // Temporary feature: allow force of HTML5 via URL: #sm2-usehtml5audio=0 or 1
  /*
  (function(){
    var a = '#sm2-usehtml5audio=', l = _wl, b = null;
    if (l.indexOf(a) !== -1) {
      b = (l.charAt(l.indexOf(a)+a.length) === '1');
      if (typeof console !== 'undefined' && typeof console.log !== 'undefined') {
        console.log((b?'Enabling ':'Disabling ')+'useHTML5Audio via URL parameter');
      }
      _s.useHTML5Audio = b;
    }
  }());
  */

  // --- public API methods ---

  this.ok = function() {
    return (_needsFlash?(_didInit && !_disabled):(_s.useHTML5Audio && _s.hasHTML5));
  };

  this.supported = this.ok; // legacy

  this.getMovie = function(smID) {
    return _isIE?_win[smID]:(_isSafari?_id(smID) || _doc[smID]:_id(smID));
  };

  this.createSound = function(oOptions) {
    var _cs = _sm+'.createSound(): ',
    thisOptions = null, oSound = null, _tO = null;
    if (!_didInit || !_s.ok()) {
      _complain(_cs + _str(!_didInit?'notReady':'notOK'));
      return false;
    }
    if (arguments.length === 2) {
      // function overloading in JS! :) ..assume simple createSound(id,url) use case
      oOptions = {
        'id': arguments[0],
        'url': arguments[1]
      };
    }
    thisOptions = _mixin(oOptions); // inherit from defaultOptions
    _tO = thisOptions; // alias
    /*
    if (_tO.id.toString().charAt(0).match(/^[0-9]$/)) {
      //_s._wD(_cs + _str('badID', _tO.id), 2);
    }
    //_s._wD(_cs + _tO.id + ' (' + _tO.url + ')', 1);
    */
    if (_idCheck(_tO.id, true)) {
      //_s._wD(_cs + _tO.id + ' exists', 1);
      return _s.sounds[_tO.id];
    }

    function make() {
      thisOptions = _loopFix(thisOptions);
      _s.sounds[_tO.id] = new SMSound(_tO);
      _s.soundIDs.push(_tO.id);
      return _s.sounds[_tO.id];
    }

    if (_html5OK(_tO)) {
      oSound = make();
      //_s._wD('Loading sound '+_tO.id+' via HTML5');
      oSound._setup_html5(_tO);
    } else {
      if (_fV > 8 && _s.useMovieStar) {
        if (_tO.isMovieStar === null) {
          _tO.isMovieStar = ((_tO.serverURL || (_tO.type?_tO.type.match(_s.netStreamPattern):false)||_tO.url.match(_s.netStreamPattern))?true:false);
        }
        if (_tO.isMovieStar) {
          //_s._wD(_cs + 'using MovieStar handling');
        }
        if (_tO.isMovieStar) {
          if (_tO.usePeakData) {
            //_wDS('noPeak');
            _tO.usePeakData = false;
          }
          if (_tO.loops > 1) {
            //_wDS('noNSLoop');
          }
        }
      }
      _tO = _policyFix(_tO, _cs);
      oSound = make();
      if (_fV === 8) {
        _s.o._createSound(_tO.id, _tO.onjustbeforefinishtime, _tO.loops||1, _tO.usePolicyFile);
      } else {
        _s.o._createSound(_tO.id, _tO.url, _tO.onjustbeforefinishtime, _tO.usePeakData, _tO.useWaveformData, _tO.useEQData, _tO.isMovieStar, (_tO.isMovieStar?_tO.bufferTime:false), _tO.loops||1, _tO.serverURL, _tO.duration||null, _tO.autoPlay, true, _tO.autoLoad, _tO.usePolicyFile);
        if (!_tO.serverURL) {
          // We are connected immediately
          oSound.connected = true;
          if (_tO.onconnect) {
            _tO.onconnect.apply(oSound);
          }
        }
      }

      if ((_tO.autoLoad || _tO.autoPlay) && !_tO.serverURL) {
        oSound.load(_tO); // call load for non-rtmp streams
      }
    }

    if (_tO.autoPlay && !_tO.serverURL) { // rtmp will play in onconnect
      oSound.play();
    }
    return oSound;
  };

  this.destroySound = function(sID, _bFromSound) {
    // explicitly destroy a sound before normal page unload, etc.
    if (!_idCheck(sID)) {
      return false;
    }
    var oS = _s.sounds[sID], i;
    oS._iO = {}; // Disable all callbacks while the sound is being destroyed
    oS.stop();
    oS.unload();
    for (i = 0; i < _s.soundIDs.length; i++) {
      if (_s.soundIDs[i] === sID) {
        _s.soundIDs.splice(i, 1);
        break;
      }
    }
    if (!_bFromSound) {
      // ignore if being called from SMSound instance
      oS.destruct(true);
    }
    oS = null;
    delete _s.sounds[sID];
    return true;
  };

  this.load = function(sID, oOptions) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].load(oOptions);
  };

  this.unload = function(sID) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].unload();
  };

  this.play = function(sID, oOptions) {
    var fN = _sm+'.play(): ';
    if (!_didInit || !_s.ok()) {
      _complain(fN + _str(!_didInit?'notReady':'notOK'));
      return false;
    }
    if (!_idCheck(sID)) {
      if (!(oOptions instanceof Object)) {
        oOptions = {
          url: oOptions
        }; // overloading use case: play('mySound','/path/to/some.mp3');
      }
      if (oOptions && oOptions.url) {
        // overloading use case, create+play: .play('someID',{url:'/path/to.mp3'});
        //_s._wD(fN + 'attempting to create "' + sID + '"', 1);
        oOptions.id = sID;
        return _s.createSound(oOptions).play();
      } else {
        return false;
      }
    }
    return _s.sounds[sID].play(oOptions);
  };

  this.start = this.play; // just for convenience

  this.setPosition = function(sID, nMsecOffset) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].setPosition(nMsecOffset);
  };

  this.stop = function(sID) {
    if (!_idCheck(sID)) {
      return false;
    }
    //_s._wD(_sm+'.stop(' + sID + ')', 1);
    return _s.sounds[sID].stop();
  };

  this.stopAll = function() {
    //_s._wD(_sm+'.stopAll()', 1);
    for (var oSound in _s.sounds) {
      if (_s.sounds[oSound] instanceof SMSound) {
        _s.sounds[oSound].stop(); // apply only to sound objects
      }
    }
  };

  this.pause = function(sID) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].pause();
  };

  this.pauseAll = function() {
    for (var i = _s.soundIDs.length; i--;) {
      _s.sounds[_s.soundIDs[i]].pause();
    }
  };

  this.resume = function(sID) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].resume();
  };

  this.resumeAll = function() {
    for (var i = _s.soundIDs.length; i--;) {
      _s.sounds[_s.soundIDs[i]].resume();
    }
  };

  this.togglePause = function(sID) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].togglePause();
  };

  this.setPan = function(sID, nPan) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].setPan(nPan);
  };

  this.setVolume = function(sID, nVol) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].setVolume(nVol);
  };

  this.mute = function(sID) {
    var fN = _sm+'.mute(): ',
    i = 0;
    if (typeof sID !== 'string') {
      sID = null;
    }
    if (!sID) {
      //_s._wD(fN + 'Muting all sounds');
      for (i = _s.soundIDs.length; i--;) {
        _s.sounds[_s.soundIDs[i]].mute();
      }
      _s.muted = true;
    } else {
      if (!_idCheck(sID)) {
        return false;
      }
      //_s._wD(fN + 'Muting "' + sID + '"');
      return _s.sounds[sID].mute();
    }
    return true;
  };

  this.muteAll = function() {
    _s.mute();
  };

  this.unmute = function(sID) {
    var fN = _sm+'.unmute(): ', i;
    if (typeof sID !== 'string') {
      sID = null;
    }
    if (!sID) {
      //_s._wD(fN + 'Unmuting all sounds');
      for (i = _s.soundIDs.length; i--;) {
        _s.sounds[_s.soundIDs[i]].unmute();
      }
      _s.muted = false;
    } else {
      if (!_idCheck(sID)) {
        return false;
      }
      //_s._wD(fN + 'Unmuting "' + sID + '"');
      return _s.sounds[sID].unmute();
    }
    return true;
  };

  this.unmuteAll = function() {
    _s.unmute();
  };

  this.toggleMute = function(sID) {
    if (!_idCheck(sID)) {
      return false;
    }
    return _s.sounds[sID].toggleMute();
  };

  this.getMemoryUse = function() {
    if (_fV === 8) {
      return 0;
    }
    if (_s.o) {
      return parseInt(_s.o._getMemoryUse(), 10);
    }
  };

  this.disable = function(bNoDisable) {
    // destroy all functions
    if (typeof bNoDisable === 'undefined') {
      bNoDisable = false;
    }
    if (_disabled) {
      return false;
    }
    _disabled = true;
    //_wDS('shutdown', 1);
    for (var i = _s.soundIDs.length; i--;) {
      _disableObject(_s.sounds[_s.soundIDs[i]]);
    }
    _initComplete(bNoDisable); // fire "complete", despite fail
    _event.remove(_win, 'load', _initUserOnload);
    return true;
  };

  this.canPlayMIME = function(sMIME) {
    var result;
    if (_s.hasHTML5) {
      result = _html5CanPlay({type:sMIME});
    }
    if (!_needsFlash || result) {
      // no flash, or OK
      return result;
    } else {
      return (sMIME?(sMIME.match(_s.mimePattern)?true:false):null);
    }
  };

  this.canPlayURL = function(sURL) {
    var result;
    if (_s.hasHTML5) {
      result = _html5CanPlay(sURL);
    }
    if (!_needsFlash || result) {
      // no flash, or OK
      return result;
    } else {
      return (sURL?(sURL.match(_s.filePattern)?true:false):null);
    }
  };

  this.canPlayLink = function(oLink) {
    if (typeof oLink.type !== 'undefined' && oLink.type) {
      if (_s.canPlayMIME(oLink.type)) {
        return true;
      }
    }
    return _s.canPlayURL(oLink.href);
  };

  this.getSoundById = function(sID, suppressDebug) {
    if (!sID) {
      throw new Error(_sm+'.getSoundById(): sID is null/undefined');
    }
    var result = _s.sounds[sID];
    if (!result && !suppressDebug) {
      //_s._wD('"' + sID + '" is an invalid sound ID.', 2);
    }
    return result;
  };

  this.onready = function(oMethod, oScope) {
    var sType = 'onready';
    if (oMethod && oMethod instanceof Function) {
      if (_didInit) {
        //_wDS('queue', sType);
      }
      if (!oScope) {
        oScope = _win;
      }
      _addOnEvent(sType, oMethod, oScope);
      _processOnEvents();
      return true;
    } else {
      throw _str('needFunction', sType);
    }
  };

  this.ontimeout = function(oMethod, oScope) {
    var sType = 'ontimeout';
    if (oMethod && oMethod instanceof Function) {
      if (_didInit) {
        //_wDS('queue');
      }
      if (!oScope) {
        oScope = _win;
      }
      _addOnEvent(sType, oMethod, oScope);
      _processOnEvents({type:sType});
      return true;
    } else {
      throw _str('needFunction', sType);
    }
  };

  this.getMoviePercent = function() {
    return (_s.o && typeof _s.o.PercentLoaded !== 'undefined'?_s.o.PercentLoaded():null);
  };

  this._writeDebug = function(sText, sType, bTimestamp) {
    // pseudo-private console.log()-style output
    /*
    var sDID = 'soundmanager-debug', o, oItem, sMethod;
    if (!_s.debugMode) {
      return false;
    }
    if (typeof bTimestamp !== 'undefined' && bTimestamp) {
      sText = sText + ' | ' + new Date().getTime();
    }
    if (_hasConsole && _s.useConsole) {
      sMethod = _debugLevels[sType];
      if (typeof console[sMethod] !== 'undefined') {
        console[sMethod](sText);
      } else {
        console.log(sText);
      }
      if (_s.useConsoleOnly) {
        return true;
      }
    }
    try {
      o = _id(sDID);
      if (!o) {
        return false;
      }
      oItem = _doc.createElement('div');
      if (++_wdCount % 2 === 0) {
        oItem.className = 'sm2-alt';
      }
      if (typeof sType === 'undefined') {
        sType = 0;
      } else {
        sType = parseInt(sType, 10);
      }
      oItem.appendChild(_doc.createTextNode(sText));
      if (sType) {
        if (sType >= 2) {
          oItem.style.fontWeight = 'bold';
        }
        if (sType === 3) {
          oItem.style.color = '#ff3333';
        }
      }
      // o.appendChild(oItem); // top-to-bottom
      o.insertBefore(oItem, o.firstChild); // bottom-to-top
    } catch(e) {
      // oh well
    }
    o = null;
    */
    return true;
  };
  this._wD = this._writeDebug; // alias

  this._debug = function() {
    /*
    //_wDS('currentObj', 1);
    for (var i = 0, j = _s.soundIDs.length; i < j; i++) {
      _s.sounds[_s.soundIDs[i]]._debug();
    }
    */
  };

  this.reboot = function() {
    // attempt to reset and init SM2
    //_s._wD(_sm+'.reboot()');
    if (_s.soundIDs.length) {
      //_s._wD('Destroying ' + _s.soundIDs.length + ' SMSound objects...');
    }
    var i, j;
    for (i = _s.soundIDs.length; i--;) {
      _s.sounds[_s.soundIDs[i]].destruct();
    }
    // trash ze flash
    try {
      if (_isIE) {
        _oRemovedHTML = _s.o.innerHTML;
      }
      _oRemoved = _s.o.parentNode.removeChild(_s.o);
      //_s._wD('Flash movie removed.');
    } catch(e) {
      // uh-oh.
      //_wDS('badRemove', 2);
    }
    // actually, force recreate of movie.
    _oRemovedHTML = _oRemoved = null;
    _s.enabled = _didInit = _waitingForEI = _initPending = _didAppend = _appendSuccess = _disabled = _s.swfLoaded = false;
    _s.soundIDs = _s.sounds = [];
    _s.o = null;
    for (i in _on_queue) {
      if (_on_queue.hasOwnProperty(i)) {
        for (j = _on_queue[i].length; j--;) {
          _on_queue[i][j].fired = false;
        }
      }
    }
    //_s._wD(_sm + ': Rebooting...');
    _win.setTimeout(function() {
      _s.beginDelayedInit();
    }, 20);
  };

  this.destruct = function() {
    //_s._wD(_sm+'.destruct()');
    _s.disable(true);
  };

  this.beginDelayedInit = function() {
    // //_s._wD(_sm+'.beginDelayedInit()');
    _windowLoaded = true;
   _dcLoaded();
    setTimeout(_beginInit, 20);
    _delayWaitForEI();
  };


  // Wrap html5 event handlers so we don't call them on destroyed sounds
  function _html5_event(oFn) {
    return function(e) {
      if (!this._t || !this._t._a) {
        if (this._t && this._t.sID) {
          //_s._wD(_h5+'ignoring '+e.type+': '+this._t.sID);
        } else {
          //_s._wD(_h5+'ignoring '+e.type);
        }
        return null;
      } else {
        return oFn.call(this, e);
      }
    };
  }

  this._html5_events = {

    // HTML5 event-name-to-handler map
    abort: _html5_event(function(e) {
      //_s._wD(_h5+'abort: '+this._t.sID);
    }),

    // enough has loaded to play
    canplay: _html5_event(function(e) {
      //_s._wD(_h5+'canplay: '+this._t.sID+', '+this._t.url);
      this._t._onbufferchange(0);
      var position1K = (!isNaN(this._t.position)?this._t.position/1000:null);
      // set the position if position was set before the sound loaded
      this._t._html5_canplay = true;
      if (this._t.position && this.currentTime !== position1K) {
        //_s._wD(_h5+'canplay: setting position to '+position1K+'');
        try {
          this.currentTime = position1K;
        } catch(ee) {
          //_s._wD(_h5+'setting position failed: '+ee.message, 2);
        }
      }
    }),

    load: _html5_event(function(e) {
      if (!this._t.loaded) {
        this._t._onbufferchange(0);
        // should be 1, and the same
        this._t._whileloading(this._t.bytesTotal, this._t.bytesTotal, this._t._get_html5_duration());
        this._t._onload(true);
      }
    }),

    emptied: _html5_event(function(e) {
      //_s._wD(_h5+'emptied: '+this._t.sID);
    }),

    ended: _html5_event(function(e) {
      //_s._wD(_h5+'ended: '+this._t.sID);
      this._t._onfinish();
    }),

    error: _html5_event(function(e) {
      //_s._wD(_h5+'error: '+this.error.code);
      // call load with error state?
      this._t._onload(false);
    }),

    loadeddata: _html5_event(function(e) {
      //_s._wD(_h5+'loadeddata: '+this._t.sID);
    }),

    loadedmetadata: _html5_event(function(e) {
      //_s._wD(_h5+'loadedmetadata: '+this._t.sID);
    }),

    loadstart: _html5_event(function(e) {
      //_s._wD(_h5+'loadstart: '+this._t.sID);
      // assume buffering at first
      this._t._onbufferchange(1);
    }),

    play: _html5_event(function(e) {
      //_s._wD(_h5+'play: '+this._t.sID+', '+this._t.url);
      // once play starts, no buffering
      this._t._onbufferchange(0);
    }),

    // TODO: verify if this is actually implemented anywhere yet.
    playing: _html5_event(function(e) {
      //_s._wD(_h5+'playing: '+this._t.sID+', '+this._t.url);
      // once play starts, no buffering
      this._t._onbufferchange(0);
    }),

    progress: _html5_event(function(e) {

      if (this._t.loaded) {
        return false;
      }

      var i, j, str, loadSum = 0, buffered = 0,
          isProgress = (e.type === 'progress'),
          ranges = e.target.buffered,
          loaded = (e.loaded||0), // firefox 3.6 implements e.loaded/total (bytes)
          total = (e.total||1);

      if (ranges && ranges.length) {

        // if loaded is 0, try TimeRanges implementation as % of load
        // https://developer.mozilla.org/en/DOM/TimeRanges
        for (i=ranges.length; i--;) {
          buffered = (ranges.end(i) - ranges.start(i));
        }

        // linear case, buffer sum; does not account for seeking and HTTP partials / byte ranges
        loaded = buffered/e.target.duration;

        /*
        if (isProgress && ranges.length > 1) {
          str = [];
          j = ranges.length;
          for (i=0; i<j; i++) {
            str.push(e.target.buffered.start(i) +'-'+ e.target.buffered.end(i));
          }
          //_s._wD(_h5+'progress: timeRanges: '+str.join(', '));
        }
        */

        if (isProgress) {
          //_s._wD(_h5+'progress: '+this._t.sID+': ' + Math.floor(loaded*100)+'% loaded');
        }

      }

      this._t._onbufferchange(0); // if progress, likely not buffering
      this._t._whileloading(loaded, total, this._t._get_html5_duration());

      if (loaded && total && loaded === total) {
        // in case "onload" doesn't fire (eg. gecko 1.9.2)
        _s._html5_events.load.call(this, e);
      }

    }),

    ratechange: _html5_event(function(e) {
      //_s._wD(_h5+'ratechange: '+this._t.sID);
    }),

    suspend: _html5_event(function(e) {
      // download paused/stopped, may have finished (eg. onload)
      //_s._wD(_h5+'suspend: '+this._t.sID);
      _s._html5_events.progress.call(this, e);
    }),

    stalled: _html5_event(function(e) {
      //_s._wD(_h5+'stalled: '+this._t.sID);
    }),

    timeupdate: _html5_event(function(e) {
      this._t._onTimer();
    }),

    waiting: _html5_event(function(e) { // see also: seeking
      //_s._wD(_h5+'waiting: '+this._t.sID);
      // playback faster than download rate, etc.
      this._t._onbufferchange(1);
    })
  };

  /*

  _HTML5_networkStates = {
    0: 'NETWORK_EMPTY',
    1: 'NETWORK_IDLE',
    2: 'NETWORK_LOADING',
    3: 'NETWORK_LOADED',
    4: 'NETWORK_NO_SOURCE' // @see http://www.chipwreck.de/blog/2010/03/01/html-5-video-dom-attributes-and-events/
  };

  _HTML5_readyStates = {
    0: 'HAVE_NOTHING',
    1: 'HAVE_METADATA',
    2: 'HAVE_CURRENT_DATA',
    3: 'HAVE_FUTURE_DATA',
    4: 'HAVE_ENOUGH_DATA'
  };

  _HTML5_errorCodes = {
    1: 'MEDIA_ERR_ABORTED', // User aborted playback
    2: 'MEDIA_ERR_NETWORK', // Network error (could not read the stream)
    3: 'MEDIA_ERR_DECODE',  // Decoding error
    4: 'MEDIA_ERR_SRC_NOT_SUPPORTED' // Unsupported format
  };

  */

  // --- SMSound (sound object) instance ---

  SMSound = function(oOptions) {
    var _t = this, _resetProperties, _stop_html5_timer, _start_html5_timer;
    this.sID = oOptions.id;
    this.url = oOptions.url;
    this.options = _mixin(oOptions);
    this.instanceOptions = this.options; // per-play-instance-specific options
    this._iO = this.instanceOptions; // short alias
    // assign property defaults
    this.pan = this.options.pan;
    this.volume = this.options.volume;
    this._lastURL = null;
    this.isHTML5 = false;
    this._a = null;

    // --- public methods ---

    this.id3 = {};

    this._debug = function() {
      /*
      // pseudo-private console.log()-style output
      if (_s.debugMode) {
        var stuff = null, msg = [], sF, sfBracket, maxLength = 64;
        for (stuff in _t.options) {
          if (_t.options[stuff] !== null) {
            if (_t.options[stuff] instanceof Function) {
              // handle functions specially
              sF = _t.options[stuff].toString();
              sF = sF.replace(/\s\s+/g, ' '); // normalize spaces
              sfBracket = sF.indexOf('{');
              msg.push(' ' + stuff + ': {' + sF.substr(sfBracket + 1, (Math.min(Math.max(sF.indexOf('\n') - 1, maxLength), maxLength))).replace(/\n/g, '') + '... }');
            } else {
              msg.push(' ' + stuff + ': ' + _t.options[stuff]);
            }
          }
        }
        //_s._wD('SMSound() merged options: {\n' + msg.join(', \n') + '\n}');
      }
      */
    };

    this._debug();

    this.load = function(oOptions) {
      var oS = null;
      if (typeof oOptions !== 'undefined') {
        _t._iO = _mixin(oOptions);
        _t.instanceOptions = _t._iO;
      } else {
        oOptions = _t.options;
        _t._iO = oOptions;
        _t.instanceOptions = _t._iO;
        if (_t._lastURL && _t._lastURL !== _t.url) {
          //_wDS('manURL');
          _t._iO.url = _t.url;
          _t.url = null;
        }
      }
      if (!_t._iO.url) {
        _t._iO.url = _t.url;
      }
      //_s._wD('SMSound.load(): ' + _t._iO.url, 1);
      if (_t._iO.url === _t.url && _t.readyState !== 0 && _t.readyState !== 2) {
        //_wDS('onURL', 1);
        return _t;
      }
      _t._lastURL = _t.url;
      _t.loaded = false;
      _t.readyState = 1;
      _t.playState = 0;
      if (_html5OK(_t._iO)) {
        oS = _t._setup_html5(_t._iO);
        if (!oS._called_load) {
          //_s._wD(_h5+'load: '+_t.sID);
          oS.load();
          oS._called_load = true;
          if (_t._iO.autoPlay) {
            _t.play();
          }
        } else {
          //_s._wD('HTML5 ignoring request to load again: '+_t.sID);
        }
      } else {
        try {
          _t.isHTML5 = false;
          _t._iO = _policyFix(_loopFix(_t._iO));
          if (_fV === 8) {
            _s.o._load(_t.sID, _t._iO.url, _t._iO.stream, _t._iO.autoPlay, (_t._iO.whileloading?1:0), _t._iO.loops||1, _t._iO.usePolicyFile);
          } else {
            _s.o._load(_t.sID, _t._iO.url, _t._iO.stream?true:false, _t._iO.autoPlay?true:false, _t._iO.loops||1, _t._iO.autoLoad?true:false, _t._iO.usePolicyFile);
          }
        } catch(e) {
          //_wDS('smError', 2);
          //_debugTS('onload', false);
          _die();
        }
      }
      return _t;
    };

    this.unload = function() {
      // Flash 8/AS2 can't "close" a stream - fake it by loading an empty MP3
      // Flash 9/AS3: Close stream, preventing further load
      if (_t.readyState !== 0) {
        //_s._wD('SMSound.unload(): "' + _t.sID + '"');
        if (!_t.isHTML5) {
          if (_fV === 8) {
            _s.o._unload(_t.sID, _s.nullURL);
          } else {
            _s.o._unload(_t.sID);
          }
        } else {
          _stop_html5_timer();
          if (_t._a) {
            // abort()-style method here, stop loading? (doesn't exist?)
            _t._a.pause();
// if (!_useGlobalHTML5Audio || (_useGlobalHTML5Audio && _t.playState)) { // if global audio, only unload if actively playing
            _t._a.src = ''; // https://developer.mozilla.org/En/Using_audio_and_video_in_Firefox#Stopping_the_download_of_media
// }
          }
        }
        // reset load/status flags
        _resetProperties();
      }
      return _t;
    };

    this.destruct = function(_bFromSM) {
      //_s._wD('SMSound.destruct(): "' + _t.sID + '"');
      if (!_t.isHTML5) {
        // kill sound within Flash
        // Disable the onfailure handler
        _t._iO.onfailure = null;
        _s.o._destroySound(_t.sID);
      } else {
        _stop_html5_timer();
        if (_t._a) {
          // abort()-style method here, stop loading? (doesn't exist?)
          _t._a.pause();
          _t._a.src = ''; // https://developer.mozilla.org/En/Using_audio_and_video_in_Firefox#Stopping_the_download_of_media
          if (!_useGlobalHTML5Audio) {
            _t._remove_html5_events();
          }
        }
      }
      if (!_bFromSM) {
        _s.destroySound(_t.sID, true); // ensure deletion from controller
      }
    };

    this.play = function(oOptions, _updatePlayState) {
      var fN = 'SMSound.play(): ', allowMulti;
      _updatePlayState = _updatePlayState === undefined ? true : _updatePlayState; // default true
      if (!oOptions) {
        oOptions = {};
      }
      _t._iO = _mixin(oOptions, _t._iO);
      _t._iO = _mixin(_t._iO, _t.options);
      _t.instanceOptions = _t._iO;
      if (_t._iO.serverURL) {
        if (!_t.connected) {
          if (!_t.getAutoPlay()) {
            //_s._wD(fN+' Netstream not connected yet - setting autoPlay');
            _t.setAutoPlay(true);
          }
          return _t; // play will be called in _onconnect()
        }
      }
      if (_html5OK(_t._iO)) {
        _t._setup_html5(_t._iO);
        _start_html5_timer();
      }
      if (_t.playState === 1 && !_t.paused) {
        allowMulti = _t._iO.multiShot;
        if (!allowMulti) {
          //_s._wD(fN + '"' + _t.sID + '" already playing (one-shot)', 1);
          return _t;
        } else {
          //_s._wD(fN + '"' + _t.sID + '" already playing (multi-shot)', 1);
          if (_t.isHTML5) {
            // TODO: BUG?
            _t.setPosition(_t._iO.position);
          }
        }
      }
      if (!_t.loaded) {
        if (_t.readyState === 0) {
          //_s._wD(fN + 'Attempting to load "' + _t.sID + '"', 1);
          // try to get this sound playing ASAP
          if (!_t.isHTML5) {
            _t._iO.autoPlay = true; // assign directly because setAutoPlay() increments the instanceCount
            _t.load(_t._iO);
          } else {
            _t.load(_t._iO);
            // _t.readyState = 1; // redundant
          }
        } else if (_t.readyState === 2) {
          //_s._wD(fN + 'Could not load "' + _t.sID + '" - exiting', 2);
          return _t;
        } else {
          //_s._wD(fN + '"' + _t.sID + '" is loading - attempting to play..', 1);
        }
      } else {
        //_s._wD(fN + '"' + _t.sID + '"');
      }
      // Streams will pause when their buffer is full if they are being loaded.
      // In this case paused is true, but the song hasn't started playing yet. If
      // we just call resume() the onplay() callback will never be called.  So
      // only call resume() if the position is > 0.
      // Another reason is because options like volume won't have been applied yet.
      if (_t.paused && _t.position && _t.position > 0) { // https://gist.github.com/37b17df75cc4d7a90bf6
        //_s._wD(fN + '"' + _t.sID + '" is resuming from paused state',1);
        _t.resume();
      } else {
        //_s._wD(fN+'"'+ _t.sID+'" is starting to play');
        _t.playState = 1;
        _t.paused = false;
        if (!_t.instanceCount || _t._iO.multiShotEvents || (_fV > 8 && !_t.isHTML5 && !_t.getAutoPlay())) {
          _t.instanceCount++;
        }
        _t.position = (typeof _t._iO.position !== 'undefined' && !isNaN(_t._iO.position)?_t._iO.position:0);
        if (!_t.isHTML5) {
          _t._iO = _policyFix(_loopFix(_t._iO));
        }
        if (_t._iO.onplay && _updatePlayState) {
          _t._iO.onplay.apply(_t);
          _t._onplay_called = true;
        }
        _t.setVolume(_t._iO.volume, true);
        _t.setPan(_t._iO.pan, true);
        if (!_t.isHTML5) {
          _s.o._start(_t.sID, _t._iO.loops || 1, (_fV === 9?_t.position:_t.position / 1000));
        } else {
          _start_html5_timer();
          _t._setup_html5().play();
        }
      }
      return _t;
    };

    this.start = this.play; // just for convenience

    this.stop = function(bAll) {
      if (_t.playState === 1) {
        _t._onbufferchange(0);
        _t.resetOnPosition(0);
        if (!_t.isHTML5) {
          _t.playState = 0;
        }
        _t.paused = false;
        if (_t._iO.onstop) {
          _t._iO.onstop.apply(_t);
        }
        if (!_t.isHTML5) {
          _s.o._stop(_t.sID, bAll);
          // hack for netStream: just unload
          if (_t._iO.serverURL) {
            _t.unload();
          }
        } else {
          if (_t._a) {
            _t.setPosition(0); // act like Flash, though
            _t._a.pause(); // html5 has no stop()
            _t.playState = 0;
            _t._onTimer(); // and update UI
            _stop_html5_timer();
            _t.unload();
          }
        }
        _t.instanceCount = 0;
        _t._iO = {};
      }
      return _t;
    };

    this.setAutoPlay = function(autoPlay) {
      //_s._wD('sound '+_t.sID+' turned autoplay ' + (autoPlay ? 'on' : 'off'));
      _t._iO.autoPlay = autoPlay;
      if (_t.isHTML5) {
        if (_t._a && autoPlay) {
          _t.play(); // HTML5 onload isn't reliable
        }
      } else {
        _s.o._setAutoPlay(_t.sID, autoPlay);
      }
      if (autoPlay) {
        // only increment the instanceCount if the sound isn't loaded (TODO: verify RTMP)
        if (!_t.instanceCount && _t.readyState === 1) {
          _t.instanceCount++;
          //_s._wD('sound '+_t.sID+' incremented instance count to '+_t.instanceCount);
        }
      }
    };

    this.getAutoPlay = function() {
      return _t._iO.autoPlay;
    };

    this.setPosition = function(nMsecOffset, bNoDebug) {
      if (nMsecOffset === undefined) {
        nMsecOffset = 0;
      }
      // Use the duration from the instance options, if we don't have a track duration yet.
      var original_pos, position, position1K, offset = (_t.isHTML5 ? Math.max(nMsecOffset,0) : Math.min(_t.duration || _t._iO.duration, Math.max(nMsecOffset, 0))); // position >= 0 and <= current available (loaded) duration
      original_pos = _t.position;
      _t.position = offset;
      position1K = _t.position/1000;
      _t.resetOnPosition(_t.position);
      _t._iO.position = offset;
      if (!_t.isHTML5) {
        position = _fV === 9 ? _t.position : position1K;
        _s.o._setPosition(_t.sID, position, (_t.paused || !_t.playState)); // if paused or not playing, will not resume (by playing)
      } else if (_t._a) {
        // Set the position in the canplay handler if the sound is not ready yet
        if (_t._html5_canplay) {
          if (_t._a.currentTime !== position1K) {
            // Only set the position if we need to.
            // DOM/JS errors/exceptions to watch out for:
            // if seek is beyond (loaded?) position, "DOM exception 11"
            // "INDEX_SIZE_ERR": DOM exception 1
            //_s._wD('setPosition('+position1K+'): setting position');
            try {
              _t._a.currentTime = position1K;
            } catch(e) {
              //_s._wD('setPosition('+position1K+'): setting position failed: '+e.message, 2);
            }
          }
        } else {
          //_s._wD('setPosition('+position1K+'): delaying, sound not ready');
        }
      }
      if (_t.isHTML5) {
        if (_t.paused) { // if paused, refresh UI right away
          _t._onTimer(true); // force update
        }
      }
      return _t;
    };

    this.pause = function(bCallFlash) {
      if (_t.paused || (_t.playState === 0 && _t.readyState !== 1)) {
        return _t;
      }
      //_s._wD('SMSound.pause()');
      _t.paused = true;
      if (!_t.isHTML5) {
        if (bCallFlash || bCallFlash === undefined) {
          _s.o._pause(_t.sID);
        }
      } else {
        _t._setup_html5().pause();
        _stop_html5_timer();
      }
      if (_t._iO.onpause) {
        _t._iO.onpause.apply(_t);
      }
      return _t;
    };

    // When auto-loaded streams pause on buffer full they have a playState of 0.
    // We need to make sure that the playState is set to 1 when these streams "resume".
    //
    // When a paused stream is resumed, we need to trigger the onplay() callback if it
    // hasn't been called already.  In this case since the sound is being played for the
    // first time, I think it's more appropriate to call onplay() rather than onresume().
    this.resume = function() {
      if (!_t.paused) {
        return _t;
      }
      //_s._wD('SMSound.resume()');
      _t.paused = false;
      _t.playState = 1;
      if (!_t.isHTML5) {
        _s.o._pause(_t.sID); // flash method is toggle-based (pause/resume)
        if (_t._iO.isMovieStar && _isWebkit) {
          // Bizarre Webkit bug (Chrome reported via 8tracks.com dudes): AAC content paused for 30+ seconds(?) will not resume without a reposition.
          _t.setPosition(_t.position);
        }
      } else {
        _t._setup_html5().play();
        _start_html5_timer();
      }
      if (!_t._onplay_called && _t._iO.onplay) {
        _t._iO.onplay.apply(_t);
        _t._onplay_called = true;
      } else if (_t._iO.onresume) {
        _t._iO.onresume.apply(_t);
      }
      return _t;
    };

    this.togglePause = function() {
      //_s._wD('SMSound.togglePause()');
      if (_t.playState === 0) {
        _t.play({
          position: (_fV === 9 && !_t.isHTML5 ? _t.position:_t.position / 1000)
        });
        return _t;
      }
      if (_t.paused) {
        _t.resume();
      } else {
        _t.pause();
      }
      return _t;
    };

    this.setPan = function(nPan, bInstanceOnly) {
      if (typeof nPan === 'undefined') {
        nPan = 0;
      }
      if (typeof bInstanceOnly === 'undefined') {
        bInstanceOnly = false;
      }
      if (!_t.isHTML5) {
        _s.o._setPan(_t.sID, nPan);
      } // else { no HTML5 pan? }
      _t._iO.pan = nPan;
      if (!bInstanceOnly) {
        _t.pan = nPan;
      }
      return _t;
    };

    this.setVolume = function(nVol, bInstanceOnly) {
      if (typeof nVol === 'undefined') {
        nVol = 100;
      }
      if (typeof bInstanceOnly === 'undefined') {
        bInstanceOnly = false;
      }
      if (!_t.isHTML5) {
        _s.o._setVolume(_t.sID, (_s.muted && !_t.muted) || _t.muted?0:nVol);
      } else if (_t._a) {
        _t._a.volume = nVol/100;
      }
      _t._iO.volume = nVol;
      if (!bInstanceOnly) {
        _t.volume = nVol;
      }
      return _t;
    };

    this.mute = function() {
      _t.muted = true;
      if (!_t.isHTML5) {
        _s.o._setVolume(_t.sID, 0);
      } else if (_t._a) {
        _t._a.muted = true;
      }
      return _t;
    };

    this.unmute = function() {
      _t.muted = false;
      var hasIO = typeof _t._iO.volume !== 'undefined';
      if (!_t.isHTML5) {
        _s.o._setVolume(_t.sID, hasIO?_t._iO.volume:_t.options.volume);
      } else if (_t._a) {
        _t._a.muted = false;
      }
      return _t;
    };

    this.toggleMute = function() {
      return (_t.muted?_t.unmute():_t.mute());
    };

    this.onposition = function(nPosition, oMethod, oScope) {
      // TODO: allow for ranges, too? eg. (nPosition instanceof Array)
      _t._onPositionItems.push({
        position: nPosition,
        method: oMethod,
        scope: (typeof oScope !== 'undefined'?oScope:_t),
        fired: false
      });
      return _t;
    };

    this.processOnPosition = function() {
      var i, item, j = _t._onPositionItems.length;
      if (!j || !_t.playState || _t._onPositionFired >= j) {
        return false;
      }
      for (i=j; i--;) {
        item = _t._onPositionItems[i];
        if (!item.fired && _t.position >= item.position) {
          item.method.apply(item.scope,[item.position]);
          item.fired = true;
          _s._onPositionFired++;
        }
      }
      return true;
    };

    this.resetOnPosition = function(nPosition) {
      // reset "fired" for items interested in this position
      var i, item, j = _t._onPositionItems.length;
      if (!j) {
        return false;
      }
      for (i=j; i--;) {
        item = _t._onPositionItems[i];
        if (item.fired && nPosition <= item.position) {
          item.fired = false;
          _s._onPositionFired--;
        }
      }
      return true;
    };

    // pseudo-private soundManager reference

    this._onTimer = function(bForce) {
      // HTML5-only _whileplaying() etc.
      var time, x = {};
      if (_t._hasTimer || bForce) {
        if (_t._a && (bForce || ((_t.playState > 0 || _t.readyState === 1) && !_t.paused))) { // TODO: May not need to track readyState (1 = loading)
          _t.duration = _t._get_html5_duration();
          _t.durationEstimate = _t.duration;
          time = _t._a.currentTime?_t._a.currentTime*1000:0;
          _t._whileplaying(time,x,x,x,x);
          return true;
        } else {
         //_s._wD('_onTimer: Warn for "'+_t.sID+'": '+(!_t._a?'Could not find element. ':'')+(_t.playState === 0?'playState bad, 0?':'playState = '+_t.playState+', OK'));
          return false;
        }
      }
    };

    // --- private internals ---

    this._get_html5_duration = function() {
      var d = (_t._a ? _t._a.duration*1000 : (_t._iO ? _t._iO.duration : undefined));
      return (d && !isNaN(d) && d !== Infinity ? d : (_t._iO ? _t._iO.duration : null));
    };

    _start_html5_timer = function() {
      if (_t.isHTML5) {
        _startTimer(_t);
      }
    };

    _stop_html5_timer = function() {
      if (_t.isHTML5) {
        _stopTimer(_t);
      }
    };

    _resetProperties = function(bLoaded) {
      _t._onPositionItems = [];
      _t._onPositionFired = 0;
      _t._hasTimer = null;
      _t._onplay_called = false;
      _t._a = null;
      _t._html5_canplay = false;
      _t.bytesLoaded = null;
      _t.bytesTotal = null;
      _t.position = null;
      _t.duration = (_t._iO && _t._iO.duration?_t._iO.duration:null);
      _t.durationEstimate = null;
      _t.failures = 0;
      _t.loaded = false;
      _t.playState = 0;
      _t.paused = false;
      _t.readyState = 0; // 0 = uninitialised, 1 = loading, 2 = failed/error, 3 = loaded/success
      _t.muted = false;
      _t.didBeforeFinish = false;
      _t.didJustBeforeFinish = false;
      _t.isBuffering = false;
      _t.instanceOptions = {};
      _t.instanceCount = 0;
      _t.peakData = {
        left: 0,
        right: 0
      };
      _t.waveformData = {
        left: [],
        right: []
      };
      _t.eqData = []; // legacy: 1D array
      _t.eqData.left = [];
      _t.eqData.right = [];
    };

    _resetProperties();

    // pseudo-private methods used by soundManager

    this._setup_html5 = function(oOptions) {
      var _iO = _mixin(_t._iO, oOptions), d = decodeURI,
          _a = _useGlobalHTML5Audio ? _s._global_a : _t._a;
      if (_a) {
        if (_a._t && d(_a.src) === d(_iO.url)) {
          return _a; // same url, ignore request
        }
        //_s._wD('setting new URL on existing object: '+_iO.url);
        /*
         * "First things first, I, Poppa.." (reset the previous state of the old sound, if playing)
         * Fixes case with devices that can only play one sound at a time
         * Otherwise, other sounds in mid-play will be terminated without warning and in a stuck state
         */
        if (_useGlobalHTML5Audio && _a._t.playState && _a._t && _a._t.url && d(_a._t.url) !== d(_iO.url)) {
          _a._t.stop();
        }
        _a.src = _iO.url;
      } else {
        //_s._wD('creating HTML5 Audio() element with URL: '+_iO.url);
        _a = new Audio(_iO.url);
        if (_useGlobalHTML5Audio) {
          _s._global_a = _a;
        }
      }
      _a._called_load = false;
      _t.isHTML5 = true;
      _t._a = _a; // store a ref on the track
      _a._t = _t; // store a ref on the audio
      _t._add_html5_events();
      _a.loop = (_iO.loops>1?'loop':'');
      if (_iO.autoLoad || _iO.autoPlay) {
        _a.autobuffer = 'auto'; // early HTML5 implementation (non-standard)
        _a.preload = 'auto'; // standard
        _t.load();
      } else {
        _a.autobuffer = false; // early HTML5 implementation (non-standard)
        _a.preload = 'none'; // standard
      }
      _a.loop = (_iO.loops>1?'loop':''); // boolean instead of "loop", for webkit? - spec says string. http://www.w3.org/TR/html-markup/audio.html#audio.attrs.loop
      return _a;
    };

    // related private methods

    this._add_html5_events = function() {
      if (_t._a._added_events) {
        return false;
      }

      var f;

      function add(oEvt, oFn, bCapture) {
        return _t._a ? _t._a.addEventListener(oEvt, oFn, bCapture||false) : null;
      }

      //_s._wD(_h5+'adding event listeners: '+_t.sID);
      _t._a._added_events = true;

      for (f in _s._html5_events) {
        if (_s._html5_events.hasOwnProperty(f)) {
          add(f, _s._html5_events[f]);
        }
      }

      /*
      // Debug issues with HTML5 streams.  Look for changes in audio object
      // attributes every few milliseconds.
      if (!_t._a._debug_interval) {
        _t._a._debug_last_values = {};
        _t._a._debug_interval = setInterval(function() {
          var _a = _t._a, rs, ns, ec;
          if (_a && _a._t) {
            rs = _a.readyState;
            ns = _a.networkState;
            if (_a._debug_last_values.readyState !== rs) {
              _a._debug_last_values.readyState = rs;
              //_s._wD(_h5+'readyState '+ (_HTML5_readyStates[rs] || rs)+', sound '+_t.sID);
            }
            if (_a._debug_last_values.networkState !== ns) {
              _a._debug_last_values.networkState = ns;
              //_s._wD(_h5+'networkState '+ (_HTML5_networkStates[ns] || ns)+', sound '+_t.sID);
            }
            if (_a.error && _a._debug_last_values.errorCode !== _a.error.code) {
              ec = _a.error.code;
              _a._debug_last_values.errorCode = ec;
              //_s._wD(_h5+'errorCode '+ (_HTML5_errorCodes[ec] || ec)+', sound '+_t.sID);
            }
          }
        }, 50);
      }
      */
      return true;
    };

    // Keep this externally accessible
    this._remove_html5_events = function() {
      // Remove event listeners
      function remove(oEvt, oFn, bCapture) {
        return (_t._a ? _t._a.removeEventListener(oEvt, oFn, bCapture||false) : null);
      }
      //_s._wD(_h5+'removing event listeners: '+_t.sID);
      _t._a._added_events = false;

      for (var f in _s._html5_events) {
        if (_s._html5_events.hasOwnProperty(f)) {
          remove(f, _s._html5_events[f]);
        }
      }

      /*
      // Clear the extra HTML5 logging interval
      if (_t._a._debug_interval) {
        clearInterval(_t._a._debug_interval);
        _t._a._debug_interval = undefined;
      }
      */
    };

    // --- "private" methods called by Flash ---

    this._whileloading = function(nBytesLoaded, nBytesTotal, nDuration, nBufferLength) {
      _t.bytesLoaded = nBytesLoaded;
      _t.bytesTotal = nBytesTotal;
      _t.duration = Math.floor(nDuration);
      _t.bufferLength = nBufferLength;
      if (!_t._iO.isMovieStar) {
        if (_t._iO.duration) {
          // use options, if specified and larger
          _t.durationEstimate = (_t.duration > _t._iO.duration) ? _t.duration : _t._iO.duration;
        } else {
          _t.durationEstimate = parseInt((_t.bytesTotal / _t.bytesLoaded) * _t.duration, 10);
        }
        if (_t.durationEstimate === undefined) {
          _t.durationEstimate = _t.duration;
        }
        if (_t.readyState !== 3 && _t._iO.whileloading) {
          _t._iO.whileloading.apply(_t);
        }
      } else {
        _t.durationEstimate = _t.duration;
        if (_t.readyState !== 3 && _t._iO.whileloading) {
          _t._iO.whileloading.apply(_t);
        }
      }
    };

    this._onid3 = function(oID3PropNames, oID3Data) {
      // oID3PropNames: string array (names)
      // ID3Data: string array (data)
      //_s._wD('SMSound._onid3(): "' + this.sID + '" ID3 data received.');
      var oData = [], i, j;
      for (i = 0, j = oID3PropNames.length; i < j; i++) {
        oData[oID3PropNames[i]] = oID3Data[i];
      }
      _t.id3 = _mixin(_t.id3, oData);
      if (_t._iO.onid3) {
        _t._iO.onid3.apply(_t);
      }
    };

    this._whileplaying = function(nPosition, oPeakData, oWaveformDataLeft, oWaveformDataRight, oEQData) {
      if (isNaN(nPosition) || nPosition === null) {
        return false; // flash safety net
      }
      if (_t.playState === 0 && nPosition > 0) {
        // invalid position edge case for end/stop
        nPosition = 0;
      }
      _t.position = nPosition;
      _t.processOnPosition();
      if (_fV > 8 && !_t.isHTML5) {
        if (_t._iO.usePeakData && typeof oPeakData !== 'undefined' && oPeakData) {
          _t.peakData = {
            left: oPeakData.leftPeak,
            right: oPeakData.rightPeak
          };
        }
        if (_t._iO.useWaveformData && typeof oWaveformDataLeft !== 'undefined' && oWaveformDataLeft) {
          _t.waveformData = {
            left: oWaveformDataLeft.split(','),
            right: oWaveformDataRight.split(',')
          };
        }
        if (_t._iO.useEQData) {
          if (typeof oEQData !== 'undefined' && oEQData && oEQData.leftEQ) {
            var eqLeft = oEQData.leftEQ.split(',');
            _t.eqData = eqLeft;
            _t.eqData.left = eqLeft;
            if (typeof oEQData.rightEQ !== 'undefined' && oEQData.rightEQ) {
              _t.eqData.right = oEQData.rightEQ.split(',');
            }
          }
        }
      }
      if (_t.playState === 1) {
        // special case/hack: ensure buffering is false if loading from cache (and not yet started)
        if (!_t.isHTML5 && _s.flashVersion === 8 && !_t.position && _t.isBuffering) {
          _t._onbufferchange(0);
        }
        if (_t._iO.whileplaying) {
          _t._iO.whileplaying.apply(_t); // flash may call after actual finish
        }
        if ((_t.loaded || (!_t.loaded && _t._iO.isMovieStar)) && _t._iO.onbeforefinish && _t._iO.onbeforefinishtime && !_t.didBeforeFinish && _t.duration - _t.position <= _t._iO.onbeforefinishtime) {
          _t._onbeforefinish();
        }
      }
      return true;
    };

    // Only applies to RTMP
    this._onconnect = function(bSuccess) {
      var fN = 'SMSound._onconnect(): ';
      bSuccess = (bSuccess === 1);
      //_s._wD(fN+'"'+_t.sID+'"'+(bSuccess?' connected.':' failed to connect? - '+_t.url), (bSuccess?1:2));
      _t.connected = bSuccess;
      if (bSuccess) {
        _t.failures = 0;
        if (_idCheck(_t.sID)) {
          if (_t.getAutoPlay()) {
            _t.play(undefined, _t.getAutoPlay()); // only update the play state if auto playing
          } else if (_t._iO.autoLoad) {
            _t.load();
          }
        }
        if (_t._iO.onconnect) {
          _t._iO.onconnect.apply(_t,[bSuccess]);
        }
      }
    };

    this._onload = function(nSuccess) {
      var fN = 'SMSound._onload(): ', loadOK = (nSuccess?true:false);
      //_s._wD(fN + '"' + _t.sID + '"' + (loadOK?' loaded.':' failed to load? - ' + _t.url), (loadOK?1:2));
      /*
      if (!loadOK && !_t.isHTML5) {
        if (_s.sandbox.noRemote === true) {
          //_s._wD(fN + _str('noNet'), 1);
        }
        if (_s.sandbox.noLocal === true) {
          //_s._wD(fN + _str('noLocal'), 1);
        }
      }
      */
      _t.loaded = loadOK;
      _t.readyState = loadOK?3:2;
      _t._onbufferchange(0);
      if (_t._iO.onload) {
        _t._iO.onload.apply(_t, [loadOK]);
      }
      return true;
    };

    // fire onfailure() only once at most
    // at this point we just recreate failed sounds rather than trying to reconnect.
    this._onfailure = function(msg, level, code) {
      _t.failures++;
      //_s._wD('SMSound._onfailure(): "'+_t.sID+'" count '+_t.failures);
      if (_t._iO.onfailure && _t.failures === 1) {
        _t._iO.onfailure(_t, msg, level, code);
      } else {
        //_s._wD('SMSound._onfailure(): ignoring');
      }
    };

    this._onbeforefinish = function() {
      if (!_t.didBeforeFinish) {
        _t.didBeforeFinish = true;
        if (_t._iO.onbeforefinish) {
          //_s._wD('SMSound._onbeforefinish(): "' + _t.sID + '"');
          _t._iO.onbeforefinish.apply(_t);
        }
      }
    };

    this._onjustbeforefinish = function(msOffset) {
      if (!_t.didJustBeforeFinish) {
        _t.didJustBeforeFinish = true;
        if (_t._iO.onjustbeforefinish) {
          //_s._wD('SMSound._onjustbeforefinish(): "' + _t.sID + '"');
          _t._iO.onjustbeforefinish.apply(_t);
        }
      }
    };

    this._onfinish = function() {
      // //_s._wD('SMSound._onfinish(): "' + _t.sID + '" got instanceCount '+_t.instanceCount);
      _t._onbufferchange(0);
      _t.resetOnPosition(0);
      if (_t._iO.onbeforefinishcomplete) {
        _t._iO.onbeforefinishcomplete.apply(_t);
      }
      // reset some state items
      _t.didBeforeFinish = false;
      _t.didJustBeforeFinish = false;
      if (_t.instanceCount) {
        _t.instanceCount--;
        if (!_t.instanceCount) {
          // reset instance options
          _t.playState = 0;
          _t.paused = false;
          _t.instanceCount = 0;
          _t.instanceOptions = {};
          _stop_html5_timer();
        }
        if (!_t.instanceCount || _t._iO.multiShotEvents) {
          // fire onfinish for last, or every instance
          if (_t._iO.onfinish) {
            //_s._wD('SMSound._onfinish(): "' + _t.sID + '"');
            _t._iO.onfinish.apply(_t);
          }
        }
      }
    };

    this._onbufferchange = function(nIsBuffering) {
      var fN = 'SMSound._onbufferchange()';
      if (_t.playState === 0) {
        // ignore if not playing
        return false;
      }
      if ((nIsBuffering && _t.isBuffering) || (!nIsBuffering && !_t.isBuffering)) {
        return false;
      }
      _t.isBuffering = (nIsBuffering === 1);
      if (_t._iO.onbufferchange) {
        //_s._wD(fN + ': ' + nIsBuffering);
        _t._iO.onbufferchange.apply(_t);
      }
      return true;
    };

    this._ondataerror = function(sError) {
      // flash 9 wave/eq data handler
      if (_t.playState > 0) { // hack: called at start, and end from flash at/after onfinish()
        //_s._wD('SMSound._ondataerror(): ' + sError);
        if (_t._iO.ondataerror) {
          _t._iO.ondataerror.apply(_t);
        }
      }
    };

  }; // SMSound()

  // --- private SM2 internals ---

  _getDocument = function() {
    return (_doc.body?_doc.body:(_doc._docElement?_doc.documentElement:_doc.getElementsByTagName('div')[0]));
  };

  _id = function(sID) {
    return _doc.getElementById(sID);
  };

  _mixin = function(oMain, oAdd) {
    // non-destructive merge
    var o1 = {}, i, o2, o;
    for (i in oMain) { // clone c1
      if (oMain.hasOwnProperty(i)) {
        o1[i] = oMain[i];
      }
    }
    o2 = (typeof oAdd === 'undefined'?_s.defaultOptions:oAdd);
    for (o in o2) {
      if (o2.hasOwnProperty(o) && typeof o1[o] === 'undefined') {
        o1[o] = o2[o];
      }
    }
    return o1;
  };

  _event = (function() {

    var old = (_win.attachEvent),
    evt = {
      add: (old?'attachEvent':'addEventListener'),
      remove: (old?'detachEvent':'removeEventListener')
    };

    function getArgs(oArgs) {
      var args = _slice.call(oArgs), len = args.length;
      if (old) {
        args[1] = 'on' + args[1]; // prefix
        if (len > 3) {
          args.pop(); // no capture
        }
      } else if (len === 3) {
        args.push(false);
      }
      return args;
    }

    function apply(args, sType) {
      var element = args.shift(),
          method = [evt[sType]];
      if (old) {
        element[method](args[0], args[1]);
      } else {
        element[method].apply(element, args);
      }
    }

    function add() {
      apply(getArgs(arguments), 'add');
    }

    function remove() {
      apply(getArgs(arguments), 'remove');
    }

    return {
      'add': add,
      'remove': remove
    };

  }());

  _html5OK = function(iO) {
    return (!iO.serverURL && (iO.type?_html5CanPlay({type:iO.type}):_html5CanPlay(iO.url)||_html5Only)); // Use type, if specified. If HTML5-only mode, no other options, so just give 'er
  };

  _html5CanPlay = function(sURL) {
    // try to find MIME, test and return truthiness
    if (!_s.useHTML5Audio || !_s.hasHTML5) {
      return false;
    }
    var result, mime, offset, fileExt, item, aF = _s.audioFormats;
    if (!_html5Ext) {
      _html5Ext = [];
      for (item in aF) {
        if (aF.hasOwnProperty(item)) {
          _html5Ext.push(item);
          if (aF[item].related) {
            _html5Ext = _html5Ext.concat(aF[item].related);
          }
        }
      }
      _html5Ext = new RegExp('\\.('+_html5Ext.join('|')+')','i');
    }
    mime = (typeof sURL.type !== 'undefined'?sURL.type:null);
    fileExt = (typeof sURL === 'string'?sURL.toLowerCase().match(_html5Ext):null); // TODO: Strip URL queries, etc.
    if (!fileExt || !fileExt.length) {
      if (!mime) {
        return false;
      } else {
        // audio/mp3 -> mp3, result should be known
        offset = mime.indexOf(';');
        fileExt = (offset !== -1?mime.substr(0,offset):mime).substr(6); // strip "audio/X; codecs.."
      }
    } else {
      fileExt = fileExt[0].substr(1); // "mp3", for example
    }
    if (fileExt && typeof _s.html5[fileExt] !== 'undefined') {
      // result known
      return _s.html5[fileExt];
    } else {
      if (!mime) {
        if (fileExt && _s.html5[fileExt]) {
          return _s.html5[fileExt];
        } else {
          // best-case guess, audio/whatever-dot-filename-format-you're-playing
          mime = 'audio/'+fileExt;
        }
      }
      result = _s.html5.canPlayType(mime);
      _s.html5[fileExt] = result;
      // //_s._wD('canPlayType, found result: '+result);
      return result;
    }
  };

  _testHTML5 = function() {
    if (!_s.useHTML5Audio || typeof Audio === 'undefined') {
      return false;
    }
    var a = (typeof Audio !== 'undefined' ? new Audio():null), item, support = {}, aF, i, _hasFlash = _detectFlash();
    function _cp(m) {
      var canPlay, i, j, isOK = false;
      if (!a || typeof a.canPlayType !== 'function') {
        return false;
      }
      if (m instanceof Array) {
        // iterate through all mime types, return any successes
        for (i=0, j=m.length; i<j && !isOK; i++) {
          if (_s.html5[m[i]] || a.canPlayType(m[i]).match(_s.html5Test)) {
            isOK = true;
            _s.html5[m[i]] = true;
          }
        }
        return isOK;
      } else {
        canPlay = (a && typeof a.canPlayType === 'function' ? a.canPlayType(m) : false);
        return (canPlay && (canPlay.match(_s.html5Test)?true:false));
      }
    }
    // test all registered formats + codecs
    aF = _s.audioFormats;
    for (item in aF) {
      if (aF.hasOwnProperty(item)) {
        support[item] = _cp(aF[item].type);
        // assign result to related formats, too
        if (aF[item] && aF[item].related) {
          for (i=aF[item].related.length; i--;) {
            _s.html5[aF[item].related[i]] = support[item];
          }
        }
      }
    }
    support.canPlayType = (a?_cp:null);
    _s.html5 = _mixin(_s.html5, support);
    return true;
  };

  _strings = {
    /*
    notReady: 'Not loaded yet - wait for soundManager.onload()/onready()',
    notOK: 'Audio support is not available.',
    appXHTML: _smc + 'createMovie(): appendChild/innerHTML set failed. May be app/xhtml+xml DOM-related.',
    spcWmode: _smc + 'createMovie(): Removing wmode, preventing win32 below-the-fold SWF loading issue',
    swf404: _sm + ': Verify that %s is a valid path.',
    tryDebug: 'Try ' + _sm + '.debugFlash = true for more security details (output goes to SWF.)',
    checkSWF: 'See SWF output for more debug info.',
    localFail: _sm + ': Non-HTTP page (' + _doc.location.protocol + ' URL?) Review Flash player security settings for this special case:\nhttp://www.macromedia.com/support/documentation/en/flashplayer/help/settings_manager04.html\nMay need to add/allow path, eg. c:/sm2/ or /users/me/sm2/',
    waitFocus: _sm + ': Special case: Waiting for focus-related event..',
    waitImpatient: _sm + ': Getting impatient, still waiting for Flash%s...',
    waitForever: _sm + ': Waiting indefinitely for Flash (will recover if unblocked)...',
    needFunction: _sm + ': Function object expected for %s',
    badID: 'Warning: Sound ID "%s" should be a string, starting with a non-numeric character',
    noMS: 'MovieStar mode not enabled. Exiting.',
    currentObj: '--- ' + _sm + '._debug(): Current sound objects ---',
    waitEI: _smc + 'initMovie(): Waiting for ExternalInterface call from Flash..',
    waitOnload: _sm + ': Waiting for window.onload()',
    docLoaded: _sm + ': Document already loaded',
    onload: _smc + 'initComplete(): calling soundManager.onload()',
    onloadOK: _sm + '.onload() complete',
    init: '-- ' + _smc + 'init() --',
    didInit: _smc + 'init(): Already called?',
    flashJS: _sm + ': Attempting to call Flash from JS..',
    noPolling: _sm + ': Polling (whileloading()/whileplaying() support) is disabled.',
    secNote: 'Flash security note: Network/internet URLs will not load due to security restrictions. Access can be configured via Flash Player Global Security Settings Page: http://www.macromedia.com/support/documentation/en/flashplayer/help/settings_manager04.html',
    badRemove: 'Warning: Failed to remove flash movie.',
    noPeak: 'Warning: peakData features unsupported for movieStar formats',
    shutdown: _sm + '.disable(): Shutting down',
    queue: _sm + ': Queueing %s handler',
    smFail: _sm + ': Failed to initialise.',
    smError: 'SMSound.load(): Exception: JS-Flash communication failed, or JS error.',
    fbTimeout: 'No flash response, applying .'+_s.swfCSS.swfTimedout+' CSS..',
    fbLoaded: 'Flash loaded',
    fbHandler: _smc+'flashBlockHandler()',
    manURL: 'SMSound.load(): Using manually-assigned URL',
    onURL: _sm + '.load(): current URL already assigned.',
    badFV: _sm + '.flashVersion must be 8 or 9. "%s" is invalid. Reverting to %s.',
    as2loop: 'Note: Setting stream:false so looping can work (flash 8 limitation)',
    noNSLoop: 'Note: Looping not implemented for MovieStar formats',
    needfl9: 'Note: Switching to flash 9, required for MP4 formats.',
    mfTimeout: 'Setting flashLoadTimeout = 0 (infinite) for off-screen, mobile flash case',
    mfOn: 'mobileFlash::enabling on-screen flash repositioning',
    policy: 'Enabling usePolicyFile for data access'
    */
  };

  _str = function() { // o [,items to replace]
    /*
    var args = _slice.call(arguments), // real array, please
    o = args.shift(), // first arg
    str = (_strings && _strings[o]?_strings[o]:''), i, j;
    if (str && args && args.length) {
      for (i = 0, j = args.length; i < j; i++) {
        str = str.replace('%s', args[i]);
      }
    }
    return str;
    */
  };

  _loopFix = function(sOpt) {
    // flash 8 requires stream = false for looping to work
    if (_fV === 8 && sOpt.loops > 1 && sOpt.stream) {
      //_wDS('as2loop');
      sOpt.stream = false;
    }
    return sOpt;
  };

  _policyFix = function(sOpt, sPre) {
    if (sOpt && !sOpt.usePolicyFile && (sOpt.onid3 || sOpt.usePeakData || sOpt.useWaveformData || sOpt.useEQData)) {
      //_s._wD((sPre?sPre+':':'') + _str('policy'));
      sOpt.usePolicyFile = true;
    }
    return sOpt;
  };

  _complain = function(sMsg) {
    if (typeof console !== 'undefined' && typeof console.warn !== 'undefined') {
      console.warn(sMsg);
    } else {
      //_s._wD(sMsg);
    }
  };

  _doNothing = function() {
    return false;
  };

  _disableObject = function(o) {
    for (var oProp in o) {
      if (o.hasOwnProperty(oProp) && typeof o[oProp] === 'function') {
        o[oProp] = _doNothing;
      }
    }
    oProp = null;
  };

  _failSafely = function(bNoDisable) {
    // general failure exception handler
    if (typeof bNoDisable === 'undefined') {
      bNoDisable = false;
    }
    if (_disabled || bNoDisable) {
      //_wDS('smFail', 2);
      _s.disable(bNoDisable);
    }
  };

  _normalizeMovieURL = function(smURL) {
    var urlParams = null;
    if (smURL) {
      if (smURL.match(/\.swf(\?.*)?$/i)) {
        urlParams = smURL.substr(smURL.toLowerCase().lastIndexOf('.swf?') + 4);
        if (urlParams) {
          return smURL; // assume user knows what they're doing
        }
      } else if (smURL.lastIndexOf('/') !== smURL.length - 1) {
        smURL = smURL + '/';
      }
    }
    return (smURL && smURL.lastIndexOf('/') !== - 1?smURL.substr(0, smURL.lastIndexOf('/') + 1):'./') + _s.movieURL;
  };

  _setVersionInfo = function() {
    if (_fV !== 8 && _fV !== 9) {
      //_s._wD(_str('badFV', _fV, _defaultFlashVersion));
      _s.flashVersion = _defaultFlashVersion;
    }
    var isDebug = (_s.debugMode || _s.debugFlash?'_debug.swf':'.swf'); // debug flash movie, if applicable
    if (_s.useHTML5Audio && !_html5Only && _s.audioFormats.mp4.required && _s.flashVersion < 9) {
      //_s._wD(_str('needfl9'));
      _s.flashVersion = 9;
    }
    _fV = _s.flashVersion; // short-hand for internal use
    _s.version = _s.versionNumber + (_html5Only?' (HTML5-only mode)':(_fV === 9?' (AS3/Flash 9)':' (AS2/Flash 8)'));
    // set up default options
    if (_fV > 8) {
      _s.defaultOptions = _mixin(_s.defaultOptions, _s.flash9Options);
      _s.features.buffering = true;
    }
    if (_fV > 8 && _s.useMovieStar) {
      // flash 9+ support for movieStar formats as well as MP3
      _s.defaultOptions = _mixin(_s.defaultOptions, _s.movieStarOptions);
      _s.filePatterns.flash9 = new RegExp('\\.(mp3|' + _s.netStreamTypes.join('|') + ')(\\?.*)?$', 'i');
      _s.mimePattern = _s.netStreamMimeTypes;
      _s.features.movieStar = true;
    } else {
      _s.useMovieStar = false;
      _s.features.movieStar = false;
    }
    _s.filePattern = _s.filePatterns[(_fV !== 8?'flash9':'flash8')];
    _s.movieURL = (_fV === 8?'soundmanager2.swf':'soundmanager2_flash9.swf').replace('.swf',isDebug);
    _s.features.peakData = _s.features.waveformData = _s.features.eqData = (_fV > 8);
  };

  _setPolling = function(bPolling, bHighPerformance) {
    if (!_s.o || !_s.allowPolling) {
      return false;
    }
    _s.o._setPolling(bPolling, bHighPerformance);
  };

  function _initDebug() {
    if (_s.debugURLParam.test(_wl)) {
      _s.debugMode = true; // allow force of debug mode via URL
    }
    /*
    if (_id(_s.debugID)) {
      return false;
    }
    var oD, oDebug, oTarget, oToggle, tmp;
    if (_s.debugMode && !_id(_s.debugID) && ((!_hasConsole || !_s.useConsole) || (_s.useConsole && _hasConsole && !_s.consoleOnly))) {
      oD = _doc.createElement('div');
      oD.id = _s.debugID + '-toggle';
      oToggle = {
        'position': 'fixed',
        'bottom': '0px',
        'right': '0px',
        'width': '1.2em',
        'height': '1.2em',
        'lineHeight': '1.2em',
        'margin': '2px',
        'textAlign': 'center',
        'border': '1px solid #999',
        'cursor': 'pointer',
        'background': '#fff',
        'color': '#333',
        'zIndex': 10001
      };
      oD.appendChild(_doc.createTextNode('-'));
      oD.onclick = _toggleDebug;
      oD.title = 'Toggle SM2 debug console';
      if (_ua.match(/msie 6/i)) {
        oD.style.position = 'absolute';
        oD.style.cursor = 'hand';
      }
      for (tmp in oToggle) {
        if (oToggle.hasOwnProperty(tmp)) {
          oD.style[tmp] = oToggle[tmp];
        }
      }
      oDebug = _doc.createElement('div');
      oDebug.id = _s.debugID;
      oDebug.style.display = (_s.debugMode?'block':'none');
      if (_s.debugMode && !_id(oD.id)) {
        try {
          oTarget = _getDocument();
          oTarget.appendChild(oD);
        } catch(e2) {
          throw new Error(_str('appXHTML'));
        }
        oTarget.appendChild(oDebug);
      }
    }
    oTarget = null;
    */
  }

  _createMovie = function(smID, smURL) {

    var specialCase = null,
    remoteURL = (smURL?smURL:_s.url),
    localURL = (_s.altURL?_s.altURL:remoteURL),
    oEmbed, oMovie, oTarget = _getDocument(), tmp, movieHTML, oEl, extraClass = _getSWFCSS(), s, x, sClass, side = '100%', isRTL = null, html = _doc.getElementsByTagName('html')[0];
    isRTL = (html && html.dir && html.dir.match(/rtl/i));
    smID = (typeof smID === 'undefined'?_s.id:smID);

    if (_didAppend && _appendSuccess) {
      return false; // ignore if already succeeded
    }

    function _initMsg() {
      //_s._wD('-- SoundManager 2 ' + _s.version + (!_html5Only && _s.useHTML5Audio?(_s.hasHTML5?' + HTML5 audio':', no HTML5 audio support'):'') + (!_html5Only ? (_s.useMovieStar?', MovieStar mode':'') + (_s.useHighPerformance?', high performance mode, ':', ') + (( _s.flashPollingInterval ? 'custom (' + _s.flashPollingInterval + 'ms)' : (_s.useFastPolling?'fast':'normal')) + ' polling') + (_s.wmode?', wmode: ' + _s.wmode:'') + (_s.debugFlash?', flash debug mode':'') + (_s.useFlashBlock?', flashBlock mode':'') : '') + ' --', 1);
    }

    if (_html5Only) {
      _setVersionInfo();
      _initMsg();
      _s.oMC = _id(_s.movieID);
      _init();
      // prevent multiple init attempts
      _didAppend = true;
      _appendSuccess = true;
      return false;
    }

    _didAppend = true;

    // safety check for legacy (change to Flash 9 URL)
    _setVersionInfo();
    _s.url = _normalizeMovieURL(_s._overHTTP?remoteURL:localURL);
    smURL = _s.url;

    _s.wmode = (!_s.wmode && _s.useHighPerformance && !_s.useMovieStar?'transparent':_s.wmode);

    if (_s.wmode !== null && !_isIE && !_s.useHighPerformance && navigator.platform.match(/win32/i)) {
      _s.specialWmodeCase = true;
      // extra-special case: movie doesn't load until scrolled into view when using wmode = anything but 'window' here
      // does not apply when using high performance (position:fixed means on-screen), OR infinite flash load timeout
      //_wDS('spcWmode');
      _s.wmode = null;
    }

    oEmbed = {
      'name': smID,
      'id': smID,
      'src': smURL,
      'width': side,
      'height': side,
      'quality': 'high',
      'allowScriptAccess': _s.allowScriptAccess,
      'bgcolor': _s.bgColor,
      'pluginspage': 'http://www.macromedia.com/go/getflashplayer',
      'type': 'application/x-shockwave-flash',
      'wmode': _s.wmode
    };

    if (_s.debugFlash) {
      oEmbed.FlashVars = 'debug=1';
    }

    if (!_s.wmode) {
      delete oEmbed.wmode; // don't write empty attribute
    }

    if (_isIE) {
      // IE is "special".
      oMovie = _doc.createElement('div');
      movieHTML = '<object id="' + smID + '" data="' + smURL + '" type="' + oEmbed.type + '" width="' + oEmbed.width + '" height="' + oEmbed.height + '"><param name="movie" value="' + smURL + '" /><param name="AllowScriptAccess" value="' + _s.allowScriptAccess + '" /><param name="quality" value="' + oEmbed.quality + '" />' + (_s.wmode?'<param name="wmode" value="' + _s.wmode + '" /> ':'') + '<param name="bgcolor" value="' + _s.bgColor + '" />' + (_s.debugFlash?'<param name="FlashVars" value="' + oEmbed.FlashVars + '" />':'') + '<!-- --></object>';
    } else {
      oMovie = _doc.createElement('embed');
      for (tmp in oEmbed) {
        if (oEmbed.hasOwnProperty(tmp)) {
          oMovie.setAttribute(tmp, oEmbed[tmp]);
        }
      }
    }

    _initDebug();
    extraClass = _getSWFCSS();
    oTarget = _getDocument();

    if (oTarget) {
      _s.oMC = _id(_s.movieID)?_id(_s.movieID):_doc.createElement('div');
      if (!_s.oMC.id) {
        _s.oMC.id = _s.movieID;
        _s.oMC.className = _s.swfCSS.swfDefault + ' ' + extraClass;
        // "hide" flash movie
        s = null;
        oEl = null;
        if (!_s.useFlashBlock) {
          if (_s.useHighPerformance) {
            s = {
              'position': 'fixed',
              'width': '8px',
              'height': '8px',
              // >= 6px for flash to run fast, >= 8px to start up under Firefox/win32 in some cases. odd? yes.
              'bottom': '0px',
              'left': '0px',
              'overflow': 'hidden',
              'hasPriority': 'true' // http://help.adobe.com/en_US/as3/mobile/WS4bebcd66a74275c36cfb8137124318eebc6-7ffd.html
            };
          } else {
            s = {
              'position': 'absolute',
              'width': '6px',
              'height': '6px',
              'top': '-9999px',
              'left': '-9999px',
              'hasPriority': 'true'
            };
            if (isRTL) {
              s.left = Math.abs(parseInt(s.left,10))+'px';
            }
          }
        }
        if (_isWebkit) {
          _s.oMC.style.zIndex = 10000; // soundcloud-reported render/crash fix, safari 5
        }
        if (!_s.debugFlash) {
          for (x in s) {
            if (s.hasOwnProperty(x)) {
              _s.oMC.style[x] = s[x];
            }
          }
        }
        try {
          if (!_isIE) {
            _s.oMC.appendChild(oMovie);
          }
          oTarget.appendChild(_s.oMC);
          if (_isIE) {
            oEl = _s.oMC.appendChild(_doc.createElement('div'));
            oEl.className = _s.swfCSS.swfBox;
            oEl.innerHTML = movieHTML;
          }
          _appendSuccess = true;
        } catch(e) {
          throw new Error(_str('appXHTML'));
        }
      } else {
        // it's already in the document.
        sClass = _s.oMC.className;
        _s.oMC.className = (sClass?sClass+' ':_s.swfCSS.swfDefault) + (extraClass?' '+extraClass:'');
        _s.oMC.appendChild(oMovie);
        if (_isIE) {
          oEl = _s.oMC.appendChild(_doc.createElement('div'));
          oEl.className = _s.swfCSS.swfBox;
          oEl.innerHTML = movieHTML;
        }
        _appendSuccess = true;
      }
    }

    if (specialCase) {
      //_s._wD(specialCase);
    }

    _initMsg();
    //_s._wD(_smc+'createMovie(): Trying to load ' + smURL + (!_s._overHTTP && _s.altURL?' (alternate URL)':''), 1);

    return true;
  };

  _idCheck = this.getSoundById;

  _initMovie = function() {
    if (_html5Only) {
      _createMovie();
      return false;
    }
    // attempt to get, or create, movie
    if (_s.o) {
      return false; // may already exist
    }
    _s.o = _s.getMovie(_s.id); // inline markup
    if (!_s.o) {
      if (!_oRemoved) {
        // try to create
        _createMovie(_s.id, _s.url);
      } else {
        // try to re-append removed movie after reboot()
        if (!_isIE) {
          _s.oMC.appendChild(_oRemoved);
        } else {
          _s.oMC.innerHTML = _oRemovedHTML;
        }
        _oRemoved = null;
        _didAppend = true;
      }
      _s.o = _s.getMovie(_s.id);
    }
    if (_s.o) {
      //_s._wD(_smc+'initMovie(): Got '+_s.o.nodeName+' element ('+(_didAppend?'created via JS':'static HTML')+')');
      //_wDS('waitEI');
    }
    if (_s.oninitmovie instanceof Function) {
      setTimeout(_s.oninitmovie, 1);
    }
    return true;
  };

  _go = function(sURL) {
    // where it all begins.
    if (sURL) {
      _s.url = sURL;
    }
    _initMovie();
  };

  _delayWaitForEI = function() {
    setTimeout(_waitForEI, 500);
  };

  _waitForEI = function() {
    if (_waitingForEI) {
      return false;
    }
    _waitingForEI = true;
    _event.remove(_win, 'load', _delayWaitForEI);
    if (_tryInitOnFocus && !_isFocused) {
      //_wDS('waitFocus');
      return false;
    }
    var p;
    if (!_didInit) {
      p = _s.getMoviePercent();
      //_s._wD(_str('waitImpatient', (p === 100?' (SWF loaded)':(p > 0?' (SWF ' + p + '% loaded)':''))));
    }
    setTimeout(function() {
      p = _s.getMoviePercent();
      if (!_didInit) {
        //_s._wD(_sm + ': No Flash response within expected time.\nLikely causes: ' + (p === 0?'Loading ' + _s.movieURL + ' may have failed (and/or Flash ' + _fV + '+ not present?), ':'') + 'Flash blocked or JS-Flash security error.' + (_s.debugFlash?' ' + _str('checkSWF'):''), 2);
        if (!_s._overHTTP && p) {
          //_wDS('localFail', 2);
          if (!_s.debugFlash) {
            //_wDS('tryDebug', 2);
          }
        }
        if (p === 0) {
          // if 0 (not null), probably a 404.
          //_s._wD(_str('swf404', _s.url));
        }
        //_debugTS('flashtojs', false, ': Timed out' + _s._overHTTP?' (Check flash security or flash blockers)':' (No plugin/missing SWF?)');
      }
      // give up / time-out, depending
      if (!_didInit && _okToDisable) {
        if (p === null) {
          // SWF failed. Maybe blocked.
          if (_s.useFlashBlock || _s.flashLoadTimeout === 0) {
            if (_s.useFlashBlock) {
              _flashBlockHandler();
            }
            //_wDS('waitForever');
          } else {
            // old SM2 behaviour, simply fail
            _failSafely(true);
          }
        } else {
          // flash loaded? Shouldn't be a blocking issue, then.
          if (_s.flashLoadTimeout === 0) {
             //_wDS('waitForever');
          } else {
            _failSafely(true);
          }
        }
      }
    }, _s.flashLoadTimeout);
  };

  _go = function(sURL) {
    // where it all begins.
    if (sURL) {
      _s.url = sURL;
    }
    _initMovie();
  };

  /*
  _wDS = function(o, errorLevel) {
    if (!o) {
      return '';
    } else {
      return //_s._wD(_str(o), errorLevel);
    }
  };

  if (_wl.indexOf('debug=alert') + 1 && _s.debugMode) {
    _s._wD = function(sText) {window.alert(sText);};
  }

  _toggleDebug = function() {
    var o = _id(_s.debugID),
    oT = _id(_s.debugID + '-toggle');
    if (!o) {
      return false;
    }
    if (_debugOpen) {
      // minimize
      oT.innerHTML = '+';
      o.style.display = 'none';
    } else {
      oT.innerHTML = '-';
      o.style.display = 'block';
    }
    _debugOpen = !_debugOpen;
  };

  _debugTS = function(sEventType, bSuccess, sMessage) {
    // troubleshooter debug hooks
    if (typeof sm2Debugger !== 'undefined') {
      try {
        sm2Debugger.handleEvent(sEventType, bSuccess, sMessage);
      } catch(e) {
        // oh well
      }
    }
    return true;
  };
  */

  _getSWFCSS = function() {
    var css = [];
    if (_s.debugMode) {
      css.push(_s.swfCSS.sm2Debug);
    }
    if (_s.debugFlash) {
      css.push(_s.swfCSS.flashDebug);
    }
    if (_s.useHighPerformance) {
      css.push(_s.swfCSS.highPerf);
    }
    return css.join(' ');
  };

  _flashBlockHandler = function() {
    // *possible* flash block situation.
    var name = _str('fbHandler'), p = _s.getMoviePercent(), css = _s.swfCSS;
    if (!_s.ok()) {
      if (_needsFlash) {
        // make the movie more visible, so user can fix
        _s.oMC.className = _getSWFCSS() + ' ' + css.swfDefault + ' ' + (p === null?css.swfTimedout:css.swfError);
        //_s._wD(name+': '+_str('fbTimeout')+(p?' ('+_str('fbLoaded')+')':''));
      }
      _s.didFlashBlock = true;
      _processOnEvents({type:'ontimeout',ignoreInit:true}); // fire onready(), complain lightly
      if (_s.onerror instanceof Function) {
        _s.onerror.apply(_win);
      }
    } else {
      // SM2 loaded OK (or recovered)
      if (_s.didFlashBlock) {
        //_s._wD(name+': Unblocked');
      }
      if (_s.oMC) {
        _s.oMC.className = [_getSWFCSS(), css.swfDefault, css.swfLoaded + (_s.didFlashBlock?' '+css.swfUnblocked:'')].join(' ');
      }
    }
  };

  _handleFocus = function() {
    function cleanup() {
      _event.remove(_win, 'focus', _handleFocus);
      _event.remove(_win, 'load', _handleFocus);
    }
    if (_isFocused || !_tryInitOnFocus) {
      cleanup();
      return true;
    }
    _okToDisable = true;
    _isFocused = true;
    //_s._wD(_smc+'handleFocus()');
    if (_isSafari && _tryInitOnFocus) {
      // giant Safari 3.1 hack - assume mousemove = focus given lack of focus event
      _event.remove(_win, 'mousemove', _handleFocus);
    }
    // allow init to restart
    _waitingForEI = false;
    cleanup();
    return true;
  };

  _initComplete = function(bNoDisable) {
    if (_didInit) {
      return false;
    }
    if (_html5Only) {
      // all good.
      //_s._wD('-- SoundManager 2: loaded --');
      _didInit = true;
      _processOnEvents();
      _initUserOnload();
      return true;
    }
    var sClass = _s.oMC.className,
    wasTimeout = (_s.useFlashBlock && _s.flashLoadTimeout && !_s.getMoviePercent());
    if (!wasTimeout) {
      _didInit = true;
    }
    //_s._wD('-- SoundManager 2 ' + (_disabled?'failed to load':'loaded') + ' (' + (_disabled?'security/load error':'OK') + ') --', 1);
    if (_disabled || bNoDisable) {
      if (_s.useFlashBlock) {
        _s.oMC.className = _getSWFCSS() + ' ' + (_s.getMoviePercent() === null?_s.swfCSS.swfTimedout:_s.swfCSS.swfError);
      }
      _processOnEvents({type:'ontimeout'});
      //_debugTS('onload', false);
      if (_s.onerror instanceof Function) {
        _s.onerror.apply(_win);
      }
      return false;
    } else {
      //_debugTS('onload', true);
    }
    _event.add(window, 'unload', _doNothing); // prevent browser from showing cached state via back button, because flash will be dead
    if (_s.waitForWindowLoad && !_windowLoaded) {
      //_wDS('waitOnload');
      _event.add(_win, 'load', _initUserOnload);
      return false;
    } else {
      if (_s.waitForWindowLoad && _windowLoaded) {
        //_wDS('docLoaded');
      }
      _initUserOnload();
    }
    return true;
  };

  _addOnEvent = function(sType, oMethod, oScope) {
    if (typeof _on_queue[sType] === 'undefined') {
      _on_queue[sType] = [];
    }
    _on_queue[sType].push({
      'method': oMethod,
      'scope': (oScope || null),
      'fired': false
    });
  };

  _processOnEvents = function(oOptions) {
    if (!oOptions) { // assume onready, if unspecified
      oOptions = {
        type: 'onready'
      };
    }
    if (!_didInit && oOptions && !oOptions.ignoreInit) {
      // not ready yet.
      return false;
    }
    var status = {
      success: (oOptions && oOptions.ignoreInit?_s.ok():!_disabled)
    },
    srcQueue = (oOptions && oOptions.type?_on_queue[oOptions.type]||[]:[]), // queue specified by type, or none
    queue = [], i, j,
    canRetry = (_needsFlash && _s.useFlashBlock && !_s.ok());
    for (i = 0; i < srcQueue.length; i++) {
      if (srcQueue[i].fired !== true) {
        queue.push(srcQueue[i]);
      }
    }
    if (queue.length) {
      //_s._wD(_sm + ': Firing ' + queue.length + ' '+oOptions.type+'() item' + (queue.length === 1?'':'s'));
      for (i = 0, j = queue.length; i < j; i++) {
        if (queue[i].scope) {
          queue[i].method.apply(queue[i].scope, [status]);
        } else {
          queue[i].method(status);
        }
        if (!canRetry) { // flashblock case doesn't count here
          queue[i].fired = true;
        }
      }
    }
    return true;
  };

  _initUserOnload = function() {
    _win.setTimeout(function() {
      if (_s.useFlashBlock) {
        _flashBlockHandler();
      }
      _processOnEvents();
      // call user-defined "onload", scoped to window
      if (_s.onload instanceof Function) {
        //_wDS('onload', 1);
        _s.onload.apply(_win);
        //_wDS('onloadOK', 1);
      }
      if (_s.waitForWindowLoad) {
        _event.add(_win, 'load', _initUserOnload);
      }
    },1);
  };

  _detectFlash = function() {

    // hat tip: Flash Detect library (BSD, (C) 2007) by Carl "DocYes" S. Yestrau - http://featureblend.com/javascript-flash-detection-library.html / http://featureblend.com/license.txt

    if (_hasFlash !== undefined) {
      // this work has already been done.
      return _hasFlash;
    }

    var hasPlugin = false, i, n = navigator, nP = n.plugins, mime, obj, type, types, axBase, axNames, getAX, AX = window.ActiveXObject;

    if (nP && nP.length) {

      type = 'application/x-shockwave-flash';
      types = n.mimeTypes;
      if (types && types[type] && types[type].enabledPlugin && types[type].enabledPlugin.description) {
        hasPlugin = true;
      }

    } else if (typeof AX !== 'undefined') {

      try {
        obj = new AX('ShockwaveFlash.ShockwaveFlash');
      } catch(e) {
        // oh well
      }
      hasPlugin = (!!obj);

    }

    return hasPlugin;

  };

  _featureCheck = function() {
    var needsFlash, item,
    isSpecial = (_ua.match(/iphone os (1|2|3_0|3_1)/i)?true:false); // iPhone <= 3.1 has broken HTML5 audio(), but firmware 3.2 (iPad) + iOS4 works.
    if (isSpecial) {
      _s.hasHTML5 = false; // has Audio(), but is broken; let it load links directly.
      _html5Only = true; // ignore flash case, however
      if (_s.oMC) {
        _s.oMC.style.display = 'none';
      }
      return false;
    }
    if (_s.useHTML5Audio) {
      if (!_s.html5 || !_s.html5.canPlayType) {
        //_s._wD('SoundManager: No HTML5 Audio() support detected.');
        _s.hasHTML5 = false;
        return true;
      } else {
        _s.hasHTML5 = true;
      }
      if (_isBadSafari) {
        _hasFlash = _detectFlash();
        //_s._wD(_smc+'Note: Buggy HTML5 Audio in Safari on OS X 10.6.[3|4|5], see https://bugs.webkit.org/show_bug.cgi?id=32159 - '+(!_hasFlash?' would use flash fallback for MP3/MP4, but none detected.':'will use flash fallback for MP3/MP4, if available'),1);
        if (_hasFlash) {
          return true;
        }
      }
    } else {
      // flash required.
      return true;
    }
    for (item in _s.audioFormats) {
      if (_s.audioFormats.hasOwnProperty(item) && _s.audioFormats[item].required && !_s.html5.canPlayType(_s.audioFormats[item].type)) {
        // may need flash for this format?
        needsFlash = true;
      }
    }
    // sanity check..
    if (_s.ignoreFlash) {
      needsFlash = false;
    }
    _html5Only = (_s.useHTML5Audio && _s.hasHTML5 && !needsFlash && !_s.requireFlash);
    return needsFlash;
  };

  _init = function() {
    var item, tests = [];
    //_wDS('init');

    // called after onload()
    if (_didInit) {
      //_wDS('didInit');
      return false;
    }

    function _cleanup() {
      _event.remove(_win, 'load', _s.beginDelayedInit);
    }

    if (_s.hasHTML5) {
      for (item in _s.audioFormats) {
        if (_s.audioFormats.hasOwnProperty(item)) {
          tests.push(item+': '+_s.html5[item]);
        }
      }
      //_s._wD('-- SoundManager 2: HTML5 support tests ('+_s.html5Test+'): '+tests.join(', ')+' --',1);
    }

    if (_html5Only) {
      if (!_didInit) {
        // we don't need no steenking flash!
        _cleanup();
        _s.enabled = true;
        _initComplete();
      }
      return true;
    }

    // flash path
    _initMovie();
    try {
      //_wDS('flashJS');
      _s.o._externalInterfaceTest(false); // attempt to talk to Flash
      if (!_s.allowPolling) {
        //_wDS('noPolling', 1);
      } else {
        _setPolling(true, _s.flashPollingInterval ? _s.flashPollingInterval : (_s.useFastPolling ? 10 : 50));
      }
      if (!_s.debugMode) {
        _s.o._disableDebug();
      }
      _s.enabled = true;
      //_debugTS('jstoflash', true);
    } catch(e) {
      //_s._wD('js/flash exception: ' + e.toString());
      //_debugTS('jstoflash', false);
      _failSafely(true); // don't disable, for reboot()
      _initComplete();
      return false;
    }
    _initComplete();
    // event cleanup
    _cleanup();
    return true;
  };

  _beginInit = function() {
    if (_initPending) {
      return false;
    }
    _createMovie();
    _initMovie();
    _initPending = true;
    return true;
  };

  _dcLoaded = function() {
    if (_didDCLoaded) {
      return false;
    }
    _didDCLoaded = true;
    _initDebug();
    _testHTML5();
    _s.html5.usingFlash = _featureCheck();
    _needsFlash = _s.html5.usingFlash;
    _didDCLoaded = true;
    if (_doc.removeEventListener) {
      _doc.removeEventListener('DOMContentLoaded', _dcLoaded, false);
    }
    _go();
    return true;
  };

  _startTimer = function(oSound) {
    if (!oSound._hasTimer) {
      oSound._hasTimer = true;
    }
  };

  _stopTimer = function(oSound) {
    if (oSound._hasTimer) {
      oSound._hasTimer = false;
    }
  };

  _die = function() {
    if (_s.onerror instanceof Function) {
      _s.onerror();
    }
    _s.disable();
  };

  _badSafariFix = function() {
    // special case: "bad" Safari can fall back to flash for MP3/MP4
    if (!_isBadSafari || !_hasFlash) {
      return false; // doesn't apply
    }
    var aF = _s.audioFormats, i, item;
    for (item in aF) {
      if (aF.hasOwnProperty(item)) {
        // special case: "bad" Safari can fall back to flash for MP3/MP4
        if (item === 'mp3' || item === 'mp4') {
          //_s._wD(_sm+': Using flash fallback for '+item+' format');
          _s.html5[item] = false;
          // assign result to related formats, too
          if (aF[item] && aF[item].related) {
            for (i<aF[item].related.length; i--;) {
              _s.html5[aF[item].related[i]] = false;
            }
          }
        }
      }
    }
  };

  // pseudo-private methods called by Flash

  this._setSandboxType = function(sandboxType) {
    /*
    var sb = _s.sandbox;
    sb.type = sandboxType;
    sb.description = sb.types[(typeof sb.types[sandboxType] !== 'undefined'?sandboxType:'unknown')];
    //_s._wD('Flash security sandbox type: ' + sb.type);
    if (sb.type === 'localWithFile') {
      sb.noRemote = true;
      sb.noLocal = false;
      //_wDS('secNote', 2);
    } else if (sb.type === 'localWithNetwork') {
      sb.noRemote = false;
      sb.noLocal = true;
    } else if (sb.type === 'localTrusted') {
      sb.noRemote = false;
      sb.noLocal = false;
    }
    */
  };

  this._externalInterfaceOK = function(flashDate) {
    // flash callback confirming flash loaded, EI working etc.
    // flashDate = approx. timing/delay info for JS/flash bridge
    if (_s.swfLoaded) {
      return false;
    }
    var eiTime = new Date().getTime();
    //_s._wD(_smc+'externalInterfaceOK()' + (flashDate?' (~' + (eiTime - flashDate) + ' ms)':''));
    //_debugTS('swf', true);
    //_debugTS('flashtojs', true);
    _s.swfLoaded = true;
    _tryInitOnFocus = false;
    if (_isBadSafari) {
      _badSafariFix();
    }
    if (_isIE) {
      // IE needs a timeout OR delay until window.onload - may need TODO: investigating
      setTimeout(_init, 100);
    } else {
      _init();
    }
  };

  _dcIE = function() {
    if (_doc.readyState === 'complete') {
      _dcLoaded();
      _doc.detachEvent('onreadystatechange', _dcIE);
    }
    return true;
  };

  // focus and window load, init
  if (!_s.hasHTML5 || _needsFlash) {
    // only applies to Flash mode
    _event.add(_win, 'focus', _handleFocus);
    _event.add(_win, 'load', _handleFocus);
    _event.add(_win, 'load', _delayWaitForEI);
    if (_isSafari && _tryInitOnFocus) {
      _event.add(_win, 'mousemove', _handleFocus); // massive Safari focus hack
    }
  }

  if (_doc.addEventListener) {
    _doc.addEventListener('DOMContentLoaded', _dcLoaded, false);
  } else if (_doc.attachEvent) {
    _doc.attachEvent('onreadystatechange', _dcIE);
  } else {
    // no add/attachevent support - safe to assume no JS -> Flash either
    //_debugTS('onload', false);
    _die();
  }

  if (_doc.readyState === 'complete') {
    setTimeout(_dcLoaded,100);
  }

} // SoundManager()

// SM2_DEFER details: http://www.schillmania.com/projects/soundmanager2/doc/getstarted/#lazy-loading
if (typeof SM2_DEFER === 'undefined' || !SM2_DEFER) {
  soundManager = new SoundManager();
}

// public interfaces
window.SoundManager = SoundManager; // constructor
window.soundManager = soundManager; // public API, flash callbacks etc

}(window));
soundManager.onload = function() {
	soundManager.createSound('messenger', 'sound/messenger.mp3');
	soundManager.createSound('buy', 'sound/buy.mp3');
	client.sound = true;
}
soundManager.url = 'sound/';
var texts = {};texts["chatbox_closed"] = "De chatbox is gesloten.";texts["cat_nofurni"] = "Er staan geen meubels in deze collectie.";texts["cat_noaccess"] = "Je hebt geen toegang tot deze collectie.";texts["cat_error"] = "Technische fout! Collectie kon niet geladen worden.";texts["client_title"] = "Jabbo";texts["client_description"] = "Jabbo is jouw virtuele wereld waar je vrienden kunt maken en ontmoeten!";texts["chromeframe_ask"] = "Je hebt Google Frame nodig<br>om Jabbo te kunnen spelen.";texts["closed_maintainance"] = "Het Hotel is tijdelijk gesloten voor onderhoud.";texts["chromeframe_bar"] = "Internet Explorer mist software om deze webpagina weer te kunnen geven. Klik hier om te updaten.";texts["close_client"] = "Sluit Hotel";texts["jabbo_1"] = "Jabbo";texts["jabbo_multi"] = "Jabbo's";texts["client_online"] = "in het Hotel";texts["debug_start"] = "Jabbo client - debug";texts["team_title"] = "Jabbo Team";texts["login_firstTimeHere"] = "Eerste keer hier?";texts["login_haventGotUser"] = "Heb je nog geen Jabbo?";texts["login_create1here"] = "Hier kun je er een maken.";texts["login_whatsJabboCalled"] = "Hoe heet jouw Jabbo?";texts["login_name"] = "Naam van je Jabbo";texts["login_pass"] = "Wachtwoord";texts["button_ok"] = "OK";texts["login_connecting"] = "Verbinden...";texts["reg_details"] = "Jabbo details";texts["reg_connecting"] = "Verbinding maken...";texts["reg_balloon_text"] = "Welkom bij Jabbo Hotel!";texts["reg_createjabbo"] = "Maak je eigen Jabbo";texts["reg_welcometext1"] = "Aan de slag! Nog snel even registreren en je bent klaar om te beginnen. Dat is zo gebeurd! Je kunt alles later veranderen behalve je Jabbonaam.";texts["reg_welcometext2"] = "BEWAAR DE GEGEVENS DIE JE HIER INVULT GOED! Je hebt ze later nodig om iets aan je account te kunnen veranderen!";texts["reg_cancel"] = "Cancel";texts["reg_underage"] = "Ik ben jonger dan 12 jaar";texts["reg_olderage"] = "Ik ben 12 jaar of ouder";texts["reg_legal_header"] = "Lees onderstaande tekst zorgvuldig. Je moet de voorwaarden accepteren voordat je het Hotel in kunt.";texts["reg_legal_agree"] = "Ja, ik heb de Algemene Voorwaarden van het Hotel gelezen en ik ga ermee akkoord.";texts["reg_back"] = "Terug";texts["reg_next"] = "Volgende";texts["reg_name_note"] = "Nu is het tijd om je Jabbo karakter aan te maken. Om te beginnen kies je een naam voor je Jabbo.";texts["reg_pwd_note"] = "Maak je wachtwoord moeilijk te raden. Gebruik 6 tot 9 tekens, waaronder cijfers, grote en kleine letters.";texts["reg_pass1"] = "Wachtwoord";texts["reg_pass2"] = "Opnieuw";texts["reg_email_note"] = "Typ hier je e-mailadres. Belangrijk! Voor sommige helpfuncties of diensten heb je een werkend e-mailadres nodig.";texts["reg_birthdayformat"] = "Geboortedatum:*";texts["reg_birthday_example"] = "(voorbeeld: '24-5-1993')";texts["reg_promise"] = "*Jouw gegevens zullen aan niemand anders gegeven of worden voor marktonderzoek. Je zal geen mails ontvangen als je dat niet wilt.";texts["reg_check_info"] = "CONTROLEER JE GEGEVENS";texts["reg_check_confirm"] = "Controleer je gegevens, in het bijzonder je geboortedatum en je naam. Deze zijn hierna niet meer te veranderen. Als je nog iets wilt wijzigen, klik dan op de 'terug' knop.<br><br>BEWAAR DEZE GEGEVENS, je hebt ze later nodig om iets aan je Jabbo account te kunnen veranderen!";texts["reg_spam"] = "Ja, ik ontvang graag mails met informatie over events, campagnes en andere funstuff via e-mail";texts["reg_doneheader"] = "<b>Gefeliciteerd!<br><br>Je bent een Jabbo!</b>";texts["reg_donetext"] = "Dadelijk zie je de Hotel Navigator, met daarin alle kamers waar je heen kunt gaan - de gastenkamers, clubs, restaurants, zwembaden en meer!";texts["reg_done_button"] = "Klaar";texts["alert_connection_lost"] = "Verbinding verbroken";texts["alert_restart"] = "Herstart Jabbo Hotel!";texts["alert_restart_wait"] = "Als het probleem opnieuw voorkomt, wacht dan even voor het<br>herstarten.";texts["connection_lost_ok"] = "ok=>";texts["inv_back"] = "<<";texts["inv_next"] = ">>";texts["inv_close"] = "Sluit";texts["vote_text"] = "Stem op kamer";texts["room_loading"] = "Wacht even...<br>Kamer wordt geladen";texts["config_furni_stop"] = "X";texts["config_furni_session"] = "Sessie";texts["config_furni_save"] = "Opslaan";texts["buy_presentIsFor"] = "Dit kado is voor:";texts["buy_presentMessage"] = "Type hier je bericht<br>(vergeet ook je naam niet te vermelden!)";texts["buy_stop"] = "Afbreken";texts["buy_ok"] = "OK";texts["buy_amount"] = "Aantal:";texts["team_code"] = "Code";texts["team_hosting"] = "Hosting";texts["team_managers"] = "Managers";texts["team_staff"] = "Staff";texts["team_other"] = "Andere teamleden";texts["team_close"] = "OK";texts["crcode_title"] = "Jabbo Credit Codes";texts["crcode_enter"] = "Typ de Code hier:";texts["crcode_send"] = "Zend";texts["crcode_help"] = "Hier kun je een Jabbo Credit Code omzetten in Credits of meubels. Een Credit Code bestaat uit cijfers, hoofdletters en kleine letters. Als je zo'n speciale code hebt, neem de Code dan exact zo over als je hem gekregen hebt in het veld hierboven.";texts["crcode_stop"] = "Afbreken";texts["camera_ask_title"] = "Camera";texts["camera_ask_text"] = "Wil je een foto maken of je<br> camera in je kamer neerzetten?";texts["camera_ask_room"] = "Kamer";texts["camera_ask_shoot"] = "Foto";texts["chatbox_title"] = "Chatbox";texts["chatbox_loading"] = "Bezig met laden...";texts["messenger_title"] = "Jabbo Console";texts["messenger_newmessages"] = "Nieuw(e) bericht(en)";texts["messenger_newrequests"] = "Vriendenverzoek(en)";texts["messenger_write_button"] = "Schrijf een bericht";texts["messenger_delbuddy"] = "Verwijder";texts["messenger_search_button"] = "Zoek";texts["messenger_askbuddy"] = "Vraag om vrienden te worden";texts["messenger_help_title"] = "Console help";texts["messenger_help_text"] = "Je kunt de Console gebruiken om in contact te blijven met je vrienden, zelfs als die offline zijn. Je oude berichten kan je bekijken op de website.";texts["messenger_askbuddy_ok"] = "OK";texts["messenger_askbuddy_no"] = "Weigeren";texts["messenger_askbuddy_yes"] = "Accept";texts["messenger_friendhelp_title"] = "Vrienden";texts["messenger_friendhelp_text"] = "Dit scherm bevat een lijst met al je Jabbo Vrienden. Je kunt er zien of ze online zijn, of, als ze niet zijn ingecheckt, wanneer ze er voor het laatst waren.";texts["messenger_friendhelp_ok"] = "Terug";texts["messenger_frienddel_no"] = "Afbreken";texts["messenger_frienddel_yes"] = "Verwijderen";texts["messenger_recipient"] = "Ontvanger";texts["messenger_stopwrite"] = "Afbreken";texts["messenger_sendwrite"] = "Verzenden";texts["messenger_writehelp_title"] = "Berichten sturen";texts["messenger_writehelp_text"] = "Hier kun je berichten sturen naar je vrienden. Zo kun je in contact blijven, ook als ze offline zijn! Oude berichten kun je altijd raadplegen op de homepage.";texts["messenger_writehelp_ok"] = "Terug";texts["messenger_reply"] = "Antwoorden";texts["messenger_delmessage"] = "Verwijder";texts["lang"] = "nl";texts["navi_title"] = "Hotel Navigator";texts["navi_publics"] = "Openbare ruimten";texts["navi_guestrooms"] = "Gastenkamers";texts["navi_search"] = "Zoek";texts["navi_ownrooms"] = "Eigen kamer(s)";texts["navi_favo"] = "Favorieten";texts["navi_searchresults"] = "Zoekresultaten";texts["navi_yourRooms"] = "Jouw kamers";texts["navi_yourFavos"] = "Jouw favoriete kamers";texts["navi_change_stop"] = "Cancel";texts["navi_change_del"] = "Verwijder kamer";texts["navi_change_ok"] = "OK";texts["navi_change_roomname"] = "Naam van de kamer";texts["navi_change_roomdesc"] = "Beschrijving kamer";texts["navi_change_lock1"] = "Deur is open";texts["navi_change_lock2"] = "Deur is op slot, alleen jij kan binnen";texts["navi_change_lock3"] = "Wachtwoord voor beperkte toegang";texts["navi_delroom1"] = "KAMER VERWIJDEREN<br>Als je de spullen in de kamer wilt<br>houden, zet ze dan in je grote hand<br>voor je verder gaat!";texts["navi_delroom2"] = "Weet je zeker dat je DEZE kamer<br>wilt verwijderen? Alle meubels die er<br>nog in staan ben je dan ook kwijt.<br>Dit kan niet worden teruggedraaid!";texts["navi_roomgone"] = "Kamer verwijderd!";texts["navi_stopdel"] = "Cancel";texts["navi_delroom"] = "Verwijder kamer";texts["navi_roomgone_ok"] = "OK";texts["navi_enterpwd_stop"] = "Cancel";texts["navi_enterpwd_ok"] = "Ok";texts["navi_search_help"] = "Zoek kamers op eigenaar of kamernaam.";texts["navi_search_button"] = "Zoek";texts["navi_matic_help"] = "Hier kun je je eigen kamer maken.";texts["navi_matic_button"] = "Maak een kamer";texts["navi_publics_help"] = "Dit zijn de openbare ruimten van de hotel. Waar wacht je nog op? Ga erheen en ontmoet er anderen!";texts["navi_room_go"] = "Ga";texts["navi_addfavo"] = "Zet bij favorieten";texts["navi_delfavo"] = "Verwijder uit favorieten";texts["navi_change_button"] = "Wijzig";texts["catalogue_crhelp_text"] = "Voor elke minuut dat je Jabbo speelt, krijg je 1 Credit.<br>Wanneer je inactief bent, krijgt je geen Credits bij.<br>Leden van <b>Jabbo Club</b> krijgen 2 Credits per minuut bij.";texts["catalogue_crhelp"] = "Hoe krijg ik ze?";texts["catalogue_codes_button"] = "Codes";texts["catalogue_transactions_button"] = "Bekijk transacties";texts["catalogue_buy"] = "Koop";texts["catalogue_selectproduct"] = "Kies artikel";texts["catalogue_page"] = "Pagina";texts["camera_savefail"] = "De foto kon niet ontwikkeld worden.";texts["notify_error"] = "Fout";texts["cat_loadfail"] = "De collectie kon niet ingeladen worden.";texts["cat_nosuchfurni"] = "De collectie kon niet geladen worden<br>omdat hij onbekende meubels bevat.";texts["notify_default"] = "Let op!";texts["client_error_title"] = "Oeps, foutje!";texts["client_error_message"] = "Herstart Jabbo Hotel!";texts["notify_ok"] = "OK";texts["login_fill_both"] = "Vul je naam en wachtwoord allebei in!";texts["login_welcome"] = "Welkom";texts["login_fail"] = "Inloggen mislukt.";texts["login_max"] = "Je hebt je maximale aantal loginpogingen overschreden.";texts["login_mail"] = "Voordat je op Jabbo kan spelen moet je eerst je e-mailadres valideren.";texts["login_full"] = "Het Hotel zit vol.<br>Probeer het zometeen nog eens.";texts["login_ban"] = "Je bent verbannen voor het overtreden van de Jabbo Regels.";texts["login_wrong"] = "Onjuiste naam of wachtwoord. Probeer het nog eens!";texts["room_loadfail"] = "Kon de kamer niet laden.";texts["room_full"] = "De kamer is vol.";texts["room_score"] = "Deze kamer scoort:";texts["room_chatclick"] = "Klik hier om te praten...";texts["room_spam"] = "Je praat te snel!";texts["room_owner"] = "Eigenaar";texts["navi_go"] = "Ga";texts["navi_no_guestrooms"] = "Er zijn nog geen kamers van anderen.";texts["navi_no_publics"] = "Er zijn nog geen openbare ruimten.";texts["navi_guestrooms_help"] = "Dit zijn de kamers van de anderen. Ga eens bij een van hen op bezoek. En eeh, heb je zelf al een kamer?";texts["navi_searchhelp"] = "Zoek je iets? Hier kun je de kamers van anderen opzoeken. Typ de naam van de kamer of van de eigenaar.";texts["navi_ownrooms_help"] = "Voor het geval je het nog niet wist: dit zijn jouw kamers! Hier kun je ze veranderen of, als je wilt, nieuwe maken.";texts["navi_favo_help"] = "Dit zijn je favoriete kamers. Mooie selectie heb je hier!";texts["navi_roomowner"] = "Eigenaar";texts["navi_locked"] = "Deze kamer is op slot!";texts["navi_pass"] = "De kamer is afgesloten met een wachtwoord.";texts["messenger_nofriends"] = "Er staan geen vrienden in je lijst. Je kunt een Vriendenverzoek sturen via de Zoek knop.";texts["messenger_notfound"] = "Jabbo niet gevonden";texts["messenger_sender"] = "Afzender";texts["messenger_request_text"] = "wil graag vrienden met<br>je worden.<br><br>Als je dit verzoek<br>accepteert zal hij/zij<br>aan je Vriendenlijst<br>toegevoegd worden.<br>Dan kun je elkaar<br>berichten sturen.";texts["messenger_request_sent"] = "<br>heeft je verzoek<br>ontvangen.<br><br>Hij/zij zal aan jouw<br>vriendenlijst worden<br>toegevoegd als je<br>verzoek wordt<br>geaccepteerd.";texts["messenger_ask_del1"] = "Weet je zeker dat je";texts["messenger_ask_del2"] = "wilt verwijderen uit je<br>Vriendenlijst?";texts["money1"] = "Credit";texts["money2"] = "Credits";texts["catalogue_youHave"] = "Je hebt";texts["catalogue_earned1"] = "Je hebt al";texts["catalogue_earned2"] = "verdiend sinds je inlogde";texts["buy_poor"] = "Je hebt niet genoeg Credits in je Portemonnee.";texts["buy_error"] = "Fout tijdens het kopen.";texts["crcode_wait"] = "Code wordt gecontroleerd...";texts["catalogue_costs"] = "kost";texts["catalogue_yourMoney1"] = "Je hebt nog";texts["catalogue_yourMoney2"] = "in je Portemonnee.";texts["catalogue_buyAsPresent"] = "Koop als kado";texts["catalogue_itsyours"] = "Jeuh! Je aankoop wordt bezorgd.";texts["nofirebug"] = "Gelieve Firebug uit te schakelen.";texts["matic_size"] = "Kamergrootte";texts["matic_tiles"] = "tegels";texts["matic_descr"] = "Richt je kamer in zoals jij het wilt. Niemand kan je vertellen dat je op moet ruimen of je muziek zachter moet zetten. En je mag lekker zelf weten of je iedereen binnenlaat, of alleen je beste vrienden. Of doe de deur op slot en in je eentje op de bank zitten. Wees creatief en in een paar minuten heb jij ook je eigen kamer.";texts["matic_createyourown"] = "Ontwerp je eigen kamer";texts["matic_canmodifysettings"] = "Je kunt deze instellingen later aanpassen";texts["matic_chooselayout"] = "Kies het plattegrond van je kamer";texts["matic_security"] = "Beveiliging";texts["matic_longpwd"] = "Kamerwachtwoord mag maar 15 tekens lang zijn";texts["matic_checkpwd"] = "Controleer je wachtwoord";texts["matic_shortpwd"] = "Kamerwachtwoord is te kort";texts["matic_fillall"] = "Niet alles is ingevuld";texts["matic_roomname"] = "Naam kamer";texts["matic_owner"] = "Eigenaar";texts["matic_roomdescr"] = "Omschrijving kamer";texts["matic_safe1"] = "Deur is open";texts["matic_safe2"] = "Deur is op slot, alleen jij kan binnen";texts["matic_safe3"] = "Wachtwoord voor beperkte toegang";texts["matic_pass1"] = "Wachtwoord";texts["matic_pass2"] = "Opnieuw";texts["matic_fail"] = "Er is een fout opgetreden bij het maken van de kamer.<br>Probeer het nog eens!";texts["navi_no_ownrooms"] = "Je hebt geen eigen kamers.";texts["navi_no_finds"] = "Geen kamers gevonden op jouw zoekvraag.";texts["register_closed"] = "Registreren is tijdelijk niet mogelijk.";texts["club_title"] = "Jabbo Club";texts["club_intro_header"] = "Welkom bij Jabbo Club, voor Jabbo's die het meeste uit hun Jabbotijd willen halen.";texts["club_intro_text"] = "Leden van Jabbo Club krijgen een hoop leuke extra's!<br>Met Jabbo Club krijg je 400 extra plaatsen in je vriendenlijst, extra keuze uit kleren, elke maand een gratis Club meubel en nog veel meer!";texts["club_button_close"] = "Scherm sluiten";texts["club_intro_more"] = "Lees meer over Jabbo Club!";texts["club_buy"] = "Koop";texts["club_price"] = "1 maand (31 dagen) =  <b>&euro;2</b>";texts["club_buy_website"] = "Je kan Jabbo Club kopen op de website.";texts["messenger_list_full"] = "Je vriendenlijst is vol. Je kunt nu geen nieuwe namen toevoegen.<br>Je kunt maximaal 200 Jabbo's in je vriendenlijst hebben.<br><br>Jabbo Clubleden kunnen maximaal 600 vrienden in hun lijst hebben.";texts["club_expires_soon"] = "Je Jabbo Club lidmaatschap verloopt binnenkort! Tijd om te verlengen?";texts["voucher_success1"] = "Je hebt";texts["voucher_success2"] = "gekregen!";texts["voucher_novalue"] = "De Credit Code had geen waarde.";texts["voucher_error"] = "Technische fout! Code kon niet worden ingewisseld.";texts["voucher_incorrect"] = "Onjuiste Credit Code.";texts["camera_caption"] = "Bijschrift";texts["donate_text"] = "Waarschuwing! Als je iets in deze kamer plaatst kun je het niet meer weghalen.";texts["donate_proceed"] = "Doneer";texts["donate_stop"] = "Afbreken";texts["untradeable"] = "Dit kun je niet ruilen.";